#! /bin/sh

APPS_NAME="DSL API Driver"

if [ -e sysconfig.sh ]; then
	. sysconfig.sh
	. config.sh
	. model_config.sh
	. gen_funcs.sh
	. build_functions.sh 
else
        echo "Application "$APPS_NAME" not configured"
        exit 0
fi
display_info "----------------------------------------------------------------------"
display_info "-------------------- build $APPS_NAME ----------------------------"
display_info "----------------------------------------------------------------------"

parse_args $@

cd drv_dsl_cpe_api

if [ $BUILD_CLEAN -eq 1 ]; then
	if [ -e .config_ok ]; then
		make -s distclean
		rm -f .config_ok configure
	fi
	[ ! $BUILD_CONFIGURE -eq 1 ] && exit 0
fi
if [ ! -f .config_ok -o $BUILD_CONFIGURE -eq 1 ]; then
	if [ ! -e configure ]; then
		autoreconf -i
		if [ ! -e NEWS ]; then
		    touch NEWS
		fi
		if [ ! -e README ]; then
		    touch README
		fi
		if [ ! -e AUTHORS ]; then
		    touch AUTHORS
		fi
		if [ ! -e ChangeLog ]; then
		    touch ChangeLog
		fi
		if [ ! -e TODO ]; then
		    touch TODO
		fi
	fi
fi

if [ "$1" = "config_only" ] ;then
	exit 0
fi

KERNEL_INCLUDE=$KERNEL_INCLUDE_PATH
KERNEL_BUILD=
DRIVER_INCLUDE=-I$PWD/src/include
ARCH=mips
DEVICE=danube
DRIVER_CFLAGS="-Os -G 0 -mno-abicalls -mlong-calls -fno-pic \
 -finline-limit=100000 -mabi=32 -march=mips32 -Wa,-32 -Wa,-march=mips32 \
 -Wa,-mips32 -Wa,--trap -I${KERNEL_INCLUDE}/asm/mach-generic/"

#    --enable-ifxos
#    --enable-ifxos-include- \
echo ${KERNEL_INCLUDE}
ls -l ${KERNEL_INCLUDE}
./configure \
    CC=$IFX_CC \
    AR=$IFX_AR \
    LD=$IFX_LD \
    AS=$IFX_AS \
    STRIP=$IFX_STRIP \
    --build=i386-linux \
    --host=$ARCH-linux-uclibc \
    --enable-kernel-include=$KERNEL_INCLUDE \
    --enable-kernel-build="$KERNEL_BUILD" \
    --enable-driver-include="$DRIVER_INCLUDE" \
    --enable-add-drv-cflags="$DRIVER_CFLAGS" \
    --enable-$DEVICE \
    --with-channels-per-line=1 \
    --enable-library-build \
    --enable-debug=no\
    --enable-debug-prints=no \
    --enable-model=full\
    --enable-debug-tool-interface=no \
    --enable-dsl-g997-per-tone=no \
    --enable-dsl-delt=no \
    --enable-dsl-pm-channel-thresholds=no \
    --enable-dsl-pm-datapath-thresholds=no \
    --enable-dsl-pm-line-thresholds=no \
    --enable-dsl-pm-history=no \
    --enable-dsl-pm-optional-parameters=no \
    --enable-firmware-memory-free=no
#    --with-channels-per-line=1 \
#    --enable-debug=yes \
#    --enable-debug-prints=yes \
#    --enable-adsl-led \
#    --with-adsl-data-led-flash-frequency=4 \
#    --enable-adsl-mib \
#    --enable-dsl-procfiles \
#    --enable-dsl-ceoc \
#    --enable-dsl-delt-static \
#    --enable-dsl-pm \
#    --enable-dsl-pm-channel-counters \
#    --enable-dsl-pm-datapath-counters \
#    --enable-dsl-pm-line-counters \
#    --enable-dsl-pm-channel-thresholds \
#    --enable-dsl-pm-datapath-thresholds \
#    --enable-dsl-pm-line-thresholds \
#    --enable-dsl-pm-total \
#    --enable-dsl-pm-history \
#    --enable-dsl-pm-showtime \
#    --enable-dsl-pm-optional-parameters \
#    --enable-firmware-memory-free \
#    --enable-adsl-trace


ifx_error_check $?

make 
ifx_error_check $?

cd -
