/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

#define DSL_INTERN

#include "drv_dsl_cpe_api.h"
#include "drv_dsl_cpe_pm_core.h"

#if defined (INCLUDE_DSL_CPE_API_DANUBE) || defined (INCLUDE_DSL_CPE_API_AMAZON_SE) || \
    defined (INCLUDE_DSL_CPE_API_AMAZON_S)

#undef DSL_DBG_BLOCK
#define DSL_DBG_BLOCK DSL_DBG_PM

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_Start(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* No limitations to data access in case of Danube, Amazon-Se, Amazon-S*/
   DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid = DSL_FALSE;

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_Restart(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* Nothing to do*/

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_DEV_Suspend(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* Nothing to do*/
   /* Reset bPmDataValid flag*/
   DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid = DSL_FALSE;

   return nErrCode;
}

#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_ChannelCountersGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_XTUDir_t nDirection,
   DSL_PM_ChannelData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nIdx = 0;
   DSL_uint16_t nAddrCRC = 0, nAddrFEC = 0, nVal = 0;
   DSL_uint16_t nAdslMode = 0, nAdslMode1 = 0;
   DSL_uint32_t nLineState = DSL_LINESTATE_UNKNOWN;
   DSL_LatencyPath_t nLPath = DSL_LATENCY_DISABLED;

   DSL_uint16_t hdlcCmd[2] = {0};
   DSL_uint16_t hdlcRxBuffer[23];
   DSL_int_t nHdlcRxLen = 0;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_ChannelCountersCurrentGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_ATU_DIRECTION(nDirection);

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

   if (nDirection == DSL_FAR_END)
   {
       memcpy(pCounters, DSL_DRV_PM_PTR_CHANNEL_COUNTERS_CURR(nChannel, DSL_FAR_END),
          sizeof(DSL_PM_ChannelData_t));

      if (nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
      {
         return nErrCode;
      }
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode, nAdslMode);
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode1,
      nAdslMode1);

   if (DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Couldn't lock device mutex"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if (nDirection == DSL_NEAR_END)
   {
      nAddrCRC = DSL_CMV_ADDRESS_PLAM_CRC_NE;
      nAddrFEC = DSL_CMV_ADDRESS_PLAM_FEC_NE;
   }
   else
   {
      nAddrCRC = DSL_CMV_ADDRESS_PLAM_CRC_FE;
      nAddrFEC = DSL_CMV_ADDRESS_PLAM_FEC_FE;
   }

   if ((nAdslMode > 8 || nAdslMode1 != 0) && nDirection == DSL_FAR_END)
   {
      hdlcCmd[0] = 0x105;

      if ((nErrCode = DSL_DRV_DANUBE_HdlcWrite(pContext, DSL_FALSE,
         (DSL_uint8_t *)hdlcCmd, 2)) == DSL_SUCCESS)
      {
         DSL_WAIT (1);

         nErrCode = DSL_DRV_DANUBE_HdlcRead(pContext, DSL_FALSE,
            (DSL_uint8_t*)hdlcRxBuffer, 46, &nHdlcRxLen);
         if (nErrCode == DSL_SUCCESS)
         {
            pCounters->nFEC =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[1]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[2]);
            pCounters->nCodeViolations =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[3]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[4]);
         }
      }
   }
   else
   {
      if (DSL_DRV_DANUBE_ActLatencyGet(pContext, nChannel, &nLPath) >= DSL_SUCCESS)
      {
         if (nLPath == DSL_LATENCY_IP_LP0) nIdx = 0;
         else nIdx = 1;

         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            nAddrCRC, nIdx, 1, &nVal);
         if (nErrCode == DSL_SUCCESS)
         {
            pCounters->nCodeViolations = nVal;
         }

         if (nErrCode == DSL_SUCCESS)
         {
            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
               nAddrFEC, nIdx, 1, &nVal);
            if (nErrCode == DSL_SUCCESS)
            {
               pCounters->nFEC = nVal;
            }
         }
      }
      else
      {
         pCounters->nCodeViolations = 0;
         pCounters->nFEC = 0;
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: DSL_DRV_PM_DEV_ChannelCountersCurrentGet:"
      DSL_DRV_CRLF "\tnChannel = %lu"DSL_DRV_CRLF"\tnDirection=%lu"DSL_DRV_CRLF"\tnCRC=%lu"
      DSL_DRV_CRLF"\tnFEC=%lu"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
      nChannel, nDirection, pCounters->nCodeViolations, pCounters->nFEC));

   if (nErrCode == DSL_ERR_DEVICE_NO_DATA)
   {
      DSL_DEBUG( DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Timed out while waiting for HDLC data"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
      nErrCode = DSL_WRN_DEVICE_NO_DATA;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_ChannelCountersCurrentGet (retCode=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_ChannelCountersSet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_XTUDir_t nDirection,
   DSL_PM_ChannelData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nIdx = 0;
   DSL_uint16_t nAddrCRC, nAddrFEC, nVal;
   DSL_uint16_t nAdslMode = 0, nAdslMode1 = 0;
   DSL_LatencyPath_t nLPath = DSL_LATENCY_DISABLED;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_ChannelCountersCurrentSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_CHANNEL_RANGE(nChannel);

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if( nDirection != DSL_NEAR_END )
   {
      return DSL_SUCCESS;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Couldn't lock device mutex"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nAddrCRC = DSL_CMV_ADDRESS_PLAM_CRC_NE;
   nAddrFEC = DSL_CMV_ADDRESS_PLAM_FEC_NE;

   if ((nAdslMode > 8 || nAdslMode1 != 0) && (DSL_CHANNELS_PER_LINE > 1))
   {
      if (DSL_DRV_DANUBE_ActLatencyGet(pContext, nChannel, &nLPath) >= DSL_SUCCESS)
      {
         if (nLPath == DSL_LATENCY_IP_LP0) nIdx = 0;
         else nIdx = 1;

         nVal = pCounters->nCodeViolations;
         nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM, nAddrCRC,
            nIdx, 1, &nVal);

         if (nErrCode == DSL_SUCCESS)
         {
            nVal = pCounters->nFEC;
            nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM, nAddrFEC,
               nIdx, 1, &nVal);
         }
      }
   }
   else
   {
      /* In case of ADSL1 or single latency write values to both interl/LP0 and
         fast/LP1 */
      for (nIdx = 0; nIdx < 2; nIdx++)
      {
         nVal = pCounters->nCodeViolations;
         nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM, nAddrCRC,
            nIdx, 1, &nVal);

         if (nErrCode < DSL_SUCCESS) break;

         nVal = pCounters->nFEC;
         nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM, nAddrFEC,
            nIdx, 1, &nVal);

         if (nErrCode < DSL_SUCCESS) break;
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_ChannelCountersCurrentSet (retCode=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_DataPathCountersGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_XTUDir_t nDirection,
   DSL_PM_DataPathData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nAddrHEC, nAddrIBE, nAddrTotalCells, nAddrUserTotalCells = -1, nVal[2];
   DSL_uint16_t nAdslMode = 0, nAdslMode1 = 0;
   DSL_uint32_t nLineState = DSL_LINESTATE_UNKNOWN;

   DSL_uint16_t hdlcCmd[2] = {0};
   DSL_uint16_t hdlcRxBuffer[23];
   DSL_int_t nHdlcRxLen = 0;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_DataPathCountersCurrentGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_ATU_DIRECTION(nDirection);

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

   if (nDirection == DSL_FAR_END)
   {
       memcpy(pCounters, DSL_DRV_PM_PTR_DATAPATH_COUNTERS_CURR(nChannel, DSL_FAR_END),
          sizeof(DSL_PM_DataPathData_t));

      if (nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
      {
         return nErrCode;
      }
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode, nAdslMode);
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode1,
      nAdslMode1);

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Couldn't lock device mutex"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if (nDirection == DSL_NEAR_END)
   {
      nAddrHEC = DSL_CMV_ADDRESS_PLAM_HEC_NE;
      nAddrTotalCells = DSL_CMV_ADDRESS_PLAM_TOTAL_CELLS_NE;
      nAddrUserTotalCells = DSL_CMV_ADDRESS_PLAM_USER_TOTAL_CELLS_NE;
      nAddrIBE = DSL_CMV_ADDRESS_PLAM_IBE_NE;
   }
   else
   {
      nAddrHEC = DSL_CMV_ADDRESS_PLAM_HEC_FE;
      nAddrTotalCells = DSL_CMV_ADDRESS_PLAM_TOTAL_CELLS_FE;
/*$$      nAddrUserTotalCells = DSL_CMV_ADDRESS_PLAM_USER_TOTAL_CELLS_FE;*/
      nAddrIBE = DSL_CMV_ADDRESS_PLAM_IBE_FE;
   }

   if ((nAdslMode > 8 || nAdslMode1 != 0) && nDirection == DSL_FAR_END)
   {
      hdlcCmd[0] = 0x105;

      if ((nErrCode = DSL_DRV_DANUBE_HdlcWrite(pContext, DSL_FALSE,
         (DSL_uint8_t *)hdlcCmd, 2))
         == DSL_SUCCESS)
      {
         DSL_WAIT (1);

         nErrCode = DSL_DRV_DANUBE_HdlcRead(pContext, DSL_FALSE,
            (DSL_uint8_t*)hdlcRxBuffer, 46, &nHdlcRxLen);
         if (nErrCode == DSL_SUCCESS)
         {
            pCounters->nHEC =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[15]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[16]);
            pCounters->nTotalCells =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[17]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[18]);
            pCounters->nUserTotalCells =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[19]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[20]);
            pCounters->nIBE =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[21]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[22]);
         }
      }
   }
   else
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
         nAddrHEC, 2*nChannel, 1, &nVal[0]);
      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            nAddrHEC, 2*nChannel + 1, 1, &nVal[1]);

         if (nErrCode == DSL_SUCCESS)
         {
            pCounters->nHEC = (nVal[1] << 16) | nVal[0];
         }
      }

      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            nAddrTotalCells, 2*nChannel, 1, &nVal[0]);
         if (nErrCode == DSL_SUCCESS)
         {
            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
               nAddrTotalCells, 2*nChannel + 1, 1, &nVal[1]);

            if (nErrCode == DSL_SUCCESS)
            {
               pCounters->nTotalCells = (nVal[1] << 16) | nVal[0];
            }
         }
      }

      if (nErrCode == DSL_SUCCESS && nDirection == DSL_NEAR_END)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            nAddrUserTotalCells, 2*nChannel, 1, &nVal[0]);
         if (nErrCode == DSL_SUCCESS)
         {
            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
               nAddrUserTotalCells, 2*nChannel + 1, 1, &nVal[1]);

            if (nErrCode == DSL_SUCCESS)
            {
               pCounters->nUserTotalCells = (nVal[1] << 16) | nVal[0];
            }
         }
      }

      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            nAddrIBE, 2*nChannel, 1, &nVal[0]);
         if (nErrCode == DSL_SUCCESS)
         {
            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
               nAddrIBE, 2*nChannel + 1, 1, &nVal[1]);

            if (nErrCode == DSL_SUCCESS)
            {
               pCounters->nIBE = (nVal[1] << 16) | nVal[0];
            }
         }
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: DSL_DRV_PM_DEV_DataPathCountersCurrentGet:"
      DSL_DRV_CRLF "\tnChannel = %lu"DSL_DRV_CRLF"\tnDirection=%lu"DSL_DRV_CRLF"\tnHEC=%lu"
      DSL_DRV_CRLF"\tnTotalCells=%lu"DSL_DRV_CRLF"\tnUserTotalCells=%lu"
      DSL_DRV_CRLF"\tnIBE=%lu"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
      nChannel, nDirection, pCounters->nHEC,
      pCounters->nTotalCells, pCounters->nUserTotalCells, pCounters->nIBE));

   if (nErrCode == DSL_ERR_DEVICE_NO_DATA)
   {
      DSL_DEBUG( DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Timed out while waiting for HDLC data"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
      nErrCode = DSL_WRN_DEVICE_NO_DATA;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_DataPathCountersCurrentGet (retCode=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_DataPathCountersSet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_XTUDir_t nDirection,
   DSL_PM_DataPathData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nAddrHEC, nAddrIBE, nAddrTotalCells, nAddrUserTotalCells, nVal[2];

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_DataPathCountersCurrentSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_CHANNEL_RANGE(nChannel);

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if( nDirection != DSL_NEAR_END )
   {
      return DSL_SUCCESS;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Couldn't lock device mutex"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nAddrHEC = DSL_CMV_ADDRESS_PLAM_HEC_NE;
   nAddrTotalCells = DSL_CMV_ADDRESS_PLAM_TOTAL_CELLS_NE;
   nAddrUserTotalCells = DSL_CMV_ADDRESS_PLAM_USER_TOTAL_CELLS_NE;
   nAddrIBE = DSL_CMV_ADDRESS_PLAM_IBE_NE;

   nVal[0] = pCounters->nHEC & 0xFFFF;
   nVal[1] = (pCounters->nHEC >> 16) & 0xFFFF;
   nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
      nAddrHEC, 2*nChannel, 1, &nVal[0]);
   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
         nAddrHEC, 2*nChannel + 1, 1, &nVal[1]);
   }

   nVal[0] = pCounters->nTotalCells & 0xFFFF;
   nVal[1] = (pCounters->nTotalCells >> 16) & 0xFFFF;
   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
         nAddrTotalCells, 2*nChannel, 1, &nVal[0]);
      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
            nAddrTotalCells, 2*nChannel + 1, 1, &nVal[1]);
      }
   }

   nVal[0] = pCounters->nUserTotalCells & 0xFFFF;
   nVal[1] = (pCounters->nUserTotalCells >> 16) & 0xFFFF;
   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
         nAddrUserTotalCells, 2*nChannel, 1, &nVal[0]);
      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
            nAddrUserTotalCells, 2*nChannel + 1, 1, &nVal[1]);
      }
   }

   nVal[0] = pCounters->nIBE & 0xFFFF;
   nVal[1] = (pCounters->nIBE >> 16) & 0xFFFF;
   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
         nAddrIBE, 2*nChannel, 1, &nVal[0]);
      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
            nAddrIBE, 2*nChannel + 1, 1, &nVal[1]);
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_DataPathCountersCurrentSet (retCode=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_LineSecCountersGet(
   DSL_Context_t *pContext,
   DSL_XTUDir_t nDirection,
   DSL_PM_LineSecData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nAddrES, nAddrSES, nAddrLOSS, nAddrUAS = -1, nVal;
   DSL_uint16_t nAdslMode = 0, nAdslMode1 = 0;
   DSL_uint32_t nLineState = DSL_LINESTATE_UNKNOWN;

   DSL_uint16_t hdlcCmd[2] = {0};
   DSL_uint16_t hdlcRxBuffer[23];
   DSL_int_t nHdlcRxLen = 0;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_LineSecCountersGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ATU_DIRECTION(nDirection);

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

   if (nDirection == DSL_FAR_END)
   {
       memcpy(pCounters, DSL_DRV_PM_PTR_LINE_SEC_COUNTERS_CURR(DSL_FAR_END),
          sizeof(DSL_PM_LineSecData_t));

      if (nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
      {
         return nErrCode;
      }
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode, nAdslMode);
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode1,
      nAdslMode1);

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Couldn't lock device mutex"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if (nDirection == DSL_NEAR_END)
   {
      nAddrES = DSL_CMV_ADDRESS_PLAM_ES_NE;
      nAddrSES = DSL_CMV_ADDRESS_PLAM_SES_NE;
      nAddrLOSS = DSL_CMV_ADDRESS_PLAM_LOSS_NE;
      nAddrUAS = DSL_CMV_ADDRESS_PLAM_UAS_NE;
   }
   else
   {
      nAddrES = DSL_CMV_ADDRESS_PLAM_ES_FE;
      nAddrSES = DSL_CMV_ADDRESS_PLAM_SES_FE;
      nAddrLOSS = DSL_CMV_ADDRESS_PLAM_LOSS_FE;
      nAddrUAS = DSL_CMV_ADDRESS_PLAM_UAS_FE;
   }

   if ((nAdslMode > 8 || nAdslMode1 != 0) && nDirection == DSL_FAR_END)
   {
      hdlcCmd[0] = 0x105;

      if ((nErrCode = DSL_DRV_DANUBE_HdlcWrite(pContext, DSL_FALSE,
         (DSL_uint8_t *)hdlcCmd, 2))
         == DSL_SUCCESS)
      {
         DSL_WAIT (1);

         nErrCode = DSL_DRV_DANUBE_HdlcRead(pContext, DSL_FALSE,
            (DSL_uint8_t*)hdlcRxBuffer, 46, &nHdlcRxLen);
         if (nErrCode == DSL_SUCCESS)
         {
            pCounters->nES =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[7]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[8]);
            pCounters->nSES =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[9]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[10]);
            pCounters->nLOSS =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[11]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[12]);
            pCounters->nUAS =
               ((DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[13]) << 16) +
               (DSL_uint32_t) DSL_Le2Cpu(hdlcRxBuffer[14]);
         }
      }
   }
   else
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
         nAddrES, 0, 1, &nVal);
      if (nErrCode == DSL_SUCCESS)
      {
         pCounters->nES = (DSL_uint32_t)nVal;
      }

      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            nAddrSES, 0, 1, &nVal);
         if (nErrCode == DSL_SUCCESS)
         {
            pCounters->nSES = (DSL_uint32_t)nVal;
         }
      }

      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            nAddrLOSS, 0, 1, &nVal);
         if (nErrCode == DSL_SUCCESS)
         {
            pCounters->nLOSS = (DSL_uint32_t)nVal;
         }
      }

      if (nErrCode == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
            nAddrUAS, 0, 1, &nVal);
         if (nErrCode == DSL_SUCCESS)
         {
            pCounters->nUAS = (DSL_uint32_t)nVal;
         }
      }
   }

   pCounters->nLOFS = 0;

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: DSL_DRV_PM_DEV_LineSecCountersGet:"
      DSL_DRV_CRLF "\tnDirection=%lu"DSL_DRV_CRLF"\tnES=%lu"
      DSL_DRV_CRLF"\tnSES=%lu"DSL_DRV_CRLF"\tnLOSS=%lu"
      DSL_DRV_CRLF"\tnUAS=%lu"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
      nDirection, pCounters->nES, pCounters->nSES,
      pCounters->nLOSS, pCounters->nUAS));

   if (nErrCode == DSL_ERR_DEVICE_NO_DATA)
   {
      DSL_DEBUG( DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Timed out while waiting for HDLC data"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
      nErrCode = DSL_WRN_DEVICE_NO_DATA;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_LineSecCountersGet (retCode=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_LineInitCountersGet(
   DSL_Context_t *pContext,
   DSL_PM_LineInitData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_LineInitCountersCurrentGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Couldn't lock data mutex"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERR_SEMAPHORE_GET;
   }

   memcpy(pCounters, &pContext->pDevCtx->data.lineInitCounters,
      sizeof(DSL_PM_LineInitData_t));

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_LineInitCountersCurrentGet (retCode=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_DEV_LineInitCountersSet(
   DSL_Context_t *pContext,
   DSL_PM_LineInitData_t *pCounters)
{
   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_LineInitCountersSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_LineInitCountersSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return DSL_SUCCESS;
}
#endif
/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_LineSecCountersSet(
   DSL_Context_t *pContext,
   DSL_XTUDir_t nDirection,
   DSL_PM_LineSecData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_LineSecCountersSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ATU_DIRECTION(nDirection);

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if (nDirection != DSL_NEAR_END)
   {
      return DSL_SUCCESS;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Couldn't lock device mutex"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nVal = pCounters->nES;
   nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
      DSL_CMV_ADDRESS_PLAM_ES_NE, 0, 1, &nVal);
   if (nErrCode == DSL_SUCCESS)
   {
      nVal = pCounters->nSES;
      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
         DSL_CMV_ADDRESS_PLAM_SES_NE, 0, 1, &nVal);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nVal = pCounters->nLOSS;
      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
         DSL_CMV_ADDRESS_PLAM_LOSS_NE, 0, 1, &nVal);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nVal = pCounters->nUAS;

      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
         DSL_CMV_ADDRESS_PLAM_UAS_NE, 0, 1, &nVal);
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: DSL_DRV_PM_DEV_LineSecCountersSet:"
      DSL_DRV_CRLF "\tnDirection=%lu"DSL_DRV_CRLF"\tnES=%lu"
      DSL_DRV_CRLF"\tnSES=%lu"DSL_DRV_CRLF"\tnLOSS=%lu"
      DSL_DRV_CRLF"\tnUAS=%lu"DSL_DRV_CRLF/*"\tnNewUAS=%lu"DSL_DRV_CRLF*/,
      DSL_DEV_NUM(pContext), nDirection, pCounters->nES,
      pCounters->nSES, pCounters->nLOSS, pCounters->nUAS/*, nVal*/));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_LineSecCountersSet (retCode=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}
#endif /* #if defined (INCLUDE_DSL_CPE_API_DANUBE) || defined (INCLUDE_DSL_CPE_API_AMAZON_SE) ||
              defined (INCLUDE_DSL_CPE_API_AMAZON_S)*/
