/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/
#define DSL_INTERN

#include "drv_dsl_cpe_api.h"

#ifdef INCLUDE_DSL_CPE_API_VINAX

#include "drv_dsl_cpe_device_g997.h"

#undef DSL_DBG_BLOCK
#define DSL_DBG_BLOCK DSL_DBG_DEVICE

/*
  VINAX chip strings and codes
  will be truncated to MAX_INFO_STRING_LEN
 */
static VNX_CodeString_t ChipString[]=
{
   {0x00000001, "VINAX-DFE_V1.1"},
   {0x00000002, "VINAX-DFE_V1.2"},
   {0x00000003, "VINAX-DFE_V1.3_shared_reticle"},
   {0x00000004, "VINAX-DFE_V1.3_mono_reticle"},
   {0x00000005, "VINAX-DFE_V1.4"},
   {0x00000000, DSL_NULL}
};

#ifndef DSL_DEBUG_DISABLE
/** Blacklist for Vinax message dump
   all message IDs contained here will occur in the
   message dump only from level MSG downwards */
VNX_MsgId_t DSL_DRV_VNX_g_MsgDumpBlacklist[]=
{
   {(DSL_uint16_t)(CMD_MODEMFSM_STATEGET)   },
   {(DSL_uint16_t)(CMD_LINEPERFCOUNTNE_GET) },
   {(DSL_uint16_t)(CMD_LINEPERFCOUNTFE_GET) },
   {(DSL_uint16_t)(CMD_LINEINITCOUNTGET)    },
   {(DSL_uint16_t)(CMD_G997_PTM_STATSGET)   },
   {(DSL_uint16_t)(CMD_G997_ATM_STATSGET)   },
   {(DSL_uint16_t)(CMD_CRC_STATSNE_GET)     },
   {(DSL_uint16_t)(CMD_FEC_STATSNE_GET)     },
   {(DSL_uint16_t)(CMD_CRC_STATSFE_GET)     },
   {(DSL_uint16_t)(CMD_FEC_STATSFE_GET)     },
   { 0xFFFF }
};
#endif /* #ifdef DSL_DEBUG_DISABLE*/

/* supported modes in ADSL Annex A FW */
static const DSL_uint8_t DSL_XtseAdslA[DSL_G997_NUM_XTSE_OCTETS] =
{0x05, 0x00, 0x04, 0x00, 0x4C, 0x01, 0x04, 0x00};

/* supported modes in ADSL Annex B FW */
static const DSL_uint8_t DSL_XtseAdslB[DSL_G997_NUM_XTSE_OCTETS] =
{0x11, 0x00, 0x10, 0x00, 0x00, 0x04, 0x00, 0x00};

/* supported modes in VDSL2 FW */
static const DSL_uint8_t DSL_XtseVdsl2[DSL_G997_NUM_XTSE_OCTETS] =
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07};


static DSL_Error_t DSL_DRV_VNX_CheckTruncParamRange(
   DSL_Context_t *pContext,
   DSL_int_t nMin, DSL_int_t nMax, DSL_int_t *pVal, DSL_char_t *pName);

static DSL_Error_t DSL_DRV_VNX_InitDeviceDrv(
   DSL_Context_t *pContext,
   DSL_DeviceLowLevelConfig_t *pLineCfg);

static DSL_DEV_CamStates_t DSL_DRV_VNX_CamFsmStateGet(
   DSL_Context_t *pContext);

#ifndef DSL_DEBUG_DISABLE
/**
   Checks if a message is on a defined blacklist.

   \param pContext Pointer to dsl library context structure, [I]
   \param nMsgId   Message Id of the VNX messages to check, [I]
*/
static DSL_boolean_t DSL_DRV_VNX_CheckMessageDumpBlacklist(
   DSL_Context_t *pContext,
   const DSL_uint16_t nMsgId)
{
   DSL_boolean_t nRet = DSL_FALSE;
   VNX_MsgId_t *pTable = &DSL_DRV_VNX_g_MsgDumpBlacklist[0];

   for ( ; pTable->nMsgId != 0xFFFF; pTable++)
   {
      if ( nMsgId == pTable->nMsgId )
      {
         nRet = DSL_TRUE;
         break;
      }
   }

   return nRet;
}

static DSL_void_t DSL_DRV_VNX_DumpMessage(
   DSL_Context_t *pContext,
   const DSL_char_t *sName,
   const DSL_uint16_t nMsgId,
   const DSL_uint16_t *pData,
   const DSL_uint16_t nSize )
{
   static const DSL_uint16_t *pMsg16;
   static const DSL_uint32_t *pMsg32;
   DSL_DBG_ModuleLevel_t dbgmlData;
   DSL_boolean_t bNoNotDump = DSL_FALSE;
   DSL_uint8_t i;

   dbgmlData.data.nDbgLevel  = DSL_DBG_NONE;
   dbgmlData.data.nDbgModule = DSL_DBG_MESSAGE_DUMP;

   if (DSL_DRV_DBG_ModuleLevelGet(pContext,&dbgmlData) != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Debug module level get failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return;
   }

   if((pData == DSL_NULL) || (nSize < 4))
   {
      return ;
   }

   pMsg16 = (DSL_uint16_t*)(pData+2);
   pMsg32 = (DSL_uint32_t*)(pData+2);

   if (dbgmlData.data.nDbgLevel < DSL_DBG_ERR)
   {
      return ;
   }

   if (dbgmlData.data.nDbgLevel < DSL_DBG_MSG)
   {
      bNoNotDump = DSL_DRV_VNX_CheckMessageDumpBlacklist(pContext, nMsgId);
   }
   else
   {
      bNoNotDump = DSL_FALSE;
   }

   if (bNoNotDump == DSL_FALSE)
   {
      DSL_DRV_debug_printf(pContext, "DSL[%s]: cmd: 0x%04x len: 0x%04x" DSL_DRV_CRLF , sName, nMsgId, nSize);

      DSL_DRV_debug_printf(pContext, "Index: 0x%04x Length: 0x%04x" DSL_DRV_CRLF, pData[0], pData[1]);

      /* decide wether to interpret the rest as 16 or 32 bit */
      if (nMsgId & VDSL2_MBX_MSG_ID_IFX_MASK)
      {
         /* IFX message: 32 bit */
         for (i=0; i<((nSize-4)/4); i++)
         {
            DSL_DRV_debug_printf(pContext, "%08X ", pMsg32[i]);
         }
      }
      else
      {
         /* no IFX message: 16 bit */
         for (i=0; i<((nSize-4)/2); i++)
         {
            DSL_DRV_debug_printf(pContext, "%04X ", pMsg16[i]);
         }
      }
   }

   DSL_DRV_debug_printf(pContext, DSL_DRV_CRLF);
}
#endif /* #ifndef DSL_DEBUG_DISABLE*/

static DSL_Error_t DSL_DRV_VNX_ChReadMessage(
   DSL_Context_t *pContext,
   IOCTL_VINAX_message_t *pMsg)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_Handle_t dev;
   DSL_uint16_t buf_size;

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->nfc_lowHandle);
   DSL_CHECK_ERR_CODE();

   dev = pContext->pDevCtx->nfc_lowHandle;

   buf_size = pMsg->paylSize_byte;

   if( VINAX_InternalNfcMsgRead( (VINAX_DYN_CNTRL_T*)dev, pMsg) >= 0 )
   {
      if ((pMsg->paylSize_byte > 0) && (buf_size != pMsg->paylSize_byte))
         nErrCode = DSL_SUCCESS;
      else
         nErrCode = DSL_ERROR;
   }
   else
      nErrCode = DSL_ERROR;

   return (nErrCode);
}


static DSL_Error_t DSL_DRV_VNX_ChRequestMessage(
   DSL_Context_t *pContext,
   IOCTL_VINAX_messageSend_t *pRw)
{
   DSL_DEV_Handle_t dev;
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t ret;

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->lowHandle);
   DSL_CHECK_ERR_CODE();

#ifndef DSL_DEBUG_DISABLE
   DSL_DRV_VNX_DumpMessage(pContext, "tx", pRw->write_msg.msgId,
      (DSL_uint16_t *)pRw->write_msg.pPayload,
      (DSL_uint16_t)pRw->write_msg.paylSize_byte);
#endif /* #ifndef DSL_DEBUG_DISABLE*/

   /* reset for later interpretation */
   pRw->ack_msg.msgClassifier = 0xFFFF;

   /* Get device handle*/
   dev = DSL_DEVICE_LOWHANDLE(pContext);

   ret =  VINAX_InternalMsgSend((VINAX_DYN_CNTRL_T*)dev, pRw);

   if (ret < 0)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: Error %d during %s (MsgID=0x%02X), FctCode 0x%02X!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), pRw->ictl.retCode,
         "FIO_VINAX_MBOX_MSG_SEND",
         pRw->write_msg.msgId, pRw->ack_msg.msgClassifier));
      nErrCode = DSL_ERROR;
   }

#ifndef DSL_DEBUG_DISABLE
   /* if ret was ok, pRw contains the proper answer already */
   DSL_DRV_VNX_DumpMessage(pContext, "rx", pRw->ack_msg.msgId,
      (DSL_uint16_t *)pRw->ack_msg.pPayload,
      (DSL_uint16_t)pRw->ack_msg.paylSize_byte);
#endif /* #ifndef DSL_DEBUG_DISABLE*/

   return (nErrCode);
}


/*
   This function exchanges a command/acknowledge couple with the driver.
   The return code is checked and several retries might be done, or the
   line might be locked.

   \param pContext Pointer to DSL CPE library context structure, [I]
   \param pMsg     pointer to message exchange data, [I/O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed or message ID of acknowledge wrong
*/
static DSL_Error_t DSL_DRV_VNX_MsgTransmit(
   DSL_Context_t *pContext,
   IOCTL_VINAX_messageSend_t *pMsg)
{
   DSL_Error_t nRet = DSL_SUCCESS;
   DSL_boolean_t bBlockLine = DSL_FALSE;
   DSL_uint16_t i;
   DSL_uint16_t nLength, nLenAck;
   DSL_uint8_t nMsgFctOpCode = 0;

   nLength = pMsg->write_msg.paylSize_byte;
   nLenAck = pMsg->ack_msg.paylSize_byte;

   for (i=0; i<3; i++)
   {
      /* for any retry set the CMD and ACK Buffer length again,
         because the driver is changing these fields to indicate
         the bytes processed */
      pMsg->write_msg.paylSize_byte  = nLength;
      pMsg->ack_msg.paylSize_byte    = nLenAck;
      nRet = DSL_DRV_VNX_ChRequestMessage(pContext, pMsg);
      /* Send and wait for recieved data */
      if(nRet == DSL_SUCCESS)
      {
         break;
      }
      else
      {
         nMsgFctOpCode = (pMsg->ack_msg.msgClassifier & 0xFF00) >> 8;
         /* interprete return code */
         switch(pMsg->ictl.retCode)
         {
            case -e_VINAX_ERR_INVAL_STATE:
            case -e_VINAX_ERR_DEV_NO_RESP:
               /* line not available - stop sending, block this line */
               DSL_DEBUG(DSL_DBG_ERR,(pContext,
                  "DSL[%02d]: Config Error or line not available - block!"
                  DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

               bBlockLine = DSL_TRUE;
               nRet = DSL_ERR_MSG_EXCHANGE;
               i = 3;
               break;

            case -e_VINAX_ERR_DEV_NEG_RESP:
            case -e_VINAX_ERR_DEV_INVAL_RESP:
            case -e_VINAX_ERR_DEV_BUSY:
               /* negative response - retry sending, keep the line alive */
               DSL_DEBUG(DSL_DBG_WRN,
                  (pContext, "DSL[%02d]: negative response for "
                     "MsgID=0x%04X (Class=0x%08X) - on try %d!" DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext),
                     pMsg->write_msg.msgId, pMsg->ack_msg.msgClassifier, i));
               if ( (nMsgFctOpCode == D2H_ERROR_CMD_NOT_ALLOWED)
#if (PRELIMINARY_ADSL_SUPPORT == 1)
                    || (nMsgFctOpCode == 0x31) ||
                       (nMsgFctOpCode == 0x61) ||
                       (nMsgFctOpCode == 0x71)
#endif
                    )
               {
                  nRet = DSL_WRN_FIRMWARE_MSG_DENIED;
                  i = 3;
               }
               else if(nMsgFctOpCode == D2H_CMV_CURRENTLY_UNAVAILABLE)
               {
                  /* currently unavailable, try again */
               }
               else
               {
                  nRet = DSL_ERR_MSG_EXCHANGE;
                  i = 3;
               }
               break;

            case e_VINAX_ERR_OP_FAILED:
               /* MBX busy, try again */
               DSL_DEBUG(DSL_DBG_WRN,
                     (pContext, "DSL[%02d]: MBX busy, "
                     "MsgID=0x%04X (Class=0x%08X) - on try %d!" DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext),
                     pMsg->write_msg.msgId, pMsg->ack_msg.msgClassifier, i));
               break;

            default:
#if (PRELIMINARY_ADSL_SUPPORT == 1)
               /* special ADSL FW return code for unsupported messages */
               if ((nMsgFctOpCode == 0x31) ||
                   (nMsgFctOpCode == 0x61) ||
                   (nMsgFctOpCode == 0x71))
               {
                  DSL_DEBUG(DSL_DBG_WRN,
                     (pContext, "DSL[%02d]: message not yet supported, "
                     "MsgID=0x%04X (Class=0x%08X) - on try %d!" DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext), pMsg->write_msg.msgId,
                     pMsg->ack_msg.msgClassifier, i));
                  nRet = DSL_WRN_FIRMWARE_MSG_DENIED;
                  i = 3;
                  break;
               }
               else
#endif /* (PRELIMINARY_ADSL_SUPPORT == 1) */
               /* try it again */
               if ( (pMsg->write_msg.paylSize_byte != nLength) &&
                    (pMsg->ack_msg.msgClassifier != 0xFFFF) )
               {
                  /* Write Error! */
                  DSL_DEBUG(DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: error[%d] WRITE VINAX Cmd"
                     "(MsgID=0x%04X, Class=0x%08X) on try %d!" DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext),
                     nRet, pMsg->write_msg.msgId,
                     pMsg->write_msg.msgClassifier, i));
               }
               else
#if (PRELIMINARY_ADSL_SUPPORT == 1)
               {
                  DSL_DEBUG(DSL_DBG_WRN,
                     (pContext, "DSL[%02d]: message not yet supported, "
                     "MsgID=0x%04X (Class=0x%08X) - on try %d!" DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext),
                     pMsg->write_msg.msgId, pMsg->ack_msg.msgClassifier, i));
                  nRet = DSL_WRN_FIRMWARE_MSG_DENIED;
                  i = 3;
                  break;
               }
#else
               {
                  /* Ack Error/Timeout! */
                  DSL_DEBUG(DSL_DBG_WRN,
                     (pContext, "DSL[%02d]: error[%d] during waiting for VINAX Ack "
                     "(MsgID=0x%04X, Class=0x%08X) on try %d!" DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext),
                     nRet, pMsg->write_msg.msgId,
                     pMsg->ack_msg.msgClassifier, i));

                  if (pMsg->ack_msg.msgClassifier != 0xFFFF)
                  {
                     /* an ACK has been received - stop sending, keep line */
                     i = 3;
                     break;
                  }
               }
#endif /* (PRELIMINARY_ADSL_SUPPORT == 1) */

               if (i <= 2)
                  bBlockLine = DSL_TRUE;

               nRet = DSL_ERR_FUNCTION_WAITING_TIMEOUT;
               break;
         }     /* switch(nRet) {...} */

         DSL_DRV_MSecSleep(700);
      }
   }

   if(nRet != DSL_SUCCESS)
   {
      /* Error - Line not available */
      DSL_DEBUG((bBlockLine ? DSL_DBG_ERR : DSL_DBG_WRN),
         (pContext, "DSL[%02d]: Error for send CMD MsgID=0x%02X - %s line!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext),
         pMsg->write_msg.msgId, bBlockLine ? "BLOCK" : "KEEP"));
   }

   return nRet;
}

/*
   This function triggers to send a specified message which requests data from
   the local device and waits for appropriate answer.

   \param pContext Pointer to dsl cpe drive context structure, [I]
   \param nMsgID   Specifies the message ID as defined in the VINAX firmware
                   message specification. It includes the message type and
                   subtype, [I]
   \param nLength  number of bytes of the message payload, [I]
   \param nData    pointer to the message payload data, [I]
   \param nLenAck  available buffer size for received ack, [I]
   \param pDataAck pointer to buffer for receiving ack message, [O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed or message ID of acknowledge wrong
*/
DSL_Error_t DSL_DRV_VNX_SendMessage(
   DSL_Context_t *pContext,
   const DSL_uint32_t nMsgID,
   const DSL_uint16_t nLength,
   const DSL_uint8_t *pData,
   const DSL_uint16_t nLenAck,
   DSL_uint8_t *pDataAck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   IOCTL_VINAX_messageSend_t sMsg;

   if ((nLenAck == 0) || (pDataAck == DSL_NULL))
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: invalid buffer for VINAX Ack!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERR_POINTER;
      
      pContext->nErrNo = nErrCode;
      
      return nErrCode;
   }

   if (DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Getting mei driver semaphore failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   sMsg.write_msg.msgId          = (DSL_uint16_t)(nMsgID & 0xFFFF);
   sMsg.write_msg.pPayload       = (unsigned char *)pData;
   sMsg.write_msg.paylSize_byte  = nLength;
   /* to receive the answer hand over the payload buffer */
   sMsg.ack_msg.pPayload         = (unsigned char *)pDataAck;

   sMsg.ack_msg.msgId = 0;
   sMsg.write_msg.paylSize_byte  = nLength;
   sMsg.ack_msg.paylSize_byte    = nLenAck;

   nErrCode = DSL_DRV_VNX_MsgTransmit(pContext, &sMsg);

   /* check if received ID was the same as the sent one */
   if ( ((DSL_uint16_t)(nMsgID & 0xFFFF)) != sMsg.ack_msg.msgId && nErrCode > DSL_ERROR)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Ack mismatch in MsgID (0x%04X vs 0x%04X expected)!"
         DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), sMsg.ack_msg.msgId, nMsgID));
         
      nErrCode = DSL_ERR_FUNCTION_WAITING;
      
      pContext->nErrNo = nErrCode;
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   return nErrCode;
}

static DSL_Error_t DSL_DRV_VNX_SendMsgFwVerInfoGet(
   DSL_Context_t *pContext,
    DSL_uint32_t *pVerNum)
{
   CMD_VersionInfoGet_t  sCmd = {0, 3};
   ACK_VersionInfoGet_t  sAck;
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgFwVerInfoGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   nErrCode = DSL_DRV_VNX_SendMessage(pContext,
      (DSL_uint16_t)CMD_VERSIONINFOGET,
      sizeof(sCmd), (DSL_uint8_t*)&sCmd,
      sizeof(ACK_VersionInfoGet_t), (DSL_uint8_t*)&sAck);

   if(nErrCode == DSL_SUCCESS)
   {
      *pVerNum = (DSL_uint32_t)sAck.FW_Version;
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,(pContext,
         DSL_DRV_CRLF "DSL[%02d]: ERROR - Failed to retrieve FW Version!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }


   DSL_DEBUG(DSL_DBG_MSG,(pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgFwVerInfoGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_SendMsgHwVerInfoGet(
   DSL_Context_t *pContext,
   DSL_uint32_t *pVerNum)
{
   CMD_VersionInfoGet_t  sCmd = {0, 3};
   ACK_VersionInfoGet_t  sAck;
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgHwVerInfoGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   nErrCode = DSL_DRV_VNX_SendMessage(pContext,
      (DSL_uint16_t)CMD_VERSIONINFOGET,
      sizeof(sCmd), (DSL_uint8_t*)&sCmd,
      sizeof(ACK_VersionInfoGet_t), (DSL_uint8_t*)&sAck);

   if(nErrCode == DSL_SUCCESS)
   {
      *pVerNum = (DSL_uint32_t)sAck.HW_Version;
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,(pContext,
         DSL_DRV_CRLF "DSL[%02d]: ERROR - Failed to retrieve HW Version!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }


   DSL_DEBUG(DSL_DBG_MSG,(pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgHwVerInfoGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


/*
   Gets out of the 32bit version number the detailled information
   of the Firmware
   release and separates it in its logical parts.

   \param pContext   Pointer to dsl cpe library context structure, [I]
   \param nVersion   Fw Version Number to decode, [I]
   \param pData      pointer to detailled version information, [O]

   \return
   None
*/
static DSL_Error_t DSL_DRV_VNX_FirmwareVersionDecode(
   DSL_Context_t *pContext,
   const DSL_uint32_t nVersion,
   DSL_VNX_FwVersion_t *pData)
{
   pData->nPlatform        = (nVersion & 0xFF000000) >> 24;
   pData->nFeatureSet      = (nVersion & 0x00FF0000) >> 16;
   pData->nMajor           = (nVersion & 0x0000F000) >> 12;
   pData->nMinor           = (nVersion & 0x00000F00) >> 8;
   pData->nReleaseStatus   = (nVersion & 0x000000C0) >> 6;
   pData->nApplication     = (nVersion & 0x0000003F);

   return DSL_SUCCESS;
}

static DSL_Error_t DSL_DRV_VNX_HardwareVersionDecode(
   DSL_Context_t *pContext,
   const DSL_uint32_t nVersion,
   DSL_char_t *pData)
{
   DSL_int32_t i;
   VNX_CodeString_t *pCS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Chip set version: 0x%04X" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nVersion));

   pCS = ChipString;
   for (i = 0; pCS[i].string; i++)
   {
      if (pCS[i].code == nVersion)
      {
         strncpy(pData, pCS[i].string, MAX_INFO_STRING_LEN);
         break;
      }
   }

   if (pCS[i].string == DSL_NULL)
      strncpy(pData, "Unknown", MAX_INFO_STRING_LEN);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Chip set string : %s" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), pData));

   DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: Chip set type: %s" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), DSL_CHIPSET_TYPE_STRING));

   return DSL_SUCCESS;
}

DSL_boolean_t DSL_DRV_VNX_FwFeatureCheck(
   DSL_Context_t           *pContext,
   const DSL_VNX_Feature_t nFeature)
{
   DSL_VNX_ApplicationCode_t nAppType =
      pContext->pDevCtx->data.fwFeatures.nApplication;
   DSL_boolean_t bReturn = DSL_FALSE;

   switch (nFeature)
   {
   case DSL_VNX_FW_VDSL1:
      if (  (nAppType == DSL_APP_VDSL1           ) ||
            (nAppType == DSL_APP_VDSL1_2         ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLA   ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLB   ) ||
            (nAppType == DSL_APP_VDSL1_HL        ) ||
            (nAppType == DSL_APP_VDSL1_2_HL      ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLA_HL) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLB_HL))
         bReturn = DSL_TRUE;
      break;
   case DSL_VNX_FW_VDSL2:
      if (  (nAppType == DSL_APP_VDSL1_2          ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLA    ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLB    ) ||
            (nAppType == DSL_APP_VDSL1_2_HL       ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLA_HL ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLB_HL ))
         bReturn = DSL_TRUE;
      break;
   case DSL_VNX_FW_ADSLA:
      if (  (nAppType == DSL_APP_VDSL1_2_ADSLA    ) ||
            (nAppType == DSL_APP_ADSLA            ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLA_HL ) ||
            (nAppType == DSL_APP_ADSLA_HL         ))
         bReturn = DSL_TRUE;
      break;
   case DSL_VNX_FW_ADSLB:
      if (  (nAppType == DSL_APP_VDSL1_2_ADSLB    ) ||
            (nAppType == DSL_APP_ADSLB            ) ||
            (nAppType == DSL_APP_VDSL1_2_ADSLB_HL ) ||
            (nAppType == DSL_APP_ADSLB_HL         ))
         bReturn = DSL_TRUE;
      break;
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: DSL_DRV_VNX_FwFeatureCheck %d vs FW %d, result %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nFeature, nAppType, bReturn));

   return (bReturn);
}

/*
   This functions makes the driver low level initialization.
   Therefore the current driver state will be check and the
   if necessary the low level config values will be set.

   \param pContext - Pointer to DSL CPE library context structure, [I]
   \param pLineCfg - Pointer to Line configuration parameters, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_InitDeviceDrv(
   DSL_Context_t *pContext,
   DSL_DeviceLowLevelConfig_t *pLowCfg)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_Handle_t dev;
   IOCTL_VINAX_reqCfg_t  fioVinaxDrvCfg;
   IOCTL_VINAX_devInit_t fioVinaxDrvInit;

   DSL_CHECK_POINTER(pContext, pLowCfg);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->lowHandle);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->nfc_lowHandle);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_InitDeviceDrv()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get device handle*/
   dev = DSL_DEVICE_LOWHANDLE(pContext);

   memset(&fioVinaxDrvCfg, 0x00, sizeof(IOCTL_VINAX_reqCfg_t));
   /* Request the configuration of the VINAX Driver*/
   VINAX_InternalRequestConfig( (VINAX_DYN_CNTRL_T*)dev, &fioVinaxDrvCfg );

   /* Check current driver state*/
   if (fioVinaxDrvCfg.currDrvState != 0)
   {
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: VINAX init, already initialized"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: VINAX driver configuration:"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "  devNum            : %d"DSL_DRV_CRLF,fioVinaxDrvCfg.devNum));
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "  phyBaseAddr       : %x"DSL_DRV_CRLF,fioVinaxDrvCfg.phyBaseAddr));
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "  usedIRQ           : %d"DSL_DRV_CRLF,fioVinaxDrvCfg.usedIRQ));
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "  bootMode          : %d"DSL_DRV_CRLF,fioVinaxDrvCfg.bootMode));
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "  currDrvState      : %d"DSL_DRV_CRLF,fioVinaxDrvCfg.currDrvState));
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "  currModemFsmState : %d"DSL_DRV_CRLF,fioVinaxDrvCfg.currModemFsmState));
   }
   else
   {
      /*...device not initialized, do the initialization */
      if ( (pLowCfg->nBaseAddr != 0xFFFFFFFF) &&
           (pLowCfg->nBaseAddr != 0x0) &&
           (pLowCfg->nIrqNum != -1) )
      {
         fioVinaxDrvInit.meiBaseAddr = pLowCfg->nBaseAddr;
         fioVinaxDrvInit.usedIRQ     = pLowCfg->nIrqNum;

         /* Do the basic init of the VINAX*/
         if( VINAX_InternalInitDevice( dev, &fioVinaxDrvInit) < 0 )
         {
            DSL_DEBUG( DSL_DBG_ERR, (pContext,
               "DSL[%02d]: VINAX init failed, error %d" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), fioVinaxDrvInit.ictl.retCode));
               
            return DSL_ERROR;
         }
      }
      else
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: VINAX init failed, no valid data" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ERROR;
      }
   }

   /* Get handle to the NFC opened instance*/
   dev = pContext->pDevCtx->nfc_lowHandle;

   /* Enable the autonomious messages */
   if( VINAX_InternalNfcEnable( (VINAX_DYN_CNTRL_T*)dev ) < 0 )
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext, "DSL[%02d]: VINAX NFC enable failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_InitDeviceDrv()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return DSL_SUCCESS;
}

/*
   This function send the firmware message "CMD_ModemFSM_StateSet".

   \param pContext Pointer to dsl cpe library context structure, [I]
   \param nMode    Specifies the command for the message, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
DSL_Error_t DSL_DRV_VNX_SendMsgModemStateSet(
   DSL_Context_t *pContext,
   const DSL_uint16_t nMode)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_ModemFSM_StateSet_t sCmd;
   ACK_ModemFSM_StateSet_t sAck;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgModemStateSet()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   memset(&sCmd, 0, sizeof(sCmd));
   /* fill up the message to be sent */
   sCmd.Length  = 1;
   sCmd.LinkControl = nMode;

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_MODEMFSM_STATESET,
                  sizeof(CMD_ModemFSM_StateSet_t), (DSL_uint8_t *)&sCmd,
                  sizeof(ACK_ModemFSM_StateSet_t), (DSL_uint8_t *)&sAck);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgModemStateSet()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


/*
   This function sends the VINAX firmware message CMD_HW_ConfigSet
   to configure some chip hardware related issues.

   \param pContext - Pointer to DSL CPE library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgHwConfigSet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_HW_ConfigSet_t sCmd;
   ACK_HW_ConfigSet_t sAck;
   DSL_DEV_Clockout_t nClockoutEnable = DSL_DEV_CLOCKOUT_NA;

   DSL_CTX_READ(pContext, nErrCode,
                pDevCtx->data.deviceCfg.cfg.nClockoutEnable,
                nClockoutEnable);

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = DSL_VNX_32BIT_WR_MSG_LEN_GET(sCmd);

   sCmd.CLKO1      = ((nClockoutEnable == DSL_DEV_CLOCKOUT_ON) ?
                     VNX_ENABLE : VNX_DISABLE);
   sCmd.CLKO2      = VNX_DISABLE;
   sCmd.LED_Status = VNX_ENABLE;

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_HW_CONFIGSET,
                  sizeof(CMD_HW_ConfigSet_t), (DSL_uint8_t*)&sCmd,
                  sizeof(ACK_HW_ConfigSet_t), (DSL_uint8_t*)&sAck );

   return (nErrCode);
}


/*
   This function sends the firmware message "CMD_HS_ToneGroupSet".
   It configures the hand shake tone set according to the given config.

   \param pContext Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgHsToneGroupSet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_HS_ToneGroupSet_t sCmd;
   ACK_HS_ToneGroupSet_t sAck;
   DSL_uint16_t nToneSet = 0;
   DSL_DEV_LineMode_t nLineMode = DSL_DEV_LINEMODE_NA;
   DSL_VNX_FwVersion_t  sFwVersion;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgHsToneGroupSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get Line mode*/
   DSL_CTX_READ(pContext, nErrCode,
                pDevCtx->data.deviceCfg.cfg.nLineMode,
                nLineMode);

   /* Get FW info*/
   DSL_CTX_READ(pContext, nErrCode,
                pDevCtx->data.fwFeatures,
                sFwVersion);

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);

   /* using new scheme for ADSL CPE FW or for special 9.6.5 FW or for 9.7
      for Versions X.0.0.0.y.z (debug) assume latests FS */
   if( !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2)     ||
      ((sFwVersion.nFeatureSet >= 7) || (sFwVersion.nFeatureSet  == 0) ||
      ((sFwVersion.nPlatform   == 9) && (sFwVersion.nFeatureSet  == 6) &&
       (sFwVersion.nMajor      == 5) && (sFwVersion.nApplication == 2))))
   {
      if(nLineMode & DSL_DEV_LINEMODE_VDSL2_B43)  nToneSet |= 0x0004;
      if(nLineMode & DSL_DEV_LINEMODE_VDSL2_A43)  nToneSet |= 0x0001;
      if(nLineMode & DSL_DEV_LINEMODE_VDSL2_V43)  nToneSet |= 0x0040;
      if(nLineMode & DSL_DEV_LINEMODE_VDSL1_V43P) nToneSet |= 0x0080;
      if(nLineMode & DSL_DEV_LINEMODE_VDSL1_V43I) nToneSet |= 0x0100;
      if(nLineMode & DSL_DEV_LINEMODE_ADSL1_C43)  nToneSet |= 0x0010;
      if(nLineMode & DSL_DEV_LINEMODE_ADSL2_J43)  nToneSet |= 0x0020;
      if(nLineMode & DSL_DEV_LINEMODE_ADSL2_B43C) nToneSet |= 0x0008;
      if(nLineMode & DSL_DEV_LINEMODE_ADSL2_A43C) nToneSet |= 0x0002;

    }
   else
   {
      switch (nLineMode)
      {
      case DSL_DEV_LINEMODE_VDSL2_B43:
         nToneSet = 2; /* CAR_SET_B43;*/
         break;
      case DSL_DEV_LINEMODE_VDSL2_A43:
         nToneSet = 0; /*CAR_SET_A43;*/
         break;
      case DSL_DEV_LINEMODE_VDSL2_V43:
         nToneSet = 6;/*CAR_SET_V43;*/
         break;
      case DSL_DEV_LINEMODE_VDSL1_V43P:
         nToneSet = 7;/*CAR_SET_V43P;*/
         break;
      case DSL_DEV_LINEMODE_VDSL1_V43I:
         nToneSet = 8;/*CAR_SET_V43I;*/
         break;
      default:
         nToneSet = 2;/*CAR_SET_B43;*/
         break;
      }
   }

   memcpy(((DSL_uint16_t*)&sCmd)+2, &nToneSet, 2);

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_HS_TONEGROUPSET,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(ACK_HS_ToneGroupSet_t), (DSL_uint8_t*)&sAck );

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgHsToneGroupSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


/*
   This function sends the gain calibration values if they are enabled
   in the DSL_DeviceLowLevelLineConfig_t.

   \param pContext Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgGainCalibrationSet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_GainCalibrationSet_t sCmd;
   ACK_GainCalibrationSet_t sAck;
   DSL_DEV_RxTxGainSelection_t nGainSelection = DSL_DEV_RX_TX_GAINS_NA;
   DSL_DEV_RxTxGainSettings_t nUserGainSettings = {0,0};

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgGainCalibrationSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CTX_READ(pContext, nErrCode,
                pDevCtx->data.deviceCfg.cfg.nGainSelection,
                nGainSelection);

   DSL_CTX_READ(pContext, nErrCode,
                pDevCtx->data.deviceCfg.cfg.nUserGainSettings,
                nUserGainSettings);

   if (nGainSelection == DSL_DEV_RX_TX_GAINS_USER)
   {
      memset(&sCmd, 0, sizeof(sCmd));
      sCmd.Length = 2;

      sCmd.gainRx = nUserGainSettings.nRxGain;
      sCmd.gainTx = nUserGainSettings.nTxGain;

      nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_GAINCALIBRATIONSET,
                     sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                     sizeof(sAck), (DSL_uint8_t*)&sAck );
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgGainCalibrationSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   This function sends the VINAX firmware message CMD_ModemFSM_OptionsSet
   to configure some chip hardware related issues.

   \param pContext - Pointer to DSL CPE library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgModemFsmOptionsSet(
   DSL_Context_t *pContext,
   const DSL_boolean_t bAutoRestart,
   const DSL_boolean_t bDiagMode)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_ModemFSM_OptionsSet_t sCmd;
   ACK_ModemFSM_OptionsSet_t sAck;
   DSL_VNX_FwVersion_t  sFwVersion;

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);
   sCmd.E0 = (bAutoRestart == DSL_FALSE) ? VNX_DISABLE : VNX_ENABLE;
   sCmd.E2 = (bDiagMode    == DSL_FALSE) ? VNX_DISABLE : VNX_ENABLE;

   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      /* Enable Virtual Noise support*/
      sCmd.E3 = VNX_ENABLE;
      sCmd.E4 = VNX_ENABLE;
   }
   else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) || 
            DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      /* Get FW info*/
      DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);

      if (sFwVersion.nFeatureSet < 4)
      {
          DSL_DEBUG(DSL_DBG_WRN,
            (pContext, "DSL[%02d]: WRN - Modem FSM option set not supported by the FW!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
          return DSL_SUCCESS;
      }
   }

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_MODEMFSM_OPTIONSSET,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(sAck), (DSL_uint8_t*)&sAck );

   return (nErrCode);
}


/*
   This function writes the configuration settings for the UTOPIA PHY Address
   from the DSL library configuration memory (which have been programmed using
   the configuration  functions) for all bearer channels
   to the device.

   \param pContext - Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgUtopiaL2Modify(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nBC[DSL_CHANNELS_PER_LINE] = { 0 };
   DSL_uint32_t nChannel = 0;
   CMD_UTOPIA_L2_Modify_t sCmd;
   ACK_UTOPIA_L2_Modify_t sAck;
   DSL_Parity_t nUtopiaParity = DSL_DEV_PARITY_NA;

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgUtopiaL2Modify()" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

   for (nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++)
   {
      if (nChannel == 0)
      {
         /* Get UTOPIA PHY address*/
         DSL_CTX_READ_SCALAR(
            pContext, nErrCode,
            pDevCtx->data.deviceCfg.cfg.nUtopiaPhyAdr,
            nBC[nChannel]);
      }
   }

   /* Get UTOPIA parity*/
   DSL_CTX_READ_SCALAR(
      pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nUtopiaParity,
      nUtopiaParity);

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length       = 4;
   sCmd.UDF2_bytes   = 0;

   sCmd.ParityMode = (nUtopiaParity == DSL_DEV_PARITY_EVEN) ? 1 : 0;
   sCmd.CDPE       = 0;
   sCmd.PortNum    = (0x00FF<<16) | (nBC[1]<<8) | (nBC[0]);

   nErrCode =  DSL_DRV_VNX_SendMessage( pContext, CMD_UTOPIA_L2_MODIFY,
      sizeof(CMD_UTOPIA_L2_Modify_t), (DSL_uint8_t *)&sCmd,
      sizeof(ACK_UTOPIA_L2_Modify_t), (DSL_uint8_t *)&sAck);

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgUtopiaL2Modify()" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


/*
   This function send the firmware messages for Utopia configuration.

   \param pContext Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_WriteConfigUtopia(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_UTOPIA_L2_Init_t sCmd;
   ACK_UTOPIA_L2_Init_t sAck;
   DSL_UtopiaBusWidth_t nDataWidth = DSL_BUS_WIDTH_16B;

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: IN - DSL_DRV_VNX_WriteConfigUtopia()" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

   /* The Init command must be given before any
      other Utopia command */

   /* Get Configuration data from the internal VINAX device context*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nUtopiaBusWidth, nDataWidth);

   memset(&sCmd, 0, sizeof(sCmd));
   /* fill up the message to be sent */
   sCmd.Length = DSL_VNX_32BIT_WR_MSG_LEN_GET(sCmd);

   sCmd.Width = nDataWidth == DSL_BUS_WIDTH_8B ? 0 : 1;

   nErrCode =  DSL_DRV_VNX_SendMessage( pContext, CMD_UTOPIA_L2_INIT,
                   sizeof(sCmd), (DSL_uint8_t *)&sCmd,
                   sizeof(sAck), (DSL_uint8_t *)&sAck );

   if (nErrCode >= DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_VNX_SendMsgUtopiaL2Modify(pContext);   
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: OUT - DSL_DRV_VNX_WriteConfigUtopia()" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   This function writes the configuration settings for the POSPHY Interface
   to the device.

   \param pContext - Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgPosphyL2Modify(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nBC[DSL_CHANNELS_PER_LINE] = { 0 };
   DSL_uint32_t nChannel = 0;
   CMD_POSPHY_L2_Modify_t sCmd;
   ACK_POSPHY_L2_Modify_t sAck;
   DSL_Parity_t nPosphyParity = DSL_DEV_PARITY_NA;
   DSL_int32_t nPosphyChunkSize = 0;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgPosphyL2Modify()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   for (nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++)
   {
      if (nChannel == 0)
      {
         /* Get UTOPIA PHY address*/
         DSL_CTX_READ_SCALAR(
            pContext, nErrCode,
            pDevCtx->data.deviceCfg.cfg.nPosPhyAdr,
            nBC[nChannel]);
      }
   }

   /* Get POSPHY parity*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nPosPhyParity,
      nPosphyParity );

   /* Get POSPHY Chunk Size*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nPosphyChunkSize,
      nPosphyChunkSize);

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length     = 5;
   sCmd.ParityMode = (nPosphyParity == DSL_DEV_PARITY_EVEN) ? 1 : 0;
   sCmd.ChunkSize  = nPosphyChunkSize/4;
   sCmd.Frequency  = 50; /* FIXME: this has to be switchable */
   sCmd.PMPE       = 0;
   sCmd.PortNum    = (0x0000<<16) | (nBC[1]<<8) | (nBC[0]);

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_POSPHY_L2_MODIFY,
                  sizeof(CMD_POSPHY_L2_Modify_t),(DSL_uint8_t *)&sCmd,
                  sizeof(ACK_POSPHY_L2_Modify_t),(DSL_uint8_t *)&sAck);

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgPosphyL2Modify()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   This function send the firmware messages for Posphy configuration.

   \param pContext Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_WriteConfigPosphy(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_POSPHY_L2_Init_t sCmd;
   ACK_POSPHY_L2_Init_t sAck;

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: IN - DSL_DRV_VNX_WriteConfigPosphy()" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

   /* The Init command must be given before any
      other Posphy command */

   memset(&sCmd, 0, sizeof(sCmd));
   /* no params in this message */
   nErrCode =  DSL_DRV_VNX_SendMessage( pContext, CMD_POSPHY_L2_INIT,
                   sizeof(CMD_POSPHY_L2_Init_t), (DSL_uint8_t *)&sCmd,
                   sizeof(ACK_POSPHY_L2_Init_t), (DSL_uint8_t *)&sAck );

   if (nErrCode >= DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_VNX_SendMsgPosphyL2Modify(pContext);   
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: OUT - DSL_DRV_VNX_WriteConfigPosphy()" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


/*
   This function send the firmware messages for MII configuration.

   \param pContext Pointer to dsl cpe library context structure, [I]
   \param nAccess    Specifies a read or write access, [I]
   \param pData      pointer to clause 45 data, [I/O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgMdioEfmPhyAccess(
   DSL_Context_t           *pContext,
   const DSL_MdioDirection_t nAccess,
   DSL_MDIO_Clause45_t     *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   MdioFrame45_t  *pMdioFrame;
   CMD_MDIO_EFM_PHY_Access_t sCmd;
   ACK_MDIO_EFM_PHY_Access_t sAck;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   memset(&sCmd, 0, sizeof(sCmd));
   /* All params are stored in this one message:
      several MDIO register contents can be modified */
   sCmd.Length = 2;

   /* Set the address of the MDIO register to be accessed */
   pMdioFrame = (MdioFrame45_t*)&sCmd.MDIO_Cmd[0];
   pMdioFrame->OP    = 0; /* MdioFrame45_MDIO_ADDR set Address*/
   pMdioFrame->DEVAD = pData->nMmdAddress;
   pMdioFrame->TA    = MdioFrame45_MDIO_TA;
   pMdioFrame->DATA  = pData->nRegisterAddress;

   /* do the read/write operation */
   pMdioFrame = (MdioFrame45_t*)&sCmd.MDIO_Cmd[1];

   pMdioFrame->OP = nAccess == DSL_VNX_MDIO_READ ? MdioFrame45_MDIO_RD :
                                                   MdioFrame45_MDIO_WR;

   pMdioFrame->DEVAD = pData->nMmdAddress;
   pMdioFrame->TA    = MdioFrame45_MDIO_TA;
   /* value to be written, this has no meaning for a read operation */
   pMdioFrame->DATA  = pData->nValue;

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_MDIO_EFM_PHY_ACCESS,
                 (DSL_uint16_t)(4 + sCmd.Length*sizeof(MdioFrame45_t)),
                 (DSL_uint8_t *)&sCmd,
                 sizeof(ACK_MDIO_EFM_PHY_Access_t),
                 (DSL_uint8_t *)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: DSL_DRV_VNX_LineWriteConfigMii(): ERROR - sending Clause45 access"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if (nAccess == DSL_VNX_MDIO_READ)
   {
      pData->nValue = pMdioFrame->DATA;
   }

   return (nErrCode);
}


/*
   This function send the firmware messages for MII configuration.

   \param pContext Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_WriteConfigMii(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nTcModeMsg = 0;
   DSL_TcLayerSelection_t nTcModeSel = 0;
   DSL_DEV_TcMode_t nTcMode = DSL_DEV_TCMODE_NA;
   DSL_EFM_Duplex_t nDuplxSel = DSL_EFM_DUPLEX_FULL;
   DSL_EFM_Speed_t nEfmSpeed = DSL_EFM_SPEED_100;
   DSL_DEV_MiiMode_t nMiiMode = DSL_DEV_MIIMODE_NA;
   DSL_DEV_MiiClockChain_t nMiiClockChain = DSL_DEV_MIICLKCHAIN_NA;
   DSL_EFM_FlowCtrl_t nFlowCtrl = DSL_EFM_FLOWCTRL_OFF;
   DSL_MDIO_Clause45_t  sMdioData;

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_VNX_WriteConfigMii" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   /* Get TC mode Selection*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.sysCIF.nTcLayer,
      nTcModeSel);

   switch (nTcModeSel)
   {
   case DSL_TC_EFM:
      nTcModeMsg = 1;
      break;
   case DSL_TC_HDLC:
      nTcModeMsg = 0;
      break;
   default:
      DSL_DEBUG( DSL_DBG_WRN, (pContext,
         "DSL[%02d]: DSL_DRV_VNX_LineWriteConfigMii(): invalid TC mode %d" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nTcMode));
   }

   sMdioData.nMmdAddress   = 31; /* VS_DEV_2 */
   sMdioData.nRegisterAddress  = 9; /* address of MODE_SETTINGS register */

   sMdioData.nValue  = 0x0000 | (0x0003 & nTcModeMsg);

   /* Get TC mode*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nTcMode,
      nTcMode);

   /* set MAC or Virt Phy Mode */
   switch (nTcMode)
   {
   case DSL_DEV_TCMODE_VPHY:
      sMdioData.nValue |= 0x0100;
      break;
   case DSL_DEV_TCMODE_MAC:
   default:
      sMdioData.nValue &= ~0x0700;
      break;
   }

   /* Get Duplex mode configuration*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.MacCfg.nEfmDuplex,
      nDuplxSel);

   /* set Duplex Mode */
   switch (nDuplxSel)
   {
   case DSL_EFM_DUPLEX_FULL:
      sMdioData.nValue |= 0x0004;
      break;
   case DSL_EFM_DUPLEX_HALF:
   default:
      sMdioData.nValue &= ~0x0004;
      break;
   }

   /* Get EFm speed setting*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.MacCfg.nEfmSpeed,
      nEfmSpeed);

   /* set Speed */
   switch (nEfmSpeed)
   {
   case DSL_EFM_SPEED_100:
      sMdioData.nValue |= 0x0008;
      break;
   case DSL_EFM_SPEED_10:
   default:
      sMdioData.nValue &= ~0x0018;
      break;
   }

   /* Get MII mode setting*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nMiiMode,
      nMiiMode);

   /* set MII Mode */
   switch (nMiiMode)
   {
   case DSL_DEV_MIIMODE_RMII:
      sMdioData.nValue |= 0x0020;
      break;
   case DSL_DEV_MIIMODE_SMII:
      sMdioData.nValue |= 0x0040;
      break;
   case DSL_DEV_MIIMODE_SSSMII:
      sMdioData.nValue |= 0x0060;
      break;
   case DSL_DEV_MIIMODE_OFF:
      sMdioData.nValue |= 0x00E0;
      break;
   case DSL_DEV_MIIMODE_MII:
   default:
      sMdioData.nValue &= ~0x00E0;
      break;
   }

   /* Get MII Clock Chain setting*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nMiiClockChain,
      nMiiClockChain);

   /* set the MII Clock Chain */
   switch (nMiiClockChain)
   {
   case DSL_DEV_MIICLKCHAIN_ON:
      sMdioData.nValue |= 0x1000;
      break;
   default:
      sMdioData.nValue &= ~0x1000;
      break;
   }

   /* Get MII flow control setting*/
   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.MacCfg.nFlowControl,
      nFlowCtrl);

   /* set the MII Flow Control */
   switch (nFlowCtrl)
   {
   case DSL_EFM_FLOWCTRL_ON:
      sMdioData.nValue |= 0x0800;
      break;
   default:
      sMdioData.nValue &= ~0x0800;
      break;
   }

   nErrCode = DSL_DRV_VNX_SendMsgMdioEfmPhyAccess(
                  pContext,
                  DSL_VNX_MDIO_WRITE,
                  &sMdioData);

   if( nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: DSL_DRV_VNX_LineWriteConfigMii(): ERROR -"
         " writing clause 45 MODE_SETTINGS register" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_WriteConfigMii" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


/*
   This function sends all necessary VINAX firmware messages to initialize
   the TC layer.

   \param pContext - Pointer to DSL CPE library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgFlowModify(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_TcLayerSelection_t nTcMode = 0/*, nCifMode = 0*/;
   CMD_FlowModify_t sCmd;
   ACK_FlowModify_t sAck;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgFlowModify()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CTX_READ_SCALAR( pContext, nErrCode,
      pDevCtx->data.deviceCfg.sysCIF.nTcLayer,
      nTcMode);

   /* Reset message structure */
   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = DSL_VNX_32BIT_WR_MSG_LEN_GET(sCmd);

   switch (nTcMode)
   {
   case DSL_TC_HDLC:
      sCmd.TC = 0; /* HDLC_TC */
      break;
   case DSL_TC_EFM:
      sCmd.TC = 1; /* EFM_TC */
      break;
   case DSL_TC_ATM:
      sCmd.TC = 3; /* ATM_TC */
      break;
   default:
      return DSL_ERROR;
   }

   /* Send VINAX message */
   nErrCode =  DSL_DRV_VNX_SendMessage( pContext, CMD_FLOWMODIFY,
                   sizeof(CMD_FlowModify_t), (DSL_uint8_t *)&sCmd,
                   sizeof(ACK_FlowModify_t), (DSL_uint8_t *)&sAck);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgFlowModify()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


static DSL_Error_t DSL_DRV_VNX_SendMsgLinkControl(
   DSL_Context_t *pContext,
   const DSL_boolean_t bEnable)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_LinkControl_t sCmd;
   ACK_LinkControl_t sAck;

   /* Reset message structure */
   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.Length  = DSL_VNX_32BIT_CH_WR_MSG_LEN_GET(sCmd);
   sCmd.LinkNo  = 0;

   sCmd.TX_Mode = sCmd.RX_Mode = bEnable ? 1 : 0;

   /* Send VINAX message */
   nErrCode =  DSL_DRV_VNX_SendMessage(pContext, CMD_LINKCONTROL,
                   sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                   sizeof(sAck), (DSL_uint8_t*)&sAck);

   return(nErrCode);
}


static DSL_Error_t DSL_DRV_VNX_SendMsgSarConfig(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_SegmentationReassemblyConfig_t sCmd;
   ACK_SegmentationReassemblyConfig_t sAck;
   DSL_SAR_SegClassFilter_t nSegClassFilter = DSL_SAR_FILTER_MAC_ADDR;
   DSL_SAR_MuxType_t nMuxType = DSL_SAR_MUX_VC_MUX;
   DSL_boolean_t FCS_Present = DSL_FALSE;
   DSL_uint16_t tmpVal = 0;

   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.Length = 7;
   sCmd.CPCS_Protocol   = SAR_LLC_ENCAP_BP;
   sCmd.FCS_Present     = IFX_TRUE;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.SARcfg.nSegClassFilter, nSegClassFilter);

   switch (nSegClassFilter)
   {
      case DSL_SAR_FILTER_MAC_ADDR:
         sCmd.SegClassFilter  =  SAR_FILTER_MAC_ADDR;
         break;
      case DSL_SAR_FILTER_VLAN_ID:
         sCmd.SegClassFilter  =  SAR_FILTER_VLAN_ID;
         break;
      case DSL_SAR_FILTER_VLAN_PRIO:
         sCmd.SegClassFilter  =  SAR_FILTER_VLAN_PRIO;
         break;
      default:
         sCmd.SegClassFilter  =  SAR_FILTER_MAC_ADDR;
         break;
   }

   sCmd.SegDefAction    =  SAR_DEF_VPI_VCI;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.SARcfg.nDefVci, tmpVal);
   sCmd.SegDef_VCI =  tmpVal;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.SARcfg.nDefVpi, tmpVal);
   sCmd.SegDef_VPI =  tmpVal;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.SARcfg.nMuxType, nMuxType);

   switch (nMuxType)
   {
      case DSL_SAR_MUX_LLC:
         sCmd.CPCS_Protocol   = SAR_LLC_ENCAP_BP;
         break;
      case DSL_SAR_MUX_VC_MUX:
      default:
         sCmd.CPCS_Protocol   = SAR_VC_MUX_BP;
         break;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.SARcfg.bFcsPreserve, FCS_Present);
   sCmd.FCS_Present = FCS_Present;

   nErrCode = DSL_DRV_VNX_SendMessage(pContext, CMD_SEGMENTATIONREASSEMBLYCONFIG,
      sizeof(sCmd), (DSL_uint8_t*)&sCmd,
      sizeof(sAck), (DSL_uint8_t*)&sAck);

   return(nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_SendMsgSraControl(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_VNX_FwVersion_t sFwVersion;
   DSL_G997_RateAdaptationConfigData_t SraConfigData;
   CMD_SRA_Control_t sCmd;
   ACK_SRA_Control_t sAck;

   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);
      
   if ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) && 
       ((sFwVersion.nFeatureSet >= 3) || 
       ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor > 0)) ||
       ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor == 0) && (sFwVersion.nMinor >= 15)))) ||
       (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB) && (sFwVersion.nFeatureSet >= 5)))
   {
      memset(&sCmd, 0, sizeof(sCmd));   
      sCmd.Length = 0x1;
      /* Get DS configuration data in the DSL CPE Context*/
      DSL_CTX_READ(pContext, nErrCode, SraConfigDataDs, SraConfigData);
      sCmd.autoSRA_DS = SraConfigData.RA_MODE == DSL_G997_RA_MODE_DYNAMIC ? VNX_ENABLE : VNX_DISABLE;

      nErrCode = DSL_DRV_VNX_SendMessage(
                    pContext, CMD_SRA_CONTROL,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck);
   }
   else
   {
      DSL_DEBUG( DSL_DBG_WRN,(pContext, "DSL[%02d]: WARNING - SRA not supported by the FW",
         DSL_DEV_NUM(pContext)));
   }

   return nErrCode;
}

static DSL_Error_t DSL_DRV_VNX_SegmentationTableSet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nArrayIdx = 0, i = 0, nNumEntries = 0, tmpVal_16 = 0;
   DSL_uint32_t tmpVal_32 = 0;
   DSL_uint8_t tmpVal_8 = 0;
   DSL_uint32_t nMac[2] = {0,0};
   union
   {
      CMD_SegmentationClassFilter_Add_t FilterAdd;
      CMD_Segmentation_VCC_Config_t     VccConfig;
   } sCmd;
   union
   {
      ACK_SegmentationClassFilter_Add_t FilterAdd;
      ACK_Segmentation_VCC_Config_t     VccConfig;
   } sAck;

   /* do the CVV entries first, that the nSegClassFilters can refer to */
   memset (&sCmd, 0, sizeof(sCmd));

   sCmd.VccConfig.LinkNo = 0;
   sCmd.VccConfig.Length = DSL_VNX_32BIT_CH_WR_MSG_LEN_GET(sCmd.VccConfig);
   nArrayIdx = 0;


   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.SegmentationTable.nNumEntries, nNumEntries);

   for (nArrayIdx=0; nArrayIdx<nNumEntries; nArrayIdx++)
   {
      sCmd.VccConfig.VCC_id = nArrayIdx;

      DSL_CTX_READ_SCALAR(pContext, nErrCode,
         pDevCtx->data.deviceCfg.SegmentationTable.pTableEntry[nArrayIdx].nVpi,
         tmpVal_16);
      sCmd.VccConfig.VPI    = tmpVal_16;
      DSL_CTX_READ_SCALAR(pContext, nErrCode,
         pDevCtx->data.deviceCfg.SegmentationTable.pTableEntry[nArrayIdx].nVci,
         tmpVal_16);
      sCmd.VccConfig.VCI    = tmpVal_16;

      DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: CMD_Segmentation_VCC_Config",
         DSL_DEV_NUM(pContext)));
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, " VCC ID %d, ", sCmd.VccConfig.VCC_id));
      DSL_DEBUG( DSL_DBG_MSG,(pContext, " VPI %d, ", sCmd.VccConfig.VPI));
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, " VCI %d" DSL_DRV_CRLF, sCmd.VccConfig.VCI));

      /* send the message */
      nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_SEGMENTATION_VCC_CONFIG,
                     sizeof(CMD_Segmentation_VCC_Config_t),
                     (DSL_uint8_t *)&sCmd, sizeof(sAck),
                     (DSL_uint8_t *)&sAck );

      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - CMD_Segmentation_VCC_Config failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return(nErrCode);
      }
   }

   memset (&sCmd.FilterAdd, 0, sizeof(sCmd.FilterAdd));

   sCmd.FilterAdd.LinkNo = 0;
   sCmd.FilterAdd.Length = DSL_VNX_32BIT_CH_WR_MSG_LEN_GET(sCmd.FilterAdd);
   nArrayIdx = 0;


   for (nArrayIdx=0; nArrayIdx<nNumEntries; nArrayIdx++)
   {
      memset(&nMac[0], 0, sizeof(nMac));

      for (i=0; i<4; i++)
      {
         DSL_CTX_READ_SCALAR(pContext, nErrCode,
            pDevCtx->data.deviceCfg.SegmentationTable.pTableEntry[nArrayIdx].pMacAddress[i],
            tmpVal_8);
         nMac[i/4] |= ((DSL_uint32_t)tmpVal_8)<<(8*(3-(i%4)));
      }
      for (i=4; i<6; i++)
      {
         DSL_CTX_READ_SCALAR(pContext, nErrCode,
            pDevCtx->data.deviceCfg.SegmentationTable.pTableEntry[nArrayIdx].pMacAddress[i],
            tmpVal_8);
         nMac[i/4] |= ((DSL_uint32_t)tmpVal_8)<<(8*(1-(i%4)));
      }
      sCmd.FilterAdd.VCC_id    = nArrayIdx;
      sCmd.FilterAdd.MAC_Address_1 = nMac[0];
      sCmd.FilterAdd.MAC_Address_2 = nMac[1];

      DSL_CTX_READ_SCALAR(pContext, nErrCode,
         pDevCtx->data.deviceCfg.SegmentationTable.pTableEntry[nArrayIdx].nVlanId,
         tmpVal_32);
      sCmd.FilterAdd.VLAN_id   = tmpVal_32;

      DSL_CTX_READ_SCALAR(pContext, nErrCode,
         pDevCtx->data.deviceCfg.SegmentationTable.pTableEntry[nArrayIdx].nVlanPrio,
         tmpVal_32);
      sCmd.FilterAdd.VLAN_prio = tmpVal_32;

      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: CMD_SegmentationClassFilter_Add:",
         DSL_DEV_NUM(pContext)));
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, " VCC ID %d, ", sCmd.FilterAdd.VCC_id));
      DSL_DEBUG( DSL_DBG_MSG,(pContext, " MacAddress %04X%02X, ",
         sCmd.FilterAdd.MAC_Address_1, sCmd.FilterAdd.MAC_Address_2));
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, " VLAN ID %d, ", sCmd.FilterAdd.VLAN_id));
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, " VLAN Prio %d" DSL_DRV_CRLF, sCmd.FilterAdd.VLAN_id));

      /* send the message */
      nErrCode = DSL_DRV_VNX_SendMessage(
                     pContext, CMD_SEGMENTATIONCLASSFILTER_ADD,
                     sizeof(CMD_SegmentationClassFilter_Add_t),
                     (DSL_uint8_t *)&sCmd,
                     sizeof(sAck), (DSL_uint8_t *)&sAck );

      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - CMD_SegmentationClassFilter_Add failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }
   }

   return(nErrCode);
}


static DSL_Error_t DSL_DRV_VNX_SendMsgReassemblyClassFilterAdd(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nArrayIdx = 0, nNumEntries = 0, tmpVal_16 = 0;
   CMD_ReassemblyClassFilter_Add_t sCmd;
   ACK_ReassemblyClassFilter_Add_t sAck;

   memset (&sCmd, 0, sizeof(sCmd));

   sCmd.LinkNo = 0;
   sCmd.Length = DSL_VNX_32BIT_CH_WR_MSG_LEN_GET(sCmd);
   nArrayIdx = 0;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ReassemblyTable.nNumEntries, nNumEntries);

   for (nArrayIdx=0; nArrayIdx<nNumEntries; nArrayIdx++)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode,
         pDevCtx->data.deviceCfg.ReassemblyTable.pTableEntry[nArrayIdx].nVpi,
         tmpVal_16);
      sCmd.VPI = tmpVal_16;

      DSL_CTX_READ_SCALAR(pContext, nErrCode,
         pDevCtx->data.deviceCfg.ReassemblyTable.pTableEntry[nArrayIdx].nVci,
         tmpVal_16);
      sCmd.VCI = tmpVal_16;

      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: CMD_ReassemblyClassFilter_Add:",
         DSL_DEV_NUM(pContext)));
      DSL_DEBUG( DSL_DBG_MSG,(pContext, " VPI %d, ", sCmd.VPI));
      DSL_DEBUG( DSL_DBG_MSG,(pContext, " VCI %d" DSL_DRV_CRLF, sCmd.VCI));

      /* send the message */
      nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_REASSEMBLYCLASSFILTER_ADD,
                     sizeof(sCmd), (DSL_uint8_t *)&sCmd,
                     sizeof(sAck), (DSL_uint8_t *)&sAck );

      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - CMD_ReassemblyClassFilter_Add failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }
   }

   return(nErrCode);
}


static DSL_Error_t DSL_DRV_VNX_SAR_ConfigSet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_SystemInterfaceSelection_t nSystemIf = DSL_SYSTEMIF_UTOPIA;
   DSL_TcLayerSelection_t nTcLayer = DSL_TC_ATM;

   /* Get CIF mode*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.sysCIF.nSystemIf, nSystemIf);

   /* Get TC Layer setting*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.sysCIF.nTcLayer, nTcLayer);

   /* check whether the System interface and TC configuration
      requires an interworking flow */
   switch (nSystemIf)
   {
   case DSL_SYSTEMIF_POSPHY:
      switch (nTcLayer)
      {
      case DSL_TC_EFM:
      case DSL_TC_HDLC:
         /* no Interworking required */
         return(nErrCode);
      case DSL_TC_ATM:
         break;
      default:
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext, "DSL[%02d]: DSL_VNX_SAR_ConfigSet: "
            "unsupported or unknown SystemIF/TC-Layer configuration "
            "for interworking." DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         return nErrCode;
      }
      break;
   case DSL_SYSTEMIF_MII:
      switch (nTcLayer)
      {
      case DSL_TC_EFM:
      case DSL_TC_HDLC:
         /* no Interworking required */
         return(nErrCode);
      case DSL_TC_ATM:
         break;
      default:
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext, "DSL[%02d]: DSL_VNX_SAR_ConfigSet: unsupported or unknown "
            "SystemIF/TC-Layer configuration for interworking." DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
      break;
   case DSL_SYSTEMIF_UTOPIA:
      switch (nTcLayer)
      {
      case DSL_TC_ATM:
         /* no Interworking required */
         return(nErrCode);
      case DSL_TC_EFM:
      case DSL_TC_HDLC:
         break;
      default:
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext, "DSL[%02d]: DSL_VNX_SAR_ConfigSet: unsupported or unknown "
            "SystemIF/TC-Layer configuration for interworking." DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
      break;
   default:
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_VNX_SAR_ConfigSet: unsupported or unknown "
            "SystemIF (%d) configuration for interworking."DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nSystemIf) );
            
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_DEVICE;
      
      return nErrCode;
   }

   nErrCode = DSL_DRV_VNX_SendMsgSarConfig(pContext);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - DSL_VNX_SendMsgSarConfig: failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return(nErrCode);
   }

   nErrCode = DSL_DRV_VNX_SegmentationTableSet(pContext);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - DSL_VNX_SegmentationTableSet: failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return(nErrCode);
   }

   nErrCode = DSL_DRV_VNX_SendMsgReassemblyClassFilterAdd(pContext);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_VNX_SendMsgReassemblyClassFilterAdd: failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      return(nErrCode);
   }

   return(nErrCode);
}


/*
   This function sends all necessary VINAX firmware messages to configure
   the Custumer Interface settings.

   \param pContext - Pointer to DSL CPE library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_WriteConfigCif(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_SystemInterfaceSelection_t nCifMode = DSL_SYSTEMIF_UNDEFINED;

   /* Get CIF mode*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.sysCIF.nSystemIf, nCifMode);

   switch (nCifMode)
   {
   case DSL_SYSTEMIF_UTOPIA:
      nErrCode = DSL_DRV_VNX_WriteConfigUtopia(pContext);
      break;
   case DSL_SYSTEMIF_POSPHY:
      nErrCode = DSL_DRV_VNX_WriteConfigPosphy(pContext);
      break;
   case DSL_SYSTEMIF_MII:
      nErrCode = DSL_DRV_VNX_WriteConfigMii(pContext);
      break;
   default:
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_VNX_WriteConfigCif - Undefined CIF mode (%d)"
         DSL_DRV_CRLF , DSL_DEV_NUM(pContext), nCifMode));
         
      return DSL_SUCCESS;
   }

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_VNX_WriteConfigCif: ERROR - CIF mode set"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nErrCode = DSL_DRV_VNX_SendMsgFlowModify(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_VNX_WriteConfigCif: ERROR - Flow Modify"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }


   /* SAR flows have to be configured with LinkControl off */
   /* FIXME: to support SAR flows with MII the interface mode within
      the vendor specific register has to be disabled, then the SAR
      has to be set and the MII config has to be set afterwards */
   nErrCode = DSL_DRV_VNX_SAR_ConfigSet(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_VNX_WriteConfigCif: ERROR - SAR Config"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if (nCifMode != DSL_SYSTEMIF_MII)
   {
      /* LinkControl command is only valid for UTOPIA/POSPHY */
      nErrCode = DSL_DRV_VNX_SendMsgLinkControl(pContext, DSL_TRUE);
   }

   return (nErrCode);
}

/*
   This function writes the configuration settings for the ATU System Enabling
   from the DSL library configuration memory (which have been programmed using
   the configuration  functions) for the specified line to the device.

   \param pContext - Pointer to dsl cpe library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgVtseConfigure(
   DSL_Context_t *pContext)
{
#define XTSE_CMD_SET(CmdBit, XtseByte, XtseBit) (sCmd.CmdBit = \
        (((XTSE.XTSE[(XtseByte)-1] & (XtseBit)) != 0) ? \
         VNX_ENABLE : VNX_DISABLE))

   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t i = 0;
   DSL_G997_XTUSystemEnablingData_t XTSE;
   CMD_XTSE_Configure_t   sCmd;
   ACK_XTSE_Configure_t   sAck;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: DSL_DRV_VNX_SendMsgVtseConfigure()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /*DSL_GET_CHANNEL_NUMBER(nLine, nCh);*/
   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   for (i = 0; i < DSL_G997_NUM_XTSE_OCTETS; i++)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[i], XTSE.XTSE[i]);
   }

   sCmd.Length = 2;

   if ((DSL_DRV_VNX_CamFsmStateGet(pContext) == DSL_CAM_ADSL_RETRY))
   {
      /* VDSL bits will be masked in case of ADSL FW*/
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: VDSL bits masked out" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      /* VDSL bits */
      if (((XTSE.XTSE[8-1] & XTSE_8_01_A) != 0) ||
          ((XTSE.XTSE[8-1] & XTSE_8_02_B) != 0) ||
          ((XTSE.XTSE[8-1] & XTSE_8_03_C) != 0))
      {
         sCmd.V2 = VNX_ENABLE;
      }
      /* this is a preliminary bit not defined by G997 yet */
      XTSE_CMD_SET(V0, 8, XTSE_8_08);
   }

   /* Check for the VDSL only mode*/
   if ((DSL_DRV_VNX_CamFsmStateGet(pContext) == DSL_CAM_VDSL_FORCED))
   {
      /* ADSL bits will be masked*/
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: ADSL bits masked out" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
   /* ADSL bits */
#if (PRELIMINARY_ADSL_SUPPORT == 1)
      /* ANSI T1.413 1998 */
      XTSE_CMD_SET(A0, 1, XTSE_1_01_A_T1_413);
      /* G.992.5, Annex M */
      XTSE_CMD_SET(A1, 7, XTSE_7_03_M_5_NO);
      /* G992.1 Annex A */
      XTSE_CMD_SET(A2, 1, XTSE_1_03_A_1_NO);
      /* G992.1 Annex B */
      XTSE_CMD_SET(A3, 1, XTSE_1_05_B_1_NO);
      /* G.992.3, Annex A */
      XTSE_CMD_SET(A8, 3, XTSE_3_03_A_3_NO);
      /* G.992.3, Annex B */
      XTSE_CMD_SET(A9, 3, XTSE_3_05_B_3_NO);
      /* G.992.3, Annex L US mask 1 */
      XTSE_CMD_SET(A11, 5, XTSE_5_03_L_3_NO);
      /* G.992.3, Annex L US mask 2 */
      XTSE_CMD_SET(A12, 5, XTSE_5_04_L_3_NO);
      /* G.992.3, Annex M */
      XTSE_CMD_SET(A13, 5, XTSE_5_07_M_3_NO);
      /* G.992.5, Annex B */
      XTSE_CMD_SET(A14, 6, XTSE_6_03_B_5_NO);
      /* G.992.5, Annex A */
      XTSE_CMD_SET(A15, 6, XTSE_6_01_A_5_NO);
#endif
   }

   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      if ((sCmd.V2 == 0) && (sCmd.V0 == 0))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - No VDSL mode selected in the XTSE configuration!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }
   }
   else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA))
   {
      if ((sCmd.A0 == 0) && (sCmd.A1 == 0) && (sCmd.A2 == 0) && (sCmd.A8 == 0) &&
          (sCmd.A11 == 0) && (sCmd.A12 == 0) && (sCmd.A13 == 0) && (sCmd.A15 == 0))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - No valid mode selected in the XTSE configuration!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }
   }
   else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      if ((sCmd.A0 == 0) && (sCmd.A3 == 0) && (sCmd.A9 == 0) && (sCmd.A14 == 0))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - No valid mode selected in the XTSE configuration!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Unknown FW type!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_ERROR;
   }

   return DSL_DRV_VNX_SendMessage( pContext, CMD_XTSE_CONFIGURE,
             sizeof(sCmd), (DSL_uint8_t*)&sCmd,
             sizeof(sAck), (DSL_uint8_t*)&sAck);
}

/*
   This function converts a data rate specified according to G.997.1 (coded in
   bit/s) to its VINAX message parameter values (MAX_Rate/MAX_Rate_4_8 for
   maximum data rate respective MIN_Rate/MIN_Rate_4_8 for minimum data rate)

   \param nDataRate Specifies the data rate to convert coded in bit/s, [I]
   \param nMode     Specifies for which ADSL mode the function
                    should apply, [I]
                    0: FULL/LITE (G.992.3/4)
                    1: PLUS (G.992.5)
                    2: G.992.1/2
   \param nRate32   Returns the data rate in multiple of 32kbit/s, [O]
   \param nRate4_8  Returns the adder for data rate in multiple of 4kbit/s for
                    G.992.3/4 respective 8kbit/s for G.992.5, [O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_GetMsgDataRateValues(
   const DSL_uint32_t nDataRate,
   const DSL_uint32_t nMode,
   DSL_uint16_t *nRate32,
   DSL_uint8_t *nRate4_8 )
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t fVal = 0;

   *nRate32 = (DSL_uint16_t)(nDataRate / 32000);

   fVal = ((DSL_uint32_t)nDataRate) - *nRate32 * 32000;

   /* Special rate adder are not available for G.992.1/2 */
   if ( nMode <=1 )
   {
      if (nMode == 0)
      {
         /* For G.992.3/4 multiple of 4kbit/s are possible */
         fVal /= 4000;
      }
      else
      {
         /* For G.992.5 multiple of 8kbit/s are possible */
         fVal /= 8000;
      }
      /* Round mathematically to next integer value */
      *nRate4_8 = (DSL_uint8_t)(fVal);
   }

   return (nErrCode);
}


/*
   This function writes the configuration settings for the bearer channels from
   the DSL library configuration memory (which have been programmed using the
   configuration  functions) for the specified line to the device.
   NOTE: Dual latency is not supported yet. Thus only bearer channel 0 will be
         programmed at the moment.

   \param pContext - Pointer to DSL CPE library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgBearerChSet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nRate32 = 0, tmpVal_16 = 0, nRateMin = 0, nRateMax = 0;
   DSL_int_t nRateEst = 0;
   DSL_uint8_t nRate4_8 = 0, nVdslMode = 0,
               tmpVal_8 = 0, xtse6 = 0, xtse7 = 0;
   DSL_uint32_t tmpVal_32 = 0;

   union
   {
      CMD_BearerCh0_US_Set_t UsCfg;
      CMD_BearerCh0_DS_Set_t DsCfg;
   } sCmd;
   union
   {
      ACK_BearerCh0_US_Set_t UsCfg;
      ACK_BearerCh0_DS_Set_t DsCfg;
   } sAck;
   DSL_uint32_t nMode = 0;

   DSL_DEBUG( DSL_DBG_MSG,
     (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgBearerChSet()" DSL_DRV_CRLF,
     DSL_DEV_NUM(pContext)));

   /* use the xTSE bits to select between
      ADSL = ATM Control block and
      VDSL = PTM Control block */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[7], nVdslMode);
   nVdslMode = nVdslMode ? 1 : 0;

   /* Programm downstream configuration */

   /* Common Bearer Channel configuration parameter
      for upstream/downstream. */
   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.DsCfg.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd.DsCfg);
   /* Select to set ATM or packet parameters */
   sCmd.DsCfg.ATMControl = nVdslMode ? 0 : 1;
   sCmd.DsCfg.PTMControl = nVdslMode ? 1 : 0;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ChannelConfigData[DSL_DOWNSTREAM].MaxIntDelay,
      tmpVal_16);

   sCmd.DsCfg.maxDelay0_PTMds = (DSL_uint8_t)tmpVal_16;
   sCmd.DsCfg.maxDelay0_ATMds = (DSL_uint8_t)tmpVal_16;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ChannelConfigData[DSL_DOWNSTREAM].MinINP,
      tmpVal_8);

   /* special value for 0.5 */
   sCmd.DsCfg.minINP_ATMds = (tmpVal_8 == 1) ? 0x800 : tmpVal_8/2;
   sCmd.DsCfg.minINP_PTMds = (tmpVal_8 == 1) ? 0x800 : tmpVal_8/2;

   if( !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      /* ADSL specific limits
         data rates in units of 4kbit */
      nRateMin = DSL_DEV_MIN_DATARATE / 4000;

      DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[6], xtse7);
      DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[5], xtse6);

      if( xtse6 || xtse7 )
      {
         nRateMax = DSL_DEV_MAX_DATARATE_DS_ADSL2P / 4000; /* ADSL2+ */
      }
      else
      {
         nRateMax = DSL_DEV_MAX_DATARATE_DS_ADSL2 / 4000; /* ADSL1/2 */
      }
   }
   else
   {
      /* VDSL specific limits
         data rates in units of 4kbit */
      nRateMin = 64 / 4;
      nRateMax = 0xF000 ; /* 246Mbit */
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ChannelConfigData[DSL_DOWNSTREAM].MinDataRate,
      tmpVal_32);

   /* The datarates have to be entered in multiples of 4 kbps */
   DSL_DRV_VNX_GetMsgDataRateValues(tmpVal_32, nMode, &nRate32, &nRate4_8);
   nRateEst = nRate32 * 8 + nRate4_8;

   nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)nRateMin,
                    (DSL_int_t)nRateMax,
                    &nRateEst,
                    "SendMsgSetPsd: MinRate");
   if( nErrCode < DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - DSL_VNX_CheckTruncParamRange!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   sCmd.DsCfg.minRate0_PTMds = (DSL_uint16_t)nRateEst;
   sCmd.DsCfg.minRate0_ATMds = sCmd.DsCfg.minRate0_PTMds;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ChannelConfigData[DSL_DOWNSTREAM].MaxDataRate,
      tmpVal_32);

   DSL_DRV_VNX_GetMsgDataRateValues(tmpVal_32, nMode, &nRate32, &nRate4_8);
   nRateEst = nRate32 * 8 + nRate4_8;

   nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)nRateMin,
                    (DSL_int_t)nRateMax,
                    &nRateEst,
                    "SendMsgSetPsd: MaxRate");
   if( nErrCode < DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - DSL_VNX_CheckTruncParamRange!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   sCmd.DsCfg.maxRate0_PTMds = (DSL_uint16_t)nRateEst;
   sCmd.DsCfg.maxRate0_ATMds = sCmd.DsCfg.maxRate0_PTMds;

   if (!DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      /* Select to set ATM or packet parameters */
      sCmd.DsCfg.minResRate0_PTMds = 8;
      sCmd.DsCfg.minResRate0_ATMds = 8;
      sCmd.DsCfg.BER_PTMds = 2;
      sCmd.DsCfg.BER_ATMds = 2;
   }

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_BEARERCH0_DS_SET,
      sizeof(CMD_BearerCh0_DS_Set_t), (DSL_uint8_t *)&sCmd,
      sizeof(ACK_BearerCh0_DS_Set_t), (DSL_uint8_t *)&sAck);

   /* Programm upstream configuration */

    /* Common Bearer Channel configuration parameter
      for upstream/downstream. */
   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.UsCfg.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd.UsCfg);
   /* Select to set ATM or Packet parameters */
   sCmd.UsCfg.ATMControl = nVdslMode ? 0 : 1;
   sCmd.UsCfg.PTMControl = nVdslMode ? 1 : 0;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ChannelConfigData[DSL_UPSTREAM].MaxIntDelay,
      tmpVal_16);

   sCmd.UsCfg.maxDelay0_PTMus = (DSL_uint8_t)tmpVal_16;
   sCmd.UsCfg.maxDelay0_ATMus = (DSL_uint8_t)tmpVal_16;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ChannelConfigData[DSL_UPSTREAM].MinINP,tmpVal_8);

   /* special value for 0.5 */
   sCmd.UsCfg.minINP_ATMus = (tmpVal_8 == 1) ? 0x800 : tmpVal_8/2;
   sCmd.UsCfg.minINP_PTMus = (tmpVal_8 == 1) ? 0x800 : tmpVal_8/2;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ChannelConfigData[DSL_UPSTREAM].MinDataRate,
      tmpVal_32);

   if (!DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      /* ADSL specific limits
         data rates in units of 4kbit */
      nRateMin = DSL_DEV_MIN_DATARATE / 4000;
      nRateMax = 0x60 * 32 / 4;
   }
   else
   {
      /* VDSL specific limits
         data rates in units of 4kbit */
      nRateMin = 64 / 4;
      nRateMax = 0xF000 ; /* 246Mbit */
   }

   /* The datarates have to be entered in multiples of 4 kbps */
   DSL_DRV_VNX_GetMsgDataRateValues(tmpVal_32, nMode, &nRate32, &nRate4_8);
   nRateEst = nRate32 * 8 + nRate4_8;

   nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)nRateMin,
                    (DSL_int_t)nRateMax,
                    &nRateEst,
                    "SendMsgSetPsd: MinRate");
   if( nErrCode < DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - DSL_VNX_CheckTruncParamRange!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }
   sCmd.UsCfg.minRate0_PTMus = (DSL_uint16_t)nRateEst;
   sCmd.UsCfg.minRate0_ATMus = sCmd.UsCfg.minRate0_PTMus;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.ChannelConfigData[DSL_UPSTREAM].MaxDataRate,
      tmpVal_32);

   DSL_DRV_VNX_GetMsgDataRateValues(tmpVal_32, nMode, &nRate32, &nRate4_8);
   nRateEst = nRate32 * 8 + nRate4_8;

   nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)nRateMin,
                    (DSL_int_t)nRateMax,
                    &nRateEst,
                    "SendMsgSetPsd: MaxRate");
   if( nErrCode < DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - DSL_VNX_CheckTruncParamRange!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   sCmd.UsCfg.maxRate0_PTMus = (DSL_uint16_t)nRateEst;
   sCmd.UsCfg.maxRate0_ATMus = sCmd.UsCfg.maxRate0_PTMus;

   if (!DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      /* Select to set ATM or packet parameters */
      sCmd.UsCfg.minResRate0_PTMus = 8;
      sCmd.UsCfg.minResRate0_ATMus = 8;
      sCmd.UsCfg.BER_PTMus = 2;
      sCmd.UsCfg.BER_ATMus = 2;
   }


   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_BEARERCH0_US_SET,
      sizeof(CMD_BearerCh0_US_Set_t), (DSL_uint8_t *)&sCmd,
      sizeof(ACK_BearerCh0_US_Set_t), (DSL_uint8_t *)&sAck);

   DSL_DEBUG( DSL_DBG_MSG,
     (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgBearerChSet()" DSL_DRV_CRLF,
     DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   This function writes the configuration settings for the ATU System Enabling
   from the DSL CPE library configuration memory (which have been
   programmed using the configuration  functions).

   \param pContext - Pointer to dsl library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgVdsl2ProfileControl(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t i = 0;
   DSL_G997_XTUSystemEnablingData_t XTSE;
   CMD_HS_VDSL2ProfileControl_t   sCmd;
   ACK_HS_VDSL2ProfileControl_t   sAck;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgVdsl2ProfileControl" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   for (i = 0; i < DSL_G997_NUM_XTSE_OCTETS; i++)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[i], XTSE.XTSE[i]);
   }

   if (((XTSE.XTSE[8-1] & XTSE_8_01_A) == 0) &&
       ((XTSE.XTSE[8-1] & XTSE_8_02_B) == 0) &&
       ((XTSE.XTSE[8-1] & XTSE_8_03_C) == 0))
   {
      /* no VDSL2 bit set, message is not needed */
      return (nErrCode);
   }

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);

   /* both sides support all profiles */
   sCmd.profileSup7 = sCmd.profileSup6 = sCmd.profileSup5 =
   sCmd.profileSup4 = sCmd.profileSup3 = sCmd.profileSup2 =
   sCmd.profileSup1 = sCmd.profileSup0 = 1;

   /* According MsgCat All "support" bits should be enable. */
   /*
   ITU_ANNEX_A_M1_EU32 */
   sCmd.A_US0PsdSup0 = 1;
   sCmd.A_US0PsdSup1 = 1;
   sCmd.A_US0PsdSup2 = 1;
   sCmd.A_US0PsdSup3 = 1;
   sCmd.A_US0PsdSup4 = 1;
   sCmd.A_US0PsdSup5 = 1;
   sCmd.A_US0PsdSup6 = 1;
   sCmd.A_US0PsdSup7 = 1;
   /*
   ITU_ANNEX_A_M9_EU64 */
   sCmd.A_US0PsdSup8 = 1;
   /*
   ITU_ANNEX_A_M1_ADLU32 */
   sCmd.A_US0PsdSup9 = 1;
   sCmd.A_US0PsdSup10 = 1;
   sCmd.A_US0PsdSup11 = 1;
   sCmd.A_US0PsdSup12 = 1;
   sCmd.A_US0PsdSup13 = 1;
   sCmd.A_US0PsdSup14 = 1;
   sCmd.A_US0PsdSup15 = 1;
   sCmd.A_US0PsdSup16 = 1;
   /*
   ITU_ANNEX_A_M9_ADLU64 */
   sCmd.A_US0PsdSup17 = 1;
   sCmd.A_US0PsdSup18 = 1;
   sCmd.A_US0PsdSup19 = 1;

   
   /*
   US0 in 25 - 138 kHz */
   sCmd.B_US0PsdSup0 = 1;
   /*
   US0 in 25 - 276 kHz */
   sCmd.B_US0PsdSup1 = 1;
   /*
   US0 in 120 - 276 kHz */
   sCmd.B_US0PsdSup2 = 1;
   sCmd.B_US0PsdSup3 = 1;
   sCmd.B_US0PsdSup4 = 1;


   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_HS_VDSL2PROFILECONTROL,
                       sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                       sizeof(sAck), (DSL_uint8_t*)&sAck);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgVdsl2ProfileControl" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   This function send the firmware message "CMD_AlgorithmControlSet_t".

   \param pContext     Pointer to dsl cpe library context structure, [I]
   \param nInitWord    Specifies the config word for initialisation, [I]
   \param nSteadyWord  Specifies the config word for SteadyState, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgAlgorithmControl(
   DSL_Context_t *pContext,
   const DSL_uint16_t nInitWord,
   const DSL_uint16_t nSteadyWord)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_AlgorithmControlSet_t sCmd;
   ACK_AlgorithmControlSet_t sAck;
   DSL_TestModeControlSet_t nTestModeControl = DSL_TESTMODE_DISABLE;
   DSL_boolean_t bTmpVal = DSL_FALSE;

   /* in this message the bits are negated: enable = 0
      set by default anything to 1 therefore */
   memset(&sCmd, 0, sizeof(sCmd));

   /* fill up the message to be sent */
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);

   /* ADSL bits */
   sCmd.AAC1   = VNX_DISABLE_N;  /* Reserved:           off */
   sCmd.AAC2   = VNX_ENABLE_N;   /* DEC adapt:          on */
   sCmd.AAC3   = VNX_DISABLE_N;  /* Reserved:           off */
   sCmd.AAC4   = VNX_ENABLE_N;   /* S1/2:               on */

   sCmd.IAC0   = VNX_ENABLE_N;  /* PGA train:           on */
   sCmd.IAC1   = VNX_ENABLE_N;  /* hybrid train:        on */

   sCmd.IAC2   = VNX_DISABLE_N;

   sCmd.IAC4   = VNX_ENABLE_N;   /* Medley Multi-Synch:  on */

   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      sCmd.IAC6   = VNX_ENABLE_N;   /* VDSL, windowing:         on */
   }
   else
   {
      sCmd.IAC6   = VNX_DISABLE_N;   /* ADSL, windowing:        off */
   }

   /* US PBO for VDSL1 (Ref PSD Mask): off, Max PSD only */
   sCmd.IAC7   = CMD_AlgorithmControlSet_MAXPSD;
   sCmd.IAC8   = VNX_ENABLE_N;  /* auto pilot tone selection: on */

   sCmd.IAC3   = VNX_DISABLE_N;   /* MFDQ:                off */

   /* Trellis Coding Tx  */
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      lineFeatureDataCfg[DSL_UPSTREAM].bTrellisEnable,
      bTmpVal);
   sCmd.SAC0   = bTmpVal ? VNX_ENABLE_N : VNX_DISABLE_N;

   /* Trellis Coding Rx */
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      lineFeatureDataCfg[DSL_DOWNSTREAM].bTrellisEnable,
      bTmpVal);
   sCmd.SAC1   = bTmpVal ? VNX_ENABLE_N : VNX_DISABLE_N;

   /* Tx Bitswap */
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      lineFeatureDataCfg[DSL_UPSTREAM].bBitswapEnable,
      bTmpVal);
   sCmd.SAC2   = bTmpVal ? VNX_ENABLE_N : VNX_DISABLE_N;

   /* Rx Bitswap */
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      lineFeatureDataCfg[DSL_DOWNSTREAM].bBitswapEnable,
      bTmpVal);
   sCmd.SAC3   = bTmpVal ? VNX_ENABLE_N : VNX_DISABLE_N;

   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      DSL_VNX_FwVersion_t  sFwVersion;

      /* Get FW info*/
      DSL_CTX_READ(pContext, nErrCode,
                   pDevCtx->data.fwFeatures,
                   sFwVersion);

      /* for VDSL FS6 or less always disable bitswap */
      if (sFwVersion.nFeatureSet <= 6)
      {
         /* Tx Bitswap */
         sCmd.SAC2   = VNX_DISABLE_N;
         /* Rx Bitswap */
         sCmd.SAC3   = VNX_DISABLE_N;
      }
   }

   sCmd.SAC4   = VNX_ENABLE_N; /* SFDQ Training:        on */
   sCmd.SAC6   = VNX_ENABLE_N; /* Steady-State SNR:     on */

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nTestModeControl, nTestModeControl);

   sCmd.SAC7 = (nTestModeControl == DSL_TESTMODE_SHOWTIME_LOCK
                || nTestModeControl == DSL_TESTMODE_TRAINING_LOCK) ?
                  VNX_DISABLE_N : VNX_ENABLE_N;
   sCmd.SAC8   = VNX_ENABLE_N; /* Steady-State Tx PSD Scaling:     on */
   sCmd.SAC9   = 0;              /* Network timing reference (swapped in FS7):      off */
   sCmd.SAC10  = VNX_ENABLE_N;   /* Tone Reordering Table:           on */


   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_ALGORITHMCONTROLSET,
                  sizeof(CMD_AlgorithmControlSet_t), (DSL_uint8_t *)&sCmd,
                  sizeof(ACK_AlgorithmControlSet_t), (DSL_uint8_t *)&sAck);

   return (nErrCode);
}


/*
   This function send the firmware message "CMD_DigitalFilterSet_t".

   \param pContext Pointer to DSL CPE library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgDigitalFilter(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_DigitalFilterSet_t sCmd;
   ACK_DigitalFilterSet_t sAck;
   DSL_DEV_Filter_t nFilter = DSL_DEV_FILTER_NA;

   memset(&sCmd, 0, sizeof(sCmd));

   /* fill up the message to be sent */
   sCmd.Length = 2;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nFilter, nFilter);

   /* Enable 'autoControl' independently from filter setting. Later on an
      additional configuration option might be used for configuration. */ 
   sCmd.autoControl = VNX_ENABLE;

   switch (nFilter)
   {
   case DSL_DEV_FILTER_POTS:
      sCmd.OpMode = CMD_DigitalFilterSet_POTS;
      sCmd.RxControl = VNX_ENABLE;
      sCmd.TxControl = VNX_ENABLE;      
      break;
   case DSL_DEV_FILTER_ISDN:
      sCmd.OpMode = CMD_DigitalFilterSet_ISDN;
      sCmd.RxControl = VNX_ENABLE;
      sCmd.TxControl = VNX_ENABLE;      
      break;
   case DSL_DEV_FILTER_POTS_2:
      sCmd.OpMode = CMD_DigitalFilterSet_POTS_2;
      sCmd.RxControl = VNX_ENABLE;
      sCmd.TxControl = VNX_ENABLE;      
      break;
   case DSL_DEV_FILTER_POTS_3:
      sCmd.OpMode = CMD_DigitalFilterSet_POTS_3;
      sCmd.RxControl = VNX_ENABLE;
      sCmd.TxControl = VNX_ENABLE;      
      break;
   case DSL_DEV_FILTER_OFF:
   default:
      /* no POTS or ISDN, but mode none is not defined */
      sCmd.OpMode = CMD_DigitalFilterSet_POTS;
      sCmd.RxControl = VNX_DISABLE;
      sCmd.TxControl = VNX_DISABLE;      
      break;
   }

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_DIGITALFILTERSET,
                  sizeof(sCmd), (DSL_uint8_t *)&sCmd,
                  sizeof(sAck), (DSL_uint8_t *)&sAck);

   return (nErrCode);
}

#if (PRELIMINARY_ADSL_SUPPORT == 1)
#if (ADSL_ALPHEUS_SUPPORT == 1)
static DSL_Error_t DSL_DRV_VNX_SendMsgHsAdslControl(
   DSL_Context_t           *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_HS_ADSL_Control_t sCmd;
   ACK_HS_ADSL_Control_t sAck;

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);

   sCmd.b0 = VNX_ENABLE; /* GHS */
   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_HS_ADSL_CONTROL,
                  sizeof(sCmd), (DSL_uint8_t *)&sCmd,
                  sizeof(sAck), (DSL_uint8_t *)&sAck);

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_SendMsgAdslLinkSetupControl(
   DSL_Context_t *pContext)
{
   CMD_ADSL_LinkSetupControl_t sCmd;
   ACK_ADSL_LinkSetupControl_t sAck;

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);

   sCmd.B10 = IFX_ENABLE;
   sCmd.B8 = IFX_ENABLE;
   return DSL_DRV_VNX_SendMessage( pContext, CMD_ADSL_LINKSETUPCONTROL,
             sizeof(sCmd), (DSL_uint8_t *)&sCmd,
             sizeof(sAck), (DSL_uint8_t *)&sAck);
}
#endif /* (ADSL_ALPHEUS_SUPPORT == 1) */
#endif /* (PRELIMINARY_ADSL_SUPPORT == 1) */

/*
   This functions stores the VINAX configuration data in the device context.

   \param pContext - Pointer to DSL CPE library context structure, [I]
   \param pData    - Pointer to configuration parameters, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_DeviceCfgStore(
   DSL_Context_t *pContext,
   DSL_Init_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   memcpy( &(pContext->pDevCtx->data.deviceCfg),
           &(pData->data.nDeviceCfg),
           sizeof(DSL_DeviceConfig_t) );

   /* Call device specific implementation*/
   nErrCode = DSL_DRV_DEV_LowLevelConfigurationSet(pContext, &(pData->data.nDeviceCfg.cfg));

   if( nErrCode == DSL_ERROR )
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - Low Level Configuration set failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   return (nErrCode);
}


static DSL_Error_t DSL_DRV_VNX_SendMsgSelectedProfileVdsl2Get(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   /* The message structures are identical for CO and CPE */
   ACK_HS_SelectedProfileVDSL2Get_t sAck;
   CMD_HS_SelectedProfileVDSL2Get_t sCmd;
   DSL_uint8_t i = 0, j = 0, nBitCount = 0;
   DSL_uint8_t actualProfile = 0;
   DSL_ProfileType_t nProfile = DSL_PROFILE_LAST;
   DSL_G997_XTUSystemEnablingData_t data = {{0,0,0,0,0,0,0,0}};

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgSelectedProfileVdsl2Get" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   /* Read data from data base */
   for (i = 0; i < DSL_G997_NUM_XTSE_OCTETS; i++)
   {
      data.XTSE[i] = pContext->xtseCurr[i];
   }

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);
   nErrCode = DSL_DRV_VNX_SendMessage(pContext, CMD_HS_SELECTEDPROFILEVDSL2GET,
      sizeof(sCmd), (DSL_uint8_t*)&sCmd,
      sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS)
   {
      return DSL_ERROR;
   }

   /* get the data */
   data.XTSE[8-1] &= ~(DSL_uint8_t)(XTSE_8_01_A|XTSE_8_02_B|XTSE_8_03_C);
   if (sAck.A_US0PsdSel21 ||
      sAck.A_US0PsdSel20 ||
      sAck.A_US0PsdSel17 ||
      sAck.A_US0PsdSel16 ||
      sAck.A_US0PsdSel15 ||
      sAck.A_US0PsdSel14 ||
      sAck.A_US0PsdSel13 ||
      sAck.A_US0PsdSel12 ||
      sAck.A_US0PsdSel11 ||
      sAck.A_US0PsdSel10 ||
      sAck.A_US0PsdSel9 ||
      sAck.A_US0PsdSel8 ||
      sAck.A_US0PsdSel7 ||
      sAck.A_US0PsdSel6 ||
      sAck.A_US0PsdSel5 ||
      sAck.A_US0PsdSel4 ||
      sAck.A_US0PsdSel3 ||
      sAck.A_US0PsdSel2 ||
      sAck.A_US0PsdSel1 ||
      sAck.A_US0PsdSel0)
      data.XTSE[8-1] |= (DSL_uint8_t)XTSE_8_01_A;
   if (sAck.B_US0PsdSel2 ||
      sAck.B_US0PsdSel1 ||
      sAck.B_US0PsdSel0)
      data.XTSE[8-1] |= (DSL_uint8_t)XTSE_8_02_B;
   if (sAck.C_US0PsdSel5 ||
      sAck.C_US0PsdSel4 ||
      sAck.C_US0PsdSel1 ||
      sAck.C_US0PsdSel0)
      data.XTSE[8-1] |= (DSL_uint8_t)XTSE_8_03_C;

   /* Check consistency of ATSE status setting - Only one bit should be set at
      a time */
   for (i = 0, nBitCount = 0; i < DSL_G997_NUM_XTSE_OCTETS; i++)
   {
      for (j = 0; j < 8; j++)
      {
         if ( ((data.XTSE[i] >> j) & 0x01) != 0)
         {
            nBitCount++;
         }
      }
   }
   if (nBitCount != 1)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext,
         "DSL[%02d]: Invalid XTSE status setting - more than one (%d) mode set"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nBitCount));
   }


   /* Get Actual Profile Information*/
   actualProfile = (sAck.profileSel7 << 7) | (sAck.profileSel6 << 6) |
                   (sAck.profileSel5 << 5) | (sAck.profileSel4 << 4) |
                   (sAck.profileSel3 << 3) | (sAck.profileSel2 << 2) |
                   (sAck.profileSel1 << 1) | (sAck.profileSel0);

   /* Decode Actual Profile*/
   switch(actualProfile)
   {
   case DSL_VNX_PROFILES_8A:
      nProfile = DSL_PROFILE_8A;
      break;
   case DSL_VNX_PROFILES_8B:
      nProfile = DSL_PROFILE_8B;
      break;
   case DSL_VNX_PROFILES_8C:
      nProfile = DSL_PROFILE_8C;
      break;
   case DSL_VNX_PROFILES_8D:
      nProfile = DSL_PROFILE_8D;
      break;
   case DSL_VNX_PROFILES_12A:
      nProfile = DSL_PROFILE_12A;
      break;
   case DSL_VNX_PROFILES_12B:
      nProfile = DSL_PROFILE_12B;
      break;
   case DSL_VNX_PROFILES_17A:
      nProfile = DSL_PROFILE_17A;
      break;
   case DSL_VNX_PROFILES_30A:
      nProfile = DSL_PROFILE_30A;
      break;
   }

   /* TODO: Get Actual Band Plan Information*/

   /* Set Profile information in the DSL CPE Vinax device context*/
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.bandPlanStatusNe.nProfile, nProfile);

   DSL_CTX_WRITE(pContext, nErrCode, xtseCurr, data.XTSE);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgSelectedProfileVdsl2Get" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


/*
   This function handles the XTSE status bits

   \param pContext Pointer to dsl library context structure, [I/O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgXtseStatusGet(
   DSL_Context_t *pContext)
{
/* map the operation status bits to the ATSE octets */
#define VNX_OP_TO_XTSE(BIT, idx, mask) if(pAck->BIT) \
         data.XTSE[idx] |= mask;

   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_G997_XTUSystemEnablingData_t data;
   DSL_uint8_t i = 0, j = 0, nBitCount = 0;
   CMD_XTSE_StatusGet_t sCmd;
   ACK_XTSE_StatusGet_t sAck, *pAck = &sAck;

   memset(&data, 0, sizeof(DSL_G997_XTUSystemEnabling_t));

   /* fill up the message to be sent */
   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_XTSE_STATUSGET,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(sAck), (DSL_uint8_t*)&sAck );

   /* Copy data only if successful */
   if (nErrCode != 0)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext,
         "DSL[%02d]: ERROR - Retrieving XTSE Status failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* read the Ack */
   data.XTSE[7] &= ~XTSE_8_08;
   VNX_OP_TO_XTSE(V0, 7, XTSE_8_08)
   
   VNX_OP_TO_XTSE(V2, 7, XTSE_8_07)

   /* ANSI T1.413 1998 */
   VNX_OP_TO_XTSE(A0, 0, XTSE_1_01_A_T1_413)
   /* G.992.5, Annex M */
   VNX_OP_TO_XTSE(A1, 6, XTSE_7_03_M_5_NO)
   /* G992.1 Annex A */
   VNX_OP_TO_XTSE(A2, 0, XTSE_1_03_A_1_NO)
   /* G992.1 Annex B */
   VNX_OP_TO_XTSE(A3, 0, XTSE_1_05_B_1_NO)

   /* G.992.3, Annex A */
   VNX_OP_TO_XTSE(A8, 2, XTSE_3_03_A_3_NO)
   /* G.992.3, Annex B */
   VNX_OP_TO_XTSE(A9, 2, XTSE_3_05_B_3_NO)

   /* G.992.3, Annex L, US Mask 1
      wide upstream */
   VNX_OP_TO_XTSE(A11, 4, XTSE_5_03_L_3_NO);
   /* G.992.3, Annex L, US Mask 2
      narrow upstream */
   VNX_OP_TO_XTSE(A12, 4, XTSE_5_04_L_3_NO);
   /* G.992.3, Annex M */
   VNX_OP_TO_XTSE(A13, 4, XTSE_5_07_M_3_NO);
   /* G.992.5, Annex B */
   VNX_OP_TO_XTSE(A14, 5, XTSE_6_03_B_5_NO);
   /* G.992.5, Annex A */
   VNX_OP_TO_XTSE(A15, 5, XTSE_6_01_A_5_NO);


   /* Check consistency of XTSE status setting - Only one bit should be set at
      a time */
   for (i = 0, nBitCount = 0; i < DSL_G997_NUM_XTSE_OCTETS; i++)
   {
      for (j = 0; j < 8; j++)
      {
         if ( ((data.XTSE[i] >> j) & 0x01) != 0)
         {
            nBitCount++;
         }
      }
   }

   if (nBitCount > 1)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext,
         "DSL[%02d]: Invalid XTSE status setting - more than one (%d) mode set."
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nBitCount));
   }

   /* Copy data to the DSL CPE internal memory*/
   memcpy(pContext->xtseCurr, data.XTSE, DSL_G997_NUM_XTSE_OCTETS);

   return (nErrCode);
}

/*
   This function sends the firmware message "CMD_BandConfig_US/DS_Get"
   and returns the retrieved Data .

   \param pContext Pointer to dsl library context structure, [I]
   \param nDirection Upstream or Downstream, [I]
   \param pBandList  Retrieved Band Configuration, [O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
DSL_Error_t DSL_DRV_VNX_SendMsgBandControlGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_BandList_t *pBandList)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_BandControl_DS_Get_t  sCmd;
   ACK_BandControl_DS_Get_t  sAck;
   DSL_uint32_t nMsgId = 0;
   DSL_uint16_t i;
   static const DSL_uint8_t nMaxNumOfBands =
                               sizeof(sAck.band)/sizeof(sAck.band[0]);
   DSL_uint8_t XTSE[DSL_G997_NUM_XTSE_OCTETS] = {0};

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgBandControlGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
      DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      nMsgId = nDirection == DSL_DOWNSTREAM ? CMD_BANDCONTROL_DS_GET :
                                           CMD_BANDCONTROL_US_GET;

      memset(&sCmd, 0, sizeof(sCmd));
      sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);

      nErrCode = DSL_DRV_VNX_SendMessage( pContext, nMsgId,
                     sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                     sizeof(sAck), (DSL_uint8_t*)&sAck );

      /* Copy data only if successful */
      if (nErrCode >= 0)
      {
         if (sAck.NumBandsDS <= DSL_MIN(DSL_MAX_NUMBER_OF_BANDS, nMaxNumOfBands))
         {
            pBandList->nNumData = sAck.NumBandsDS;
         }
         else
         {
            pBandList->nNumData = DSL_MIN(DSL_MAX_NUMBER_OF_BANDS,nMaxNumOfBands);
         
            DSL_DEBUG(DSL_DBG_WRN,(pContext,
               "DSL[%02d]: Band List truncated from %d to %d!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), sAck.NumBandsDS, pBandList->nNumData ));
            
            nErrCode = DSL_ERROR;
         }

         for (i = 0; i < pBandList->nNumData; i++)
         {
            pBandList->nBand[i].nFirstToneIndex = sAck.band[i].first;
            pBandList->nBand[i].nLastToneIndex  = sAck.band[i].last;
         }
      }
      else
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, DSL_DRV_CRLF "DSL[%02d]: ERROR - Failed to retrieve Band Config!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      }
   }
   else
   {
      /*Get current xTSE octets*/
      DSL_CTX_READ(pContext, nErrCode, xtseCurr, XTSE);
      /* Only one band in ADSL mode*/
      pBandList->nNumData = 1;
      pBandList->nBand[0].nFirstToneIndex = 0;

      /*
      ADSL   Annex   US/DS
      1      A       0...32/0...256
      1      B       0...64/0...256
      2      A,I,L   0...32/0...256
      2      B,J     0...64/0...256
      2+     A,I,L   0...32/0...512
      2+     B,J,M   0...64/0...512
      */
      if (nDirection == DSL_DOWNSTREAM)
      {
         if ( (XTSE[6-1] & (XTSE_6_01_A_5_NO |  XTSE_6_03_B_5_NO | XTSE_6_07_I_5_NO)) ||
              (XTSE[7-1] & (XTSE_7_01_J_5_NO |  XTSE_7_03_M_5_NO)))
         {
            pBandList->nBand[0].nLastToneIndex = 511;
         }
         else if( (XTSE[1-1] & (XTSE_1_03_A_1_NO | XTSE_1_05_B_1_NO | XTSE_1_01_A_T1_413)) ||
                  (XTSE[3-1] & (XTSE_3_03_A_3_NO |  XTSE_3_05_B_3_NO)) ||
                  (XTSE[4-1] & (XTSE_4_05_I_3_NO |  XTSE_4_07_J_3_NO)) ||
                  (XTSE[5-1] & (XTSE_5_03_L_3_NO |  XTSE_5_04_L_3_NO | XTSE_5_07_M_3_NO)) )
         {
            pBandList->nBand[0].nLastToneIndex = 255;
         }
         else
         {
            pBandList->nBand[0].nLastToneIndex = 0;         
            
            nErrCode = DSL_ERROR;
            
            DSL_DEBUG(DSL_DBG_ERR,
              (pContext, "DSL[%02d]: ERROR - XTSE current settings not detected!"DSL_DRV_CRLF,
              DSL_DEV_NUM(pContext)));
         }
      }
      else
      {
         if ( (XTSE[1-1] & (XTSE_1_05_B_1_NO)) ||
              (XTSE[3-1] & (XTSE_3_05_B_3_NO)) ||
              (XTSE[4-1] & (XTSE_4_07_J_3_NO)) ||
              (XTSE[6-1] & (XTSE_6_03_B_5_NO)) ||
              (XTSE[7-1] & (XTSE_7_01_J_5_NO | XTSE_7_03_M_5_NO)) )
         {
            pBandList->nBand[0].nLastToneIndex = 63;
         }
         else if( (XTSE[1-1] & (XTSE_1_03_A_1_NO | XTSE_1_01_A_T1_413)) ||
                  (XTSE[3-1] & (XTSE_3_03_A_3_NO)) ||
                  (XTSE[4-1] & (XTSE_4_05_I_3_NO )) ||
                  (XTSE[5-1] & (XTSE_5_03_L_3_NO |  XTSE_5_04_L_3_NO)) ||
                  (XTSE[6-1] & (XTSE_6_01_A_5_NO |  XTSE_6_07_I_5_NO)) )
         {
            pBandList->nBand[0].nLastToneIndex = 31;
         }
         else
         {
            pBandList->nBand[0].nLastToneIndex = 0;         
            
            nErrCode = DSL_ERROR;
            
            DSL_DEBUG(DSL_DBG_ERR,
              (pContext, "DSL[%02d]: ERROR - XTSE current settings not detected!"DSL_DRV_CRLF,
              DSL_DEV_NUM(pContext)));
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgBandControlGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return
   value please refer to the description in the header file
   'drv_dsl_cpe_device_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_BitAllocationTableGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_G997_NSCData8_t *pData,
   DSL_uint32_t *pBitsPerSymbol )
{
   DSL_Error_t    nErrCode = DSL_SUCCESS;
   DSL_uint32_t   nMsgId = 0, nTonesMax = 0;
   DSL_uint16_t   i = 0, nTonesLeft = 0, nTonesCurrent = 0,
                  nAckSize = 0, nAckIndex = 0, nAckLength = 0;
   DSL_LineStateValue_t   nCurrentState = DSL_LINESTATE_UNKNOWN;
   /* assume US/DS ADSL/VDSL to have the same CMD Structures */
   CMD_BAT_TableEntriesGet_t  sCmd;
   union
   {
      ACK_BAT_TableEntriesGet_t  Vdsl;
      ACK_ADSL_BAT_US_Get_t      AdslUs;
      ACK_ADSL_BAT_DS_Get_t      AdslDs;
   }sAck;
   DSL_BandList_t  sBandList;
   DSL_Band_t     *pBand = DSL_NULL;
   DSL_uint16_t   nBand = 0;
   DSL_uint8_t    nMaxNumOfEntries = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_BitAllocationTableGet(nDirection=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nDirection));

   if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
      DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      if (nDirection == DSL_UPSTREAM)
      {
         nMsgId = CMD_ADSL_BAT_US_GET;
         nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.AdslUs.BAT);
         nAckSize = sizeof(sAck.AdslUs);
      }
      else
      {
         nMsgId = CMD_ADSL_BAT_DS_GET;
         nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.AdslDs.BAT);
         nAckSize = sizeof(sAck.AdslDs);
      }
   }
   else
   {
      /* assume VDSL */
      nMsgId = CMD_BAT_TABLEENTRIESGET;
      nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.Vdsl.BAT);
      nAckSize = sizeof(sAck.Vdsl);
   }

   /* the maximum number of tone data retreivable with one message */
   nTonesMax = 2 * nMaxNumOfEntries;

   /* Only proceed if the specified line is in SHOWTIME state.
      In other cases set number of used data elements to zero. */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   if ((nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC) ||
       (nCurrentState == DSL_LINESTATE_SHOWTIME_NO_SYNC))
   {
      /* init the return structure */
      if (pData != DSL_NULL)
      {
         memset (pData->nNSCData, 0, DSL_MAX_NSC*sizeof(pData->nNSCData[0]));
      }
      else
      {
         return DSL_ERROR;
      }

      memset(&sBandList, 0x0, sizeof(DSL_BandList_t));

      /* get number uf currently used bands */
      nErrCode = DSL_DRV_VNX_SendMsgBandControlGet(pContext, nDirection, &sBandList);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: ERROR -  getting current band config failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
         return nErrCode;
      }
      
      /* get number uf currently used bands */
      if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      {
         pData->nNumData = 4096;
      }
      else
      {
         pData->nNumData = sBandList.nBand[0].nLastToneIndex + 1;
      }

      if (pBitsPerSymbol != DSL_NULL)
         *pBitsPerSymbol = 0;

      /* retrieve data only for the appropriate bands
         in sBandList */
      for (nBand=0; nBand<sBandList.nNumData; nBand++)
      {
         pBand = &sBandList.nBand[nBand];
         /*  several messages for max 128*2 values have to be sent */
         nTonesLeft = pBand->nLastToneIndex - pBand->nFirstToneIndex + 1;

         while (nTonesLeft)
         {
            nTonesCurrent = nTonesLeft > nTonesMax ? nTonesMax : nTonesLeft;

            /* the Msg structure allows retreival in pairs only */
            sCmd.Index = (pBand->nLastToneIndex - (nTonesLeft - 1)) / 2;
            sCmd.Length = nTonesCurrent / 2;
            nErrCode = DSL_DRV_VNX_SendMessage(pContext, nMsgId,
                           sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                           nAckSize, (DSL_uint8_t*)&sAck);

            nAckIndex  = sAck.Vdsl.Index;
            nAckLength = sAck.Vdsl.Length;

            /* Copy data only if successful and valid */
            if( (nErrCode == DSL_SUCCESS) &&
                (nAckIndex <= DSL_MAX_NSC - nAckLength) )
            {
               /* Copy received data to structure */
               DSL_DEBUG(DSL_DBG_MSG,(pContext,
                  "DSL[%02d]: Bit Allocation data received (idx %d - %d):" DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), nAckIndex, nAckIndex + nAckLength));

               if (nMaxNumOfEntries < nAckLength)
               {
                  DSL_DEBUG(DSL_DBG_WRN, (pContext,
                     "DSL[%02d]: received BAT Elements truncated from %d to %d"
                     DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nAckLength, nMaxNumOfEntries));
               }

               /* The indexed words contain 2 values each */
               for(i = 0;
                  (i<nAckLength) && (i<nTonesMax/2) && (i<nMaxNumOfEntries);
                  i++)
               {
                  /* VDSL and ADSL Ack structures differ
                     only in length of entries */
                  if (pData != DSL_NULL)
                  {
                     pData->nNSCData[(i + nAckIndex) * 2] =
                        sAck.Vdsl.BAT[i].data_00;
                     pData->nNSCData[(i+nAckIndex)*2 + 1] =
                        sAck.Vdsl.BAT[i].data_01;

                     DSL_DEBUG(DSL_DBG_MSG,(pContext,
                        "%04X, ", pData->nNSCData[(i+nAckIndex)*2]));
                     DSL_DEBUG(DSL_DBG_MSG,(pContext,
                        "%04X, ", pData->nNSCData[(i+nAckIndex)*2 + 1]));
                  }

                  if (pBitsPerSymbol != DSL_NULL)
                  {
                     *pBitsPerSymbol += sAck.Vdsl.BAT[i].data_00;
                     *pBitsPerSymbol += sAck.Vdsl.BAT[i].data_01;
                  }
               }
            }
            else
            {
               DSL_DEBUG(DSL_DBG_ERR, (pContext,
                  DSL_DRV_CRLF "DSL[%02d]: ERROR - Failed to retrieve BAT %d to %d!" DSL_DRV_CRLF ,
                  DSL_DEV_NUM(pContext), sCmd.Index * 2, (sCmd.Index + sCmd.Length) * 2));

               nErrCode = DSL_ERR_FUNCTION_WAITING_TIMEOUT;
               return nErrCode;
            }

            nTonesLeft -= nTonesCurrent;
         }
      }
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext,
         "DSL[%02d]: ERROR - Function is only available if line is in showtime!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_BitAllocationTableGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_GainAllocationTableGet(
   DSL_IN DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_IN_OUT DSL_G997_NSCData16_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_VNX_FwVersion_t  sFwVersion;
   DSL_uint32_t   nMsgId = 0, nTonesMax = 0;
   DSL_uint16_t   i = 0, nTonesLeft = 0, nTonesCurrent = 0,
                  nAckSize = 0, nAckIndex = 0, nAckLength = 0;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
   /* assume US/DS ADSL/VDSL to have the same CMD Structures */
   CMD_GainTableEntriesGet_t  sCmd;
   union
   {
      ACK_GainTableEntriesGet_t  Vdsl;
      ACK_ADSL_GainTableUS_Get_t AdslUs;
      ACK_ADSL_GainTableDS_Get_t AdslDs;
   } sAck;
   DSL_BandList_t  sBandList;
   DSL_Band_t     *pBand = DSL_NULL;
   DSL_uint16_t   nBand = 0;
   DSL_uint8_t    nMaxNumOfEntries = 0;
   DSL_boolean_t  bAdslCpe = DSL_FALSE, bATUCformat = DSL_FALSE;
   /* Excess Margin Reduction*/
   DSL_uint16_t eExMrgRed = 0;
   CMD_ADSL_ExMarginReductionGet_t sCmdEMR;
   ACK_ADSL_ExMarginReductionGet_t sAckEMR;   

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_GainAllocationTableGet(nDirection=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nDirection));

   /* Only proceed if the specified line is in SHOWTIME state.
      In other cases set number of used data elements to zero. */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   if ((nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC) ||
       (nCurrentState == DSL_LINESTATE_SHOWTIME_NO_SYNC))
   {
      /* init the return structure */
      if (pData != DSL_NULL)
      {
         memset (pData->nNSCData, 0, DSL_MAX_NSC * sizeof(pData->nNSCData[0]));
      }
      else
      {
         return DSL_ERROR;
      }

      memset(&sBandList, 0x0, sizeof(DSL_BandList_t));

      /* assume VDSL */
      nErrCode = DSL_DRV_VNX_SendMsgBandControlGet(pContext, nDirection, &sBandList);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: ERROR - getting current band config failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
         return nErrCode;
      }
      
      /* get number uf currently used bands */
      if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      {
         pData->nNumData = 4096;

         nMsgId = CMD_GAINTABLEENTRIESGET;
         nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.Vdsl.Gains);
         nAckSize = sizeof(sAck.Vdsl);
      }
      else
      {
      /*...ADSL mode*/
         pData->nNumData = sBandList.nBand[0].nLastToneIndex + 1;
         
         /* Get FW info*/
         DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);

         if ( ((sFwVersion.nApplication == 5) && (sFwVersion.nFeatureSet >= 4)) ||
              ((sFwVersion.nApplication == 6) && (sFwVersion.nFeatureSet >= 5)) )
         {
            /* Assume that the GAIN values format was aligned with the ATU-C format*/
            bATUCformat = DSL_TRUE;
         }

         bAdslCpe = DSL_TRUE;

         if (nDirection == DSL_UPSTREAM)
         {
            nMsgId = CMD_ADSL_GAINTABLEUS_GET;
            nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.AdslUs.Gains);
            nAckSize = sizeof(sAck.AdslUs);
         }
         else
         {
            nMsgId = CMD_ADSL_GAINTABLEDS_GET;
            nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.AdslDs.Gains);
            nAckSize = sizeof(sAck.AdslDs);

            /* Excess Margin Reduction needed in case we use an old Q8.8 dB format*/            
            if (!bATUCformat)
            {
               sCmdEMR.Index  = 0;
               sCmdEMR.Length = 1;
            
               /* Get Excess Margin Reduction*/
               nErrCode = DSL_DRV_VNX_SendMessage (pContext, CMD_ADSL_EXMARGINREDUCTIONGET,
                             sizeof(sCmdEMR), (DSL_uint8_t*)&sCmdEMR,
                             sizeof(sAckEMR), (DSL_uint8_t*)&sAckEMR);
               if (nErrCode != DSL_SUCCESS)
               {
                  DSL_DEBUG(DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: ERROR - Excess Margin Reduction get failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                     
                  return nErrCode;
               }
               else
               {
                  eExMrgRed = sAckEMR.eSnrmReduction;
               }
            }
         }
      }

      /* the maximum number of tone data retreivable with one message */
      nTonesMax = nMaxNumOfEntries;

      /* retrieve data only for the appropriate bands in sBandList */
      for (nBand=0; nBand<sBandList.nNumData; nBand++)
      {
         pBand = &sBandList.nBand[nBand];
         /*  several messages for max 128*2 values
         have to be sent */
         nTonesLeft = pBand->nLastToneIndex - pBand->nFirstToneIndex + 1;
         while (nTonesLeft)
         {
            nTonesCurrent = nTonesLeft > nTonesMax ? nTonesMax : nTonesLeft;

            sCmd.Index = (pBand->nLastToneIndex - (nTonesLeft - 1));
            sCmd.Length = nTonesCurrent;
            nErrCode = DSL_DRV_VNX_SendMessage(pContext, nMsgId,
                          sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                          nAckSize, (DSL_uint8_t*)&sAck);

            nAckIndex = sAck.Vdsl.Index;
            nAckLength = sAck.Vdsl.Length;

            /* Copy data only if successful and valid */
            if( (nErrCode == DSL_SUCCESS) &&
                (nAckIndex <= DSL_MAX_NSC - nAckLength) )
            {
               DSL_DEBUG(DSL_DBG_MSG,(pContext,
                  "DSL[%02d]: Gain per Tone data received:" DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));

               if (nMaxNumOfEntries < nAckLength)
               {
                  DSL_DEBUG(DSL_DBG_WRN,(pContext,
                     "DSL[%02d]: received Gain Elements truncated from %d to %d"
                     DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nAckLength, nMaxNumOfEntries));
               }

               for(i = 0;
                  (i<nAckLength) && (i<nTonesMax) && (i<nMaxNumOfEntries);
                  i++)
               {
                  /* Copy received data to structure */
                  if (nDirection == DSL_DOWNSTREAM)
                  {
                     /* Rx Data is handled */
                     if (bAdslCpe == DSL_TRUE)
                     {
                        if (sAck.AdslDs.Gains[i] == Q88_NOVALUE)
                        {
                           pData->nNSCData[i + nAckIndex] = 0;
                        }
                        else
                        {
                           if (bATUCformat)
                           {
                           /* exactly the fine gains are reported here,
                              no additional calculation needed */
                           /* FW Format is Q3.13 (unsigned),
                              API codes units of 1/512 = 2^(-9)
                              (see chapter 7.5.1.20.2 of G.997.1)
                              APIV = (FWV / 2^13) * 2^9 = FWV / 2^4 */
                              pData->nNSCData[i + nAckIndex] = sAck.AdslDs.Gains[i] >> 4;
                           }
                           else
                           {
                              /* Values in Q8.8 dB */
                              pData->nNSCData[i + nAckIndex] = sAck.AdslDs.Gains[i] - eExMrgRed;
                           }
                        }
                     }
                     else
                     {
                        /* The receive gain provides the reciprocal
                           of the fine gain */
                        if (sAck.Vdsl.Gains[i] == 0)
                        {
                           pData->nNSCData[i + nAckIndex] = 0;
                        }
                        else
                        {
                           /* FW Format is Q3.13 (unsigned),
                              API codes units of 1/512 = 2^(-9)
                              (see chapter 7.5.1.20.2 of G.997.1)
                              APIV = (1 / (FWV / 2^13)) * 2^9 = 2^22 / FWV */
                           pData->nNSCData[i + nAckIndex] =
                                 0x400000 / sAck.Vdsl.Gains[i];
                        }
                     }
                  }
                  else
                  {
                     /* Tx Data is handled */
                     if (bAdslCpe == DSL_TRUE)
                     {
                        /* exactly the fine gains are reported here,
                           no additional calculation needed */
                        /* FW Format is Q3.13 (unsigned),
                           API codes units of 1/512 = 2^(-9)
                           (see chapter 7.5.1.20.2 of G.997.1)
                           APIV = (FWV / 2^13) * 2^9 = FWV / 2^4 */
                        pData->nNSCData[i + nAckIndex] = sAck.AdslUs.Gains[i] >> 4;
                     }
                     else
                     {
                        /* The transmit gain includes both
                           the fine gain g and PSD shaping gain TSSI:
                           Gains(FW) =  g * 10^( TSSI / 20 )*/
                        /* FW Format is Q3.13 (unsigned),
                           API codes units of 1/512 = 2^(-9)
                           (see chapter 7.5.1.20.2 of G.997.1)
                           APIV = (FWV / 2^13) * 2^9 = FWV / 2^4 */
                        pData->nNSCData[i + nAckIndex] = sAck.Vdsl.Gains[i] >> 4;
                     }
                  }
                  DSL_DEBUG(DSL_DBG_MSG,(pContext,
                     "%04X, ", pData->nNSCData[(i+nAckIndex)]));
               }
            }
            else
            {
               DSL_DEBUG (DSL_DBG_ERR, (pContext,
                  DSL_DRV_CRLF "DSL[%02d]: ERROR - Failed to retrieve Gains %d to %d"
                  DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
                  sCmd.Index * 2, (sCmd.Index + sCmd.Length) * 2));

               nErrCode = DSL_ERR_FUNCTION_WAITING_TIMEOUT;
               return nErrCode;
            }

            nTonesLeft -= nTonesCurrent;
         }
      }
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext,
         "DSL[%02d]: ERROR - Function is only available if line is in showtime!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_GainAllocationTableGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file
   'drv_dsl_cpe_device_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_SnrAllocationTableNeGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_OUT DSL_G997_NSCData8_t *pData)
{
   static const DSL_uint16_t nTonesMax = 100;
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nTonesLeft = 0, nTonesCurrent;
   DSL_uint16_t i = 0;
   /* assume US/DS to have the same Msg Structures */
   CMD_SNR_NE_TableEntriesGet_t  sCmd;
   ACK_SNR_NE_TableEntriesGet_t  sAck;
   DSL_BandList_t  sBandList;
   DSL_Band_t    *pBand = DSL_NULL;
   DSL_uint16_t  nBand = 0, intPart = 0;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
   static const DSL_uint8_t nMaxNumOfEntries =
                               sizeof(sAck.SNRps)/sizeof(sAck.SNRps[0]);

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG (DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SnrAllocationTableNeGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Only proceed if the specified line is in SHOWTIME state.
      In other cases set number of used data elements to zero. */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   if ((nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC) ||
       (nCurrentState == DSL_LINESTATE_SHOWTIME_NO_SYNC))
   {
      if (pData != DSL_NULL)
      {
         /* init the return structure */
         memset (pData->nNSCData, 0xFF, DSL_MAX_NSC*sizeof(pData->nNSCData[0]));
      }
      else
      {
         return DSL_ERROR;
      }

      memset(&sBandList, 0x0, sizeof(DSL_BandList_t));

      /* get number uf currently used bands */
      nErrCode = DSL_DRV_VNX_SendMsgBandControlGet(
                    pContext, DSL_DOWNSTREAM, &sBandList);
      if( nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - getting current band config failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
         return nErrCode;
      }
      
      if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      {
         pData->nNumData = 4096;
      }
      else
      {
      /*...ADSL mode*/
         pData->nNumData = sBandList.nBand[0].nLastToneIndex + 1;
      }

      /* retrieve data only for the appropriate bands
         in sBandList */
      for (nBand=0; nBand<sBandList.nNumData; nBand++)
      {
         pBand = &sBandList.nBand[nBand];
         /*  several messages for max 128*2 values
         have to be sent */
         nTonesLeft = pBand->nLastToneIndex - pBand->nFirstToneIndex + 1;
         while (nTonesLeft)
         {
            nTonesCurrent = nTonesLeft > nTonesMax ? nTonesMax : nTonesLeft;

            sCmd.Index = (pBand->nLastToneIndex - (nTonesLeft - 1));
            sCmd.Length = nTonesCurrent;
            nErrCode = DSL_DRV_VNX_SendMessage(
                          pContext, CMD_SNR_NE_TABLEENTRIESGET,
                          sizeof(CMD_SNR_NE_TableEntriesGet_t),
                          (DSL_uint8_t*)&sCmd,
                          sizeof(ACK_SNR_NE_TableEntriesGet_t),
                          (DSL_uint8_t*)&sAck);

            /* Copy data only if successful and valid */
            if( (nErrCode == DSL_SUCCESS) &&
                (sAck.Index <= DSL_MAX_NSC-sAck.Length) )
            {
               DSL_DEBUG(DSL_DBG_MSG,
                  (pContext, "DSL[%02d]: Signal to Noise Ratio data received (idx %d - %d):"
                  DSL_DRV_CRLF, DSL_DEV_NUM(pContext),sAck.Index, sAck.Index + sAck.Length));

               if( nMaxNumOfEntries < sAck.Length )
               {
                  DSL_DEBUG(DSL_DBG_WRN, (pContext,
                     "DSL[%02d]: received SNR Elements truncated from %d to %d"
                     DSL_DRV_CRLF, DSL_DEV_NUM(pContext), sAck.Length, nMaxNumOfEntries));
               }

               for( i = 0;
                   (i<sAck.Length) && (i<nTonesMax) && (i<nMaxNumOfEntries);
                   i++)
               {
                  /* Copy received data to structure */
                  if (sAck.SNRps[i] & Q88_NOVALUE)
                  {
                     /* a 0 value indicates that no measurement took place
                        in FW. DSL CPE API indicates 0xFF in this case. */
                     pData->nNSCData[i + sAck.Index] = 255;
                  }
                  else
                  {
                     /* SNR Data from the message is Q8.8 coded,
                        API value is (SNR[dB]+32)*2
                       (see chapter 7.5.1.20.4 of G.997.1) */
                     /*pData->nNSCData[i + sAck.Index] =
                       (DSL_uint8_t)
                       ((DSL_DRV_VNX_GetDbFromQ88(sAck.SNRps[i],1)+32) * 2 + 0.5);
                     */
                     intPart = ((sAck.SNRps[i] >> 8) + 32) * 2;
                     pData->nNSCData[i + sAck.Index] =
                        (DSL_uint8_t)((sAck.SNRps[i] & 0xFF) < 128 ? intPart : intPart + 1);
                  }

                  DSL_DEBUG(DSL_DBG_MSG,
                     (pContext, "%02X, ", pData->nNSCData[(i+sAck.Index)]));
               }
            }
            else
            {
               DSL_DEBUG(DSL_DBG_ERR,
                  (pContext, DSL_DRV_CRLF "DSL[%02d]: ERROR - Failed to retrieve SNR %d to %d!"
                  DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
                  sCmd.Index*2, (sCmd.Index + sCmd.Length)*2));

               nErrCode = DSL_ERR_FUNCTION_WAITING_TIMEOUT;
               return nErrCode;
            }

            nTonesLeft -= nTonesCurrent;
         }
      }
   }
   else
   {
      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - SNR allocation table retreiving only"
         " available in the SHOWTIME!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }


   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SnrAllocationTableNeGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

static DSL_void_t DSL_DRV_VNX_Array16_to_8(
   DSL_uint16_t *pIn,
   DSL_uint8_t inSz,
   DSL_uint8_t *pOut,
   DSL_uint8_t outSz)
{
   DSL_uint8_t i = 0;

   for( i = 0; i < inSz; i++ )
   {
      if( (DSL_uint32_t)(i*2 + 1) < outSz )
      {
         pOut[i*2 + 1] = (DSL_uint8_t)(pIn[i] >> 8);
         pOut[i*2]     = (DSL_uint8_t)(pIn[i]);
      }
   }
}

/*
   Reads the Far End xTSE capabilities from the device and stores it
   into the internal structure.

   \param pContext   Pointer to dsl library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgStandardInfoFeVdsl2Get(
   DSL_Context_t *pContext,
   DSL_uint8_t *pXTSE)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   /* The message structures are identical for CO and CPE */
   ACK_HS_StandardInfoFE_VDSL2Get_t sAck;
   CMD_HS_StandardInfoFE_VDSL2Get_t sCmd;

   DSL_CHECK_POINTER(pContext, pXTSE);
   DSL_CHECK_ERR_CODE();

   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   DSL_DEBUG(DSL_DBG_MSG,(pContext,
      "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgStandardInfoFeVdsl2Get" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);
   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_HS_STANDARDINFOFE_VDSL2GET,
                 sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_MSG, (pContext,
         "DSL[%02d]: ERROR - Far-End XTSE capabilities get failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   /* get the data */
   pXTSE[8-1] &= ~(DSL_uint8_t)(XTSE_8_01_A|XTSE_8_02_B|XTSE_8_03_C);
   if (sAck.A_US0PsdSup21 ||
       sAck.A_US0PsdSup20 ||
       sAck.A_US0PsdSup19 ||
       sAck.A_US0PsdSup18 ||
       sAck.A_US0PsdSup17 ||
       sAck.A_US0PsdSup16 ||
       sAck.A_US0PsdSup15 ||
       sAck.A_US0PsdSup14 ||
       sAck.A_US0PsdSup13 ||
       sAck.A_US0PsdSup12 ||
       sAck.A_US0PsdSup11 ||
       sAck.A_US0PsdSup10 ||
       sAck.A_US0PsdSup9 ||
       sAck.A_US0PsdSup8 ||
       sAck.A_US0PsdSup7 ||
       sAck.A_US0PsdSup6 ||
       sAck.A_US0PsdSup5 ||
       sAck.A_US0PsdSup4 ||
       sAck.A_US0PsdSup3 ||
       sAck.A_US0PsdSup2 ||
       sAck.A_US0PsdSup1 ||
       sAck.A_US0PsdSup0)
       pXTSE[8-1] |= (DSL_uint8_t)XTSE_8_01_A;
   if (sAck.B_US0PsdSup4 ||
       sAck.B_US0PsdSup3 ||
       sAck.B_US0PsdSup2 ||
       sAck.B_US0PsdSup1 ||
       sAck.B_US0PsdSup0)
       pXTSE[8-1] |= (DSL_uint8_t)XTSE_8_02_B;
   if (sAck.C_US0PsdSup0 ||
       sAck.C_US0PsdSup1 ||
       sAck.C_US0PsdSup4 ||
       sAck.C_US0PsdSup5 ||
       sAck.C_US0PsdSup12 ||
       sAck.C_US0PsdSup13)
       pXTSE[8-1] |= (DSL_uint8_t)XTSE_8_03_C;

   DSL_DEBUG(DSL_DBG_MSG,(pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgStandardInfoFeVdsl2Get" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   Write all the Inventory information of the Near End to the device

   \param pContext   Pointer to dsl cpe library context structure, [I]
   \param pData      Pointer to the Inventory data to set, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_NeInventorySet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t nBytesLeft, nBytesSent, nBytesMsg, nBytes;
   DSL_uint16_t nVendor = 0;
   DSL_VNX_FwVersion_t  sFwVersion;

   union
   {
      CMD_SysVendorID_O_Set_t          VendorId;
      CMD_SysVendorVersionNum_O_Set_t  VendorVersion;
      CMD_SysVendorSerialNum_O_Set_t   VendorSerial;
      CMD_AuxInventoryInfo_O_Set_t     AuxInfo;
   } sCmd;
   ACK_SysVendorID_O_Set_t sAck;
   DSL_uint8_t i = 0;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_NeInventorySet()" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get FW info*/
   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);

   DSL_DRV_MUTEX_LOCK(pContext->dataMutex);

   nVendor |= ((DSL_uint16_t)sFwVersion.nPlatform   & 0x000F) << 12;
   nVendor |= ((DSL_uint16_t)sFwVersion.nFeatureSet & 0x001F) << 7;
   nVendor |= ((DSL_uint16_t)sFwVersion.nMajor      & 0x0007) << 4;
   nVendor |= ((DSL_uint16_t)sFwVersion.nMinor      & 0x000F);

   pContext->lineInventoryNe.G994VendorID[0] = 0xB5;
   pContext->lineInventoryNe.G994VendorID[1] = 0x00;
   pContext->lineInventoryNe.G994VendorID[2] = 0x49;
   pContext->lineInventoryNe.G994VendorID[3] = 0x46;
   pContext->lineInventoryNe.G994VendorID[4] = 0x54;
   pContext->lineInventoryNe.G994VendorID[5] = 0x4E;
   pContext->lineInventoryNe.G994VendorID[6] = (DSL_uint8_t) 0xFF & (nVendor >> 8);
   pContext->lineInventoryNe.G994VendorID[7] = (DSL_uint8_t) 0xFF & nVendor;                     

   /* do the Vendor ID */
   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.VendorId.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd.VendorId);
   /* fill the message */
   for (i = 0; i < DSL_ARRAY_LENGTH(sCmd.VendorId.sysVendorID); i++)
   {
      if( (DSL_uint32_t)(i*2 + 1) <
                   DSL_ARRAY_LENGTH(pContext->lineInventoryNe.SystemVendorID) )
         sCmd.VendorId.sysVendorID[i] =
            ((DSL_uint16_t)(pContext->lineInventoryNe.SystemVendorID[i*2 + 1]) << 8) |
             (DSL_uint16_t)(pContext->lineInventoryNe.SystemVendorID[i*2]);
   }
   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_SYSVENDORID_R_SET,
                 sizeof(sCmd.VendorId), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_MSG,(pContext,
         "DSL[%02d]: ERROR - Vendor ID set, retCode=%d!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
   }

   /* do the Vendor Version */
   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.VendorVersion.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd.VendorVersion);
   for (i = 0; i < DSL_ARRAY_LENGTH(sCmd.VendorVersion.versionNum); i++)
   {
      if( (DSL_uint32_t)(i*2 + 1) <
                    DSL_ARRAY_LENGTH(pContext->lineInventoryNe.VersionNumber) )
         sCmd.VendorVersion.versionNum[i] =
            ((DSL_uint16_t)(pContext->lineInventoryNe.VersionNumber[i*2 + 1]) << 8) |
             (DSL_uint16_t)(pContext->lineInventoryNe.VersionNumber[i*2]);
   }
   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_SYSVENDORVERSIONNUM_R_SET,
                 sizeof(sCmd.VendorVersion), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_MSG,(pContext,
         "DSL[%02d]: ERROR - Vendor Version set, retCode=%d!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
   }

   /* do the Vendor Serial Num */
   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.VendorSerial.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd.VendorSerial);
   for (i = 0; i < DSL_ARRAY_LENGTH(sCmd.VendorSerial.serialNum); i++)
   {
      if( (DSL_uint32_t)(i*2 + 1) <
                     DSL_ARRAY_LENGTH(pContext->lineInventoryNe.SerialNumber) )
         sCmd.VendorSerial.serialNum[i] =
            ((DSL_uint16_t)(pContext->lineInventoryNe.SerialNumber[i*2 + 1]) << 8) |
             (DSL_uint16_t)(pContext->lineInventoryNe.SerialNumber[i*2]);
   }
   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_SYSVENDORSERIALNUM_R_SET,
                 sizeof(sCmd.VendorSerial), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_MSG,(pContext,
         "DSL[%02d]: ERROR - Vendor Serial set, retCode=%d!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
   }

   /* not for ADSL-only mode */
   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      /* do the Aux Info, not part of G997.1 */
      memset(&sCmd, 0, sizeof(sCmd));

      nBytesLeft = pContext->auxInventoryNe.nLength;
      nBytesSent = 0;
      nBytesMsg = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd.AuxInfo) * 2;
      while( nBytesLeft )
      {
         nBytes = nBytesLeft > nBytesMsg ? nBytesMsg : nBytesLeft;

         sCmd.AuxInfo.Length  = (nBytes + 1) / 2;
         sCmd.AuxInfo.Index   = nBytesSent / 2;

         for( i = 0; i < sCmd.AuxInfo.Length; i++ )
         {
            sCmd.AuxInfo.auxInfo[i] =
               ((DSL_uint16_t)(pContext->auxInventoryNe.pData[i*2 + 1 + nBytesSent]) << 8) |
                (DSL_uint16_t)(pContext->auxInventoryNe.pData[i*2 + nBytesSent]);
         }

         nErrCode = DSL_DRV_VNX_SendMessage(
                       pContext, CMD_AUXINVENTORYINFO_R_SET,
                       (DSL_uint16_t)(4 + (sCmd.AuxInfo.Length * 2)), (DSL_uint8_t*)&sCmd,
                       sizeof(sAck), (DSL_uint8_t*)&sAck);

         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG(DSL_DBG_ERR,(pContext,
               "DSL[%02d]: ERROR - Aux Info set failed, retCode=%d!"
               DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
         }
         nBytesLeft -= nBytes;
         nBytesSent += nBytes;
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   if( nErrCode != DSL_SUCCESS )
      nErrCode = DSL_ERROR;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_NeInventorySet, retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return DSL_SUCCESS;
}

static DSL_Error_t DSL_DRV_VNX_AuxLineInventoryFeGet(
   DSL_Context_t *pContext,
   DSL_AuxLineInventoryData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_AuxInventoryInfo_O_Get_t sCmd;
   ACK_AuxInventoryInfo_O_Get_t sAck;
   DSL_uint32_t nBytes = sizeof(pData->pData);
   DSL_uint8_t i = 0;

   /* do the Aux Info, not part of G997.1 */
   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   /* there was nothing copied yet */
   pData->nLength = 0;
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);

   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_AUXINVENTORYINFO_O_GET,
                 sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Aux Line Inventory Get failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return nErrCode;
   }

   pData->nLength = (sCmd.Length * 2);

   /* get the data */
   for( i = 0; i < DSL_ARRAY_LENGTH(sAck.auxInfo); i++ )
   {
      if( (DSL_uint32_t)(i*2 + 1) < nBytes )
      {
         pData->pData[i*2 + 1] = (DSL_uint8_t)(sAck.auxInfo[i] >> 8);
         pData->pData[i*2]     = (DSL_uint8_t)(sAck.auxInfo[i]);
      }
   }

   return nErrCode;
}

/*
   Read all the Inventory information of the Far End from the device,
   and store it in the DSL CPE context

   \param pContext   Pointer to dsl library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_FeInventoryGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   union
   {
      ACK_VendorID_O_Get_t             G994VendorId;
      ACK_SysVendorID_O_Get_t          VendorId;
      ACK_SysVendorVersionNum_O_Get_t  VendorVersion;
      ACK_SysVendorSerialNum_O_Get_t   VendorSerial;
   } sAck;
   CMD_SysVendorID_O_Get_t sCmd;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_FeInventoryGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_DRV_MUTEX_LOCK(pContext->dataMutex);

   /* do the G994 Vendor ID */
   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck.G994VendorId);
   nErrCode = DSL_DRV_VNX_SendMessage(pContext, CMD_VENDORID_O_GET,
      sizeof(sCmd), (DSL_uint8_t*)&sCmd,
      sizeof(sAck.G994VendorId), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - G994 Vendor ID get failed, retCoe=%d!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
   }
   else
   {
      /* get the data */
      DSL_DRV_VNX_Array16_to_8(
         sAck.G994VendorId.vendorID,
         DSL_ARRAY_LENGTH(sAck.G994VendorId.vendorID),
         pContext->lineInventoryFe.G994VendorID,
         DSL_ARRAY_LENGTH(pContext->lineInventoryFe.G994VendorID));
   }
   
   /* do the Vendor ID */
   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck.VendorId);
   nErrCode = DSL_DRV_VNX_SendMessage(pContext, CMD_SYSVENDORID_O_GET,
      sizeof(sCmd), (DSL_uint8_t*)&sCmd,
      sizeof(sAck.VendorId), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Vendor ID get failed, retCode=%d!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
   }
   else
   {
      /* get the data */
      DSL_DRV_VNX_Array16_to_8(
         sAck.VendorId.sysVendorID,
         DSL_ARRAY_LENGTH(sAck.VendorId.sysVendorID),
         pContext->lineInventoryFe.SystemVendorID,
         DSL_ARRAY_LENGTH(pContext->lineInventoryFe.SystemVendorID));
   }

   /* do the Vendor Version */
   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck.VendorVersion);

   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_SYSVENDORVERSIONNUMO_GET,
                 sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck.VendorVersion), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Vendor Version get failed, retCode=%d!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
   }
   else
   {
      /* get the data */
      DSL_DRV_VNX_Array16_to_8(
         sAck.VendorVersion.versionNum,
         DSL_ARRAY_LENGTH(sAck.VendorVersion.versionNum),
         pContext->lineInventoryFe.VersionNumber,
         DSL_ARRAY_LENGTH(pContext->lineInventoryFe.VersionNumber));
   }

   /* do the Vendor Serial Num */
   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck.VendorSerial);

   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_SYSVENDORSERIALNUM_O_GET,
                 sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck.VendorSerial), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Vendor Serial get failed, retCode=%d!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
   }
   else
   {
      /* get the data */
      DSL_DRV_VNX_Array16_to_8(
         sAck.VendorSerial.serialNum,
         DSL_ARRAY_LENGTH(sAck.VendorSerial.serialNum),
         pContext->lineInventoryFe.SerialNumber,
         DSL_ARRAY_LENGTH(pContext->lineInventoryFe.SerialNumber));
   }

   /* not for ADSL mode*/
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      nErrCode = DSL_DRV_VNX_SendMsgStandardInfoFeVdsl2Get(
                    pContext,
                    pContext->lineInventoryFe.XTSECapabilities);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - XTSE get failed, retCode=%d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nErrCode));
      }

      /* Get FE Aux Line information*/
      nErrCode = DSL_DRV_VNX_AuxLineInventoryFeGet(
                    pContext,
                    &(pContext->auxInventoryFe));
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - FE Aux line inventory get failed, retCode=%d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nErrCode));
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   if( nErrCode != DSL_SUCCESS )
      nErrCode = DSL_ERROR;

   DSL_DEBUG(DSL_DBG_MSG,(pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_FeInventoryGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   Read all the Inventory information of the Near End from the device,
   and store it in the DSL CPE context

   \param pContext   Pointer to dsl library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_NeInventoryGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_VendorID_R_Get_t sCmd;
   ACK_VendorID_R_Get_t sAck;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_NeInventoryGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_DRV_MUTEX_LOCK(pContext->dataMutex);

   /*
      XTSE capabilities
   */

   /* refresh the current NE info */
   memset(pContext->lineInventoryNe.XTSECapabilities, 0,
      DSL_G997_NUM_XTSE_OCTETS);

   if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA))
      memcpy(
         pContext->lineInventoryNe.XTSECapabilities,
         DSL_XtseAdslA,
         DSL_G997_NUM_XTSE_OCTETS);
   if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
      memcpy(
         pContext->lineInventoryNe.XTSECapabilities,
         DSL_XtseAdslB,
         DSL_G997_NUM_XTSE_OCTETS);
   if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      memcpy(
         pContext->lineInventoryNe.XTSECapabilities,
         DSL_XtseVdsl2,
         DSL_G997_NUM_XTSE_OCTETS);

   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   sCmd.Length = 4;
   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_VENDORID_R_GET,
                 sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_ERROR)
   {
      /* get the data */
      DSL_DRV_VNX_Array16_to_8(
         sAck.vendorID,
         DSL_ARRAY_LENGTH(sAck.vendorID),
         pContext->lineInventoryNe.G994VendorID,
         DSL_ARRAY_LENGTH(pContext->lineInventoryNe.G994VendorID));
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_NeInventoryGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}


static DSL_Error_t DSL_DRV_VNX_SendMsgLineStatusGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_uint8_t *pAck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t   nMsgId;
   CMD_LineStatusUS_Get_t sCmd;
   ACK_LineStatusUS_Get_t sAck;

   /* fill up the message to be sent */
   memset(&sCmd, 0, sizeof(sCmd));

   nMsgId = nDirection == DSL_DOWNSTREAM ? CMD_LINESTATUSDS_GET :
                                           CMD_LINESTATUSUS_GET;

   /*
      Starting with VDSL-FW version w.8.x.y.z.a another16-Bit word was appended
      (--> Length=9). The same will be done in future ADSL releases (check release
      notes). Older VDSL FW-versions and all current ADSL FW-versions can
      request only 8 parameters (Length=8).
   */
   sCmd.Length = 9;

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, nMsgId,
      sizeof(CMD_LineStatusUS_Get_t), (DSL_uint8_t*)&sCmd,
      sizeof(ACK_LineStatusUS_Get_t), (DSL_uint8_t*)&sAck );

   /* Copy data only if successful */
   if (nErrCode >= 0)
   {
      memcpy(pAck,&sAck, sizeof(ACK_LineStatusUS_Get_t));
   }

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_LineStatusGet(
   DSL_Context_t *pContext,
   DSL_AccessDir_t nDirection,
   DSL_G997_LineStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   union
   {
      ACK_LineStatusUS_Get_t US;
      ACK_LineStatusDS_Get_t DS;
   } sAck;

   DSL_CHECK_POINTER(pContext,pData);
   DSL_CHECK_ERR_CODE();

   /* Get line status*/
   nErrCode = DSL_DRV_VNX_SendMsgLineStatusGet(pContext, nDirection, (DSL_uint8_t*)&sAck.US);

   /* Copy data only if successful */
   if (nErrCode >= DSL_SUCCESS)
   {
      if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) || 
          DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
      {
         pData->LATN = nDirection == DSL_DOWNSTREAM ? sAck.DS.LATNds :
                                                      sAck.US.LATNus;
         /* Check for the FW 1023 special value*/
         pData->LATN = pData->LATN == 1023 ? 1271 : pData->LATN;
         
         pData->SATN = nDirection == DSL_DOWNSTREAM ? sAck.DS.SATNds :
                                                      sAck.US.SATNus;
         /* Check for the FW 1023 special value*/
         pData->SATN = pData->SATN == 1023 ? 1271 : pData->SATN;
   
         pData->SNR  = nDirection == DSL_DOWNSTREAM ? 
            ((DSL_int16_t)sAck.DS.SNRMds == -512 ? -641 : sAck.DS.SNRMds):
            ((DSL_int16_t)sAck.US.SNRMus == -512 ? -641 : sAck.US.SNRMus);
      }
      else
      {
         pData->LATN = nDirection == DSL_DOWNSTREAM ? sAck.DS.LATNds :
                                                      -32768;

         pData->SATN = nDirection == DSL_DOWNSTREAM ? sAck.DS.SATNds :
                                                      -32768;

         pData->SNR  = nDirection == DSL_DOWNSTREAM ?
            (sAck.DS.SNRMds == 0 ? -641 : sAck.DS.SNRMds):
            (sAck.US.SNRMus == 0 ? -641 : sAck.US.SNRMus);
      }

      pData->ATTNDR  = nDirection == DSL_DOWNSTREAM ?
                          ((DSL_uint32_t)sAck.DS.ATTNDRds_LSW) |
                          (((DSL_uint32_t)sAck.DS.ATTNDRds_MSW) << 16) :
                          ((DSL_uint32_t)sAck.US.ATTNDRus_LSW) |
                          (((DSL_uint32_t)sAck.US.ATTNDRus_MSW) << 16);

      pData->ACTPS   = -901;

#ifndef HAS_TO_BE_CLARIFIED
      /* KAv: FIXME: workaround for a swap in FW, see also SMS00656158*/
      if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      {
         /* Get line status*/
         nErrCode = DSL_DRV_VNX_SendMsgLineStatusGet(
            pContext,
            nDirection == DSL_DOWNSTREAM ? DSL_UPSTREAM : DSL_DOWNSTREAM,
            (DSL_uint8_t*)&sAck.US);

         /* Copy data only if successful */
         if (nErrCode >= DSL_SUCCESS)
         {
            pData->ACTATP  = nDirection == DSL_DOWNSTREAM ? sAck.US.ACTATPus :
                                                            sAck.DS.ACTATPds;
         }
      }
      else
      {
         pData->ACTATP  = nDirection == DSL_DOWNSTREAM ? sAck.DS.ACTATPds :
                                                         sAck.US.ACTATPus;
      }
#else
      pData->ACTATP  = nDirection == DSL_DOWNSTREAM ? sAck.DS.ACTATPds :
                                                      sAck.US.ACTATPus;
#endif /* #ifndef HAS_TO_BE_CLARIFIED*/      
   }

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file
   'drv_dsl_cpe_device_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_LineStatusPerBandGet(
   DSL_Context_t *pContext,
   DSL_AccessDir_t nDirection,
   DSL_G997_LineStatusPerBandData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t nMsgId = 0;
   DSL_int_t i;
   DSL_BandList_t  sBandList;
   CMD_LineStatusPerBandDS_Get_t sCmd;
   union
   {
      ACK_LineStatusPerBandUS_Get_t US;
      ACK_LineStatusPerBandDS_Get_t DS;
   } sAck;
   static const DSL_uint8_t nMaxNumOfBands =
      sizeof(sAck.DS.LineStatus_pb_ds)/sizeof(sAck.DS.LineStatus_pb_ds[0]);

   /* init all data */
   for (i=0; i<DSL_G997_MAX_NUMBER_OF_BANDS; i++)
   {
      pData->SNR[i]  = -32768;
      pData->LATN[i] = -32768;
      pData->SATN[i] = -32768;
   }

   /* Get VDSL specific information*/
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      memset(&sBandList, 0x0, sizeof(DSL_BandList_t));
      /* get number uf currently used bands */
      nErrCode = DSL_DRV_VNX_SendMsgBandControlGet(pContext, nDirection, &sBandList);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: ERROR - getting current band config failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      memset (&sCmd, 0 , sizeof(sCmd));
      sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck.US);

      nMsgId = nDirection == DSL_UPSTREAM ? CMD_LINESTATUSPERBANDUS_GET :
                                            CMD_LINESTATUSPERBANDDS_GET;

      nErrCode = DSL_DRV_VNX_SendMessage(pContext, nMsgId,
         sizeof(sCmd), (DSL_uint8_t*)&sCmd,
         sizeof(ACK_LineStatusPerBandDS_Get_t), (DSL_uint8_t*)&sAck);

      /* Copy data only if successful */
      if (nErrCode >= 0)
      {
         /* fill only the used ones */
         for (i=0; i < sBandList.nNumData; i++)
         {
            if(i >= nMaxNumOfBands || i >= DSL_G997_MAX_NUMBER_OF_BANDS)
               break;

            if (nDirection == DSL_UPSTREAM)
            {
               pData->SNR[i]  = (DSL_int16_t)sAck.US.LineStatus_pb_us[i].SNRMpb == -512 ? -641 :
                                (DSL_int16_t)sAck.US.LineStatus_pb_us[i].SNRMpb;
               pData->LATN[i] = (DSL_int16_t)sAck.US.LineStatus_pb_us[i].LATNpb == 1023 ? 1271 :
                                (DSL_int16_t)sAck.US.LineStatus_pb_us[i].LATNpb;
               pData->SATN[i] = (DSL_int16_t)sAck.US.LineStatus_pb_us[i].SATNpb == 1023 ? 1271 :
                                (DSL_int16_t)sAck.US.LineStatus_pb_us[i].SATNpb;
            }
            else
            {
               pData->SNR[i]  = (DSL_int16_t)sAck.DS.LineStatus_pb_ds[i].SNRMpb == -512 ? -641 :
                                (DSL_int16_t)sAck.DS.LineStatus_pb_ds[i].SNRMpb;
               pData->LATN[i] = (DSL_int16_t)sAck.DS.LineStatus_pb_ds[i].LATNpb == 1023 ? 1271 :
                                (DSL_int16_t)sAck.DS.LineStatus_pb_ds[i].LATNpb;
               pData->SATN[i] = (DSL_int16_t)sAck.DS.LineStatus_pb_ds[i].SATNpb == 1023 ? 1271 :
                                (DSL_int16_t)sAck.DS.LineStatus_pb_ds[i].SATNpb;
            }
         }
      }
   }

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_SendMsgLedControl(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t i;
   CMD_LED_Control_t sCmd;
   ACK_LED_Control_t sAck;

   for (i = 0; i < 2; i++)
   {
      memset(&sCmd, 0, sizeof(sCmd));
      sCmd.Length = 8;

      sCmd.LED_RESET    = CMD_LED_Control_OFF;
      sCmd.LED_FAIL     = CMD_LED_Control_BLINK_SLOW;
      if (i == 0)
      {
         sCmd.led          = CMD_LED_Control_LED1;
         sCmd.LED_GHS_INIT = CMD_LED_Control_BLINK_SLOW;
         sCmd.LED_Showtime = CMD_LED_Control_ON;
      }
      else
      {
         sCmd.led          = CMD_LED_Control_LED2;
         sCmd.LED_GHS_INIT = CMD_LED_Control_OFF;
         sCmd.LED_Showtime = CMD_LED_Control_TRAFFIC;
      }

      nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_LED_CONTROL,
                     sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                     sizeof(sAck), (DSL_uint8_t*)&sAck );

      if(nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DRV_VNX_SendMsgLedControl: ERROR - failed to set LED%d control!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext), i));
            
         return nErrCode;
      }
   }

   return (nErrCode);
}

/*
   This function sends the VINAX firmware message CMD_HybridSelect
   to configure some chip hardware related issues.

   \param pContext - Pointer to DSL library context structure, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgHybridSelect(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_HybridSelect_t sCmd;
   CMD_HybridSelect_t sAck;
   DSL_DEV_Hybrid_t nHybrid = DSL_DEV_HYBRID_NA;

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
                pDevCtx->data.deviceCfg.cfg.nHybrid,
                nHybrid);

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);

   switch (nHybrid)
   {
   case DSL_DEV_HYBRID_AD1_138_17:
      sCmd.HybridType = CMD_HybridSelect_HYB_AD1_138_17;
      break;
   case DSL_DEV_HYBRID_AD1_25_17:
      sCmd.HybridType = CMD_HybridSelect_HYB_AD1_25_17;
      break;
   case DSL_DEV_HYBRID_AD1_138_30:
      sCmd.HybridType = CMD_HybridSelect_HYB_AD1_138_30;
      break;
   case DSL_DEV_HYBRID_AD1_138_17_CPE:
      sCmd.HybridType = CMD_HybridSelect_HYB_AD1_138_17_CPE;
      break;
   case DSL_DEV_HYBRID_F1_998:
      sCmd.HybridType = CMD_HybridSelect_HYB_F1_998;
      break;
   case DSL_DEV_HYBRID_F1_998_U0:
      sCmd.HybridType = CMD_HybridSelect_HYB_F1_998_U0;
      break;
   case DSL_DEV_HYBRID_F1_998_U017_o_ISDN:
      sCmd.HybridType = CMD_HybridSelect_HYB_F1_998_U017_O_ISDN;
      break;
   case DSL_DEV_HYBRID_F1_JAP30:
      sCmd.HybridType = CMD_HybridSelect_HYB_F1_JAP30;
      break;
   case DSL_DEV_HYBRID_F1_998E17_U0:
      sCmd.HybridType = CMD_HybridSelect_HYB_F1_998E17_U0;
      break;
   case DSL_DEV_HYBRID_NA:
   case DSL_DEV_HYBRID_LAST:
      break;
   }

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_HYBRIDSELECT,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(sAck), (DSL_uint8_t*)&sAck );

   return (nErrCode);
}

/*
   This function sends the VINAX firmware message CMD_ReInitNE_Configure
   to configure some chip hardware related issues.

   \param pContext - Pointer to DSL library context structure, [I]
   \param bEnableRestart - Enables the restart triggers, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgReInitNeConfigure(
   DSL_Context_t *pContext,
   const DSL_boolean_t bEnableRestart)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_ReInitNE_Configure_t sCmd;
   ACK_ReInitNE_Configure_t sAck;

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = 1;
   if (bEnableRestart)
   {
      sCmd.E0  = 1; /* restart after LOS */
      sCmd.E1  = 1; /* restart after LOF */
      sCmd.E3  = 1; /* restart after LOM */
      sCmd.E10 = 1; /* restart after OOS */
      sCmd.E13 = 1; /* restart after ESE */
   }

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_REINITNE_CONFIGURE,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(sAck), (DSL_uint8_t*)&sAck );

   return (nErrCode);
}

/*
   This function sends the VINAX firmware message CMD_ReInitFE_Configure
   to configure some chip hardware related issues.

   \param pContext - Pointer to DSL library context structure, [I]
   \param bEnableRestart - Enables the restart triggers, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgReInitFeConfigure(
   DSL_Context_t *pContext,
   const DSL_boolean_t bEnableRestart)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_ReInitFE_Configure_t sCmd;
   ACK_ReInitFE_Configure_t sAck;

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = 1;

   if (bEnableRestart)
   {
      sCmd.E0  = 1; /* restart after LOS */
      /* sCmd.E1  = 1;*/ /* restart after LOF */
      sCmd.E3  = 1; /* restart after LOM */
      /* sCmd.E10 = 1;*/ /* restart after OOS */
      /* sCmd.E13 = 1;*/ /* restart after ESE */
   }

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_REINITFE_CONFIGURE,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(sAck), (DSL_uint8_t*)&sAck );

   return (nErrCode);
}

/*
   This function sends the VINAX firmware message CMD_TEST_TxControlSet
   to configure some chip hardware related issues.

   \param pContext  - Pointer to DSL library context structure, [I]
   \param TxControl - Tx Control options, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgTxControlSet(
   DSL_Context_t *pContext,
   const DSL_uint16_t TxControl)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_TEST_TxControlSet_t sCmd;
   ACK_TEST_TxControlSet_t sAck;

   if( !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
       !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      return nErrCode;
   }

   switch(TxControl)
   {
   case CMD_TEST_TxControlSet_SEND_QUIET:
   case CMD_TEST_TxControlSet_SEND_TRAINING:
   case CMD_TEST_TxControlSet_SEND_MEDLEY:
      sCmd.TxControl = TxControl;

      memset(&sCmd, 0, sizeof(sCmd));
      sCmd.Length = 1;

      nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_TEST_TXCONTROLSET,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck );
   break;

   default:
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unsupported TxControl option %d!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), TxControl));
         
      nErrCode = DSL_ERROR;
   break;
   }

   return (nErrCode);
}


/*
   This function sends the firmware message "CMD_FrameDataExt1XX_Get".

   \param pContext Pointer to dsl library context structure, [I]
   \param nDirection Upstream or Downstream, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
DSL_Error_t DSL_DRV_VNX_SendMsgFrameDataExt1Get(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_uint8_t *pAck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_FrameDataExt1US_Get_t sCmd;
   /* Structures for the DS and US directions are the same.
       So we can use any for CMD and ACK*/
   ACK_FrameDataExt1US_Get_t sAck;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   /* fill up the message to be sent */
   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.Length = 26;

   nErrCode = DSL_DRV_VNX_SendMessage( pContext,
                  nDirection == DSL_DOWNSTREAM ? CMD_FRAMEDATAEXT1DS_GET :
                                                 CMD_FRAMEDATAEXT1US_GET,
                  sizeof(CMD_FrameDataExt1US_Get_t), (DSL_uint8_t*)&sCmd,
                  sizeof(ACK_FrameDataExt1US_Get_t), (DSL_uint8_t*)&sAck );

   /* Copy data only if successful */
   if (nErrCode >= 0)
   {
      memcpy(pAck,&sAck, sizeof(ACK_FrameDataExt1US_Get_t));
   }

   return (nErrCode);
}

/*
   This function sends the firmware message "CMD_FrameDataExt2XX_Get".

   \param pContext Pointer to dsl library context structure, [I]
   \param nDirection Upstream or Downstream, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
DSL_Error_t DSL_DRV_VNX_SendMsgFrameDataExt2Get(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_uint8_t *pAck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   /* Structures for the DS and US directions are the same.
       So we can use any for CMD and ACK*/
   CMD_FrameDataExt2US_Get_t sCmd;
   ACK_FrameDataExt2US_Get_t sAck;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   /* fill up the message to be sent */
   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);

   nErrCode = DSL_DRV_VNX_SendMessage( pContext,
                  nDirection == DSL_DOWNSTREAM ? CMD_FRAMEDATAEXT2DS_GET :
                                                 CMD_FRAMEDATAEXT2US_GET,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(sAck), (DSL_uint8_t*)&sAck );

   /* Copy data only if successful */
   if (nErrCode >= 0)
   {
      memcpy(pAck, &sAck, sizeof(ACK_FrameDataExt2US_Get_t));
   }

   return (nErrCode);
}

/*
   This function handles the VINAX firmware message
   for the ADSL framing parameters

   \param pContext Pointer to dsl library context structure, [I]
   \param nDir     Direction to retrieve data for, US/DS, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
DSL_Error_t DSL_DRV_VNX_SendMsgAdslFrameDataGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDir,
   DSL_uint8_t *pAck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   /* Structures for the DS and US directions are the same.
       So we can use any for CMD and ACK*/
   CMD_ADSL_FrameDataDS_LP0Get_t sCmd;
   ACK_ADSL_FrameDataDS_LP0Get_t sAck;
   DSL_boolean_t bAdsl1 = DSL_FALSE;
   DSL_uint32_t nAdsl1Mode = 0;
   DSL_uint32_t nMsgId = 0;
   DSL_uint8_t lp = 0;

   DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCurr[0], nAdsl1Mode);

   /* Check for the ADSL1 mode*/   
   if ( (nAdsl1Mode & XTSE_1_03_A_1_NO) || (nAdsl1Mode & XTSE_1_05_B_1_NO))
   {
      bAdsl1 = DSL_TRUE;
   }

   /* loop over the latency paths */
   for (lp = 0 ; lp < 2; lp++)
   {
      /* each latency path needs to be handled differently */
      if (lp == 0)
      {
         nMsgId = (nDir == DSL_UPSTREAM) ? CMD_ADSL_FRAMEDATAUS_LP0GET : CMD_ADSL_FRAMEDATADS_LP0GET;
      }
      else
      {
         /* LP1 may be used for ADSL1 only */
         if (!bAdsl1)
            break;

         nMsgId = (nDir == DSL_UPSTREAM) ? CMD_ADSL_FRAMEDATAUS_LP1GET : CMD_ADSL_FRAMEDATADS_LP1GET;
      }
   
      memset(&sCmd, 0, sizeof(sCmd));
      memset(&sAck, 0, sizeof(sAck));

      sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);

      nErrCode = DSL_DRV_VNX_SendMessage(pContext, nMsgId,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck);

      /* if the current latency path seems not to carry proper data 
         try the next */
      if (sAck.Lp == 0)
         continue;
   }

   /* Copy data only if successful */
   if (nErrCode >= 0)
   {
      memcpy(pAck, &sAck, sizeof(ACK_ADSL_FrameDataDS_LP0Get_t));
   }

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_OnEventChannelDataRateUpdate(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDir)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_G997_ChannelStatusData_t chStsData;

   nErrCode = DSL_DRV_VNX_ChannelStatusGet(pContext, 0, nDir, &chStsData);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Channel status %s get failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nDir == DSL_UPSTREAM ? "US" : "DS"));
         
      return nErrCode;
   }

   nErrCode = DSL_DRV_EventGenerate(
                  pContext, 0, nDir, DSL_XTUDIR_NA, DSL_EVENT_S_CHANNEL_DATARATE,
                  (DSL_EventData_Union_t*)&chStsData,
                  sizeof(DSL_G997_ChannelStatusData_t));

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), DSL_EVENT_S_CHANNEL_DATARATE));
         
      return nErrCode;
   }

   return nErrCode;
}

static DSL_Error_t DSL_DRV_VNX_OnEventChannelDataRateThresholdCrossingUpdate(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDir,
   DSL_uint32_t *pOldDr,
   DSL_uint32_t *pNewDr)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_boolean_t bNotifyDataRateShift = DSL_FALSE;
   DSL_uint32_t nDataRateShiftThresh = 0;
   DSL_G997_DataRateShiftThresholdCrossingData_t DataRateShiftThresholdCrossingData;
#ifdef INCLUDE_DSL_ADSL_MIB
   DSL_MIB_ADSL_Thresholds_t nMibThresholds = 0;
#endif

   DSL_CHECK_POINTER(pContext, pOldDr);
   DSL_CHECK_POINTER(pContext, pNewDr);
   DSL_CHECK_ERR_CODE();

   /* Check for the Data Rate Upshift*/
   if( *pNewDr > *pOldDr )
   {
      if(nDir == DSL_UPSTREAM)
      {
         DSL_CTX_READ_SCALAR(pContext, nErrCode,
            channelDataRateThresholdUs.nDataRateThresholdUpshift, nDataRateShiftThresh);
      }
      else
      {
         DSL_CTX_READ_SCALAR(pContext, nErrCode,
            channelDataRateThresholdDs.nDataRateThresholdUpshift, nDataRateShiftThresh);
      }

      if( *pNewDr - *pOldDr > nDataRateShiftThresh)
      {
         /* Upshift occured*/
         bNotifyDataRateShift = DSL_TRUE;
         DataRateShiftThresholdCrossingData.nDataRateThresholdType =
            DSL_G997_DATARATE_THRESHOLD_UPSHIFT;
      }
   }

   /* Check for the Data Rate Downshift*/
   if( *pNewDr < *pOldDr )
   {
      if(nDir == DSL_UPSTREAM)
      {
         DSL_CTX_READ_SCALAR(pContext, nErrCode,
            channelDataRateThresholdUs.nDataRateThresholdDownshift, nDataRateShiftThresh);
      }
      else
      {
         DSL_CTX_READ_SCALAR(pContext, nErrCode,
            channelDataRateThresholdDs.nDataRateThresholdDownshift, nDataRateShiftThresh);
      }
      if( *pOldDr - *pNewDr > nDataRateShiftThresh)
      {
         bNotifyDataRateShift = DSL_TRUE;
         /* Downshift occured*/
         DataRateShiftThresholdCrossingData.nDataRateThresholdType =
            DSL_G997_DATARATE_THRESHOLD_DOWNSHIFT;
      }
   }

   if( bNotifyDataRateShift && nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_EventGenerate(
                     pContext, 0, nDir, DSL_XTUDIR_NA,
                     DSL_EVENT_I_CHANNEL_DATARATE_SHIFT_THRESHOLD_CROSSING,
                     (DSL_EventData_Union_t*)&DataRateShiftThresholdCrossingData,
                     sizeof(DSL_G997_DataRateShiftThresholdCrossingData_t));

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext),
            DSL_EVENT_I_CHANNEL_DATARATE_SHIFT_THRESHOLD_CROSSING));
            
         return nErrCode;
      }
#ifdef INCLUDE_DSL_ADSL_MIB
      else
      {
         DSL_CTX_READ_SCALAR(pContext, nErrCode, MibAdslCtx.nThresholds,
            nMibThresholds);
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, MibAdslCtx.nThresholds,
            nMibThresholds | ((nDir == DSL_UPSTREAM) ? 
            DSL_MIB_THRESHOLD_ATUR_RATE_CHANGE_FLAG :
            DSL_MIB_THRESHOLD_ATUC_RATE_CHANGE_FLAG));
      }
#endif
   }

   return nErrCode;
}

static DSL_Error_t DSL_DRV_VNX_DataRateUpdate(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDir,
   DSL_uint32_t dataRate)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t oldDataRate = 0;

   if (nDir == DSL_UPSTREAM)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nChannelActualDataRatePrevUs[0], oldDataRate);
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRatePrevUs[0], dataRate);

      if (dataRate != oldDataRate)
	  {
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelPreviousDataRateUs[0], oldDataRate);
	  }
   }
   else
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nChannelActualDataRatePrevDs[0], oldDataRate);
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRatePrevDs[0], dataRate);

      if (dataRate != oldDataRate)
	  {
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelPreviousDataRateDs[0], oldDataRate);
	  }
   }

   if( dataRate != oldDataRate )
   {
      nErrCode = DSL_DRV_VNX_OnEventChannelDataRateUpdate(pContext, nDir);
   }

   if( oldDataRate != 0 && nErrCode == DSL_SUCCESS )
   {
      nErrCode = DSL_DRV_VNX_OnEventChannelDataRateThresholdCrossingUpdate(
                    pContext, nDir,
                    &oldDataRate, &dataRate);
   }

   return nErrCode;
}

/*
   This function sends the firmware message "CMD_BearerChsXX_Get".

   \param pContext Pointer to dsl library context structure, [I]
   \param nChannel Specifies the bearer channel for the message, [I]
   \param nDirection Upstream or Downstream, [I]
   \param pData pointer to the channel status data, [O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgBearerChannelStatusGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_uint8_t *pAck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   /* Assume that US/DS messages have same format */
   CMD_BearerChsUS_Get_t sCmd;
   union
   {
      ACK_BearerChsUS_Get_t US;
      ACK_BearerChsDS_Get_t DS;
   } sAck;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   /* fill up the message to be sent */
   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);

   nErrCode =  DSL_DRV_VNX_SendMessage( pContext,
                    nDirection == DSL_DOWNSTREAM ? CMD_BEARERCHSDS_GET :
                                                   CMD_BEARERCHSUS_GET,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck );

   /* Copy data only if successful */
   if (nErrCode >= 0)
   {
      memcpy(pAck,&sAck, sizeof(ACK_BearerChsUS_Get_t));
   }

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_SendMsgOlrStatusGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_uint8_t *pAck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   /* Assume that US/DS messages have same format */
   CMD_OLR_RxStatusGet_t sCmd;
   union
   {
      ACK_OLR_TxStatusGet_t US;
      ACK_OLR_RxStatusGet_t DS;
   } sAck;

   /* fill up the message to be sent */
   memset(&sCmd, 0, sizeof(sCmd));

   sCmd.Length = 15;

   nErrCode =  DSL_DRV_VNX_SendMessage( pContext,
                    nDirection == DSL_DOWNSTREAM ? CMD_OLR_RXSTATUSGET :
                                                   CMD_OLR_TXSTATUSGET,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck );

   /* Copy data only if successful */
   if (nErrCode >= 0)
   {
      memcpy(pAck,&sAck, sizeof(ACK_OLR_RxStatusGet_t));
   }

   return (nErrCode);
}


static DSL_Error_t DSL_DRV_VNX_Vdsl1ChStatusGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDir)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t ActINP = 0;
   DSL_uint16_t D = 0;
   DSL_uint32_t R, I, N, P, K, U;
   DSL_uint32_t R_tot, R_net;
   DSL_uint32_t f_s = 0;
   DSL_uint32_t RP = 0;
   DSL_uint32_t newDataRate = 0;
   DSL_uint32_t ActualInterleaveDelay = 0;

   union
   {
      ACK_FrameDataExt1US_Get_t US;
      ACK_FrameDataExt1DS_Get_t DS;
   } sAck;

   union
   {
      ACK_BearerChsUS_Get_t US;
      ACK_BearerChsDS_Get_t DS;
   } sAckBch;

   nErrCode = DSL_DRV_VNX_SendMsgFrameDataExt1Get(pContext,nDir, (DSL_uint8_t*)&(sAck.DS));
   if(nErrCode < 0)
      return nErrCode;

   nErrCode = DSL_DRV_VNX_SendMsgBearerChannelStatusGet(pContext, nDir, (DSL_uint8_t*)&(sAckBch.DS));
   if(nErrCode < 0)
      return nErrCode;

   if (nDir == DSL_UPSTREAM)
   {
      newDataRate = (sAckBch.US.DRusLP0_LSW | (sAckBch.US.DRusLP0_MSW << 16)) * 4000;
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRateUs[0], newDataRate);
   }
   else
   {
      newDataRate = (sAckBch.DS.DRdsLP0_LSW | (sAckBch.DS.DRdsLP0_MSW << 16)) * 4000;
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRateDs[0], newDataRate);
   }

   if (nDir == DSL_UPSTREAM)
   {
      /*
      G.992.1/2: Rf/s for fast path, Ri/S for interleaved path
      G.992.3/4/5: RP in G.992.3/4/5 (number of RS check bytes per FEC data frame)
      G.993.1: The value corresponds to R (number of RS check bytes per code word) */
      RP   = (DSL_uint32_t)sAck.US.R_LP0us;
      /* Interleaver Depth */
      D  = sAck.US.D_LP0us;
      /* RS parity check bytes per RS-codeword */
      R  = (DSL_uint32_t)sAck.US.R_LP0us;
      /* Interleaver Block Length */
      I  = (DSL_uint32_t)sAck.US.I_LP0us;
      /* RS-codeword size */
      N  = (DSL_uint32_t)sAck.US.N_LP0us;
      /* Bytes per symbol */
      P  = (DSL_uint32_t)sAck.US.P_LP0us;
      /* information bytes per RS-codeword */
      K  = (DSL_uint32_t)sAck.US.K_LP0us;
      /* data bytes per frame including dummy bytes */
      U  = (DSL_uint32_t)sAck.US.U_LP0us;
   }
   else
   {
      /*
      G.992.1/2: Rf/s for fast path, Ri/S for interleaved path
      G.992.3/4/5: RP in G.992.3/4/5 (number of RS check bytes per FEC data frame)
      G.993.1: The value corresponds to R (number of RS check bytes per code word) */
      RP   = (DSL_uint32_t)sAck.DS.R_LP0ds;
      /* Interleaver Depth */
      D    = sAck.DS.D_LP0ds;
      /* RS parity check bytes per RS-codeword */
      R  = (DSL_uint32_t)sAck.DS.R_LP0ds;
      /* Interleaver Block Length */
      I  = (DSL_uint32_t)sAck.DS.I_LP0ds;
      /* RS-codeword size */
      N  = (DSL_uint32_t)sAck.DS.N_LP0ds;
      /* Bytes per symbol */
      P  = (DSL_uint32_t)sAck.DS.P_LP0ds;
      /* information bytes per RS-codeword */
      K  = (DSL_uint32_t)sAck.DS.K_LP0ds;
      /* data bytes per frame including dummy bytes */
      U  = (DSL_uint32_t)sAck.DS.U_LP0ds;
   }

   /* FIXME: here a current value should be used
      instead of the config entry */
#if 0
   if (DSL_UINT8_GET(DSL_DBL_PROFILE) == DSL_PROFILE_30A)
      f_s = 8;
   else
      f_s = 4;
#else
   f_s = 8;
#endif

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: R = %d, I = %d, N = %d, P = %d, K = %d, U = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), R, I, N, P, K, U));

   R_net = newDataRate/1000;
   /* The calculation is not applicable for 0 delays */
   if ( R_net*(P*(K+R))*(N*U) == 0 )
   {
      ActINP = 0;
      DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: ActINP not applicable." DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      /* This formula is only valid for VDSL1 */
      R_tot =  R_net*(P*(K+R))/(N*U);
      /* This formula returns the actual Impulse Noise Protection
         as a number of half frames */
      ActINP   =  (DSL_uint32_t)(2*((R * I) / (2*N)) * (8*D*f_s) / R_tot);
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: R_net = %d, R_tot = %d, ActINP = %d" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext),
         R_net, R_tot, (2*((R * I) / (2*N)) * (8*D*f_s) / R_tot)));
   }

   ActualInterleaveDelay = nDir == DSL_UPSTREAM ? sAckBch.US.delay_usLP0 * 100 :
                                                  sAckBch.DS.delay_dsLP0 * 100;

   if (nDir == DSL_UPSTREAM)
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualInterleaveDelayUs[0], ActualInterleaveDelay);
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualImpulseNoiseProtectionUs[0], ActINP);
   }
   else
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualInterleaveDelayDs[0], ActualInterleaveDelay);
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualImpulseNoiseProtectionDs[0], ActINP);
   }

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_Vdsl2ChStatusGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDir)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t ActINP = 0;
   DSL_uint32_t Lp, Rp, Dp, q_p;
   DSL_uint32_t newDataRate = 0;
   DSL_uint32_t ActualInterleaveDelay = 0;

   union
   {
      ACK_FrameDataExt2US_Get_t US;
      ACK_FrameDataExt2DS_Get_t DS;
   } sAck;

   union
   {
      ACK_BearerChsUS_Get_t US;
      ACK_BearerChsDS_Get_t DS;
   } sAckBch;

   nErrCode = DSL_DRV_VNX_SendMsgFrameDataExt2Get(pContext, nDir, (DSL_uint8_t*)&(sAck.DS));
   if( nErrCode < 0 )
      return nErrCode;

   nErrCode = DSL_DRV_VNX_SendMsgBearerChannelStatusGet(pContext, nDir, (DSL_uint8_t*)&(sAckBch.DS));
   if(nErrCode < 0)
      return nErrCode;

   if (nDir == DSL_UPSTREAM)
   {
      newDataRate = (sAckBch.US.DRusLP0_LSW | (sAckBch.US.DRusLP0_MSW << 16)) * 4000;
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRateUs[0], newDataRate);
   }
   else
   {
      newDataRate = (sAckBch.DS.DRdsLP0_LSW | (sAckBch.DS.DRdsLP0_MSW << 16)) * 4000;
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRateDs[0], newDataRate);
   }

   if (nDir == DSL_UPSTREAM)
   {
      Lp  = sAck.US.Lp_LP0us;
      Rp  = sAck.US.Rp_LP0us;
      Dp  = sAck.US.Dp_LP0us;
      q_p = sAck.US.q_LP0us;
   }
   else
   {
      Lp  = sAck.DS.Lp_LP0ds;
      Rp  = sAck.DS.Rp_LP0ds;
      Dp  = sAck.DS.Dp_LP0ds;
      q_p = sAck.DS.q_LP0ds;
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: Lp = %d, Rp = %d, Dp = %d, q_p = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), Lp, Rp, Dp, q_p));

   /* The calculation is not applicable for 0 delays */
   if ((q_p * Lp) == 0)
   {
      ActINP = 0;
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ActINP not applicable." DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      /* Formula for VDSL2 framing:*/
      ActINP = (DSL_uint32_t)(10 * 8 * Dp * (DSL_uint32_t)(Rp / (2 * q_p)) / Lp);
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext,"DSL[%02d]: ActINP = %d" DSL_DRV_CRLF, DSL_DEV_NUM(pContext), ActINP));
   }

   ActualInterleaveDelay = nDir == DSL_UPSTREAM ? sAckBch.US.delay_usLP0 * 100 :
                                                  sAckBch.DS.delay_dsLP0 * 100;

   if (nDir == DSL_UPSTREAM)
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualInterleaveDelayUs[0], ActualInterleaveDelay);
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualImpulseNoiseProtectionUs[0], ActINP);
   }
   else
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualInterleaveDelayDs[0], ActualInterleaveDelay);
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualImpulseNoiseProtectionDs[0], ActINP);
   }

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_AdslChStatusGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDir)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_boolean_t bAdsl1 = DSL_FALSE;
   DSL_uint32_t nAdsl1Mode = 0;
   DSL_uint32_t ActINP = 0;
   ACK_ADSL_FrameDataDS_LP0Get_t sAck;
   DSL_uint32_t newDataRate = 0;
   DSL_boolean_t bTrellisEnable = DSL_FALSE;
   DSL_uint32_t ActualInterleaveDelay = 0;

   union
   {
      ACK_BearerChsUS_Get_t US;
      ACK_BearerChsDS_Get_t DS;
   } sAckBch;

   nErrCode = DSL_DRV_VNX_SendMsgAdslFrameDataGet(pContext, nDir, (DSL_uint8_t*)&sAck);
   if(nErrCode < 0)
      return nErrCode;

   /* calculate the ActINP here nActInp =  Rp*Dp*4/Lp */
   if (sAck.Lp > 0)
   {
      ActINP = ( 2 * 4 * sAck.Rp * sAck.Dp) / sAck.Lp;
   }

   nErrCode = DSL_DRV_VNX_SendMsgBearerChannelStatusGet(pContext, nDir, (DSL_uint8_t*)&(sAckBch.DS));
   if(nErrCode < 0)
      return nErrCode;

   DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCurr[0], nAdsl1Mode);

   /* Check for the ADSL1 mode*/   
   if ( (nAdsl1Mode & XTSE_1_03_A_1_NO) || (nAdsl1Mode & XTSE_1_05_B_1_NO))
   {
      bAdsl1 = DSL_TRUE;
   }

   if (nDir == DSL_UPSTREAM)
   {
      /* check whether fast path information has to be used */
      if (bAdsl1)
      {
         newDataRate = (sAckBch.US.DRusLP1_LSW | (sAckBch.US.DRusLP1_MSW << 16));
         ActualInterleaveDelay = sAckBch.US.delay_usLP1;

      }

      newDataRate += (sAckBch.US.DRusLP0_LSW | (sAckBch.US.DRusLP0_MSW << 16));
      newDataRate *= 4000;
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRateUs[0], newDataRate);

      ActualInterleaveDelay += sAckBch.US.delay_usLP0;
      ActualInterleaveDelay *= 100;
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualInterleaveDelayUs[0], ActualInterleaveDelay);

      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualImpulseNoiseProtectionUs[0], ActINP);
      
      bTrellisEnable = (DSL_boolean_t)sAckBch.US.TCMstatus_usLP0;
   }
   else
   {
      /* check whether fast path information has to be used */
      if (bAdsl1)
      {
         newDataRate = (sAckBch.DS.DRdsLP1_LSW | (sAckBch.DS.DRdsLP1_MSW << 16));
         ActualInterleaveDelay = sAckBch.DS.delay_dsLP1;
      }

      newDataRate += (sAckBch.DS.DRdsLP0_LSW | (sAckBch.DS.DRdsLP0_MSW << 16));
      newDataRate *= 4000;
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRateDs[0], newDataRate);

      ActualInterleaveDelay += sAckBch.DS.delay_dsLP0;
      ActualInterleaveDelay *= 100;
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualInterleaveDelayDs[0], ActualInterleaveDelay);

      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, ActualImpulseNoiseProtectionDs[0], ActINP);
                  
      bTrellisEnable = (DSL_boolean_t)sAckBch.DS.TCMstatus_dsLP0;
   }

   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, lineFeatureDataSts[nDir].bTrellisEnable, bTrellisEnable);

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_ChannelStatusUpdate(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_AccessDir_t nDirection)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t newDataRate = 0;
   DSL_boolean_t bTrellisEnable = DSL_FALSE, bBitswapEnable = DSL_FALSE;
   
   union
   {
      ACK_LineStatusUS_Get_t US;
      ACK_LineStatusDS_Get_t DS;
   } sAckLs;

   union
   {
      ACK_OLR_TxStatusGet_t US;
      ACK_OLR_RxStatusGet_t DS;
   } sAckOlr;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_ChannelStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) || 
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1))
   {
      /* Get line status*/
      nErrCode = DSL_DRV_VNX_SendMsgLineStatusGet(pContext, nDirection, (DSL_uint8_t*)&sAckLs.US);
      if(nErrCode < 0)
         return nErrCode;

      bTrellisEnable = nDirection == DSL_UPSTREAM ?
                      (DSL_boolean_t)sAckLs.US.TCMstatus_us :
                      (DSL_boolean_t)sAckLs.DS.TCMstatus_ds;

      DSL_CTX_WRITE_SCALAR(
         pContext, nErrCode, lineFeatureDataSts[nDirection].bTrellisEnable, bTrellisEnable);

      nErrCode = DSL_DRV_VNX_SendMsgOlrStatusGet(pContext, nDirection, (DSL_uint8_t*)&sAckOlr.US);
      if(nErrCode < 0)
         return nErrCode;

      if (sAckOlr.US.BitswapReqs || sAckOlr.US.ExtBitswapReqs ||
          sAckOlr.US.BitswapUTCs || sAckOlr.US.BitswapsDone)
      {
         bBitswapEnable = DSL_TRUE;
      }

      DSL_CTX_WRITE_SCALAR(
         pContext, nErrCode, lineFeatureDataSts[nDirection].bBitswapEnable, bBitswapEnable);
   }

   /* Get bearer channel params and calculate INP */
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      /* VDSL2 mode */
      nErrCode = DSL_DRV_VNX_Vdsl2ChStatusGet(pContext, nDirection);
   }
   else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1))
   {
      /* VDSL1 mode */
      nErrCode = DSL_DRV_VNX_Vdsl1ChStatusGet(pContext, nDirection);
   }
   else
   {
      /* ADSL mode */
      nErrCode = DSL_DRV_VNX_AdslChStatusGet(pContext, nDirection);
   }

   if(nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Channel Status Data for %s get failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nDirection == DSL_DOWNSTREAM ? "DS":"US"));
   }

   if (nDirection == DSL_UPSTREAM)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nChannelActualDataRateUs[0], newDataRate);
   }
   else
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nChannelActualDataRateDs[0], newDataRate);
   }

   /* Update Data Rate*/
   nErrCode = DSL_DRV_VNX_DataRateUpdate(pContext, nDirection, newDataRate);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Data Rate Update failed (nDirection=%d)!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nDirection));
         
      return nErrCode;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_ChannelStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_VNX_ChannelStatusGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_AccessDir_t nDirection,
   DSL_G997_ChannelStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   if (nDirection == DSL_UPSTREAM)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, 
         nChannelActualDataRateUs[nChannel], pData->ActualDataRate);

      DSL_CTX_READ_SCALAR(pContext, nErrCode, 
         nChannelPreviousDataRateUs[nChannel], pData->PreviousDataRate);

      DSL_CTX_READ_SCALAR(pContext, nErrCode, 
         ActualInterleaveDelayUs[nChannel], pData->ActualInterleaveDelay);

      DSL_CTX_READ_SCALAR(pContext, nErrCode, 
         ActualImpulseNoiseProtectionUs[nChannel], pData->ActualImpulseNoiseProtection);
   }
   else
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, 
         nChannelActualDataRateDs[nChannel], pData->ActualDataRate);

      DSL_CTX_READ_SCALAR(pContext, nErrCode, 
         nChannelPreviousDataRateDs[nChannel], pData->PreviousDataRate);

      DSL_CTX_READ_SCALAR(pContext, nErrCode, 
         ActualInterleaveDelayDs[nChannel], pData->ActualInterleaveDelay);

      DSL_CTX_READ_SCALAR(pContext, nErrCode, 
         ActualImpulseNoiseProtectionDs[nChannel], pData->ActualImpulseNoiseProtection);
   }

   return nErrCode;
}


/*
   This function sets the NE/FE data path failures to the DSL CPE context.

   \param pContext - Pointer to dsl library context structure, [I/O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
DSL_Error_t DSL_DRV_VNX_SendMsgDataPathFailuresStatusGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_DataPathFailuresGet_t sCmd;
   ACK_DataPathFailuresGet_t sAck;

   DSL_uint32_t nActDPF_NE = 0;
   DSL_uint32_t nActDPF_FE = 0;
   DSL_uint32_t nActDPFM_NE = 0;
   DSL_uint32_t nActDPFM_FE = 0 ;
   DSL_uint32_t nPreDPF_NE = 0;
   DSL_uint32_t nPreDPF_FE = 0;
   DSL_uint32_t nCfgDPFM_NE = 0;
   DSL_uint32_t nCfgDPFM_FE = 0;
   DSL_uint32_t nRelevantBitMask = 0;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;

   nRelevantBitMask = DSL_G997_DATAPATHFAILURE_NCD |
                      DSL_G997_DATAPATHFAILURE_LCD |
                      DSL_G997_DATAPATHFAILURE_OOS;

   sCmd.Index  = 0;
   sCmd.Length = 2;
   sCmd.LinkNo = 0;

   nErrCode =  DSL_DRV_VNX_SendMessage( pContext, CMD_DATAPATHFAILURESGET,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck );
   if( nErrCode < 0 )
   {
      return nErrCode;
   }

   /* Get previous state of Data Path Failures */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nDataPathFailuresFe[0], nPreDPF_FE);
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nDataPathFailuresNe[0], nPreDPF_NE);

   /* Process Near-End bits */
   DSL_FAILUREBIT_SET(OOS_NE, nPreDPF_NE, DSL_G997_DATAPATHFAILURE_OOS, nActDPF_NE, nActDPFM_NE);
   DSL_FAILUREBIT_SET(NCD_NE, nPreDPF_NE, DSL_G997_DATAPATHFAILURE_NCD, nActDPF_NE, nActDPFM_NE);
   DSL_FAILUREBIT_SET(LCD_NE, nPreDPF_NE, DSL_G997_DATAPATHFAILURE_LCD, nActDPF_NE, nActDPFM_NE);
   /* Process Far-End bits */
   DSL_FAILUREBIT_SET(NCD_FE, nPreDPF_FE, DSL_G997_DATAPATHFAILURE_NCD, nActDPF_FE, nActDPFM_FE);
   DSL_FAILUREBIT_SET(LCD_FE, nPreDPF_FE, DSL_G997_DATAPATHFAILURE_LCD, nActDPF_FE, nActDPFM_FE);


   /* Write actual settings for Data Path Failures in DSL context. Take care
      to set only failures bits that should be handled by this function
      (relevant bit mask).  */
   nActDPF_FE &= nRelevantBitMask;
   nActDPF_NE &= nRelevantBitMask;
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nDataPathFailuresFe[0], nActDPF_FE);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nDataPathFailuresNe[0], nActDPF_NE);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: No Cell Delineation failure NE (NCD) = %d" DSL_DRV_CRLF ,
      DSL_DEV_NUM(pContext),
      (nActDPF_NE & DSL_G997_DATAPATHFAILURE_NCD) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss of Cell Delineation failure NE (LCD) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      (nActDPF_NE & DSL_G997_DATAPATHFAILURE_LCD) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Out of Sync failure NE (OOS) = %d" DSL_DRV_CRLF ,
      DSL_DEV_NUM(pContext),
      (nActDPF_NE & DSL_G997_DATAPATHFAILURE_OOS) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: No Cell Delineation failure FE (NCD) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      (nActDPF_FE & DSL_G997_DATAPATHFAILURE_NCD) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss of Cell Delineation failure FE (LCD) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      (nActDPF_FE & DSL_G997_DATAPATHFAILURE_LCD) ? 1 : 0 ));

   /* Get (user) mask configuration for line failure event handling */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nDataPathFailuresNeAlarmMask, nCfgDPFM_NE);
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nDataPathFailuresFeAlarmMask, nCfgDPFM_FE);
   /* Get line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

   nActDPFM_NE &= nCfgDPFM_NE;
   nActDPFM_FE &= nCfgDPFM_FE;

   if (nActDPFM_NE != 0)
   {
      nErrCode = DSL_DRV_EventGenerate(
                     pContext, 0, DSL_ACCESSDIR_NA, DSL_NEAR_END,
                     DSL_EVENT_I_DATA_PATH_FAILURES,
                     (DSL_EventData_Union_t*)&nActDPF_NE,
                     sizeof(DSL_G997_DataPathFailuresData_t));
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_EVENT_I_DATA_PATH_FAILURES));
         return nErrCode;
      }
   }

   if (nActDPFM_FE != 0)
   {
      nErrCode = DSL_DRV_EventGenerate(
                     pContext, 0, DSL_ACCESSDIR_NA, DSL_FAR_END,
                     DSL_EVENT_I_DATA_PATH_FAILURES,
                     (DSL_EventData_Union_t*)&nActDPF_FE,
                     sizeof(DSL_G997_DataPathFailuresData_t));
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_EVENT_I_DATA_PATH_FAILURES));
            
         return nErrCode;
      }
   }

   return nErrCode;
}

/*
   This function sets the NE line failures to the DSL CPE context.

   \param pContext - Pointer to dsl library context structure, [I/O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
DSL_Error_t DSL_DRV_VNX_SendMsgLineFailuresNeStatusGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t nCfgLFM = 0;
   DSL_uint32_t nActLF = 0;
   DSL_uint32_t nActLFM = 0;
   DSL_uint32_t nPreLF = 0;
   DSL_uint32_t nRelevantBitMask = 0;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;

   CMD_LineFailureNE_Get_t sCmd;
   ACK_LineFailureNE_Get_t sAck;

   sCmd.Index  = 0;
   sCmd.Length = 1;

   /* Get Line failures status information from the FW*/
   nErrCode =  DSL_DRV_VNX_SendMessage(
                   pContext, CMD_LINEFAILURENE_GET,
                   sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                   sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode < 0 )
   {
      return nErrCode;
   }

   /* Get previous state of NE line failures */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineFailuresNe, nPreLF);

   nRelevantBitMask = DSL_G997_LINEFAILURE_LPR |
                      DSL_G997_LINEFAILURE_LOF |
                      DSL_G997_LINEFAILURE_LOS |
                      DSL_G997_LINEFAILURE_LOM |
                      DSL_G997_LINEFAILURE_LOL |
                      DSL_G997_LINEFAILURE_ESE;

   DSL_FAILUREBIT_SET(LOS_NE, nPreLF, DSL_G997_LINEFAILURE_LOS, nActLF, nActLFM);
   DSL_FAILUREBIT_SET(LOF_NE, nPreLF, DSL_G997_LINEFAILURE_LOF, nActLF, nActLFM);
   DSL_FAILUREBIT_SET(LOM_NE, nPreLF, DSL_G997_LINEFAILURE_LOM, nActLF, nActLFM);
   DSL_FAILUREBIT_SET(ESE_NE, nPreLF, DSL_G997_LINEFAILURE_ESE, nActLF, nActLFM);
   DSL_FAILUREBIT_SET(LOL_NE, nPreLF, DSL_G997_LINEFAILURE_LOL, nActLF, nActLFM);

   /* Write new settings for line failures back to the DSL CPE context.
      Take care to set only failures bits that should be handled
      by this function (relevant bit mask). */
   nActLF &= nRelevantBitMask;

   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nLineFailuresNe, nActLF);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss-of-signal failure NE (LOS) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      (nActLF & DSL_G997_LINEFAILURE_LOS) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss-of-frame failure NE (LOF) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      (nActLF & DSL_G997_LINEFAILURE_LOF) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss of margin failure NE (LOM) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      (nActLF & DSL_G997_LINEFAILURE_LOM) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss of link failure NE (LOL) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      (nActLF & DSL_G997_LINEFAILURE_LOL) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Excessive Severe Errors NE (ESE) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      (nActLF & DSL_G997_LINEFAILURE_ESE) ? 1 : 0 ));

   /* Get (user) mask configuration for line failure event handling */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineFailuresNeAlarmMask, nCfgLFM);
   /* Get line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

   /* Do not signal line failure events that are disbaled (masked) by the
      user. */
   nActLFM &= nCfgLFM;
   if (nActLFM != 0)
   {
      nErrCode = DSL_DRV_EventGenerate(
                     pContext, 0, DSL_ACCESSDIR_NA, DSL_NEAR_END,
                     DSL_EVENT_I_LINE_FAILURES,
                     (DSL_EventData_Union_t*)&nActLF,
                     sizeof(DSL_G997_LineFailuresData_t));
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_EVENT_I_LINE_FAILURES));
            
         return nErrCode;
      }
   }

   return nErrCode;
}

/*
   This function sets the FE line failures to the DSL CPE context.

   \param pContext - Pointer to dsl library context structure, [I/O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
DSL_Error_t DSL_DRV_VNX_SendMsgLineFailuresFeStatusGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t nCfgLFM = 0;
   DSL_uint32_t nActLF = 0;
   DSL_uint32_t nActLFM = 0;
   DSL_uint32_t nPreLF = 0;
   DSL_uint32_t nRelevantBitMask = 0;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;

   CMD_LineFailureFE_Get_t sCmd;
   ACK_LineFailureFE_Get_t sAck;

   sCmd.Index  = 0;
   sCmd.Length = 1;

   if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
      DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      DSL_DEBUG( DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - FE line failures retrieving not supported "
         "by the FW in ADSL mode!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Get Line failures status information from the FW*/
   nErrCode =  DSL_DRV_VNX_SendMessage(
                   pContext, CMD_LINEFAILUREFE_GET,
                   sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                   sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode < 0 )
   {
      return nErrCode;
   }

   /* Get previous state of NE line failures */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineFailuresFe, nPreLF);

   nRelevantBitMask = DSL_G997_LINEFAILURE_LPR |
                      DSL_G997_LINEFAILURE_LOF |
                      DSL_G997_LINEFAILURE_LOS |
                      DSL_G997_LINEFAILURE_LOM |
                      DSL_G997_LINEFAILURE_ESE;

   DSL_FAILUREBIT_SET(LOS_FE, nPreLF, DSL_G997_LINEFAILURE_LOS, nActLF, nActLFM);
   DSL_FAILUREBIT_SET(LOM_FE, nPreLF, DSL_G997_LINEFAILURE_LOM, nActLF, nActLFM);
   DSL_FAILUREBIT_SET(LPR_FE, nPreLF, DSL_G997_LINEFAILURE_LPR, nActLF, nActLFM);

   /* ESE, LOL and LOF are currently supported for ADSL only*/
   DSL_FAILUREBIT_SET(ESE_FE, nPreLF, DSL_G997_LINEFAILURE_ESE, nActLF, nActLFM);

   /* Write new settings for line failures back to the DSL CPE context.
      Take care to set only failures bits that should be handled
      by this function (relevant bit mask). */
   nActLF &= nRelevantBitMask;

   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nLineFailuresFe, nActLF);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss-of-signal failure FE (LOS) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), (nActLF & DSL_G997_LINEFAILURE_LOS) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss of margin failure FE (LOM) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), (nActLF & DSL_G997_LINEFAILURE_LOM) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Loss-of-power failure FE (LPR) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), (nActLF & DSL_G997_LINEFAILURE_LPR) ? 1 : 0 ));

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Excessive Severe Errors Failure FE (ESE) = %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), (nActLF & DSL_G997_LINEFAILURE_ESE) ? 1 : 0 ));

   /* Get (user) mask configuration for line failure event handling */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineFailuresFeAlarmMask, nCfgLFM);
   /* Get line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

    /* Do not signal line failure events that are disbaled (masked) by the
      user. */
   nActLFM &= nCfgLFM;
   if (nActLFM != 0)
   {
      nErrCode = DSL_DRV_EventGenerate(
                     pContext, 0, DSL_ACCESSDIR_NA, DSL_FAR_END,
                     DSL_EVENT_I_LINE_FAILURES,
                     (DSL_EventData_Union_t*)&nActLF,
                     sizeof(DSL_G997_LineFailuresData_t));
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_EVENT_I_LINE_FAILURES));
            
         return nErrCode;
      }
   }

   return nErrCode;
}

static DSL_Error_t DSL_DRV_VNX_FailReasonGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_G997_LineInit_t lineInitStatus = LINIT_UNKNOWN;
   DSL_G997_LineInitSubStatus_t lineInitSubStatus = LINIT_SUB_NONE;
   CMD_ModemFSM_FailReasonGet_t sCmd;
   ACK_ModemFSM_FailReasonGet_t sAck;

   memset(&sCmd, 0, sizeof(sCmd));
   sCmd.Length = 10;

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_MODEMFSM_FAILREASONGET,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(sAck), (DSL_uint8_t*)&sAck );

   if( nErrCode == DSL_SUCCESS )
   {
      switch(sAck.ErrorCode)
      {
      case ALM_ModemFSM_FailReasonGet_E_CONFIG:
         lineInitStatus = LINIT_CONFIG_ERROR;
      break;
      case ALM_ModemFSM_FailReasonGet_E_NOTFEASIBLE:
         lineInitStatus = LINIT_CONFIG_NOT_FEASIBLE;
      break;
      case ALM_ModemFSM_FailReasonGet_E_COMM:
      case ALM_ModemFSM_FailReasonGet_E_COMMCRC:
         lineInitStatus = LINIT_COMMUNICATION_PROBLEM;
      break;
      case ALM_ModemFSM_FailReasonGet_E_NOPEER:
          lineInitStatus = LINIT_NO_PEER_XTU;
       break;
      case ALM_ModemFSM_FailReasonGet_E_UNKNOWN:
      case ALM_ModemFSM_FailReasonGet_E_CHIPSET:
      case ALM_ModemFSM_FailReasonGet_E_RESERVED:
         lineInitStatus = LINIT_UNKNOWN;
      break;
      default:
         lineInitStatus = LINIT_UNKNOWN;
      break;
      }
   }

   switch( sAck.SubErrorCode )
   {
   case ALM_ModemFSM_FailReasonGet_S_MODE:
      lineInitSubStatus = LINIT_SUB_NO_COMMON_MODE;
   break;
   default:
      lineInitSubStatus = LINIT_SUB_NONE;
   break;
   }

   DSL_DRV_HandleLinitValue(pContext, lineInitStatus, lineInitSubStatus);

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_LineFailuresStatusUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t errCode = DSL_SUCCESS;

   /* Update Line failures information for Near-End*/
   errCode = DSL_DRV_VNX_SendMsgLineFailuresNeStatusGet(pContext);
   if(errCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Line Failures NE update failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
     return errCode;
   }

   /* Update Line failures information for Far-End*/
   errCode = DSL_DRV_VNX_SendMsgLineFailuresFeStatusGet(pContext);
   if(errCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Line Failures FE update failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return errCode;
   }

   return errCode;
}

static DSL_Error_t DSL_DRV_VNX_OnLineInventoryFe(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* Retreive all FE inventory information*/
   nErrCode = DSL_DRV_VNX_FeInventoryGet(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - FE inventory get failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   nErrCode = DSL_DRV_EventGenerate(
                 pContext, 0, DSL_ACCESSDIR_NA, DSL_FAR_END,
                 DSL_EVENT_S_FE_INVENTORY_AVAILABLE,
                 DSL_NULL,
                 0);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), DSL_EVENT_S_FE_INVENTORY_AVAILABLE));
         
      return nErrCode;
   }

   return nErrCode;
}

static DSL_Error_t DSL_DRV_VNX_OnEapsTimeout(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: EAPS timeout occured" DSL_DRV_CRLF,
       DSL_DEV_NUM(pContext)));

   /* Clear EAPS timeout ID in the DSL CPE Context*/
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nEapsTimeoutId, 0);

   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1))
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nPrevMode, FWMODE_VDSL);
   }
   else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
            DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nPrevMode, FWMODE_ADSL);
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown FW mode!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nPrevMode, FWMODE_NA);
   }

#ifdef INCLUDE_DSL_PM
   /* Handle PM device specific stuff*/
   nErrCode = DSL_DRV_PM_DEV_OnEapsTimeout(pContext);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM EAPS timeout handling failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   
      return nErrCode;
   }
#endif /* INCLUDE_DSL_PM*/

   /* Handle all other necessary stuff which depends on the EAPS timeout...*/
   
   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_SendMsgTr1Control(
   DSL_Context_t *pContext,
   DSL_boolean_t bTR1Enable,
   DSL_boolean_t bTR1EventEnable,
   DSL_uint16_t nTR1Period,
   DSL_uint16_t nTR1Control,
   DSL_uint16_t nTR1Adjust )
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_TR1ControlSet_t sCmd;
   ACK_TR1ControlSet_t sAck;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   memset(&sCmd, 0, sizeof(sCmd));

   /* fill up the message to be sent */
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);

   sCmd.period         = nTR1Period;          /* unlimited */
   sCmd.TR1enable      = bTR1Enable ? VNX_ENABLE : VNX_DISABLE;
   sCmd.TR1_EVT_Enable = bTR1EventEnable ? VNX_ENABLE : VNX_DISABLE;
   sCmd.TR1Adjust      = nTR1Adjust;
   sCmd.CntInhibitEnable = VNX_ENABLE;

   switch(nTR1Control)
   {
   case CMD_TR1ControlSet_NOT_USED:
   case CMD_TR1ControlSet_RESET_NOW:
   case CMD_TR1ControlSet_RESET_EVER:
   case CMD_TR1ControlSet_RESET_TR1:
   case CMD_TR1ControlSet_SHORTEN:
   case CMD_TR1ControlSet_COPY:
      break;
   default:
      return DSL_ERROR;
   }

   sCmd.TR1Control     = nTR1Control;

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_TR1CONTROLSET,
                  sizeof(sCmd), (DSL_uint8_t *)&sCmd,
                  sizeof(sAck), (DSL_uint8_t *)&sAck);

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_SendMsgTestParamsAuxGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_uint8_t *pAck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   /* assume US/DS to have the same Msg Structures */
   CMD_TestParamsAuxDS_Get_t  sCmd;
   ACK_TestParamsAuxDS_Get_t  sAck;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgTestParamsAuxGet(nDirection=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection));

   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);
   nErrCode = DSL_DRV_VNX_SendMessage(
      pContext,
      nDirection == DSL_DOWNSTREAM ? CMD_TESTPARAMSAUXDS_GET : CMD_TESTPARAMSAUXUS_GET,
      sizeof(sCmd), (DSL_uint8_t*)&sCmd,
      sizeof(sAck), (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_ERROR )
   {
      memcpy(pAck, &sAck, sizeof(ACK_TestParamsAuxDS_Get_t));
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgTestParamsAuxGet, retCode"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_PollDrvForModemResetState(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int32_t nWaitCount = 0;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;
   DSL_boolean_t bIdleReached = DSL_FALSE;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_PollDrvForModemResetState" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* wait for Modem Reset state max 4 sec */
   for (nWaitCount = 0; nWaitCount < 40; nWaitCount++)
   {
      /* Get line state*/
      nErrCode = DSL_DRV_DEV_LineStateGet( pContext, &nLineState);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - line state get failed, retCode=%d!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nErrCode));
            
         break;
      }

      /* Check if the DSL_LINESTATE_IDLE state reached*/
      if( nLineState == DSL_LINESTATE_IDLE )
      {
         bIdleReached = DSL_TRUE;
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: MSG - modem RESET state reached" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }
      DSL_DRV_MSecSleep(100);
   }

   if( !bIdleReached )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - RESET state not reached!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_PollDrvForModemResetState, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_SAR_VNX_SegmentationCountersGet(
   DSL_Context_t *pContext,
   DSL_SAR_SegmentationCountersData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_AAL5_SegmentationStatsGet_t sCmd;
   ACK_AAL5_SegmentationStatsGet_t sAck;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_SAR_VNX_SegmentationCountersGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   memset (pData, 0, sizeof(DSL_SAR_SegmentationCountersData_t));

   memset (&sCmd, 0, sizeof(sCmd));
   sCmd.Length = 19;
   nErrCode = DSL_DRV_VNX_SendMessage(
                  pContext, CMD_AAL5_SEGMENTATIONSTATSGET,
                  sizeof(sCmd), (DSL_uint8_t *)&sCmd,
                  sizeof(sAck), (DSL_uint8_t *)&sAck );

   /* Copy data only if successful */
   if( nErrCode !=DSL_SUCCESS )
   {
      DSL_DEBUG (DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - SegmentationStatsGet failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      /* Copy received data to structure */
      pData->nDefaultAtm      = sAck.segATM_DefVCC;
      pData->nDefaultFramesRx = sAck.segMAC_DefVCC;
      pData->nInvalidFramesRx = sAck.segMAC_NoVCCmatch;

      pData->nAtm[0] = sAck.segATM_VCC0;
      pData->nAtm[1] = sAck.segATM_VCC1;
      pData->nAtm[2] = sAck.segATM_VCC2;
      pData->nAtm[3] = sAck.segATM_VCC3;
      pData->nAtm[4] = sAck.segATM_VCC4;
      pData->nAtm[5] = sAck.segATM_VCC5;
      pData->nAtm[6] = sAck.segATM_VCC6;
      pData->nAtm[7] = sAck.segATM_VCC7;

      pData->nFramesRx[0] = sAck.segMAC_VCC0;
      pData->nFramesRx[1] = sAck.segMAC_VCC1;
      pData->nFramesRx[2] = sAck.segMAC_VCC2;
      pData->nFramesRx[3] = sAck.segMAC_VCC3;
      pData->nFramesRx[4] = sAck.segMAC_VCC4;
      pData->nFramesRx[5] = sAck.segMAC_VCC5;
      pData->nFramesRx[6] = sAck.segMAC_VCC6;
      pData->nFramesRx[7] = sAck.segMAC_VCC7;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_SAR_VNX_SegmentationCountersGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_SAR_VNX_ReassemblyCountersGet(
   DSL_Context_t *pContext,
   DSL_SAR_ReassemblyCountersData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_AAL5_ReassemblyStats1Get_t sCmd;
   ACK_AAL5_ReassemblyStats1Get_t sAck;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_SAR_VNX_ReassemblyCountersGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   memset (pData, 0, sizeof(DSL_SAR_ReassemblyCountersData_t));

   memset (&sCmd, 0, sizeof(sCmd));
   sCmd.Length = 18;
   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_AAL5_REASSEMBLYSTATS1GET,
                 sizeof(sCmd), (DSL_uint8_t *)&sCmd,
                 sizeof(sAck), (DSL_uint8_t *)&sAck );

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG (DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - ReassemblyStatsGet failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      pData->nInvalidAtm    = sAck.reaATM_NoVCCmatch;
      pData->nInvalidFrames = sAck.reaMAC_LLCinvalid;

      pData->nCrc[0] = sAck.reaCRC_VCC0;
      pData->nCrc[1] = sAck.reaCRC_VCC1;
      pData->nCrc[2] = sAck.reaCRC_VCC2;
      pData->nCrc[3] = sAck.reaCRC_VCC3;
      pData->nCrc[4] = sAck.reaCRC_VCC4;
      pData->nCrc[5] = sAck.reaCRC_VCC5;
      pData->nCrc[6] = sAck.reaCRC_VCC6;
      pData->nCrc[7] = sAck.reaCRC_VCC7;

      pData->nFrames[0] = sAck.reaMAC_VCC0;
      pData->nFrames[1] = sAck.reaMAC_VCC1;
      pData->nFrames[2] = sAck.reaMAC_VCC2;
      pData->nFrames[3] = sAck.reaMAC_VCC3;
      pData->nFrames[4] = sAck.reaMAC_VCC4;
      pData->nFrames[5] = sAck.reaMAC_VCC5;
      pData->nFrames[6] = sAck.reaMAC_VCC6;
      pData->nFrames[7] = sAck.reaMAC_VCC7;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_SAR_VNX_ReassemblyCountersGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}


/*
   This function returns the support state of a bandplan/profile
   combination.

   \param pContext Pointer to dsl cpe library context structure, [I]
   \param pData    Defines the bandplan from
                   \ref DSL_BandPlanConfigData_t, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_TRUE (1) if supported
   - DSL_FALSE (0) if not supported
*/
static DSL_boolean_t DSL_DRV_VNX_BandPlanSupportedCheck(
   DSL_Context_t *pContext,
   const DSL_BandPlanConfigData_t *pData)
{
   typedef struct
   {
      DSL_BandPlanType_t nIdx;
      DSL_uint8_t        bSupported[DSL_PROFILE_LAST];
   } DSL_BP_SupportTableEntry_t;
   
   static const DSL_BP_SupportTableEntry_t sBpSupport[] =
   {
      /* only new IDs are supported from version 3.2 onwards */
      /* Annex A     */                                                           /*  8a,8b,8c,8d,12a,12b,17a,30a,17b */
      /*                      50  */  {DSL_ITU_VDSL2_ANNEXA_M1_EU32                  , {1, 1, 1, 1, 1, 1, 1, 1, 1,}},        
      /*                      51  */  {DSL_ITU_VDSL2_ANNEXA_M2_EU36                  , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},        
      /*                      52  */  {DSL_ITU_VDSL2_ANNEXA_M3_EU40                  , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},        
      /*                      53  */  {DSL_ITU_VDSL2_ANNEXA_M4_EU44                  , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},        
      /*                      54  */  {DSL_ITU_VDSL2_ANNEXA_M5_EU48                  , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},        
      /*                      55  */  {DSL_ITU_VDSL2_ANNEXA_M6_EU52                  , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},        
      /*                      56  */  {DSL_ITU_VDSL2_ANNEXA_M7_EU56                  , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},        
      /*                      57  */  {DSL_ITU_VDSL2_ANNEXA_M8_EU60                  , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},        
      /*                      58  */  {DSL_ITU_VDSL2_ANNEXA_M9_EU64                  , {1, 1, 1, 1, 1, 1, 1, 0, 1,}},        
      /*                      59  */  {DSL_ITU_VDSL2_ANNEXA_EU128                    , {1, 1, 1, 1, 1, 1, 1, 1, 1,}},           
      /*                      70  */  {DSL_ITU_VDSL2_ANNEXA_M1_ADLU32                , {1, 1, 1, 1, 1, 0, 0, 1, 1,}},         
      /*                      71  */  {DSL_ITU_VDSL2_ANNEXA_M2_ADLU36                , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},         
      /*                      72  */  {DSL_ITU_VDSL2_ANNEXA_M3_ADLU40                , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},         
      /*                      73  */  {DSL_ITU_VDSL2_ANNEXA_M4_ADLU44                , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},         
      /*                      74  */  {DSL_ITU_VDSL2_ANNEXA_M5_ADLU48                , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},         
      /*                      75  */  {DSL_ITU_VDSL2_ANNEXA_M6_ADLU52                , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},         
      /*                      76  */  {DSL_ITU_VDSL2_ANNEXA_M7_ADLU56                , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},         
      /*                      77  */  {DSL_ITU_VDSL2_ANNEXA_M8_ADLU60                , {0, 0, 0, 0, 0, 0, 0, 0, 0,}},         
      /*                      78  */  {DSL_ITU_VDSL2_ANNEXA_M9_ADLU64                , {1, 1, 1, 1, 1, 0, 0, 1, 1,}},         
      /*                      79  */  {DSL_ITU_VDSL2_ANNEXA_ADLU128                  , {1, 1, 1, 1, 1, 0, 0, 1, 1,}},        
      /* ANNEX B     */
      /*                      100 */  {DSL_ITU_VDSL2_ANNEXB_B7_1_997_M1C_A_7         , {0, 0, 1, 0, 0, 0, 0, 0, 0,}},        
      /*                      101 */  {DSL_ITU_VDSL2_ANNEXB_B7_2_997_M1X_M_8         , {1, 1, 1, 1, 0, 0, 0, 0, 0,}},        
      /*                      102 */  {DSL_ITU_VDSL2_ANNEXB_B7_3_997_M1X_M           , {0, 0, 0, 0, 1, 1, 0, 0, 0,}},       
      /*                      103 */  {DSL_ITU_VDSL2_ANNEXB_B7_4_997_M2X_M_8         , {1, 1, 1, 1, 0, 0, 0, 0, 0,}},        
      /*                      104 */  {DSL_ITU_VDSL2_ANNEXB_B7_5_997_M2X_A           , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},       
      /*                      105 */  {DSL_ITU_VDSL2_ANNEXB_B7_6_997_M2X_M           , {0, 0, 0, 0, 1, 1, 0, 0, 0,}},       
      /*                      106 */  {DSL_ITU_VDSL2_ANNEXB_B7_7_HPE17_M1_NUS0       , {0, 0, 0, 0, 0, 0, 1, 0, 0,}},     
      /*                      107 */  {DSL_ITU_VDSL2_ANNEXB_B7_8_HPE30_M1_NUS0       , {0, 0, 0, 0, 0, 0, 0, 1, 0,}},     
      /*                      108 */  {DSL_ITU_VDSL2_ANNEXB_B7_9_997E17_M2X_NUS0     , {0, 0, 0, 0, 0, 0, 1, 0, 0,}},       
      /*                      109 */  {DSL_ITU_VDSL2_ANNEXB_B7_10_997E30_M2X_NUS0    , {0, 0, 0, 0, 0, 0, 0, 1, 0,}},     
      /*                      120 */  {DSL_ITU_VDSL2_ANNEXB_B8_1_998_M1X_A           , {1, 1, 1, 1, 1, 0, 0, 0, 0,}},       
      /*                      121 */  {DSL_ITU_VDSL2_ANNEXB_B8_2_998_M1X_B           , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},       
      /*                      122 */  {DSL_ITU_VDSL2_ANNEXB_B8_3_998_M1X_NUS0        , {1, 1, 1, 1, 0, 1, 0, 0, 0,}},         
      /*                      123 */  {DSL_ITU_VDSL2_ANNEXB_B8_4_998_M2X_A           , {1, 1, 1, 1, 1, 0, 0, 0, 0,}},       
      /*                      124 */  {DSL_ITU_VDSL2_ANNEXB_B8_5_998_M2X_M           , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},       
      /*                      125 */  {DSL_ITU_VDSL2_ANNEXB_B8_6_998_M2X_B           , {1, 1, 1, 1, 1, 0, 0, 0, 0,}},       
      /*                      126 */  {DSL_ITU_VDSL2_ANNEXB_B8_7_998_M2X_NUS0        , {1, 1, 1, 1, 0, 1, 0, 0, 0,}},         
      /*                      127 */  {DSL_ITU_VDSL2_ANNEXB_B8_8_998E17_M2X_NUS0     , {0, 0, 0, 0, 0, 0, 1, 0, 0,}},         
      /*                      128 */  {DSL_ITU_VDSL2_ANNEXB_B8_9_998E17_M2X_NUS0_M   , {0, 0, 0, 0, 0, 0, 1, 0, 0,}},      
      /*                      129 */  {DSL_ITU_VDSL2_ANNEXB_B8_10_998ADE17_M2X_NUS0_M, {0, 0, 0, 0, 0, 0, 1, 0, 0,}},        
      /*                      130 */  {DSL_ITU_VDSL2_ANNEXB_B8_11_998ADE17_M2X_A     , {0, 0, 0, 0, 0, 0, 1, 0, 1,}},       
      /*                      131 */  {DSL_ITU_VDSL2_ANNEXB_B8_12_998ADE17_M2X_B     , {0, 0, 0, 0, 0, 0, 0, 0, 1,}},       
      /*                      132 */  {DSL_ITU_VDSL2_ANNEXB_B8_13_998E30_M2X_NUS0    , {0, 0, 0, 0, 0, 0, 0, 1, 0,}},     
      /*                      133 */  {DSL_ITU_VDSL2_ANNEXB_B8_14_998E30_M2X_NUS0_M  , {0, 0, 0, 0, 0, 0, 0, 1, 0,}},       
      /*                      134 */  {DSL_ITU_VDSL2_ANNEXB_B8_15_998ADE30_M2X_NUS0_M, {0, 0, 0, 0, 0, 0, 0, 1, 0,}},        
      /*                      135 */  {DSL_ITU_VDSL2_ANNEXB_B8_16_998ADE30_M2X_NUS0_A, {0, 0, 0, 0, 0, 0, 1, 1, 0,}},        
      /*                      140 */  {DSL_ITU_VDSL2_ANNEXB_998ADE17_M2X_M           , {0, 0, 0, 0, 0, 0, 0, 0, 1,}},       
      /*                      141 */  {DSL_ITU_VDSL2_ANNEXB_998E17_M2X_A             , {0, 0, 0, 0, 0, 0, 0, 0, 1,}},
      /* ANNEX C      */
      /*                      160 */  {DSL_ITU_VDSL2_ANNEXC_FTTCAB_A                 , {0, 0, 1, 0, 1, 1, 1, 0, 1,}},         
      /*                      161 */  {DSL_ITU_VDSL2_ANNEXC_FTTCAB_M                 , {0, 0, 1, 0, 1, 1, 1, 0, 1,}},         
      /*                      162 */  {DSL_ITU_VDSL2_ANNEXC_FTTEX_A                  , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},        
      /*                      163 */  {DSL_ITU_VDSL2_ANNEXC_FTTEX_M                  , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},        
      /*                      164 */  {DSL_ITU_VDSL2_ANNEXC_O_ADSL                   , {0, 0, 1, 0, 0, 1, 1, 1, 0,}},        
      /*                      165 */  {DSL_ITU_VDSL2_ANNEXC_O_TCMISDN                , {0, 0, 1, 0, 0, 1, 1, 1, 0,}},      
      /* VDSL1       */
      /*                      180 */  {DSL_ANSI_FTTCAB_M1                            , {0, 0, 1, 0, 1, 1, 0, 0, 0,}},        
      /*                      181 */  {DSL_ANSI_FTTCAB_M2                            , {0, 0, 1, 1, 1, 1, 0, 0, 0,}},        
      /*                      182 */  {DSL_ANSI_FTTEX_M1                             , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},           
      /*                      183 */  {DSL_ANSI_FTTEX_M2                             , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},           
      /*                      184 */  {DSL_ETSI_FTTCAB_PCAB_M1                       , {0, 0, 1, 0, 1, 1, 0, 0, 0,}},           
      /*                      185 */  {DSL_ETSI_FTTCAB_PCAB_M2                       , {0, 0, 1, 0, 1, 1, 0, 0, 0,}},           
      /*                      186 */  {DSL_ETSI_FTTEX_P1_M1_O_ISDN                   , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},        
      /*                      187 */  {DSL_ETSI_FTTEX_P1_M2_O_ISDN                   , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},        
      /*                      188 */  {DSL_ETSI_FTTEX_P2_M1_O_POTS                   , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},        
      /*                      189 */  {DSL_ETSI_FTTEX_P2_M2_O_POTS                   , {1, 1, 1, 1, 1, 1, 0, 0, 0,}},        
      /*                      190 */  {DSL_ITU_VDSL1_ANNEXE_E2_PCABA_M1              , {0, 0, 1, 0, 1, 1, 0, 0, 0,}},       
      /* Others      */
      /*                      200 */  {DSL_HANARO_FTTCAB                             , {0, 0, 1, 0, 1, 1, 1, 1, 1,}},           
      /*                      201 */  {DSL_ANFP_CAL0                                 , {0, 0, 1, 0, 0, 0, 0, 0, 0,}},        

   };
   DSL_uint32_t i = 0;

   if  (pData == DSL_NULL)
      return DSL_FALSE;

   if ((pData->nBandPlan >= DSL_BANDPLAN_LAST) || (pData->nProfile >= DSL_PROFILE_LAST))
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - DSL_VNX_BandPlanSupportedCheck: (nBandplan=%d, "
         "nProfile=%d) out of range!" DSL_DRV_CRLF , DSL_DEV_NUM(pContext),
         pData->nBandPlan, pData->nProfile));
         
      return DSL_FALSE;
   }

   /* to speed up the search, break when search index gets greater than the 
      bandplan number to search for, 
     because the entries are sorted and unique so
     position <= BandplanId */
   for (i = 0; 
      (i < DSL_ARRAY_LENGTH(sBpSupport)) && 
      (i <= (DSL_uint32_t)pData->nBandPlan); 
      i++)
   {
      if (sBpSupport[i].nIdx == pData->nBandPlan)
         return (DSL_boolean_t)sBpSupport[i].bSupported[pData->nProfile];
   }
   
   /* Bandplan number not found */
   return DSL_FALSE;
}

DSL_Error_t DSL_DRV_VNX_BandPlanSupportedGet(
   DSL_Context_t *pContext,
   DSL_BandPlanSupportData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_ProfileType_t nProfile   = (DSL_ProfileType_t)0;
   DSL_BandPlanType_t nBandPlan = (DSL_BandPlanType_t)0;
   DSL_BandPlanConfigData_t sBpConfig;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: Supported (Bandplan, Profile) combinations:" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      for( nBandPlan = (DSL_BandPlanType_t)0; nBandPlan < DSL_BANDPLAN_LAST; nBandPlan++ )
      {
         sBpConfig.nBandPlan = (DSL_BandPlanType_t)nBandPlan;
         for (nProfile = (DSL_ProfileType_t)0; nProfile < DSL_PROFILE_LAST; nProfile++)
         {
            sBpConfig.nProfile = (DSL_ProfileType_t)nProfile;
            pData->bSupported[nBandPlan][nProfile] =
               DSL_DRV_VNX_BandPlanSupportedCheck(pContext, &sBpConfig);
               DSL_DEBUG( DSL_DBG_MSG, (pContext,
               "%d ", pData->bSupported[nBandPlan][nProfile]) );
         }
         DSL_DEBUG( DSL_DBG_MSG, (pContext, DSL_DRV_CRLF ) );
      }
   }
   else
   {
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }
   
   return nErrCode;
}

DSL_Error_t DSL_DRV_VNX_BandPlanStatusGet(
   DSL_Context_t *pContext,
   DSL_BandPlanConfigData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_BandPlanStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Only Available for VDSL mode*/
   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - data mutex lock failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ERROR;
      }

      /* Get Band Plan Status from the internal VINAX device context*/
      memcpy(
         pData,
         &(pContext->pDevCtx->data.deviceCfg.bandPlanStatusNe),
         sizeof(DSL_BandPlanConfigData_t));

      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   }
   else
   {
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_BandPlanStatusGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

static DSL_Error_t DSL_DRV_VNX_CheckTruncParamRange(
   DSL_Context_t *pContext,
   DSL_int_t nMin, DSL_int_t nMax, DSL_int_t *pVal, DSL_char_t *pName)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t originalValue = 0;
   DSL_CHECK_POINTER(pContext, pVal);
   DSL_CHECK_POINTER(pContext, pName);
   DSL_CHECK_ERR_CODE();

   originalValue = *pVal;

   if( *pVal < nMin )
   {
      *pVal = nMin;

      DSL_DEBUG(DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Parameter (%s, %d) truncated "
         "to min value (%d)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
         pName,originalValue,nMin));
         
      nErrCode = DSL_WRN_CONFIG_PARAM_TRUNCATED;
   }
   else if( *pVal > nMax )
   {
      *pVal = nMax;

      DSL_DEBUG(DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Parameter (%s, %d) truncated"
         " to max value (%d)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
         pName,originalValue,nMax));
         
      nErrCode = DSL_WRN_CONFIG_PARAM_TRUNCATED;
   }

   return nErrCode;
}


/*CAM*/
/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'lib_dsl_vinax.h'
*/
DSL_Error_t DSL_DRV_VNX_CamFsmStateSet(
   DSL_Context_t *pContext,
   const DSL_DEV_CamStates_t nCamStateNew)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_CamStates_t nCamStateCurr;

   /* Get CAM state information*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nCamState, nCamStateCurr);

   if (nCamStateCurr == DSL_CAM_DISABLED)
   {
      DSL_DEBUG(DSL_DBG_MSG, (pContext,
         "DSL[%02d]: DSL_VNX_CamFsmStateSet CAM off" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_SUCCESS;
   }

   if (nCamStateNew > DSL_CAM_LAST)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - DSL_VNX_CamFsmStateSet illegal state %d!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nCamStateNew));
         
      return DSL_ERROR;
   }

   /* no change */
   if (nCamStateCurr == nCamStateNew)
      return nErrCode;

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: DSL_VNX_CamFsmStateSet %d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nCamStateNew));

   /* Set CAM state information*/
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nCamState, nCamStateNew);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nCamReinits, 0);

   return(nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'lib_dsl_vinax.h'
*/
static DSL_Error_t DSL_DRV_VNX_CamFsmEnable(
   DSL_Context_t *pContext,
   const DSL_boolean_t bEnable)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   if (bEnable)
   {
      /* if already enabled, this causes a reset of the state machine */
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nCamState, DSL_CAM_INIT);
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nCamReinits, 0);
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nPrevMode, FWMODE_NA);
   }
   else
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nCamState, DSL_CAM_DISABLED);
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: DSL_VNX_CamFsmEnable %s" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), (bEnable?"true":"false")));

   return nErrCode;
}

static DSL_DEV_CamStates_t DSL_DRV_VNX_CamFsmStateGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_CamStates_t nCamStateCurr;

   /* Get CAM state information*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nCamState, nCamStateCurr);

   return nCamStateCurr;
}

static DSL_Error_t DSL_DRV_VNX_CamFwLoadNeededCheck(
   DSL_Context_t *pContext,
   DSL_boolean_t *bDownloadRequired,
   DSL_VNX_FwMode_t *nFwMode)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_CamStates_t nCamStateCurr = DSL_DRV_VNX_CamFsmStateGet(pContext);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_CamFwLoadNeededCheck"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   *nFwMode = FWMODE_VDSL;
   *bDownloadRequired = DSL_FALSE;

   if (nCamStateCurr == DSL_CAM_DISABLED)
   {
      DSL_DEBUG(DSL_DBG_MSG, (pContext,
         "DSL[%02d]: DSL_VNX_CamFwLoad CAM disabled" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return(nErrCode);
   }

   switch (nCamStateCurr)
   {
   case DSL_CAM_INIT:
      {
         DSL_uint8_t i, nAtse = 0, nAtseCurr = 0;
         /* get the ADSL bits */
         for (i = 0; i < DSL_G997_NUM_XTSE_OCTETS - 1; i++)
         {
            DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[i], nAtseCurr);
            nAtse |= nAtseCurr;
         }

         /* prefer ADSL FW also in multi mode */
         if (nAtse != 0)
            *nFwMode = FWMODE_ADSL;
         else
            *nFwMode = FWMODE_VDSL;
      }
      break;
   case DSL_CAM_MULTI:
   case DSL_CAM_ADSL_RETRY:
      *nFwMode = FWMODE_ADSL;
      break;
   case DSL_CAM_VDSL_FORCED:
   case DSL_CAM_VDSL_RETRY:
      *nFwMode = FWMODE_VDSL;
      break;
   default:
      DSL_DEBUG(DSL_DBG_WRN, (pContext,
         "DSL[%02d]: DSL_VNX_CamFwLoad unhandled state %d" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nCamStateCurr));
         
      break;
   }

   /* now check if we need a new download */
   if (*nFwMode == FWMODE_ADSL)
   {
      if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      {
         *bDownloadRequired = DSL_TRUE;
      }
   }
   else
   {
      if (!DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      {
         *bDownloadRequired = DSL_TRUE;
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: DSL_VNX_CamFwLoadNeededCheck needed=%d, type=%s" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), 
      *bDownloadRequired, ((*nFwMode == FWMODE_VDSL)?"VDSL":"ADSL")));

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_CamFwLoadNeededCheck"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return(nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_CamExceptionHandle(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_CamStates_t nCurrentState = DSL_DRV_VNX_CamFsmStateGet(pContext);
   DSL_DEV_CamStates_t nNewState = nCurrentState;
   DSL_uint8_t nCamReinits = 0;
   DSL_G997_LineInitStatusData_t lineInitStatus;
   DSL_uint8_t nSpar1 = 0, nSpar2 = 0, nSpar3 = 0,
               nSpar4 = 0, nSpar5 = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_CamExceptionHandle"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( DSL_DRV_VNX_CamFsmStateGet(pContext) != DSL_CAM_DISABLED )
   {
      DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.nCamReinits, nCamReinits);
      
      /* dependent on the current state
         retries or state changes have to be done.
         Counter resets are done as part of a state change */
      switch (nCurrentState)
      {
      case DSL_CAM_ADSL_RETRY:
         if( nCamReinits >= DSL_CAM_MAX_REINITS )
         {
            /* Reset nPrevMode value*/
            DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nPrevMode, FWMODE_NA);
            nNewState = DSL_CAM_INIT;
            DSL_DEBUG( DSL_DBG_MSG, (pContext,
               "DSL[%02d]: DSL_CAM_ADSL_RETRY reached limit at %d retries" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), nCamReinits));
         }
         else
         {
            nCamReinits++;
         }
         break;
         
      case DSL_CAM_MULTI:
         /* Get Line Init info*/
         DSL_CTX_READ(pContext, nErrCode, lineInitStatus, lineInitStatus);
         /* Get SPAR1 info*/
         DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar1, nSpar1);
         DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar2, nSpar2);
         DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar3, nSpar3);
         DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar4, nSpar4);
         DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar5, nSpar5);

         /* decide on the fail reasons state changes */
         if( lineInitStatus.nLineInitSubStatus == LINIT_SUB_NO_COMMON_MODE )
         {
            DSL_boolean_t bVdsl = DSL_FALSE, bAdsl = DSL_FALSE;

            /* check the FE capabilities */
            if( nSpar5 & 0x20 )
            {
               bVdsl = DSL_TRUE;
            }

            /* check for any ADSL mode */
            if( (nSpar1 & 0x7F) || (nSpar2 & 0x03) ||
                (nSpar3 & 0x7F) || (nSpar4 & 0x1F) || (nSpar5 & 0x15) )
            {
               bAdsl = DSL_TRUE;
            }

            if (bAdsl && bVdsl)
            {
               nNewState = DSL_CAM_VDSL_RETRY;
            }
            else if (bVdsl)
            {
               nNewState = DSL_CAM_VDSL_FORCED;
            }
            else
            {
               nNewState = DSL_CAM_INIT;
            }
         }
         else
         {
            nNewState = DSL_CAM_INIT;
         }
         break;

      case DSL_CAM_VDSL_RETRY:
         if( nCamReinits >= DSL_CAM_MAX_REINITS )
         {
            nNewState = DSL_CAM_ADSL_RETRY;
            DSL_DEBUG( DSL_DBG_MSG, (pContext,
               "DSL[%02d]: DSL_CAM_VDSL_RETRY reached limit at %d retries" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), nCamReinits));
         }
         else
         {
            nCamReinits++;
         }
         break;

      case DSL_CAM_VDSL_FORCED:
         if( nCamReinits >= DSL_CAM_MAX_REINITS )
         {
            /* Reset nPrevMode value*/
            DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nPrevMode, FWMODE_NA);
            nNewState = DSL_CAM_INIT;
            DSL_DEBUG( DSL_DBG_MSG, (pContext,
               "DSL[%02d]: DSL_CAM_VDSL_FORCED reached limit at %d retries" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), nCamReinits));
         }
         else
         {
            nCamReinits++;
         }
         break;

      default:
         break;
      }

      DSL_CTX_WRITE(pContext, nErrCode, pDevCtx->data.nCamReinits, nCamReinits);

      nErrCode = DSL_DRV_VNX_CamFsmStateSet(pContext, nNewState);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_MSG, (pContext,
            "DSL[%02d]: ERROR - CAM FSM state set failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_CamExceptionHandle"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return(nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_CamFsmUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nVtse = 0, nAtse = 0, nAtseCurr = 0, i = 0;
   DSL_VNX_FwMode_t nFwType;
   DSL_boolean_t bDownload = DSL_FALSE;
   DSL_VNX_FwMode_t nPrevMode = FWMODE_NA;
   DSL_DEV_CamStates_t nCamStateNew = DSL_CAM_MULTI;
   DSL_uint8_t nCamReinits = 0;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG, 
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_CamFsmUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* In INIT state modify the CPE Auto mode FSM 
      according to the xTSE bits */
   if( DSL_DRV_VNX_CamFsmStateGet(pContext) == DSL_CAM_INIT )
   {
      DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.nPrevMode, nPrevMode);

      switch(nPrevMode)
      {
      case FWMODE_VDSL:
         /* Try again in VDSL mode*/
         nCamStateNew = DSL_CAM_VDSL_FORCED;
         /* First reinit starts here*/
         nCamReinits++;
         break;
         
      case FWMODE_ADSL:
         /* Try again in ADSL mode*/
         nCamStateNew = DSL_CAM_ADSL_RETRY;
         /* First reinit starts here*/
         nCamReinits++;
         break;
         
      case FWMODE_NA:
      default:
         /* get the ADSL bits */
         for (i = 0; i < DSL_G997_NUM_XTSE_OCTETS - 1; i++)
         {
            DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[i], nAtseCurr);
            nAtse |= nAtseCurr;
         }

         /* get the VDSL bits */
         DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[7], nVtse);

         if (nAtse && nVtse)
         {
            nCamStateNew = DSL_CAM_MULTI;
         }
         else if (nAtse)
         {
            nCamStateNew = DSL_CAM_ADSL_RETRY;
         }
         else if (nVtse)
         {
            nCamStateNew = DSL_CAM_VDSL_FORCED;
         }
         break;
      }

      /* Set New CAM state*/
      nErrCode = DSL_DRV_VNX_CamFsmStateSet(pContext, nCamStateNew);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - CAM FSM state set failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
         return DSL_ERROR;
      }

      DSL_CTX_WRITE(pContext, nErrCode, pDevCtx->data.nCamReinits, nCamReinits);      
   }

   /* check the loaded FW and load a different one if needed */
   nErrCode = DSL_DRV_VNX_CamFwLoadNeededCheck(pContext, &bDownload, &nFwType);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - CAM FW check failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* Check if it is neccessary to reload the FW binary*/
   if( bDownload )
   {
      if( nFwType == FWMODE_VDSL )
      {
         /* intermediate workaround: 
            for VDSL set EFM TC regardless to what was configured before.
            This has to be replaced by common profile usage */
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.sysCIF.nTcLayer, DSL_TC_EFM);
         /* Set VDSL FW image type*/
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.cfg.nFwType, DSL_FW_REQUEST_VDSL);
         DSL_CHECK_ERR_CODE();
      }
      else
      {
         /* intermediate workaround: 
            for VDSL set EFM TC regardless to what was configured before.
            This has to be replaced by common profile usage */
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.sysCIF.nTcLayer, DSL_TC_ATM);
         /* Set ADSL FW image type*/
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.cfg.nFwType, DSL_FW_REQUEST_ADSL); 
         DSL_CHECK_ERR_CODE();
      }

      /* Set bCamFwReload flag*/
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.bCamFwReload, DSL_TRUE);
   }

   DSL_DEBUG(DSL_DBG_MSG, 
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_CamFsmUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return(nErrCode);
}

/*
   This function requests the far end SPAR1 bits from the handshake.

   \param pContext  Pointer to dsl library context structure, [I]
   \param nLine     Specifies on which line the message was received, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_SendMsgSpar1Get(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_HS_StandardInfoFE_SPAR1Get_t sCmd;
   ACK_HS_StandardInfoFE_SPAR1Get_t sAck;
   DSL_uint16_t *pAck = (DSL_uint16_t*)&sAck;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_SendMsgSpar1Get"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* fill up the message to be sent */
   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));
   sCmd.Length = DSL_VNX_16BIT_RD_MSG_LEN_GET(sAck);

   /* KAv: This delay is needed to get the correct Spar1 values after the exception state
      was detected*/
   DSL_DRV_MSecSleep(1000);

   nErrCode = DSL_DRV_VNX_SendMessage(
                 pContext, CMD_HS_STANDARDINFOFE_SPAR1GET,
                 sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                 sizeof(sAck), (DSL_uint8_t*)&sAck );

   if (nErrCode != 0)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Retrieving SPAR1!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return(DSL_ERROR);
   }

   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar1, (*(pAck+2)) & 0xFF);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar2, (*(pAck+2)) >> 8);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar3, (*(pAck+3)) & 0xFF);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar4, (*(pAck+3)) >> 8);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.nSpar5, (*(pAck+4)) & 0xFF);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_SendMsgSpar1Get"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_FramingParametersVdsl2StatusGet(
   DSL_Context_t *pContext,
   const DSL_uint8_t nChannel,
   const DSL_AccessDir_t nDir,
   DSL_FramingParameterStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   union
   {
      ACK_FrameDataExt2US_Get_t US;
      ACK_FrameDataExt2DS_Get_t DS;
   } sAck;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_VNX_FramingParametersVdsl2StatusGet,"
      " (nDirection=%d, nChannel=%d)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
      nDir, nChannel));

   /* Get VDSL2 framing parameters*/
   nErrCode = DSL_DRV_VNX_SendMsgFrameDataExt2Get(pContext, nDir, (DSL_uint8_t*)&(sAck.DS));
   if( nErrCode < 0 )
      return nErrCode;

   pData->nTp    = nDir == DSL_DOWNSTREAM ? sAck.DS.Tp_LP0ds : sAck.US.Tp_LP0us;
   pData->nBP    = nDir == DSL_DOWNSTREAM ? sAck.DS.B0_LP0ds : sAck.US.B0_LP0us;
   pData->nMp    = nDir == DSL_DOWNSTREAM ? sAck.DS.Mp_LP0ds : sAck.US.Mp_LP0us;
   pData->nMSGC  = nDir == DSL_DOWNSTREAM ? sAck.DS.MSGc_LP0ds : sAck.US.MSGc_LP0us;
   pData->nMSGLP = 0x0; /* not supported*/
   pData->nSEQ   = nDir == DSL_DOWNSTREAM ? sAck.DS.SEQp_LP0ds : sAck.US.SEQp_LP0us;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_FramingParametersVdsl2StatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_FramingParametersAdslStatusGet(
   DSL_Context_t *pContext,
   const DSL_uint8_t nChannel,
   const DSL_AccessDir_t nDir,
   DSL_FramingParameterStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   ACK_ADSL_FrameDataDS_LP0Get_t sAck;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_FramingParametersAdslStatusGet,"
      " (nDirection=%d, nChannel=%d)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
      nDir, nChannel));

   /* Get ADSL framing parameters*/
   nErrCode = DSL_DRV_VNX_SendMsgAdslFrameDataGet(pContext, nDir, (DSL_uint8_t*)&sAck);
   if(nErrCode < 0)
      return nErrCode;

   pData->nTp    = sAck.Tp;
   pData->nBP    = sAck.Kp - 1;
   pData->nMp    = sAck.Mp;
   pData->nMSGC  = sAck.MSGc;
   pData->nMSGLP = 0x0; /* not supported*/
   pData->nSEQ   = sAck.SEQp;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_FramingParametersAdslStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);

}

/*
   This function set the actual power management mode in the data base and also
   handles to set the previous values if necessary.
   If necessary a callback event will be processed to inform the upper layer
   software about data power management state change.

   \param pContext   Pointer to dsl library context structure, [I]
   \param nLine      Specifies on which line the message was received, [I]
   \param pData      Includes the the new power management mode, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
DSL_Error_t DSL_DRV_VNX_PowerManagementStatusSet(
   DSL_Context_t *pContext,
   const DSL_G997_PowerManagement_t nPMMode)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_G997_PowerManagementStatusData_t nPrePmMode;

   DSL_CTX_READ(pContext, nErrCode, powerMgmtStatus, nPrePmMode);

   if( nPrePmMode.nPowerManagementStatus != nPMMode )
   {
      nPrePmMode.nPowerManagementStatus = nPMMode;
      DSL_CTX_WRITE(pContext, nErrCode, powerMgmtStatus, nPrePmMode);

      /* If event procedure is enabled inform the upper software layer
         about the new power management state changes */
      nErrCode = DSL_DRV_EventGenerate(
                    pContext, 0, DSL_ACCESSDIR_NA, DSL_XTUDIR_NA,
                    DSL_EVENT_S_LINE_POWERMANAGEMENT_STATE,
                    (DSL_EventData_Union_t*)&nPrePmMode,
                    sizeof(DSL_G997_PowerManagementStatusData_t));

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_EVENT_S_LINE_POWERMANAGEMENT_STATE));
      }
   }

   return (nErrCode);
}

DSL_uint32_t DSL_DRV_VNX_G997_FramingParameterStatusVdsl1Get(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN const DSL_uint8_t nChannel,
   DSL_OUT DSL_G997_FramingParameterStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   union
   {
      ACK_FrameDataExt1US_Get_t US;
      ACK_FrameDataExt1DS_Get_t DS;
   } sAck;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_FramingParameterStatusVdsl1Get,"
      " (nDirection=%d, nChannel=%d)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection, nChannel));

   /* Get VDSL1 framing status parameters*/
   nErrCode = DSL_DRV_VNX_SendMsgFrameDataExt1Get(pContext, nDirection, (DSL_uint8_t*)&(sAck.DS));

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Framing Parameters get failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      pData->nINTLVBLOCK = nDirection == DSL_DOWNSTREAM ? sAck.DS.I_LP0ds :
                                                          sAck.US.I_LP0us;

      pData->nINTLVDEPTH = nDirection == DSL_DOWNSTREAM ? sAck.DS.D_LP0ds :
                                                          sAck.US.D_LP0us;

      pData->nLPATH      = 0;

      /* TODO: has to be clarified for the VDSL1 mode*/
      pData->nLSYMB      = 0;

      pData->nNFEC       = nDirection == DSL_DOWNSTREAM ? sAck.DS.N_LP0ds :
                                                          sAck.US.N_LP0us;

      pData->nRFEC       = nDirection == DSL_DOWNSTREAM ? sAck.DS.R_LP0ds :
                                                          sAck.US.R_LP0us;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_FramingParameterStatusVdsl1Get, retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_uint32_t DSL_DRV_VNX_G997_FramingParameterStatusVdsl2Get(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN const DSL_uint8_t nChannel,
   DSL_OUT DSL_G997_FramingParameterStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   union
   {
      ACK_FrameDataExt2US_Get_t US;
      ACK_FrameDataExt2DS_Get_t DS;
   } sAck;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_FramingParameterStatusVdsl2Get,"
      " (nDirection=%d, nChannel=%d)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
      nDirection, nChannel));

   /* Get VDSL2 framing status parameters*/
   nErrCode = DSL_DRV_VNX_SendMsgFrameDataExt2Get(pContext, nDirection, (DSL_uint8_t*)&(sAck.DS));

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Framing Parameters get failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      pData->nINTLVBLOCK = nDirection == DSL_DOWNSTREAM ? sAck.DS.Ip_LP0ds :
                                                          sAck.US.Ip_LP0us;

      pData->nINTLVDEPTH = nDirection == DSL_DOWNSTREAM ? sAck.DS.Dp_LP0ds :
                                                          sAck.US.Dp_LP0us;

      pData->nLPATH      = 0;

      pData->nLSYMB      = nDirection == DSL_DOWNSTREAM ? sAck.DS.Lp_LP0ds :
                                                          sAck.US.Lp_LP0us;

      pData->nNFEC       = nDirection == DSL_DOWNSTREAM ? sAck.DS.N_fecp_LP0ds :
                                                          sAck.US.N_fecp_LP0us;

      pData->nRFEC       = nDirection == DSL_DOWNSTREAM ? sAck.DS.Rp_LP0ds :
                                                          sAck.US.Rp_LP0us;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_FramingParameterStatusVdsl2Get, retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_uint32_t DSL_DRV_VNX_G997_FramingParameterStatusAdslGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN const DSL_AccessDir_t nDirection,
   DSL_IN const DSL_uint8_t nChannel,
   DSL_OUT DSL_G997_FramingParameterStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nNfec = 0;
   ACK_ADSL_FrameDataDS_LP0Get_t sAck;
   DSL_uint8_t XTSE[DSL_G997_NUM_XTSE_OCTETS] = {0};   

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_G997_FramingParameterStatusAdslGet,"
      " (nDirection=%d, nChannel=%d)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection, nChannel));

   /* Get ADSL framing status parameters*/
   nErrCode = DSL_DRV_VNX_SendMsgAdslFrameDataGet(pContext, nDirection, (DSL_uint8_t*)&sAck);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Framing Parameters get failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      /*Get current xTSE octets*/
      DSL_CTX_READ(pContext, nErrCode, xtseCurr, XTSE);

      /* Check for ADSL1*/
      /* Special conditions:
         - fraMode and Sp are only provided in ADSL1 mode, Sp = 0x8000 means 1/2 
         - Mp, Tp, SEQ, MSG are only provided in ADSL2/2+ */
      /* Nfec = M*K + R (ADSL2/2+) and
         Nfec = (S*L)/8 (ADSL1) */
      if ((XTSE[1-1] & (XTSE_1_03_A_1_NO | XTSE_1_05_B_1_NO)))
      {
         nNfec = (sAck.Sp == 0x8000) ? (1 * sAck.Lp)/(8 * 2) : (sAck.Sp * sAck.Lp)/8;
      }
      else
      {
         nNfec = sAck.Mp * sAck.Kp + sAck.Rp;
      }  

      pData->nINTLVBLOCK = nNfec;
      pData->nINTLVDEPTH = sAck.Dp;
      pData->nLPATH      = 0;
      pData->nLSYMB      = sAck.Lp;
      pData->nNFEC       = nNfec;
      pData->nRFEC       = sAck.Rp;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_G997_FramingParameterStatusAdslGet, retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   This function reads the H logarithmic parameters per subcarrier group
   measured during diagnostic state or training.

   \param pContext   Pointer to dsl library context structure, [I]
   \param nDirection Specifies the direction (upstream/downstream) to which
                     the function will apply. [I]
   \param nDataNum   Number of elements to request, [I]
   \param pData      Returns the requested HLOG data, [O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
   - or any other defined specific error code
*/
DSL_Error_t DSL_DRV_VNX_DeltHlogGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_G997_NSCData16_t *pData)
{
   DSL_uint16_t nTonesMax = 128, nDataNum = 512;
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t i = 0;
   DSL_uint16_t nTonesLeft, nTonesCurrent, nTonesDone;
   /* assume US/DS to have the same Msg Structures */
   CMD_HlogDS_Get_t  sCmd;
   ACK_HlogDS_Get_t  sAck;
   static const DSL_uint8_t nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.HLOGpsds);
   DSL_BandList_t  sBandList;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_DeltHlogNeGet(nDirection=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection));

   /* init the return structure */
   memset(pData->nNSCData, 0, nDataNum * sizeof(pData->nNSCData[0]));
   pData->nNumData = 0;

   /* Check current xDSL mode*/
   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      nDataNum = 512;
   }
   else
   {
      memset(&sBandList, 0x0, sizeof(DSL_BandList_t));
      
      /* get number uf currently used bands */
      nErrCode = DSL_DRV_VNX_SendMsgBandControlGet(pContext, nDirection, &sBandList);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: ERROR -  getting current band config failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
         return nErrCode;
      }

      nDataNum = sBandList.nBand[0].nLastToneIndex + 1;
   }

   nTonesLeft = nDataNum;

   nTonesDone = 0;
   while( nTonesLeft )
   {
      nTonesCurrent = nTonesLeft > nTonesMax ? nTonesMax : nTonesLeft;

      sCmd.Index  = nTonesDone;
      sCmd.Length = nTonesCurrent;
      nErrCode = DSL_DRV_VNX_SendMessage(
         pContext, nDirection == DSL_DOWNSTREAM ? CMD_HLOGDS_GET : CMD_HLOGUS_GET,
         sizeof(sCmd), (DSL_uint8_t*)&sCmd,
         sizeof(sAck), (DSL_uint8_t*)&sAck);

      /* Copy data only if successful and valid */
      if( (nErrCode >= 0) && (sAck.Index <= nDataNum - sAck.Length) )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: HLOG data received (idx %d - %d):" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), sAck.Index, sAck.Index + sAck.Length - 1));

         for( i = 0;
             (i < sAck.Length) && (i < nTonesMax) && (i < nMaxNumOfEntries);
              i++ )
         {
            /* Copy received data to structure */
            pData->nNSCData[i + sAck.Index] = sAck.HLOGpsds[i];
            DSL_DEBUG(DSL_DBG_MSG,
               (pContext, "%04X, ", pData->nNSCData[i + sAck.Index]));
         }
      }
      else
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, DSL_DRV_CRLF "DSL[%02d]: ERROR - Failed to retrieve HLOG %d to %d!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), sCmd.Index, (sCmd.Index + sCmd.Length)));

         nErrCode = DSL_ERROR;
         break;
      }

      nTonesLeft -= sAck.Length;
      nTonesDone += sAck.Length;
   }

   if( nErrCode == DSL_SUCCESS )
   {
      pData->nNumData = nTonesDone;

      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, ", nNumData=%d" DSL_DRV_CRLF, pData->nNumData));
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_DeltHlogNeGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

DSL_Error_t DSL_DRV_VNX_DeltHlinGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_G997_NSCComplexData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t i;
   DSL_uint16_t nTonesLeft, nTonesCurrent, nTonesDone, nDataNum = 512;
   CMD_HlinDS_Get_t  sCmd;
   ACK_HlinDS_Get_t  sAck;
   static const DSL_uint16_t nTonesMax = 64;
   static const DSL_uint8_t nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.HLINpsds);
   DSL_BandList_t  sBandList;

   DSL_DEBUG( DSL_DBG_MSG, (pContext, 
      "DSL[%02d]: IN - DSL_DRV_VNX_DeltHlinGet(nDirection=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection));

   /* init the return structure */
   memset(pData->nNSCComplexData, 0, nDataNum * sizeof(pData->nNSCComplexData[0]));
   pData->nNumData = 0;

   /* Check current xDSL mode*/
   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      nDataNum = 512;
   }
   else
   {
      memset(&sBandList, 0x0, sizeof(DSL_BandList_t));
      
      /* get number uf currently used bands */
      nErrCode = DSL_DRV_VNX_SendMsgBandControlGet(pContext, nDirection, &sBandList);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: ERROR -  getting current band config failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
         return nErrCode;
      }

      nDataNum = sBandList.nBand[0].nLastToneIndex + 1;
   }

   /*  several messages for max 64*2 values
   have to be sent */
   nTonesLeft = nDataNum;
   nTonesDone = 0;
   while (nTonesLeft)
   {
      nTonesCurrent = nTonesLeft > nTonesMax ? nTonesMax : nTonesLeft;

      memset(&sAck, 0, sizeof(sAck));
      sCmd.Index  = nTonesDone * 2;
      sCmd.Length = nTonesCurrent * 2;

      nErrCode = DSL_DRV_VNX_SendMessage(
                    pContext, 
                    nDirection == DSL_DOWNSTREAM ? CMD_HLINDS_GET : CMD_HLINUS_GET,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck);

      /* Copy data only if successful and valid */
      if ((nErrCode == DSL_SUCCESS) && (sAck.Index <= (nDataNum * 2) - sAck.Length))
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: Hlin data received (idx %d - %d):" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), sAck.Index / 2, (sAck.Index + sAck.Length) / 2 - 1));

         for (i = 0;
               (i < sAck.Length / 2) && (i < nTonesMax) && (i < nMaxNumOfEntries);
               i++)
         {
            /* Copy received data to structure */
            pData->nNSCComplexData[(i + sAck.Index / 2)].nImag = 
               sAck.HLINpsds[i].hlin_imag;
            pData->nNSCComplexData[(i + sAck.Index / 2)].nReal = 
               sAck.HLINpsds[i].hlin_real;
            DSL_DEBUG(DSL_DBG_MSG,(pContext, "(%d, %d) ", 
               pData->nNSCComplexData[(i + sAck.Index / 2)].nReal,
               pData->nNSCComplexData[(i + sAck.Index / 2)].nImag));
         }
      }
      else
      {
         DSL_DEBUG (DSL_DBG_WRN, (pContext, DSL_DRV_CRLF 
            "DSL[%02d]: Failed to retrieve Hlin %d to %d" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), sCmd.Index*2, (sCmd.Index + sCmd.Length)*2));

         nErrCode = DSL_ERROR;
         break;
      }

      nTonesLeft -= (sAck.Length / 2);
      nTonesDone += (sAck.Length / 2);
   }

   if( nErrCode == DSL_SUCCESS )
   {
      pData->nNumData = nTonesDone;

      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, ", nNumData=%d" DSL_DRV_CRLF, pData->nNumData));
   }
   
   DSL_DEBUG( DSL_DBG_MSG, (pContext, 
      "DSL[%02d]: OUT - DSL_DRV_VNX_DeltHlinGet(nDirection=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection));

   return (nErrCode);
}

/**
   This function calculates the SNR per subcarrier group from the 
   SNR per subcarrier values for a given direction. For each subcarrier group 
   an average is built.

   \param pContext   Pointer to dsl library context structure, [I]
   \param nLine      Specifies for which line the function will apply, [I]
   \param nDirection Specifies the direction (upstream/downstream) to which
                     the function will apply. [I]
   \param pData      Returns the SNR data , [O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
   - or any other defined specific error code
*/
DSL_Error_t DSL_DRV_VNX_SnrPerGroupCalculate(
   DSL_Context_t *pContext,
   DSL_uint8_t nGroupSize,
   DSL_G997_NSCData8_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t *pDest = DSL_NULL;
   DSL_uint32_t nAvrg = 0, i = 0;

   pDest = &(pData->nNSCData[0]);
   while(i < pData->nNumData)
   {
      if (pData->nNSCData[i] == 255)
      {
         /* as soon as one value is not valid,
            the whole group shall be invalid */
         nAvrg += (nGroupSize * 255);
      }
      else
      {
         nAvrg += pData->nNSCData[i];
      }

      if ((i % nGroupSize) == (DSL_uint8_t)(nGroupSize - 1))
      {
         /* one group is summarized */
         *pDest = DSL_MIN(255, (nAvrg / nGroupSize));
         nAvrg = 0;
         pDest++;
      }
      i++;
   }

   pData->nNumData = pData->nNumData / nGroupSize;

   return(nErrCode);
}

/**
   This function requests a set of up to 60 entries of the DELT data.

   \param pContext   DSL CPE API context
   \param nStartIndex Specifies the first tone group index to retrieve. [I]
   \param nStopIndex Specifies the last tone group index to retrieve. [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_DeltArrayParamsQuery(
   DSL_Context_t *pContext,
   const DSL_uint16_t nStartIndex,
   const DSL_uint16_t nStopIndex)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   CMD_TestParamsFE_Request_t sCmd;
   ACK_TestParamsFE_Request_t sAck;

   DSL_CHECK_CTX_POINTER(pContext);

   memset(&sCmd, 0, sizeof(sCmd));
   memset(&sAck, 0, sizeof(sAck));

   if ((nStopIndex < nStartIndex) ||
      (nStopIndex - nStartIndex > 60) ||
      (nStopIndex > 511))
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: DSL_DRV_VNX_DeltArrayParamsQuery illegal params start %d stop %d"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nStartIndex, nStopIndex));
         
      return (DSL_ERROR);
   }

   sCmd.Length     = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);
   sCmd.Control    = CMD_TestParamsFE_Request_TRIGGER;
   sCmd.StartIndex = nStartIndex;
   sCmd.EndIndex   = nStopIndex;

   nErrCode = DSL_DRV_VNX_SendMessage( pContext, CMD_TESTPARAMSFE_REQUEST,
                  sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                  sizeof(sAck), (DSL_uint8_t*)&sAck );

   return (nErrCode);
}

/**
   This function handles the following firmware messages which includes the
   array DELT parameters for Hlin upstream/downstream:
   + RSP_Diag_C_MSG2_LD for upstream G.992.3/5
   + RSP_Diag_R_MSG2_LD..RSP_Diag_R_MSG5_LD for downstream G.992.3
   + RSP_Diag_R_MSG2_LD..RSP_Diag_R_MSG9_LD for downstream G.992.5

   \param pContext   DSL library context
   \param pMsg       Pointer to received firmware message, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_DeltFeUpdate(
   DSL_Context_t *pContext,
   const EVT_PMD_TestParamsGet_t *pMsg)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t i = 0;
   DSL_uint16_t nDataLen = 0;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_DeltFeUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( pContext->DELT_SHOWTIME == DSL_NULL )
   {
      DSL_DEBUG( DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - No memory to store FE DELT parameters!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Store DELT data*/
   if (DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nDataLen = pMsg->EndIndex - pMsg->StartIndex + 1;


   /* Copy retrieved data to internal buffer*/
   for( i = 0; i < nDataLen; i++ )
   {
      pContext->DELT_SHOWTIME->hlogDataUsVdsl.deltHlog.nSCGData[pMsg->StartIndex + i] = pMsg->TestPar[i].hlog;
      pContext->DELT_SHOWTIME->qlnDataUsVdsl.deltQln.nSCGData[pMsg->StartIndex + i]   = pMsg->TestPar[i].qln;
      pContext->DELT_SHOWTIME->snrDataUsVdsl.deltSnr.nSCGData[pMsg->StartIndex + i]   = pMsg->TestPar[i].qln;
   }

   pContext->DELT_SHOWTIME->hlogDataUsVdsl.deltHlog.nNumData += nDataLen;
   pContext->DELT_SHOWTIME->qlnDataUsVdsl.deltQln.nNumData   += nDataLen;
   pContext->DELT_SHOWTIME->snrDataUsVdsl.deltSnr.nNumData   += nDataLen;
   
   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   /* Request next DELT data */
   if( pMsg->EndIndex < (VNX_MAX_SCGROUPS-1) )
   {
      nErrCode = DSL_DRV_VNX_DeltArrayParamsQuery(
                    pContext,
                    (DSL_uint16_t)(pMsg->EndIndex+1), 
                    (DSL_uint16_t)(DSL_MIN((pMsg->EndIndex + 32), (VNX_MAX_SCGROUPS-1))) );
      if(nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, 
            (pContext, "DSL[%02d]: failed to QueryDeltArrayParams!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return (nErrCode);
      }
   }
   else
   {  
      pContext->DELT_SHOWTIME->hlogDataUsVdsl.nMeasurementTime = pMsg->hlogTime;
      pContext->DELT_SHOWTIME->qlnDataUsVdsl.nMeasurementTime  = pMsg->qlnTime;
      pContext->DELT_SHOWTIME->snrDataUsVdsl.nMeasurementTime  = pMsg->snrTime;

      /* all data retrieved */
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: All DELT FE date retrieved"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_DeltFeUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   This function calculates the SNR per subcarrier group from the
   SNR per subcarrier values for a given direction. For each subcarrier group
   an average is built.

   \param pContext   Pointer to dsl library context structure, [I]
   \param nDirection Specifies the direction (upstream/downstream) to which
                     the function will apply. [I]
   \param nDataNum   Number of elements to request, [I]
   \param pData      Returns the SNR data , [O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
   - or any other defined specific error code
*/
DSL_Error_t DSL_DRV_VNX_DeltSnrGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_G997_NSCData8_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t i = 0;
   DSL_uint16_t nTonesLeft, nTonesCurrent, nTonesDone, nDataNum = 512;
   /* assume US/DS to have the same Msg Structures */
   CMD_SNR_DS_Get_t  sCmd;
   ACK_SNR_DS_Get_t  sAck;
   static const DSL_uint8_t nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.SNRpsds);
   static const DSL_uint16_t nTonesMax = 256;
   DSL_BandList_t  sBandList;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_DeltSnrGet(nDirection=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection));

   /* init the return structure */
   memset(pData->nNSCData, 0, nDataNum * sizeof(pData->nNSCData[0]));
   pData->nNumData = 0;

   /* Check current xDSL mode*/
   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      nDataNum = 512;
   }
   else
   {
      memset(&sBandList, 0x0, sizeof(DSL_BandList_t));
      
      /* get number uf currently used bands */
      nErrCode = DSL_DRV_VNX_SendMsgBandControlGet(pContext, nDirection, &sBandList);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: ERROR -  getting current band config failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
         return nErrCode;
      }

      nDataNum = sBandList.nBand[0].nLastToneIndex + 1;
   }

   /*  several messages for max 128*2 values have to be sent */
   nTonesLeft = nDataNum;
   nTonesDone = 0;

   while (nTonesLeft)
   {
      nTonesCurrent = nTonesLeft > nTonesMax ? nTonesMax : nTonesLeft;

      sCmd.Index  = nTonesDone/2;
      sCmd.Length = nTonesCurrent/2;
      nErrCode = DSL_DRV_VNX_SendMessage(
                    pContext,
                    nDirection == DSL_DOWNSTREAM ? CMD_SNR_DS_GET : CMD_SNR_US_GET,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck);

      /* Copy data only if successful and valid */
      if ((nErrCode >= 0) && (sAck.Index <= (nDataNum/2) - sAck.Length))
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: SNR data received (idx %d - %d):" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), 2*sAck.Index, 2*(sAck.Index + sAck.Length - 1)));

         for (i = 0;
               (i < sAck.Length) && (i < nTonesMax/2) && (i < nMaxNumOfEntries);
               i++)
         {
            /* Copy received data to structure */
            pData->nNSCData[2*(i + sAck.Index)] = 
               sAck.SNRpsds[i].snr_00;
            pData->nNSCData[2*(i + sAck.Index) + 1] = 
               sAck.SNRpsds[i].snr_01;
            DSL_DEBUG(DSL_DBG_MSG,(pContext, "%04X, %04X, ", 
               pData->nNSCData[2*(i + sAck.Index)], 
               pData->nNSCData[2*(i + sAck.Index) + 1]));
         }
      }
      else
      {
         DSL_DEBUG (DSL_DBG_ERR, (pContext, DSL_DRV_CRLF "DSL[%02d]: Failed to "
            "retrieve SNR %d to %d" DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
            sCmd.Index*2, (sCmd.Index + sCmd.Length)*2));
  
         nErrCode = DSL_ERROR;
         break;
      }
   
      nTonesLeft -= (sAck.Length * 2);
      nTonesDone += (sAck.Length * 2);
   }

   if( nErrCode == DSL_SUCCESS )
   {
      pData->nNumData = nTonesDone;
      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, ", nNumData=%d" DSL_DRV_CRLF, pData->nNumData));
   }
   
   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_DeltSnrGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   This function reads the Quality of line parameters per subcarrier group
   measured during diagnostic state or training.

   \param pContext   Pointer to dsl library context structure, [I]
   \param nDirection Specifies the direction (upstream/downstream) to which
                     the function will apply. [I]
   \param nDataNum   Number of elements to request, [I]
   \param pData      Returns the QLN data, [O]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
   - or any other defined specific error code
*/
DSL_Error_t DSL_DRV_VNX_DeltQlnGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_G997_NSCData8_t * pData)
{
   static const DSL_uint16_t nTonesMax = 128;
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t i = 0, nDataNum = 512;
   DSL_uint16_t nTonesLeft, nTonesCurrent, nTonesDone;
   /* assume US/DS to have the same Msg Structures */
   CMD_QLN_DS_Get_t  sCmd;
   ACK_QLN_DS_Get_t  sAck;
   static const DSL_uint8_t nMaxNumOfEntries = DSL_ARRAY_LENGTH(sAck.QLNds);
   DSL_BandList_t  sBandList;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_DeltQlnNeGet(nDirection=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection));

   /* init the return structure */
   memset(pData->nNSCData, 0, nDataNum * sizeof(pData->nNSCData[0]));
   pData->nNumData = 0;

   /* Check current xDSL mode*/
   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      nDataNum = 512;
   }
   else
   {
      memset(&sBandList, 0x0, sizeof(DSL_BandList_t));
      
      /* get number uf currently used bands */
      nErrCode = DSL_DRV_VNX_SendMsgBandControlGet(pContext, nDirection, &sBandList);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: ERROR -  getting current band config failed!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
         return nErrCode;
      }

      nDataNum = sBandList.nBand[0].nLastToneIndex + 1;
   }

   /* Request maximum allowable data independently of the current ADSL or VDSL mode*/
   nTonesLeft = nDataNum;

   nTonesDone = 0;
   while (nTonesLeft)
   {
      nTonesCurrent = nTonesLeft > nTonesMax ? nTonesMax : nTonesLeft;

      sCmd.Index  = nTonesDone/2;
      sCmd.Length = nTonesCurrent/2;
      nErrCode = DSL_DRV_VNX_SendMessage(pContext,
         nDirection == DSL_DOWNSTREAM ? CMD_QLN_DS_GET : CMD_QLN_US_GET,
         sizeof(sCmd), (DSL_uint8_t*)&sCmd,
         sizeof(sAck), (DSL_uint8_t*)&sAck);

      /* Copy data only if successful and valid */
      if ((nErrCode >= 0) && (sAck.Index <= (nDataNum/2)-sAck.Length))
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: QLN data received (idx %d - %d):" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), 2*sAck.Index, 2*(sAck.Index + sAck.Length)));

         for (i = 0;
               (i < sAck.Length) && (i < nTonesMax/2) && (i < nMaxNumOfEntries);
               i++)
         {
            /* Copy received data to structure */
            pData->nNSCData[2 * (i + sAck.Index)] =
               (DSL_uint8_t)(sAck.QLNds[i].qln_00);
            pData->nNSCData[2 * (i + sAck.Index) + 1] =
               (DSL_uint8_t)(sAck.QLNds[i].qln_01);
            DSL_DEBUG(DSL_DBG_MSG,(pContext, "%02X, %02X, ",
               pData->nNSCData[2 * (i + sAck.Index)],
               pData->nNSCData[2 * (i + sAck.Index) + 1]));
         }
      }
      else
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, DSL_DRV_CRLF "DSL[%02d]: ERROR - Failed to retrieve QLN %d to %d!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), sCmd.Index*2, (sCmd.Index + sCmd.Length)*2));
         nErrCode = DSL_ERROR;
         break;
      }

      nTonesLeft -= (sAck.Length * 2);
      nTonesDone += (sAck.Length * 2);
   }
   
   if( nErrCode == DSL_SUCCESS )
   {
      pData->nNumData = nTonesDone;

      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, ", nNumData=%d" DSL_DRV_CRLF, pData->nNumData));
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_DeltQlnGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

DSL_Error_t DSL_DRV_VNX_DiagnosticDeltDataGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   ACK_TestParamsAuxDS_Get_t  sAck;
   DSL_uint16_t i = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_AdslDeltAllDataGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( pContext->DELT == DSL_NULL )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - No memory allocated to store DELT data!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   for(i = 0; i <= (DSL_uint16_t)DSL_DOWNSTREAM; i++)
   {
      /* Get Test Params Aux data, not available for VDSL in DIAG_COMPLETE state*/
         nErrCode = DSL_DRV_VNX_SendMsgTestParamsAuxGet(
                       pContext, (DSL_AccessDir_t)i, (DSL_uint8_t*)&sAck);
   
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - DELT Aux data get failed, nDirection=%d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), i));
            
         return DSL_ERROR;
      }

      /* Update Aux Parameters*/
      if (((DSL_AccessDir_t)i) == DSL_DOWNSTREAM)
      {
         pContext->DELT->hlinScaleDataDs.nDeltHlinScale = sAck.HLINSC;
         pContext->DELT->hlogDataDs.nGroupSize          = (DSL_uint8_t)sAck.HLOGG;
         pContext->DELT->hlogDataDs.nMeasurementTime    = sAck.HLOGMT;
         pContext->DELT->qlnDataDs.nGroupSize           = (DSL_uint8_t)sAck.QLNG;
         pContext->DELT->qlnDataDs.nMeasurementTime     = sAck.QLNMT;
         pContext->DELT->snrDataDs.nGroupSize           = (DSL_uint8_t)sAck.SNRG;
         pContext->DELT->snrDataDs.nMeasurementTime     = sAck.SNRMT;
         pContext->DELT->hlinDataDs.nGroupSize          = (DSL_uint8_t)sAck.HLING;
         /* Not supported yet*/
         pContext->DELT->hlinDataDs.nMeasurementTime    = 0x0;
      }
      else
      {
         pContext->DELT->hlinScaleDataUs.nDeltHlinScale = sAck.HLINSC;
         pContext->DELT->hlogDataUs.nGroupSize          = (DSL_uint8_t)sAck.HLOGG;
         pContext->DELT->hlogDataUs.nMeasurementTime    = sAck.HLOGMT;
         pContext->DELT->qlnDataUs.nGroupSize           = (DSL_uint8_t)sAck.QLNG;
         pContext->DELT->qlnDataUs.nMeasurementTime     = sAck.QLNMT;
         pContext->DELT->snrDataUs.nGroupSize           = (DSL_uint8_t)sAck.SNRG;
         pContext->DELT->snrDataUs.nMeasurementTime     = sAck.SNRMT;
         pContext->DELT->hlinDataUs.nGroupSize          = (DSL_uint8_t)sAck.HLING;
         /* Not supported yet*/
         pContext->DELT->hlinDataUs.nMeasurementTime    = 0x0;
      }

      /* Get Hlog values, not available for VDSL in DIAG_COMPLETE state*/
      nErrCode = DSL_DRV_VNX_DeltHlogGet(
              pContext,
              (DSL_AccessDir_t)i,
              ((DSL_AccessDir_t)i) == DSL_DOWNSTREAM ?
              (DSL_G997_NSCData16_t*)&(pContext->DELT->hlogDataDs.deltHlog) :
              (DSL_G997_NSCData16_t*)&(pContext->DELT->hlogDataUs.deltHlog));

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - DELT Hlog data get failed, nDirection=%d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), i));
            
         break;
      }

      /* Get Line Status data*/
      nErrCode = DSL_DRV_VNX_LineStatusGet(
                    pContext,
                    (DSL_AccessDir_t)i,
                    &(pContext->DELT->lineStatus[i]));

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - DELT Line Status data get failed, nDirection=%d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), i));
            
         break;
      }

      /* Get Hlin values*/
      nErrCode = DSL_DRV_VNX_DeltHlinGet(
                    pContext,
                    (DSL_AccessDir_t)i,
					((DSL_AccessDir_t)i) == DSL_DOWNSTREAM ?
					(DSL_G997_NSCComplexData_t*)&(pContext->DELT->hlinDataDs.deltHlin) :
                    (DSL_G997_NSCComplexData_t*)&(pContext->DELT->hlinDataUs.deltHlin));

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - DELT Hlin data get failed, nDirection=%d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), i));
            
         break;
      }

      /* Get SNR values*/
      if ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
           DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB)) && (DSL_AccessDir_t)i == DSL_DOWNSTREAM)
      {
         DSL_DEBUG(DSL_DBG_WRN,
            (pContext, "DSL[%02d]: ADSL DELT SNR data DS not supported "
            "yet in DIAG_COMPLETE state!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
      }
      else
      {
         nErrCode = DSL_DRV_VNX_DeltSnrGet(
                       pContext,
                       (DSL_AccessDir_t)i,
   					   ((DSL_AccessDir_t)i) == DSL_DOWNSTREAM ?
   					   (DSL_G997_NSCData8_t*)&(pContext->DELT->snrDataDs.deltSnr) :
                       (DSL_G997_NSCData8_t*)&(pContext->DELT->snrDataUs.deltSnr));

         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG(DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - DELT SNR data get failed, nDirection=%d!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), i));
               
            break;
         }
      }

      /* Get QLN values*/
      nErrCode = DSL_DRV_VNX_DeltQlnGet(
                    pContext,
                    (DSL_AccessDir_t)i,
                    ((DSL_AccessDir_t)i) == DSL_DOWNSTREAM ?
                    (DSL_G997_NSCData8_t*)&(pContext->DELT->qlnDataDs.deltQln) :
                    (DSL_G997_NSCData8_t*)&(pContext->DELT->qlnDataUs.deltQln));

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - DELT QLN data get failed, nDirection=%d!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), i));
            
         break;
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_AdslDeltAllDataGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return(nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_LoopDiagnosticCompleteHandle(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nAutoCount = 0;
   DSL_G997_LDSF_t nLoopMode = 0;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_LoopDiagnosticCompleteHandle"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLoopAutoCount, nAutoCount);
   if( nAutoCount > 0 )
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nLoopAutoCount, nAutoCount - 1);
   }
 
   DSL_CTX_READ_SCALAR(pContext, nErrCode, lineActivateConfig.nLDSF, nLoopMode);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, bLoopDiagnosticsCompleted, DSL_TRUE);

#ifndef DSL_CPE_STATIC_DELT_DATA
   if (pContext->DELT == DSL_NULL)
   {
      pContext->DELT = DSL_DRV_Malloc(sizeof(DSL_G997_DeltData_t));
      if( pContext->DELT == DSL_NULL )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - memory for DELT data "
            "could not be allocated!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   }
#endif /* #ifndef DSL_CPE_STATIC_DELT_DATA*/

   memset(pContext->DELT, 0x0, sizeof(DSL_G997_DeltData_t));


   /* Update XTSE status information*/
   nErrCode = DSL_DRV_VNX_SendMsgXtseStatusGet( pContext );
   if(nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - XTSE status get failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
     return nErrCode;
   }

   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      nErrCode = DSL_DRV_VNX_SendMsgSelectedProfileVdsl2Get(pContext);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Selected Profile Vdsl2 Get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
              
         return nErrCode;
      }
   }

   /* Store all DELT diagnostic data*/
   nErrCode = DSL_DRV_VNX_DiagnosticDeltDataGet(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - DELT data save failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
       return nErrCode;
   }

   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      /* Send LINKRES message even if the CPE didn't triggered the DELT measurements.
         The configuration and state variables keep their values. FE DELT data will be requested
         later in the showtime. All NE data also will be available in the showtime*/
      nErrCode = DSL_DRV_DEV_LinkReset(pContext);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Link Reset failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      if( nLoopMode == DSL_G997_AUTO_LDSF )
      {
         nErrCode = DSL_DRV_VNX_SendMsgModemFsmOptionsSet(pContext, DSL_FALSE, DSL_FALSE);
         if( nErrCode!=DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Modem FSM options set failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return nErrCode;
         }
      }

      nErrCode = DSL_DRV_VNX_SendMsgModemStateSet(
                pContext, CMD_ModemFSM_StateSet_LINKINI);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Link Initialization failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   }
   else
   {
      /* Reboot modem*/
      nErrCode = DSL_DRV_AutobootStateSet(
                    pContext,
                    DSL_AUTOBOOTSTATE_RESTART,
                    DSL_AUTOBOOT_RESTART_POLL_TIME);
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_LoopDiagnosticCompleteHandle"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/**
   This function copies the received data to the structure \ref DSL_FctWaiting_t.

   \param pContext Pointer to dsl library context structure, [I]
   \param pMsg    - Pointer to message data, [I]
   \param nSize   - total byte count of the message, [I]

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS (0) in case of success
   - DSL_ERROR (-1) if operation failed
*/
static DSL_Error_t DSL_DRV_VNX_WaitingMessageCopyData(
   DSL_Context_t *pContext,
   DSL_uint8_t const *pMsg,
   const DSL_uint16_t nSize)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_FctWaiting_t *pFctWait = DSL_NULL;
   DSL_int_t nLength=0;

   /* Set pointer to function waiting structure that includes the actual
      settings for the messages that has to be processed here */
   pFctWait = &(pContext->pDevCtx->data.fctWait);
   
   nLength = nSize;
   if (nLength<=0)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_VNX_WaitingMessageCopyData - Length <= 0 !!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      pFctWait->bDataReceived = DSL_FALSE;
   }
   else
   {
      /* Copies the data from the firmware message to the message buffer included
         within the function waiting structure */
      memcpy(&pFctWait->nData[0], pMsg, nLength);
   }

   pFctWait->bDataReceived = DSL_TRUE;

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_VNX_HandleMessage(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t buf[512];
   IOCTL_VINAX_message_t sMsg;
   DSL_FctWaiting_t *pFctWait = DSL_NULL;
   pFctWait = &(pContext->pDevCtx->data.fctWait);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_VNX_HandleMessage"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   sMsg.paylSize_byte = sizeof(buf);
   sMsg.pPayload      = buf;

   /* check for received NFCs */
   if( DSL_DRV_VNX_ChReadMessage(pContext, &sMsg) >= DSL_SUCCESS )
   {
   /* one message received, handle it...*/
      if( sMsg.msgCtrl != VINAX_MSG_CTRL_MODEM_MSG )
      {
         DSL_DEBUG( DSL_DBG_MSG,
            (pContext, "DSL[%02d]: VINAX driver message received MsgID %x"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), sMsg.msgId));
      }
      else
      {
         switch(sMsg.msgId)
         {
         case EVT_PMD_TESTPARAMSGET:
            /* Get FE DELT parameters*/
            nErrCode = DSL_DRV_VNX_DeltFeUpdate(pContext, (EVT_PMD_TestParamsGet_t*)buf);
            break;

         case NFC_CLAUSE30_STATSLINEGET:
         case NFC_CLAUSE30_STATSSYSGET:         
            if (pFctWait->bFctWaiting == DSL_TRUE)
            {
               nErrCode = DSL_DRV_VNX_WaitingMessageCopyData(
                             pContext, (DSL_uint8_t *)buf,
                             (DSL_uint16_t)sMsg.paylSize_byte);
            }
            break;
            
         default:
            DSL_DEBUG( DSL_DBG_MSG,
               (pContext, "DSL[%02d]: VINAX control message received MsgID %x"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), sMsg.msgId));
               
            break;
         }
      }
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_VNX_HandleMessage"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_VNX_L3ShutdownRequest(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_VNX_FwVersion_t sFwVersion;
   CMD_L3ShutdownRequest_t sCmd;
   ACK_L3ShutdownRequest_t sAck;

   DSL_DEBUG( DSL_DBG_MSG,(pContext,
      "DSL[%02d]: IN - DSL_DRV_VNX_L3ShutdownRequest" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);
      
   if ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) && 
       ((sFwVersion.nFeatureSet >= 3) || 
       ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor > 0)) ||
       ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor == 0) && (sFwVersion.nMinor >= 15)))) ||
       (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB) && (sFwVersion.nFeatureSet >= 5)))
   {
      memset(&sCmd, 0, sizeof(sCmd));   
      sCmd.Length = 0x1;
      sCmd.L3shutdown = VNX_ENABLE;

      nErrCode = DSL_DRV_VNX_SendMessage(
                    pContext, CMD_L3SHUTDOWNREQUEST,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck);
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR,(pContext, "DSL[%02d]: ERROR - L3 mode not supported by the FW"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }

   DSL_DEBUG( DSL_DBG_MSG,(pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_L3ShutdownRequest" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_VNX_TxL3RequestStatusGet(
   DSL_Context_t *pContext,
   DSL_VNX_L3RequestStatus_t *pStatus)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_VNX_FwVersion_t sFwVersion;
   CMD_TxL3RequestStatusGet_t sCmd;
   ACK_TxL3RequestStatusGet_t sAck;

   DSL_CHECK_POINTER(pContext, pStatus);
   
   DSL_DEBUG( DSL_DBG_MSG,(pContext,
      "DSL[%02d]: IN - DSL_DRV_VNX_TxL3RequestStatusGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get FW version info*/
   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);
      
   if ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) && 
       ((sFwVersion.nFeatureSet >= 3) || 
       ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor > 0)) ||
       ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor == 0) && (sFwVersion.nMinor >= 15)))) ||
       (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB) && (sFwVersion.nFeatureSet >= 5)))
   {
      memset(&sCmd, 0, sizeof(sCmd));   
      sCmd.Length = 0x1;

      nErrCode = DSL_DRV_VNX_SendMessage(
                    pContext, CMD_TXL3REQUESTSTATUSGET,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck);

      if (nErrCode == DSL_SUCCESS)
      {
         switch (sAck.L3ReqStatus)
         {
         case ACK_TxL3RequestStatusGet_L3PENDING:
            *pStatus = DSL_VNX_L3_STATUS_PENDING;
            break;
         case ACK_TxL3RequestStatusGet_L3REJECTED:
            *pStatus = DSL_VNX_L3_STATUS_REJECTED;
            break;
         case ACK_TxL3RequestStatusGet_L3ACCEPTED:
            *pStatus = DSL_VNX_L3_STATUS_ACCEPTED;
            break;
         case ACK_TxL3RequestStatusGet_L3FAIL:
            *pStatus = DSL_VNX_L3_STATUS_FAIL;
            break;
         default:
            *pStatus = DSL_VNX_L3_STATUS_NA;         
            DSL_DEBUG( DSL_DBG_ERR,(pContext, "DSL[%02d]: ERROR - Unknown L3 request status (%d) received!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), sAck.L3ReqStatus));
            break;
         }
      }
      else
      {
         *pStatus = DSL_VNX_L3_STATUS_NA;
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR,(pContext, "DSL[%02d]: ERROR - L3 mode not supported by the FW"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }


   DSL_DEBUG( DSL_DBG_MSG,(pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_TxL3RequestStatusGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_VNX_TxL3RequestFailReasonGet(
   DSL_Context_t *pContext,
   DSL_VNX_L3RequestFailReason_t *pFailReason)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_VNX_FwVersion_t sFwVersion;
   CMD_TxL3ReqFailReasonGet_t sCmd;
   ACK_TxL3ReqFailReasonGet_t sAck;

   DSL_CHECK_POINTER(pContext, pFailReason);
   
   DSL_DEBUG( DSL_DBG_MSG,(pContext,
      "DSL[%02d]: IN - DSL_DRV_VNX_TxL3RequestFailReasonGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get FW version info*/
   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);
      
   if ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) && 
       ((sFwVersion.nFeatureSet >= 3) || 
       ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor > 0)) ||
       ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor == 0) && (sFwVersion.nMinor >= 15)))) ||
       (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB) && (sFwVersion.nFeatureSet >= 5)))
   {
      memset(&sCmd, 0, sizeof(sCmd));   
      sCmd.Length = 0x1;

      nErrCode = DSL_DRV_VNX_SendMessage(
                    pContext, CMD_TXL3REQFAILREASONGET,
                    sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                    sizeof(sAck), (DSL_uint8_t*)&sAck);

      if (nErrCode == DSL_SUCCESS)
      {
         switch (sAck.L3FailReason)
         {
         case ACK_TxL3ReqFailReasonGet_NOT_L0:
            *pFailReason = DSL_VNX_L3_FAIL_REASON_NOT_L0;
            break;
         case ACK_TxL3ReqFailReasonGet_TIMEOUT:
            *pFailReason = DSL_VNX_L3_FAIL_REASON_TIMEOUT;
            break;
         default:
            *pFailReason = DSL_VNX_L3_FAIL_REASON_NA;         
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Unknown L3 "
               "fail reason status (%d) received!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), sAck.L3FailReason));
            break;
         }
      }
      else
      {
         *pFailReason = DSL_VNX_L3_FAIL_REASON_NA;
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR,(pContext, "DSL[%02d]: ERROR - L3 mode not supported by the FW"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }


   DSL_DEBUG( DSL_DBG_MSG,(pContext,
      "DSL[%02d]: OUT - DSL_DRV_VNX_TxL3RequestFailReasonGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_XtseSettingsCheck(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN DSL_uint8_t *pXTSE)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   return nErrCode;
}

/**
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'lib_dsl_api.h'
*/
DSL_Error_t DSL_DRV_DEV_LinePathCounterTotalGet(
   DSL_Context_t *pContext,
   const DSL_uint8_t nChannel,
   DSL_LinePathCounterTotalData_t *pData )
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_FctWaiting_t *pFctWait = DSL_NULL;
   DSL_int32_t nWaitCount = 0;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
   
   CMD_Clause30_StatsLineGet_t sCmd;
   ACK_Clause30_StatsLineGet_t sAck;
   NFC_Clause30_StatsLineGet_t *pEfmNfc;

   DSL_LinePathCounterTotalData_t nTotalLinePathCounters;

   pFctWait = &(pContext->pDevCtx->data.fctWait);

   DSL_DEBUG( DSL_DBG_MSG,(pContext,
      "DSL[%02d]: IN - DSL_DRV_DEV_LinePathCounterTotalGet(nChannel=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nChannel));

   /* Only proceed if the specified line is in SHOWTIME state.
      In other cases set number of used data elements to zero. */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   if (nCurrentState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
   {
      DSL_DEBUG(DSL_DBG_WRN,(pContext,
         "DSL[%02d]: WARNING - Function is only available if line is in showtime state!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      memset((void*)pData, 0, sizeof(*pData));
      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   if (DSL_DRV_MUTEX_LOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex))
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Getting FctWait semaphore failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   pFctWait->bDataReceived = DSL_FALSE;
   pFctWait->bFctWaiting   = DSL_TRUE;

   memset (&sCmd, 0, sizeof(sCmd));
   sCmd.Length = 23;
   sCmd.LinkNo = nChannel;
   nErrCode = DSL_DRV_VNX_SendMessage(pContext, CMD_CLAUSE30_STATSLINEGET,
                 sizeof(CMD_Clause30_StatsLineGet_t), (DSL_uint8_t*)&sCmd,
                 sizeof(ACK_Clause30_StatsLineGet_t), (DSL_uint8_t*)&sAck);

   if (nErrCode < 0)
   {
      DSL_DEBUG( DSL_DBG_ERR,
          (pContext, "DSL[%02d]: error sending MsgID 0x%X." DSL_DRV_CRLF,
          DSL_DEV_NUM(pContext)));

      DSL_DRV_MUTEX_UNLOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex);
      return(nErrCode);
   }

   /* wait for NFC  max 3 sec */
   for (nWaitCount = 0; nWaitCount < 30; nWaitCount++)
   {
      DSL_DRV_MSecSleep(100);

      if (pFctWait->bDataReceived)
      {
         break;
      }
   }

   pFctWait->bFctWaiting   = DSL_FALSE;
      
   /* ACK was retrieved by the DSL_VNX_SendMessage,
      now check for the NFC */
   if (pFctWait->bDataReceived == DSL_FALSE)
   {
      /* Timeout! */
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Timeout during waiting for VINAX message (type=0x%02X, subtype=0x%04X)!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), pFctWait->nMsgClassifier, pFctWait->nMsgId));
         
      DSL_DRV_MUTEX_UNLOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex);
      
      return DSL_ERR_FUNCTION_WAITING_TIMEOUT;
   }

   pEfmNfc = (NFC_Clause30_StatsLineGet_t*)(&pFctWait->nData[0]);

   /* Get reference values from the Context*/
   DSL_CTX_READ(pContext, nErrCode, nTotalLinePathCounters[nChannel], nTotalLinePathCounters);

   /* ...and use them for the output values*/
   pData->UnitsTx                  = nTotalLinePathCounters.UnitsTx + pEfmNfc->Counter0;
   pData->UnitsRx                  = nTotalLinePathCounters.UnitsRx + pEfmNfc->Counter1;
   pData->FrameCheckSequenceErrors = nTotalLinePathCounters.FrameCheckSequenceErrors + pEfmNfc->Counter2;
   pData->AlignmentErrors          = nTotalLinePathCounters.AlignmentErrors + pEfmNfc->Counter3;
   pData->OctetsTx                 = nTotalLinePathCounters.OctetsTx + pEfmNfc->Counter4;
   pData->OctetsRx                 = nTotalLinePathCounters.OctetsRx + pEfmNfc->Counter6;

   pData->UnitsTxErrors  = nTotalLinePathCounters.UnitsTxErrors + pEfmNfc->Counter8;
   pData->UnitsRxErrors  = nTotalLinePathCounters.UnitsRxErrors + pEfmNfc->Counter9;
   pData->PauseFramesTx  = nTotalLinePathCounters.PauseFramesTx + pEfmNfc->Counter10;
   pData->PauseFramesRx  = nTotalLinePathCounters.PauseFramesRx + pEfmNfc->Counter11;
   pData->FramesTooLong  = nTotalLinePathCounters.FramesTooLong + pEfmNfc->Counter12;
   pData->FramesTooShort = nTotalLinePathCounters.FramesTooShort + pEfmNfc->Counter13;

   DSL_DRV_MUTEX_UNLOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_LinePathCounterTotalGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/**
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'lib_dsl_api.h'
*/
DSL_Error_t DSL_DRV_DEV_DataPathCounterTotalGet(
   DSL_Context_t *pContext,
   const DSL_uint8_t nChannel,
   DSL_DataPathCounterTotalData_t *pData )
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_FctWaiting_t *pFctWait = DSL_NULL;
   DSL_int32_t nWaitCount = 0;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
   
   CMD_Clause30_StatsSysGet_t sCmd;
   ACK_Clause30_StatsSysGet_t sAck;
   NFC_Clause30_StatsSysGet_t *pEfmNfc;

   DSL_DataPathCounterTotalData_t nTotalDataPathCounters;

   pFctWait = &(pContext->pDevCtx->data.fctWait);

   DSL_DEBUG( DSL_DBG_MSG,(pContext,
      "DSL[%02d]: IN - DSL_DRV_DEV_DataPathCounterTotalGet(nChannel=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nChannel));

   /* Only proceed if the specified line is in SHOWTIME state.
      In other cases set number of used data elements to zero. */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   if (nCurrentState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
   {
      DSL_DEBUG(DSL_DBG_WRN,(pContext,
         "DSL[%02d]: WARNING - Function is only available if line is in showtime state!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      memset((void*)pData, 0, sizeof(*pData));
      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   if (DSL_DRV_MUTEX_LOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex))
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Getting FctWait semaphore failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   pFctWait->bDataReceived = DSL_FALSE;
   pFctWait->bFctWaiting   = DSL_TRUE;

   memset (&sCmd, 0, sizeof(sCmd));
   sCmd.Length = 29;
   sCmd.LinkNo = nChannel;
   nErrCode = DSL_DRV_VNX_SendMessage(pContext, CMD_CLAUSE30_STATSSYSGET,
                 sizeof(CMD_Clause30_StatsSysGet_t), (DSL_uint8_t*)&sCmd,
                 sizeof(ACK_Clause30_StatsSysGet_t), (DSL_uint8_t*)&sAck);

   if (nErrCode < 0)
   {
      DSL_DEBUG( DSL_DBG_ERR,
          (pContext, "DSL[%02d]: error sending MsgID 0x%X." DSL_DRV_CRLF,
          DSL_DEV_NUM(pContext)));

      DSL_DRV_MUTEX_UNLOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex);
      return(nErrCode);
   }

   /* wait for NFC  max 3 sec */
   for (nWaitCount = 0; nWaitCount < 30; nWaitCount++)
   {
      DSL_DRV_MSecSleep(100);

      if (pFctWait->bDataReceived)
      {
         break;
      }
   }

   pFctWait->bFctWaiting   = DSL_FALSE;
      
   /* ACK was retrieved by the DSL_VNX_SendMessage,
      now check for the NFC */
   if (pFctWait->bDataReceived == DSL_FALSE)
   {
      /* Timeout! */
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: Timeout during waiting for VINAX message (type=0x%02X, subtype=0x%04X)!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), pFctWait->nMsgClassifier, pFctWait->nMsgId));
         
      DSL_DRV_MUTEX_UNLOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex);
      
      return DSL_ERR_FUNCTION_WAITING_TIMEOUT;
   }

   pEfmNfc = (NFC_Clause30_StatsSysGet_t*)(&pFctWait->nData[0]);

   /* Get reference values from the Context*/
   DSL_CTX_READ(pContext, nErrCode, nTotalDataPathCounters[nChannel], nTotalDataPathCounters);

   /* ...and use them for the output values*/

   pData->UnitsTx                  = nTotalDataPathCounters.UnitsTx + pEfmNfc->Counter0;
   pData->SingleCollisionFrames    = nTotalDataPathCounters.SingleCollisionFrames + pEfmNfc->Counter1;
   pData->MultipleCollisionFrames  = nTotalDataPathCounters.MultipleCollisionFrames + pEfmNfc->Counter2;
   pData->UnitsRx                  = nTotalDataPathCounters.UnitsRx + pEfmNfc->Counter3;
   pData->FrameCheckSequenceErrors = nTotalDataPathCounters.FrameCheckSequenceErrors + pEfmNfc->Counter4;
   pData->AlignmentErrors          = nTotalDataPathCounters.AlignmentErrors + pEfmNfc->Counter5;
   pData->OctetsTx                 = nTotalDataPathCounters.OctetsTx + pEfmNfc->Counter6;
   pData->OctetsRx                 = nTotalDataPathCounters.OctetsRx + pEfmNfc->Counter8;
   pData->UnitsTxErrors            = nTotalDataPathCounters.UnitsTxErrors + pEfmNfc->Counter10;
   pData->UnitsRxErrors            = nTotalDataPathCounters.UnitsRxErrors + pEfmNfc->Counter11;
   pData->PauseFramesTx            = nTotalDataPathCounters.PauseFramesTx + pEfmNfc->Counter12;
   pData->PauseFramesRx            = nTotalDataPathCounters.PauseFramesRx + pEfmNfc->Counter13;
   pData->FramesTooLong            = nTotalDataPathCounters.FramesTooLong + pEfmNfc->Counter14;
   pData->FramesTooShort           = nTotalDataPathCounters.FramesTooShort + pEfmNfc->Counter15;
   pData->UnitsCrc                 = 0; /* not supported by the FW*/
   pData->UnitsAlign               = 0; /* not supported by the FW*/
   
   DSL_DRV_MUTEX_UNLOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_LinePathCounterTotalGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_Clause30CountersSave(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nChannel = 0;
   DSL_uint16_t nWaitCount = 0;
   DSL_VNX_FwVersion_t  sFwVersion;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
   DSL_boolean_t bLineStatsReceived = DSL_FALSE,
                 bSysStatsReceived = DSL_FALSE,
                 bShowtimeReached = DSL_FALSE;
   union
   {
      DSL_LinePathCounterTotalData_t nLinePathData;
      DSL_DataPathCounterTotalData_t nDataPathData;
   } nTotalData;
   union
   {
      CMD_Clause30_StatsLineGet_t statsLine;
      CMD_Clause30_StatsSysGet_t  statsSys;
   } sCmd;
   union
   {
      ACK_Clause30_StatsLineGet_t statsLine;
      ACK_Clause30_StatsSysGet_t  statsSys;
   } sAck;

   NFC_Clause30_StatsLineGet_t *pNfcLine;
   NFC_Clause30_StatsSysGet_t  *pNfcSys;

   DSL_uint8_t buf[512];
   IOCTL_VINAX_message_t sMsg;
   
   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_Clause30CountersSave" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get Line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);
   /* Get FW info*/
   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);

   /* Check for the appropriate VDSL FW version*/
   if ( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) &&
        (nCurrentState == DSL_LINESTATE_EXCEPTION) &&
        ((sFwVersion.nFeatureSet < 9) || 
        ((sFwVersion.nFeatureSet == 9) && (sFwVersion.nMajor < 4)) ||
        ((sFwVersion.nFeatureSet == 9) && (sFwVersion.nMajor == 4) && (sFwVersion.nMinor < 4))) )
   {
      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, "DSL[%02d]: WARNING - Please use VDSL FW 9.9.4.4.0.2 or "
                    "higher to allow Clause30 counters access in FAIL state!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_WRN_FIRMWARE_MSG_DENIED;
   }

   /* Check for the appropriate ADSL Annex A FW version*/
   if ( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) &&
        (nCurrentState == DSL_LINESTATE_EXCEPTION) &&
        ((sFwVersion.nFeatureSet < 3) || 
        ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMinor < 15))) )
   {
      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, "DSL[%02d]: WARNING - Please use ADSL Annex A FW 9.3.0.15.1.5 or "
                    "higher to allow Clause30 counters access in FAIL state!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_WRN_FIRMWARE_MSG_DENIED;
   }

   /* Check for the appropriate ADSL Annex B FW version*/
   if ( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB) &&
        (nCurrentState == DSL_LINESTATE_EXCEPTION) &&
        ((sFwVersion.nFeatureSet < 5) || 
        ((sFwVersion.nFeatureSet == 5) && (sFwVersion.nMinor < 1))) )
   {
      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, "DSL[%02d]: WARNING - Please use ADSL Annex B FW 9.5.0.1.1.6 or "
                    "higher to allow Clause30 counters access in FAIL state!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_WRN_FIRMWARE_MSG_DENIED;
   }

   /* Get the Showtime indication flag*/
   DSL_CTX_READ(pContext, nErrCode, bShowtimeReached, bShowtimeReached);

   if (((nCurrentState != DSL_LINESTATE_SHOWTIME_TC_SYNC) &&
        (nCurrentState != DSL_LINESTATE_EXCEPTION)) ||
       ((nCurrentState == DSL_LINESTATE_EXCEPTION) && (bShowtimeReached != DSL_TRUE)))
   {
      DSL_DEBUG(DSL_DBG_WRN,(pContext,
         "DSL[%02d]: WARNING - Function is only available if line is in showtime or "
         "fail state entered from the showtime only!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      if (DSL_DRV_MUTEX_LOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex))
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: Getting FctWait semaphore failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         
         return DSL_ERROR;
      }
   
      for (nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++)
      {
         /* Request Line Path statistics*/
         memset (&(sCmd.statsLine), 0, sizeof(sCmd.statsLine));
         sCmd.statsLine.Length = 23;
         sCmd.statsLine.LinkNo = nChannel;
         nErrCode = DSL_DRV_VNX_SendMessage(pContext, CMD_CLAUSE30_STATSLINEGET,
                       sizeof(CMD_Clause30_StatsLineGet_t), (DSL_uint8_t*)&(sCmd.statsLine),
                       sizeof(ACK_Clause30_StatsLineGet_t), (DSL_uint8_t*)&(sAck.statsLine));

         if (nErrCode < 0)
         {
            DSL_DEBUG( DSL_DBG_ERR,
                (pContext, "DSL[%02d]: ERROR - CMD_CLAUSE30_STATSLINEGET msg send failed!" DSL_DRV_CRLF,
                DSL_DEV_NUM(pContext)));
            break;
         }


         /* Request Data Path statistics only after the previous NFC request was handled???*/
         memset (&(sCmd.statsSys), 0, sizeof(sCmd.statsSys));
         sCmd.statsSys.Length = 29;
         sCmd.statsSys.LinkNo = nChannel;
         nErrCode = DSL_DRV_VNX_SendMessage(pContext, CMD_CLAUSE30_STATSSYSGET,
                       sizeof(CMD_Clause30_StatsSysGet_t), (DSL_uint8_t*)&(sCmd.statsSys),
                       sizeof(ACK_Clause30_StatsSysGet_t), (DSL_uint8_t*)&(sAck.statsSys));

         if (nErrCode < 0)
         {
            DSL_DEBUG( DSL_DBG_ERR,
              (pContext, "DSL[%02d]: ERROR - CMD_CLAUSE30_STATSSYSGET msg send failed!" DSL_DRV_CRLF,
              DSL_DEV_NUM(pContext)));
            break;
         }

         /* wait for both NFC  max 6 sec */
         for (nWaitCount = 0; nWaitCount < 60; nWaitCount++)
         {
            sMsg.paylSize_byte = sizeof(buf);
            sMsg.pPayload      = buf;

            /* check for received NFCs */
            if( DSL_DRV_VNX_ChReadMessage(pContext, &sMsg) >= DSL_SUCCESS )
            {
               /* one message received, handle it...*/
               if( sMsg.msgCtrl == VINAX_MSG_CTRL_MODEM_MSG )
               {
                  switch(sMsg.msgId)
                  {
                  case NFC_CLAUSE30_STATSLINEGET:
                     pNfcLine = (NFC_Clause30_StatsLineGet_t*)buf;
                     bLineStatsReceived = DSL_TRUE;
                     
                     nTotalData.nLinePathData.UnitsTx                  = pNfcLine->Counter0;
                     nTotalData.nLinePathData.UnitsRx                  = pNfcLine->Counter1;
                     nTotalData.nLinePathData.FrameCheckSequenceErrors = pNfcLine->Counter2;
                     nTotalData.nLinePathData.AlignmentErrors          = pNfcLine->Counter3;
                     nTotalData.nLinePathData.OctetsTx                 = pNfcLine->Counter4;
                     nTotalData.nLinePathData.OctetsRx                 = pNfcLine->Counter6;
                     nTotalData.nLinePathData.UnitsTxErrors  = pNfcLine->Counter8;
                     nTotalData.nLinePathData.UnitsRxErrors  = pNfcLine->Counter9;
                     nTotalData.nLinePathData.PauseFramesTx  = pNfcLine->Counter10;
                     nTotalData.nLinePathData.PauseFramesRx  = pNfcLine->Counter11;
                     nTotalData.nLinePathData.FramesTooLong  = pNfcLine->Counter12;
                     nTotalData.nLinePathData.FramesTooShort = pNfcLine->Counter13;

                     /* Set reference values in the Context*/
                     DSL_CTX_WRITE(pContext, nErrCode, nTotalLinePathCounters[nChannel], nTotalData.nLinePathData);
                     break;
                  case NFC_CLAUSE30_STATSSYSGET:         
                     pNfcSys = (NFC_Clause30_StatsSysGet_t*)buf;               
                     bSysStatsReceived = DSL_TRUE;
                     
                     nTotalData.nDataPathData.UnitsTx                  = pNfcSys->Counter0;
                     nTotalData.nDataPathData.SingleCollisionFrames    = pNfcSys->Counter1;
                     nTotalData.nDataPathData.MultipleCollisionFrames  = pNfcSys->Counter2;
                     nTotalData.nDataPathData.UnitsRx                  = pNfcSys->Counter3;
                     nTotalData.nDataPathData.FrameCheckSequenceErrors = pNfcSys->Counter4;
                     nTotalData.nDataPathData.AlignmentErrors          = pNfcSys->Counter5;
                     nTotalData.nDataPathData.OctetsTx                 = pNfcSys->Counter6;
                     nTotalData.nDataPathData.OctetsRx                 = pNfcSys->Counter8;
                     nTotalData.nDataPathData.UnitsTxErrors            = pNfcSys->Counter10;
                     nTotalData.nDataPathData.UnitsRxErrors            = pNfcSys->Counter11;
                     nTotalData.nDataPathData.PauseFramesTx            = pNfcSys->Counter12;
                     nTotalData.nDataPathData.PauseFramesRx            = pNfcSys->Counter13;
                     nTotalData.nDataPathData.FramesTooLong            = pNfcSys->Counter14;
                     nTotalData.nDataPathData.FramesTooShort           = pNfcSys->Counter15;
                     nTotalData.nDataPathData.UnitsCrc                 = 0; /* not supported by the FW*/
                     nTotalData.nDataPathData.UnitsAlign               = 0; /* not supported by the FW*/

                     /* Set reference values in the Context*/
                     DSL_CTX_WRITE(pContext, nErrCode, nTotalDataPathCounters[nChannel], nTotalData.nDataPathData);
                     break;
                  
                  default:
                     break;
                  }
               }
            }
            
            if (bLineStatsReceived && bSysStatsReceived)
            {
               break;
            }
           
            /* ...sleep*/
            DSL_DRV_MSecSleep(100);            
         }
 
         /* Check if both NFCs were received*/
         if ((!bLineStatsReceived) || (!bSysStatsReceived))
         {
            DSL_DEBUG(DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Timeout while waiting for the Clause30 NFCs, (nChannel=%d)!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), nChannel));

            nErrCode = DSL_ERROR;
            break;
         }
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->pDevCtx->data.fctWait.MsgTransferMutex);      
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_Clause30CountersSave" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_LinkActivate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nLineState;
   DSL_TestModeControlSet_t nTestModeControl = DSL_TESTMODE_DISABLE;
   DSL_uint16_t linkStartMode = CMD_ModemFSM_StateSet_LINKINI;

   /* Get device line state*/
   nErrCode = DSL_DRV_DEV_LineStateGet(pContext, &nLineState);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Line state get failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   if (nLineState != DSL_LINESTATE_IDLE)
   {
      DSL_DEBUG( DSL_DBG_ERR,
        (pContext, "DSL[%02d]: ERROR - Expecting IDLE line state while Link Activation!"DSL_DRV_CRLF,
        DSL_DEV_NUM(pContext)));
        
      return DSL_ERROR;
   }

   /* Get Test Mode Control settings*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nTestModeControl, nTestModeControl);

   /* Activate LINK in TEST mode only if required. LINKINI is the default setting*/
   if( nTestModeControl == DSL_TESTMODE_QUIET )
   {
      if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) || 
          DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
      {
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext, "DSL[%02d]: WARNING - TEST state only available in the VDSL "
            "mode, using normal LINKINI!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         linkStartMode = CMD_ModemFSM_StateSet_LINKINI;
      }
      else
      {
         linkStartMode = CMD_ModemFSM_StateSet_TESTSTA;      
      }
   }

   /* Send LINKINI message*/
   nErrCode = DSL_DRV_VNX_SendMsgModemStateSet(pContext, linkStartMode);
   if ( nErrCode < DSL_SUCCESS )
   {
      /* Set LINIT value to 'LINIT_UNKNOWN' in case of an error */
      DSL_DRV_HandleLinitValue(pContext, LINIT_UNKNOWN, LINIT_SUB_NONE);

      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - modem LINKINI failed!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* SILENT line state reached*/
   DSL_DRV_LineStateSet(pContext, DSL_LINESTATE_SILENT);

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_LowLevelConfigurationSet(
   DSL_Context_t *pContext,
   DSL_DeviceLowLevelConfig_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t tmp = 0;
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_LowLevelConfigurationSet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - data mutex lock failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* Check and set nGainSelection*/
   if( pData->nGainSelection != DSL_DEV_RX_TX_GAINS_NA )
   {
      tmp = (DSL_int_t)pData->nGainSelection;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)DSL_DEV_RX_TX_GAINS_DEFAULT,
                    (DSL_int_t)(DSL_DEV_RX_TX_GAINS_LAST-1),
                    &tmp,
                    "nGainSelection");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nGainSelection = tmp;
   }

   if (pContext->pDevCtx->data.deviceCfg.cfg.nGainSelection == DSL_DEV_RX_TX_GAINS_USER)
   {
      /* Check and set nRxGain*/
      tmp = (DSL_int_t)pData->nUserGainSettings.nRxGain;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)-15,
                    (DSL_int_t)15,
                    &tmp,
                    "nRxGain");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nUserGainSettings.nRxGain = tmp;

      /* Check and set nTxGain*/
      tmp = (DSL_int_t)pData->nUserGainSettings.nTxGain;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)-15,
                    (DSL_int_t)15,
                    &tmp,
                    "nTxGain");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nUserGainSettings.nTxGain = tmp;
   }

   /* Check and set nClockoutEnable*/
   if( pData->nClockoutEnable != DSL_DEV_CLOCKOUT_NA )
   {
      tmp = (DSL_int_t)pData->nClockoutEnable;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_CLOCKOUT_LAST-1),
                    &tmp,
                    "nClockoutEnable");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nClockoutEnable = tmp;
   }

   /* Check and set nFilter*/
   if( pData->nFilter != DSL_DEV_FILTER_NA )
   {
      tmp = (DSL_int_t)pData->nFilter;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_FILTER_LAST-1),
                    &tmp,
                    "nFilter");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nFilter = tmp;
   }

   /* Check and set nHdlcIfMode*/
   if( pData->nHdlcIfMode != DSL_DEV_HDLCMODE_NA )
   {
      tmp = (DSL_int_t)pData->nHdlcIfMode;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_HDLCMODE_LAST-1),
                    &tmp,
                    "nHdlcIfMode");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nHdlcIfMode = tmp;
   }

   /* Check and set nMiiClockSource*/
   if( pData->nMiiClockSource != DSL_DEV_MIICLKSOURCE_NA )
   {
      tmp = (DSL_int_t)pData->nMiiClockSource;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_MIICLKSOURCE_LAST-1),
                    &tmp,
                    "nMiiClockSource");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nMiiClockSource = tmp;
   }

   /* Check and set nMiiClockChain*/
   if( pData->nMiiClockChain != DSL_DEV_MIICLKCHAIN_NA )
   {
      tmp = (DSL_int_t)pData->nMiiClockChain;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_MIICLKCHAIN_LAST-1),
                    &tmp,
                    "nMiiClockChain");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nMiiClockChain = tmp;
   }

   /* Check and set nTcMode*/
   if( pData->nTcMode != DSL_DEV_TCMODE_NA )
   {
      tmp = (DSL_int_t)pData->nTcMode;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_TCMODE_LAST-1),
                    &tmp,
                    "nTcMode");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nTcMode = tmp;
   }

   /* Check and set nMiiMode*/
   if( pData->nMiiMode != DSL_DEV_MIIMODE_NA )
   {
      tmp = (DSL_int_t)pData->nMiiMode;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_MIIMODE_LAST-1),
                    &tmp,
                    "nMiiMode");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nMiiMode = tmp;
   }

   /* Check and set nLineMode*/
   if( pData->nLineMode != DSL_DEV_LINEMODE_NA )
   {
      tmp = (DSL_int_t)pData->nLineMode;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)DSL_DEV_LINEMODE_VDSL2_B43,
                    (DSL_int_t)(DSL_DEV_LINEMODE_LAST >> 1),
                    &tmp,
                    "nLineMode");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nLineMode = tmp;
   }

   /* Check and set nHybrid*/
   if( pData->nHybrid != DSL_DEV_HYBRID_NA )
   {
      tmp = (DSL_int_t)pData->nHybrid;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_HYBRID_LAST-1),
                    &tmp,
                    "nHybrid");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nHybrid = tmp;
   }

   /* Check and set nBaseAddr*/
   if (pData->nBaseAddr != (DSL_uint32_t)-1)
   {
         /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nBaseAddr = pData->nBaseAddr;
   }

   /* Check and set nIrqNum*/
   if (pData->nIrqNum != -1)
   {
      tmp = (DSL_int_t)pData->nIrqNum;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)99,
                    &tmp,
                    "nIrqNum");
         /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nIrqNum = tmp;
   }

   /* Check and set nUtopiaParity*/
   if( pData->nUtopiaParity != DSL_DEV_PARITY_NA )
   {
      tmp = (DSL_int_t)pData->nUtopiaParity;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_PARITY_LAST-1),
                    &tmp,
                    "nUtopiaParity");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nUtopiaParity = tmp;
   }

   if( pData->nUtopiaPhyAdr != 0xFF )
   {
      tmp = (DSL_int_t)pData->nUtopiaPhyAdr;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(30),
                    &tmp,
                    "nUtopiaPhyAdr");
      /* Set nUtopiaPhyAdr*/
      pContext->pDevCtx->data.deviceCfg.cfg.nUtopiaPhyAdr = tmp;
   }

   /* Set nUtopiaBusWidth*/
   if( pData->nUtopiaBusWidth != DSL_BUS_WIDTH_NA )
   {
      tmp = (DSL_int_t)pData->nUtopiaBusWidth;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)DSL_BUS_WIDTH_8B,
                    (DSL_int_t)(DSL_BUS_WIDTH_LAST - 1),
                    &tmp,
                    "nUtopiaBusWidth");

      pContext->pDevCtx->data.deviceCfg.cfg.nUtopiaBusWidth = tmp;
   }

   /* Check and set nPosPhyParity*/
   if( pData->nPosPhyParity != DSL_DEV_PARITY_NA )
   {
      tmp = (DSL_int_t)pData->nPosPhyParity;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(DSL_DEV_PARITY_LAST-1),
                    &tmp,
                    "nUtopiaParity");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nPosPhyParity = tmp;
   }

   /* Set nPosPhyAdr*/
   if( pData->nPosPhyAdr != 0xFF )
   {
      tmp = (DSL_int_t)pData->nPosPhyAdr;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)(30),
                    &tmp,
                    "nPosPhyAdr");
      /* Set nUtopiaPhyAdr*/
      pContext->pDevCtx->data.deviceCfg.cfg.nPosPhyAdr = tmp;
   }

   /* Check and set nPosphyChunkSize*/
   if( pData->nPosphyChunkSize != -1 )
   {
      tmp = (DSL_int_t)pData->nPosphyChunkSize;
      nErrCode = DSL_DRV_VNX_CheckTruncParamRange(
                    pContext,
                    (DSL_int_t)0,
                    (DSL_int_t)64,
                    &tmp,
                    "nPosphyChunkSize");
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nPosphyChunkSize = tmp;
   }

   if( pData->nEapsTimeout != -1 )
   {
      tmp = (DSL_int_t)pData->nPosphyChunkSize;
      /* Set new value in the Device context*/
      pContext->pDevCtx->data.deviceCfg.cfg.nEapsTimeout = pData->nEapsTimeout;
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_LowLevelConfigurationSet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_FramingParameterStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_FramingParameterStatus_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_DIRECTION(pData->nDirection);
   DSL_CHECK_CHANNEL_RANGE(pData->nChannel);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_FramingParameterStatusGet,"
      " (nDirection=%d, nChannel=%d)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext),
      pData->nDirection, pData->nChannel));

   /* Get current line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   /* Only proceed if the line is in SHOWTIME state.*/
   if ( (nCurrentState != DSL_LINESTATE_SHOWTIME_TC_SYNC) &&
        (nCurrentState != DSL_LINESTATE_SHOWTIME_NO_SYNC) )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Function is only available in the SHOWTIME!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      /* VDSL2 mode */
      nErrCode = DSL_DRV_VNX_FramingParametersVdsl2StatusGet(
                    pContext, pData->nChannel,
                    pData->nDirection,&(pData->data));
   }
   else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1))
   {
      /* VDSL1 mode. Not supported yet */
      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }
   else
   {
      /* ADSL mode */
      nErrCode = DSL_DRV_VNX_FramingParametersAdslStatusGet(
                    pContext, pData->nChannel,
                    pData->nDirection,&(pData->data));
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_FramingParameterStatusGet, retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_UtopiaAddressConfigSet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_PhyAddressConfigData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_UtopiaAddressConfigSet(nChannel=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nChannel));

   if (pData->nPhyAdr > 30)
   {
      nErrCode = DSL_ERR_PARAM_RANGE;
   }
   else
   {
      /* Copy System Configuration data to internal VINAX device context*/   
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
         pDevCtx->data.deviceCfg.cfg.nUtopiaPhyAdr, pData->nPhyAdr);
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_UtopiaAddressConfigSet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_UtopiaAddressConfigGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_PhyAddressConfigData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_UtopiaAddressConfigGet,(nChannel=%d)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nChannel));

   /* Get Configuration data from the internal VINAX device context*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nUtopiaPhyAdr, pData->nPhyAdr);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_UtopiaAddressConfigGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_UtopiaBusWidthConfigSet(
   DSL_Context_t *pContext,
   DSL_UtopiaBusWidthConfigData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_UtopiaBusWidthConfigSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if ( pData->nDataWidth < DSL_BUS_WIDTH_8B || 
        pData->nDataWidth > (DSL_BUS_WIDTH_LAST - 1))
   {
      nErrCode = DSL_ERR_PARAM_RANGE;
   }
   else
   {
      /* Copy System Configuration data to internal VINAX device context*/   
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
         pDevCtx->data.deviceCfg.cfg.nUtopiaBusWidth, pData->nDataWidth);
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_UtopiaBusWidthConfigSet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_UtopiaBusWidthConfigGet(
   DSL_Context_t *pContext,
   DSL_UtopiaBusWidthConfigData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_UtopiaBusWidthConfigGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get Configuration data from the internal VINAX device context*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.deviceCfg.cfg.nUtopiaBusWidth, pData->nDataWidth);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_UtopiaBusWidthConfigGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_InitDataPrepare(
   DSL_Context_t *pContext,
   DSL_boolean_t bIsInKernel,
   DSL_Init_t *pInit)
{
   /* Nothing to be done for VINAX */
   return DSL_SUCCESS;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_void_t DSL_DRV_DEV_InitDataFree(DSL_Init_t *pInit)
{
   /* Nothing to be done for VINAX */
   return;
}


static DSL_Error_t DSL_DRV_DEV_ModemWriteConfig(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_boolean_t bFwReady = DSL_FALSE, bCamFwReload = DSL_FALSE;
   DSL_TestModeControlSet_t nTestModeControl = DSL_TESTMODE_DISABLE;

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->lowHandle);
   DSL_CHECK_ERR_CODE();

   /* Get FW DL information*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, bFwReady);

   if( nErrCode != DSL_SUCCESS || !bFwReady )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - no FW downloaded!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* Get bCamFwReload flag*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.bCamFwReload, bCamFwReload);
   if( bCamFwReload )
   {
      /* No further configurations needed in case of CAM FW reload*/
      return DSL_SUCCESS;
   }

   /* Get Test Mode Control settings*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nTestModeControl, nTestModeControl);

   /* Configurations dependent on the Low Level Config */

   nErrCode = DSL_DRV_VNX_SendMsgHwConfigSet(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
         "HW config write failed!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* not for ADSL-only mode */
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      nErrCode = DSL_DRV_VNX_SendMsgHybridSelect(pContext);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "hybrid config write failed!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      nErrCode = DSL_DRV_VNX_SendMsgLedControl(pContext);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "LED config write failed!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      nErrCode = DSL_DRV_VNX_SendMsgHsToneGroupSet(pContext);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "handshake tones config write!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      /* Set Near-End Re-Init configuration options depending on the 
      Test Mode Control settings*/
      nErrCode = DSL_DRV_VNX_SendMsgReInitNeConfigure(
                    pContext, 
                    (nTestModeControl != DSL_TESTMODE_SHOWTIME_LOCK 
                     && nTestModeControl != DSL_TESTMODE_TRAINING_LOCK)
                    ? DSL_TRUE : DSL_FALSE);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "re-init NE config write failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      /* Set Far-End Re-Init configuration options depending on the 
      Test Mode Control settings*/
      nErrCode = DSL_DRV_VNX_SendMsgReInitFeConfigure(
                    pContext, 
                    (nTestModeControl != DSL_TESTMODE_SHOWTIME_LOCK
                    && nTestModeControl != DSL_TESTMODE_TRAINING_LOCK) ? DSL_TRUE : DSL_FALSE);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "re-init FE config write failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   }

   nErrCode = DSL_DRV_VNX_SendMsgGainCalibrationSet(pContext);
   if( nErrCode!=DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
         "gain calibration config write!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Customer- (System-) Interface selection shall be
      the first command to be sent
      SAR Config is done in here also */
   nErrCode = DSL_DRV_VNX_WriteConfigCif(pContext);
   if( nErrCode!=DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
         "CIF config write!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* VTSE and Handshake configuration */
   nErrCode = DSL_DRV_VNX_SendMsgVtseConfigure(pContext);
   if( nErrCode!=DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
         "VTSE config write!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* this is not possible for ADSL  CPE */
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      /* Settings related to the line Configs */
      nErrCode = DSL_DRV_VNX_SendMsgBearerChSet(pContext);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "Channel config write!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      nErrCode = DSL_DRV_VNX_SendMsgVdsl2ProfileControl(pContext);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "Profile Control write!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      nErrCode = DSL_DRV_VNX_SendMsgAlgorithmControl(pContext, 0, 0);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "algorithm configuration!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      nErrCode = DSL_DRV_VNX_SendMsgDigitalFilter(pContext);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "Digital Filter config write!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   }

#if (PRELIMINARY_ADSL_SUPPORT == 1)
   /* for ADSL only */
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
#if (ADSL_ALPHEUS_SUPPORT == 1)
      /* ADSL specific configs */
      nErrCode = DSL_DRV_VNX_SendMsgHsAdslControl(pContext);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "ADSL HS config write!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      nErrCode = DSL_DRV_VNX_SendMsgAdslLinkSetupControl(pContext);
      if( nErrCode!=DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: DSL_DEV_ModemWriteConfig: ERROR - "
            "ADSL Link Setup write!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
#endif
   }
#endif /* (PRELIMINARY_ADSL_SUPPORT == 1) */

   /* Set Near-End Inventory information*/
   nErrCode = DSL_DRV_VNX_NeInventorySet(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Line Inventory set failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Set SRA control*/
   nErrCode = DSL_DRV_VNX_SendMsgSraControl(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - SRA control failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   return DSL_SUCCESS;
}


DSL_DEV_Handle_t DSL_DRV_DEV_DriverHandleGet(
   DSL_int_t nMaj,
   DSL_int_t nNum)
{
   VINAX_DYN_CNTRL_T *pVinaxDynCntrl = DSL_NULL;

   return VINAX_InternalDevOpen(
             (IFX_uint16_t)nNum,
             &pVinaxDynCntrl) ? DSL_NULL : (DSL_DEV_Handle_t)pVinaxDynCntrl;
}

DSL_Error_t DSL_DRV_DEV_DriverHandleDelete(
   DSL_DEV_Handle_t handle)
{
   return VINAX_InternalDevClose(
             (VINAX_DYN_CNTRL_T*)handle) ? DSL_ERROR : DSL_SUCCESS;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_FwDownload(
   DSL_Context_t *pContext,
   const DSL_char_t *pFirmware,
   DSL_uint32_t nFirmwareSize,
   const DSL_char_t *pFirmware2,
   DSL_uint32_t nFirmwareSize2,
   DSL_int32_t *pLoff,
   DSL_int32_t *pCurrentOff,
   DSL_boolean_t bLastChunk)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_Handle_t dev;
   IOCTL_VINAX_fwDownLoad_t Vdsl2_FwDl;
   IOCTL_VINAX_reset_t rstCfg;
   DSL_VNX_FwVersion_t sFwVersion;
   DSL_uint32_t verNum = 0;
   DSL_FirmwareRequestType_t nFwType = DSL_FW_REQUEST_NA;
#if defined(INCLUDE_DSL_PM)
   DSL_uint32_t fwDwnldStartTime = 0, fwDwnldStopTime = 0, fwDwnldTime = 0;
#endif
   DSL_FwDownloadStatusData_t fwDwnlStatus;

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->lowHandle);
   DSL_CHECK_ERR_CODE();

   /* Get device handle*/
   dev = DSL_DEVICE_LOWHANDLE(pContext);

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.cfg.nFwType, nFwType);

   memset(&Vdsl2_FwDl,0x0,sizeof(IOCTL_VINAX_fwDownLoad_t));

   switch(nFwType)
   {
   case DSL_FW_REQUEST_ADSL:
      Vdsl2_FwDl.pFwImage  = (unsigned char *)pFirmware2;
      Vdsl2_FwDl.size_byte = (unsigned long)nFirmwareSize2;
      break;
   case DSL_FW_REQUEST_VDSL:
      Vdsl2_FwDl.pFwImage  = (unsigned char *)pFirmware;
      Vdsl2_FwDl.size_byte = (unsigned long)nFirmwareSize;
      break;
   default:
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Invalid FW mode %d specified!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nFwType));
         
      return DSL_ERROR;
   break;
   }

   /* Check FW availability*/
   if ( (Vdsl2_FwDl.size_byte == 0) || (Vdsl2_FwDl.pFwImage == DSL_NULL) )
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - VINAX FW download failed - no binary for %d FW mode!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nFwType));
         
      return DSL_ERROR;
   }

   /* Reset the device*/
   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Resetting the VINAX device..." DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));
      
   rstCfg.rstMode    = e_VINAX_RESET;
   rstCfg.rstSelMask = VINAX_IOCTL_HW_RST_MASK_ALL;
   if( VINAX_InternalDevReset((VINAX_DYN_CNTRL_T*)dev, &rstCfg, 0x1) != 0 )
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - VINAX device reset failed, (nReturn = %d)!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), rstCfg.ictl.retCode));
         
      return DSL_ERROR;
   }
   else
   {
      /* Reset bFirmwareReady flag*/
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, DSL_FALSE);
   }

#if defined(INCLUDE_DSL_PM)
   fwDwnldStartTime = DSL_DRV_ElapsedTimeMSecGet(0);
#endif

   /* Do the FW Download*/
   if( VINAX_InternalFirmwareDownload( (VINAX_DYN_CNTRL_T*)dev, &Vdsl2_FwDl) != 0 )
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - VINAX FW downloading failed, (nReturn = %d)!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), Vdsl2_FwDl.ictl.retCode));
         
      return DSL_ERROR;
   }

#if defined(INCLUDE_DSL_PM)
   fwDwnldStopTime = DSL_DRV_ElapsedTimeMSecGet(0);
   fwDwnldTime     = fwDwnldStartTime > fwDwnldStopTime ?
                     fwDwnldStopTime :
                     ((fwDwnldStopTime - fwDwnldStartTime)/1000);

   /* Set FW unavailable time*/
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nFwUnavailableTime, fwDwnldTime);
#endif

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: %s FW downloaded successfully" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext),
      nFwType == DSL_FW_REQUEST_VDSL ? "VDSL" : "ADSL"));

   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, DSL_TRUE);

   /* Get the downloaded FW version information*/
   if( DSL_DRV_VNX_SendMsgFwVerInfoGet(pContext, &verNum) != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - FW version get failed, msg send error!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   DSL_DRV_VNX_FirmwareVersionDecode(pContext, verNum, &sFwVersion);

   /* Write FW application type to the DSL CPE context*/
   DSL_CTX_WRITE(pContext, nErrCode,
                 pDevCtx->data.fwFeatures,
                 sFwVersion);

   if (nFwType == DSL_FW_REQUEST_VDSL && 
       !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) &&
       !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - ADSL FW was loaded, expecting VDSL FW!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }       

   if (nFwType == DSL_FW_REQUEST_ADSL && 
       !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) &&
       !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - VDSL FW was loaded, expecting ADSL FW!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }       

   /* Fill DSL_EVENT_S_FIRMWARE_DOWNLOAD_STATUS event structure*/
   fwDwnlStatus.nError  = nErrCode == DSL_SUCCESS ? DSL_FW_LOAD_SUCCESS: DSL_FW_LOAD_ERROR;
   fwDwnlStatus.nFwType = nFwType;

   if( DSL_DRV_EventGenerate( pContext, 0, DSL_ACCESSDIR_NA,
          DSL_XTUDIR_NA, DSL_EVENT_S_FIRMWARE_DOWNLOAD_STATUS,
         (DSL_EventData_Union_t*)&fwDwnlStatus, sizeof(DSL_FwDownloadStatusData_t)) 
       != DSL_SUCCESS )
   {
       DSL_DEBUG( DSL_DBG_ERR,
          (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
          DSL_DEV_NUM(pContext), DSL_EVENT_S_FIRMWARE_DOWNLOAD_STATUS));
   }

   /* TODO:
       Some security check maybe needed to be sure that the correct
       FW was downloaded*/

   return (nErrCode);
}


/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_DeviceInit(
   DSL_Context_t *pContext,
   DSL_Init_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nVtse = 0, nAtse = 0, nAtseCurr = 0, i = 0;
   DSL_G997_RateAdaptationConfigData_t SraConfigData;
   DSL_DEV_CamStates_t nNewState = DSL_CAM_INIT;
   
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   /* Store Device specific configuration data in the device context*/
   DSL_DRV_VNX_DeviceCfgStore(pContext, pData);

   /* Set default SRA DS configuration. SRA is enabled by default*/
   SraConfigData.RA_MODE = DSL_G997_RA_MODE_DYNAMIC;
   DSL_CTX_WRITE(pContext, nErrCode, SraConfigDataDs, SraConfigData);

   /* Check current driver state, do the driver init, */
   if ( DSL_DRV_VNX_InitDeviceDrv(pContext, &pData->data.nDeviceCfg.cfg) != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - VINAX driver init failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* Enable CAM FSM */
   nErrCode = DSL_DRV_VNX_CamFsmEnable(pContext, DSL_TRUE);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - CAM FSM init failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* get the ADSL bits */
   for (i = 0; i < DSL_G997_NUM_XTSE_OCTETS - 1; i++)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[i], nAtseCurr);
      nAtse |= nAtseCurr;
   }

   /* get the VDSL bits */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, xtseCfg[7], nVtse);
    
   if (nAtse && nVtse)
   {
      /* Set ADSL FW image type for the multimode operation*/
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.cfg.nFwType, DSL_FW_REQUEST_ADSL); 
      nNewState = DSL_CAM_MULTI;
   }
   else if (nAtse)
   {
      /* Set ADSL FW image type*/
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.cfg.nFwType, DSL_FW_REQUEST_ADSL);
      nNewState = DSL_CAM_ADSL_RETRY;
   }
   else if (nVtse)
   {
      /* Set VDSL FW image type*/   
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.cfg.nFwType, DSL_FW_REQUEST_VDSL);
      nNewState = DSL_CAM_VDSL_FORCED;
   }
   else
   {
      /* Set VDSL mode by default if no XTSE configuration defined*/
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.deviceCfg.cfg.nFwType, DSL_FW_REQUEST_VDSL);         
   }

   nErrCode = DSL_DRV_VNX_CamFsmStateSet(pContext, nNewState);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - CAM FSM state set failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
            
      return nErrCode;
   }

   DSL_DRV_MUTEX_INIT(pContext->pDevCtx->data.fctWait.MsgTransferMutex);

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_boolean_t DSL_DRV_DEV_ModemIsReady(DSL_Context_t *pContext)
{
   DSL_LineStateValue_t nLineState;

   if( DSL_DRV_DEV_LineStateGet(pContext, &nLineState) != DSL_ERROR )
   {
      return nLineState == DSL_LINESTATE_IDLE ? DSL_TRUE : DSL_FALSE;
   }

   return DSL_FALSE;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_AutobootHandleL3(
   DSL_Context_t *pContext,
   DSL_boolean_t bL3Forced)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleL3"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( bL3Forced == DSL_FALSE )
   {
      nErrCode = DSL_DRV_AutobootStateSet(pContext, DSL_AUTOBOOTSTATE_INIT,
         DSL_AUTOBOOT_INIT_POLL_TIME);
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DEV_AutobootHandleL3, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_AutobootHandleStart(
   DSL_Context_t *pContext,
   DSL_boolean_t bLoopTest,
   DSL_boolean_t bShortInit)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_Autoboot_State_t nState = DSL_AUTOBOOTSTATE_UNKNOWN;

   /* print unused parameters to suppress warnings*/
   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleStart(%d, %d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), bLoopTest, bShortInit));

   /* Reset bCamFwReload flag*/
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->data.bCamFwReload, DSL_FALSE);

   /* Write Modem configuration data*/
   nErrCode = DSL_DRV_DEV_ModemWriteConfig(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - modem conf. write failed!"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Get actual autoboot state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nAutobootState, nState);
   if( nState != DSL_AUTOBOOTSTATE_INIT )
   {
      /* No further actions needed in case of the Autoboot state change*/
      return DSL_SUCCESS;
   }

   /* Diagnostic mode switch, auto-restart enable/disable */
   nErrCode = DSL_DRV_VNX_SendMsgModemFsmOptionsSet(pContext, DSL_FALSE, bLoopTest);
   if( nErrCode!=DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Modem FSM options set failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleStart" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

DSL_Error_t DSL_DRV_DEV_AutobootHandleFwRequest(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleFwRequest"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Update CAM FSM*/
   nErrCode = DSL_DRV_VNX_CamFsmUpdate(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - CAM FSM Update failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleFwRequest"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_AutobootHandleTraining(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN,
                        nPrevLineState = DSL_LINESTATE_UNKNOWN;
   DSL_TestModeControlSet_t nTestMode = DSL_TESTMODE_DISABLE;
#ifdef INCLUDE_DSL_PM
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
   DSL_uint32_t nFullInits = 0, nFailedFullInits = 0;
#endif /* INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
#endif /* INCLUDE_DSL_PM*/

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleTraining" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get Line State from the DSL CPE internal memory*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nPrevLineState);

   /* Update Line State in the DSL CPE internal memory*/
   nErrCode = DSL_DRV_LineStateUpdate(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      return nErrCode;
   }

   /* Get Line State from the DSL CPE internal memory*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

   if (nLineState != nPrevLineState)
   {
      /* Get Test mode from the DSL CPE internal memory*/
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nTestModeControl, nTestMode);

      switch (nLineState)
      {
      case DSL_LINESTATE_SILENT:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: SILENT state reached"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         break;
      case DSL_LINESTATE_SHOWTIME_TC_SYNC:
      case DSL_LINESTATE_SHOWTIME_NO_SYNC:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: SHOWTIME state reached"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         /* Clear bGotResponse flag*/
         pContext->bGotResponse = DSL_FALSE;
         /* Clear bGotFullInit flag*/
         pContext->bGotFullInit = DSL_FALSE;
         
         /* Get XTSE status info and update it in the DSL CPE internal memory*/
         nErrCode = DSL_DRV_DEV_ShowtimeStatusUpdate(pContext, DSL_TRUE);
         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG(DSL_DBG_MSG,
               (pContext, "DSL[%02d]: ERROR - Showtime status update failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
         }
         /* Move to the next Autoboot state*/
         nErrCode = DSL_DRV_AutobootStateSet(
                       pContext,
                       DSL_AUTOBOOTSTATE_SHOWTIME,
                       DSL_AUTOBOOT_SHOWTIME_POLL_TIME);

         /* Set LINIT value to 'LINIT_SUCCESSFUL' here because activation procedure
            was finished without errors */
         DSL_DRV_HandleLinitValue(pContext, LINIT_SUCCESSFUL, LINIT_SUB_NONE);

         /* Add FE Inventory Status timeout event. FE inventory information is available
             from the FW after 5 sec of the showtime state*/
         DSL_DRV_Timeout_AddEvent( pContext,
                              (DSL_int_t)DSL_TIMEOUTEVENT_FE_STATUS,
                              (DSL_uint32_t)DSL_DEV_TIMEOUT_FE_STATUS );
         break;
      case DSL_LINESTATE_EXCEPTION:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: EXCEPTION state reached"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
#ifdef INCLUDE_DSL_PM
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
         /* Check for the Failed Full init condition*/
         if (pContext->bGotFullInit)
         {
            /* Increment the number of Failed Full Inits*/
            DSL_CTX_READ(pContext, nErrCode, nFailedFullInits, nFailedFullInits);
            nFailedFullInits++;
            DSL_CTX_WRITE(pContext, nErrCode, nFailedFullInits, nFailedFullInits);
         }
#endif /* INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
#endif /* INCLUDE_DSL_PM*/
         if (nTestMode == DSL_TESTMODE_TRAINING_LOCK)
         {
            DSL_DEBUG(DSL_DBG_MSG,
               (pContext, "DSL[%02d]: EXCEPTION - training lock is set, "
               "exception state will not be set"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
         }
         else
         {
            DSL_DEBUG(DSL_DBG_MSG,
               (pContext, "DSL[%02d]: EXCEPTION - set exception state"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            nErrCode = DSL_DRV_AutobootStateSet(
               pContext,
               DSL_AUTOBOOTSTATE_EXCEPTION,
               DSL_AUTOBOOT_EXCEPTION_POLL_TIME);
         }
         break;
      case DSL_LINESTATE_HANDSHAKE:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: HANDSHAKE state reached"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         if (pContext->bGotResponse == DSL_FALSE)
         {
            pContext->bGotResponse = DSL_TRUE;
         }
         break;
      case DSL_LINESTATE_FULL_INIT:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: FULL_INIT state reached"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         if (pContext->bGotFullInit == DSL_FALSE)
         {
            pContext->bGotFullInit = DSL_TRUE;
#ifdef INCLUDE_DSL_PM
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
            /* Increment the number of Full Inits*/
            DSL_CTX_READ(pContext, nErrCode, nFullInits, nFullInits);
            nFullInits++;
            DSL_CTX_WRITE(pContext, nErrCode, nFullInits, nFullInits);
#endif /* INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
#endif /* INCLUDE_DSL_PM*/
         }
         break;
      case DSL_LINESTATE_LOOPDIAGNOSTIC_ACTIVE:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: LOOPDIAGNOSTIC_ACTIVE state reached"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         /* In case of DIAGNOSTIC mode use this state to identify FULL_INIT*/
         if (pContext->bGotFullInit == DSL_FALSE)
         {
            pContext->bGotFullInit = DSL_TRUE;
#ifdef INCLUDE_DSL_PM
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
            /* Increment the number of Full Inits*/
            DSL_CTX_READ(pContext, nErrCode, nFullInits, nFullInits);
            nFullInits++;
            DSL_CTX_WRITE(pContext, nErrCode, nFullInits, nFullInits);
#endif /* INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
#endif /* INCLUDE_DSL_PM*/
         }
         pContext->autobootStartTime = DSL_DRV_TimeMSecGet();
         pContext->nAutobootTimeoutLimit = 360;
         break;
      case DSL_LINESTATE_LOOPDIAGNOSTIC_COMPLETE:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: LOOPDIAGNOSTIC_COMPLETE state reached"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         /* Handle DSL_LINESTATE_LOOPDIAGNOSTIC_COMPLETE state */
         nErrCode = DSL_DRV_VNX_LoopDiagnosticCompleteHandle(pContext);
         break;
      case DSL_LINESTATE_IDLE:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: IDLE state reached" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         break;
      case DSL_LINESTATE_TEST:
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: TEST state reached"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         if( nTestMode == DSL_TESTMODE_QUIET)
         {
            /* Switch Off Tx Signals*/
            nErrCode = DSL_DRV_VNX_SendMsgTxControlSet(pContext, CMD_TEST_TxControlSet_SEND_QUIET);
            if( nErrCode!=DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Tx Control config write failed!" DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
               return nErrCode;
            }
         }
         break;
      default:
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: Unexpected line state (%X) while training" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nLineState));
         nErrCode = DSL_ERROR;
         break;
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleTraining, errCode(%d)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_AutobootHandleException(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_VNX_FwVersion_t  sFwVersion;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleException"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Update Power Management Status*/
   nErrCode = DSL_DRV_VNX_PowerManagementStatusSet(pContext, DSL_G997_PMS_L3);
   if(nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Power Management State set failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
     return nErrCode;
   }

   /* Get Modem Fail Reason*/
   nErrCode = DSL_DRV_VNX_FailReasonGet( pContext );
   if( nErrCode!= DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_WRN,
         (pContext, "DSL[%02d]: Can't get modem Fail reason"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   /* Request the handshake information about the far-end operating mode*/
   nErrCode = DSL_DRV_VNX_SendMsgSpar1Get(pContext);
   if( nErrCode!= DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - SPAR1 get failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Handle exception for CAM*/
   nErrCode = DSL_DRV_VNX_CamExceptionHandle(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - CAM exception handle failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Get FW info*/
   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);

   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      nErrCode = DSL_DRV_VNX_SendMsgDataPathFailuresStatusGet(pContext);
      if(nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Data Path Failures update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
        return nErrCode;
      }
   }

   /* Update Line failures information*/
   nErrCode = DSL_DRV_VNX_LineFailuresStatusUpdate(pContext);

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleException, retCode(%d)"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_LinkReset(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_LinkReset"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   nErrCode = DSL_DRV_VNX_SendMsgModemStateSet(
             pContext, CMD_ModemFSM_StateSet_LINKRES);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - FSM state set failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

    /* Poll for the modem RESET state at least 4 sec*/
   nErrCode = DSL_DRV_VNX_PollDrvForModemResetState(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - RESET state poll failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Update Power Management Status*/
   nErrCode = DSL_DRV_VNX_PowerManagementStatusSet(pContext, DSL_G997_PMS_L3);
   if(nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Power Management State set failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
     return nErrCode;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_LinkReset"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_FirmwareVersionGet(
   DSL_Context_t *pContext,
   DSL_char_t *pString)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_boolean_t bFwReady = DSL_FALSE;
   DSL_uint32_t verNum = 0;
   DSL_VNX_FwVersion_t sFwVersion;

   DSL_CHECK_POINTER(pContext, pString);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->lowHandle);
   DSL_CHECK_ERR_CODE();

   /* Get FW DL information*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, bFwReady);

   if( nErrCode == DSL_SUCCESS && bFwReady )
   {
      nErrCode = DSL_DRV_VNX_SendMsgFwVerInfoGet(pContext, &verNum);
      if( nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - FW version get failed, msg send error!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return DSL_ERROR;
      }

      DSL_DRV_VNX_FirmwareVersionDecode(pContext, verNum, &sFwVersion);

      DSL_DRV_snprintf(pString, MAX_INFO_STRING_LEN,
         "%d.%d.%d.%d.%d.%d",
         sFwVersion.nPlatform,
         sFwVersion.nFeatureSet,
         sFwVersion.nMajor,
         sFwVersion.nMinor,
         sFwVersion.nReleaseStatus,
         sFwVersion.nApplication);

      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: Firmware version: %s (plt.fts.maj.min.rel.app)"
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext), pString));
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - FW version get failed, no FW loaded!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   return DSL_SUCCESS;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_ChipHardwareVersionGet(
   DSL_Context_t *pContext,
   DSL_char_t *pString)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_boolean_t bFwReady = DSL_FALSE;
   DSL_uint32_t verNum = 0;

   DSL_CHECK_POINTER(pContext, pString);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->lowHandle);
   DSL_CHECK_ERR_CODE();

   /* Get FW DL information*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, bFwReady);

   if( nErrCode == DSL_SUCCESS && bFwReady )
   {
      nErrCode = DSL_DRV_VNX_SendMsgHwVerInfoGet(pContext, &verNum);
      if( nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - HW version get failed, msg send error!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         return DSL_ERROR;
      }

      DSL_DRV_VNX_HardwareVersionDecode(pContext, verNum, pString);

      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: Hardware version: %s" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), pString));
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - HW version get failed, no FW loaded!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   return DSL_SUCCESS;
}


/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_DriverVersionGet(
   DSL_Context_t *pContext,
   DSL_char_t *pString)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_Handle_t dev;
   IOCTL_VINAX_drvVersion_t drvVersion;
   DSL_char_t Version[80]="";

   DSL_CHECK_POINTER(pContext, pString);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->lowHandle);
   DSL_CHECK_ERR_CODE();

   if( nErrCode == DSL_SUCCESS )
   {

      /* Get device handle*/
      dev = DSL_DEVICE_LOWHANDLE(pContext);

      drvVersion.pVersionStr = Version;
      drvVersion.strSize = sizeof(Version);

      /* Request the driver version*/
      if ( VINAX_InternalDrvVersionGet((VINAX_DYN_CNTRL_T*)dev, &drvVersion) != 0 )
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: VINAX driver version get failed, error %d" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), drvVersion.ictl.retCode));
            
         return DSL_ERROR;
      }

      memcpy((void*)pString, (void*)&Version[0], MAX_INFO_STRING_LEN);

      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: Device Driver version: %s" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), pString));
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Driver version get failed, no FW loaded!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   return DSL_SUCCESS;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_LineStateGet(
   DSL_Context_t *pContext,
   DSL_LineStateValue_t *pnLineState)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_boolean_t bFwReady = DSL_FALSE, bLxStateValid = DSL_FALSE;
   DSL_G997_PowerManagement_t nPMMode = DSL_G997_PMS_NA;
   DSL_VNX_FwVersion_t sFwVersion;
   CMD_ModemFSM_StateGet_t sCmd;
   ACK_ModemFSM_StateGet_t sAck;

   DSL_CHECK_POINTER(pContext, pnLineState);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx->lowHandle);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DEV_LineStateGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get FW DL information*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, bFwReady);

   if( bFwReady )
   {
      /* fill up the message to be sent */
      memset(&sCmd, 0, sizeof(sCmd));
      /*
      Different lengths! Parameter 3 of the ACK is available only in the ADSL-CPE
      starting with 9.3.0.15.1.5 (Annex A) & 9.5.0.1.1.6 (Annex B) and must not be
      requested otherwise. ADSL-CO and VDSL will add it later (check release
      notes).
      */
      DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.fwFeatures, sFwVersion);
      
      if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) && 
         ((sFwVersion.nFeatureSet >= 3) || 
         ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor > 0)) ||
         ((sFwVersion.nFeatureSet == 3) && (sFwVersion.nMajor == 0) && (sFwVersion.nMinor >= 15))))
      {
         sCmd.Length = 0x2;
         bLxStateValid = DSL_TRUE;
      }
      else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB) &&
              (sFwVersion.nFeatureSet >= 5) )
      {
         sCmd.Length = 0x2;
         bLxStateValid = DSL_TRUE;
      }
      else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      {
         /* Not supported in VDSL mode yet*/
         sCmd.Length = 0x1;
      }
      else
      {
         sCmd.Length = 0x1;
      }

      nErrCode = DSL_DRV_VNX_SendMessage( pContext,
                     CMD_MODEMFSM_STATEGET,
                     sizeof(sCmd), (DSL_uint8_t*)&sCmd,
                     sizeof(ACK_ModemFSM_StateGet_t), (DSL_uint8_t*)&sAck );
     
      if(nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_MSG, (pContext,
            "DSL[%02d]: ERROR - Modem FSM state get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return DSL_ERROR;
      }

      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: DSL_DEV_LineStateGet: Got %hu state" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), sAck.ModemState));

      /* Map states to internal*/
      switch(sAck.ModemState)
      {
      case ACK_ModemFSM_StateGet_RESET_STATE:
         *pnLineState = DSL_LINESTATE_IDLE;
         break;
      case ACK_ModemFSM_StateGet_READY_STATE:
         *pnLineState = DSL_LINESTATE_SILENT;
         break;
      case ACK_ModemFSM_StateGet_FAIL_STATE:
         *pnLineState = DSL_LINESTATE_EXCEPTION;
         break;
      case ACK_ModemFSM_StateGet_DIAG_COMPLETE_STATE:
         *pnLineState = DSL_LINESTATE_LOOPDIAGNOSTIC_COMPLETE;
         break;
      case ACK_ModemFSM_StateGet_GHS_STATE:
         *pnLineState = DSL_LINESTATE_HANDSHAKE;
         break;
      case ACK_ModemFSM_StateGet_FULLINIT_STATE:
         *pnLineState = DSL_LINESTATE_FULL_INIT;
         break;
      case ACK_ModemFSM_StateGet_STEADY_STATE_TC_NOSYNC:
         *pnLineState = DSL_LINESTATE_SHOWTIME_NO_SYNC;
         break;
      case ACK_ModemFSM_StateGet_STEADY_STATE_TC_SYNC:
         *pnLineState = DSL_LINESTATE_SHOWTIME_TC_SYNC;
         break;
      case ACK_ModemFSM_StateGet_DIAGMODE_STATE:
         *pnLineState = DSL_LINESTATE_LOOPDIAGNOSTIC_ACTIVE;
         break;
      case ACK_ModemFSM_StateGet_TEST_STATE:
         *pnLineState = DSL_LINESTATE_TEST;
         break;
      default:
         DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: DSL_DEV_LineStateGet: "
         "DSL_LINESTATE_UNKNOWN will be returned" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
         *pnLineState = DSL_LINESTATE_UNKNOWN;
         
         break;
      }

      /* Check if the LxState is available from the FW*/
      if (bLxStateValid)
      {
         switch(sAck.LxState)
         {
         case ACK_ModemFSM_StateGet_L0:
            nPMMode = DSL_G997_PMS_L0;
            break;
         case ACK_ModemFSM_StateGet_L2:
            nPMMode = DSL_G997_PMS_L2;
            break;
         case ACK_ModemFSM_StateGet_L3:
            nPMMode = DSL_G997_PMS_L3;
            break;
         default:
            DSL_DEBUG( DSL_DBG_MSG,
               (pContext, "DSL[%02d]: ERROR - Unknown LxState=%d received!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), sAck.LxState));
            nErrCode = DSL_ERROR;
            break;
         }

         if (nErrCode == DSL_SUCCESS)
         {
            /* Update Power Management Status*/
            nErrCode = DSL_DRV_VNX_PowerManagementStatusSet(pContext, nPMMode);
            if(nErrCode != DSL_SUCCESS)
            {
               DSL_DEBUG( DSL_DBG_ERR, (pContext,
                  "DSL[%02d]: ERROR - Power Management State set failed!" DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
            }
         }
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Line state get failed, no FW loaded!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_LineStateGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_ShowtimeStatusUpdate(
   DSL_Context_t *pContext,
   DSL_boolean_t bInit)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   ACK_TestParamsAuxDS_Get_t  sAck;
   DSL_uint32_t nEapsTimeout = 0, nEapsTimeoutId;
   
   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DEV_ShowtimeStatusUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Update Channel status information*/
   nErrCode = DSL_DRV_VNX_ChannelStatusUpdate(pContext, 0, DSL_DOWNSTREAM);
   if(nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Channel status DS update failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   nErrCode = DSL_DRV_VNX_ChannelStatusUpdate(pContext, 0, DSL_UPSTREAM);
   if(nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Channel status US update failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   if( bInit )
   {
      /* Set the showtime indication flag*/
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, bShowtimeReached, DSL_TRUE);

      /* EAPS timeout value*/
      DSL_CTX_READ_SCALAR(
         pContext, nErrCode, pDevCtx->data.deviceCfg.cfg.nEapsTimeout, nEapsTimeout);
   
      /* Add EAPS timeout event.*/
      nEapsTimeoutId = DSL_DRV_Timeout_AddEvent(pContext, (DSL_int_t)DSL_TIMEOUTEVENT_EAPS, nEapsTimeout);
      /* Set EAPS timeout ID in the DSL CPE Context*/
      DSL_CTX_WRITE(pContext, nErrCode, nEapsTimeoutId, nEapsTimeoutId);
 
      /* Reset CAM state*/
      nErrCode = DSL_DRV_VNX_CamFsmStateSet(pContext, DSL_CAM_INIT);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - CAM FSM state set failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   
      /* Get Near-End inventory information*/
      nErrCode = DSL_DRV_VNX_NeInventoryGet(pContext);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - NE inventory get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
          return nErrCode;
      }

      /* Update XTSE status information*/
      nErrCode = DSL_DRV_VNX_SendMsgXtseStatusGet( pContext );
      if(nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - XTSE status get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
        return nErrCode;
      }

      if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
      {
         nErrCode = DSL_DRV_VNX_SendMsgSelectedProfileVdsl2Get(pContext);
         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR, (pContext,
               "DSL[%02d]: ERROR - Selected Profile Vdsl2 Get failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return nErrCode;
         }
      }

      /* Update Power Management Status*/
      nErrCode = DSL_DRV_VNX_PowerManagementStatusSet(pContext, DSL_G997_PMS_L0);
      if(nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Power Management State set failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
        return nErrCode;
      }

      if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
      {
#ifndef DSL_CPE_STATIC_DELT_DATA
         if (pContext->DELT_SHOWTIME == DSL_NULL)
         {
            pContext->DELT_SHOWTIME = DSL_DRV_Malloc(sizeof(DSL_G997_DeltShowtimeData_t));
            if( pContext->DELT_SHOWTIME == DSL_NULL )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - memory for DELT SHOWTIME data "
                  "could not be allocated!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                  
               return nErrCode;
            }
         }
#endif /* #ifndef DSL_CPE_STATIC_DELT_DATA*/

         memset(pContext->DELT_SHOWTIME, 0x0, sizeof(DSL_G997_DeltShowtimeData_t));

         /* Get FE Aux Test parameters here only for the VDSL US*/
         nErrCode = DSL_DRV_VNX_SendMsgTestParamsAuxGet(pContext, DSL_UPSTREAM, (DSL_uint8_t*)&sAck);
         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG(DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - DELT Aux US data get failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return DSL_ERROR;
         }

         /* Update VDSL FE group size parameter*/
         pContext->DELT_SHOWTIME->hlogDataUsVdsl.nGroupSize = sAck.HLOGG;
         pContext->DELT_SHOWTIME->qlnDataUsVdsl.nGroupSize  = sAck.QLNG;
         pContext->DELT_SHOWTIME->snrDataUsVdsl.nGroupSize  = sAck.SNRG;
           
         /* Request firts portion of DELT data from the FE*/
         nErrCode = DSL_DRV_VNX_DeltArrayParamsQuery(pContext, 0, 31);
      }
   }
   else
   {
      nErrCode = DSL_DRV_VNX_SendMsgDataPathFailuresStatusGet(pContext);
      if(nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Data Path Failures update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
        return nErrCode;
      }

      /* Update Line failures information*/
      nErrCode = DSL_DRV_VNX_LineFailuresStatusUpdate(pContext);
      if(nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Line Failures update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
        return nErrCode;
      }

      /* Handle VINAX driver autonomous messages*/
      nErrCode = DSL_DRV_VNX_HandleMessage(pContext);
   }


   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_ShowtimeStatusUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   VINAX Device specific implementation of the DSL_DRV_OnTimeoutEvent() function
*/
DSL_Error_t DSL_DRV_DEV_OnTimeoutEvent(
   DSL_Context_t *pContext,
   DSL_int_t nEventType,
   DSL_uint32_t nTimeoutID)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,(pContext, "DSL[%02d]: DSL_DRV_VNX_OnTimeoutEvent: "
      "nEventType=%d, nTimeoutID=0x%08X" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nEventType, nTimeoutID));

   switch (nEventType)
   {
   case DSL_TIMEOUTEVENT_FE_STATUS:
      nErrCode = DSL_DRV_VNX_OnLineInventoryFe(pContext);
      break;
   case DSL_TIMEOUTEVENT_EAPS:
      nErrCode = DSL_DRV_VNX_OnEapsTimeout(pContext);
      break;
   default:
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: Unknown nEventType (%d)" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nEventType));
      break;
   }

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'lib_dsl_cpe_api.h'
*/
DSL_Error_t DSL_DRV_DEV_DBG_DeviceMessageSend(
   DSL_Context_t *pContext,
   DSL_DeviceMessageData_t *pMsg )
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   IOCTL_VINAX_messageSend_t sRw;

   if (pMsg->nSizeTx < 6)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - The device message has to consist of ID, "
          "Index and Length minimum!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
          
      nErrCode = DSL_ERROR;

      return (nErrCode);
   }
   if (pMsg->nSizeRx < 6)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - The device Acknowledge has to keep ID, "
          "Index and Length minimum!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
          
      nErrCode = DSL_ERROR;

      return (nErrCode);
   }

   sRw.write_msg.msgId          = *(DSL_uint16_t*)pMsg->pMsg;
   sRw.write_msg.pPayload       = pMsg->pMsg+2;
   sRw.write_msg.paylSize_byte  = pMsg->nSizeTx-2;
   /* to receive the answer hand over the payload buffer */
   sRw.ack_msg.pPayload         = pMsg->pMsg+2;
   sRw.ack_msg.paylSize_byte    = pMsg->nSizeRx-2;

   nErrCode = DSL_DRV_VNX_ChRequestMessage(pContext, &sRw);
   if (nErrCode < 0)
   {
      return (DSL_ERROR);
   }
   else
   {
      *(DSL_uint16_t*)pMsg->pMsg = sRw.ack_msg.msgId;
      pMsg->nSizeRx = sRw.ack_msg.paylSize_byte+2;
   }

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_LED_FirmwareInit(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_ERR_NOT_IMPLEMENTED;

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_SystemInterfaceConfigSet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_SystemInterfaceConfig_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_SystemInterfaceConfigSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Copy System Configuration data to internal VINAX device context*/
   DSL_CTX_WRITE(pContext, nErrCode, pDevCtx->data.deviceCfg.sysCIF, pData->data);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_SystemInterfaceConfigSet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_SystemInterfaceConfigGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_SystemInterfaceConfig_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_SystemInterfaceConfigGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get System Configuration from the internal VINAX device context*/
   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.deviceCfg.sysCIF, pData->data);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_SystemInterfaceConfigGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_SystemInterfaceStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_SystemInterfaceStatus_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_SystemInterfaceStatusGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get System Configuration from the internal VINAX device context.
      KAv: Configuration request temporary added. Should be changed later to the
           status information.*/
   DSL_CTX_READ(pContext, nErrCode, pDevCtx->data.deviceCfg.sysCIF, pData->data);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_SystemInterfaceStatusGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

DSL_Error_t DSL_DRV_DEV_TestModeControlSet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_TestModeControl_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_TestModeControlSet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
      DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      if (pData->data.nTestMode == DSL_TESTMODE_SHOWTIME_LOCK ||
          pData->data.nTestMode == DSL_TESTMODE_QUIET ||
          pData->data.nTestMode == DSL_TESTMODE_TRAINING_LOCK)
      {
         nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
      }
   }

   if (nErrCode == DSL_SUCCESS)
   {
      DSL_CTX_WRITE(pContext, nErrCode, nTestModeControl, pData->data.nTestMode);
   }
     
   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_TestModeControlSet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

#endif /* #ifdef INCLUDE_DSL_CPE_API_VINAX*/
