/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

#define DSL_INTERN
/* lower level BSP driver stuff */
#include "drv_dsl_cpe_api.h"
#if defined (INCLUDE_DSL_CPE_API_DANUBE) || defined (INCLUDE_DSL_CPE_API_AMAZON_SE) || \
    defined (INCLUDE_DSL_CPE_API_AMAZON_S)

#include "drv_dsl_cpe_device_g997.h"
#include "drv_dsl_cpe_device_danube.h"

#undef DSL_DBG_BLOCK
#define DSL_DBG_BLOCK DSL_DBG_DEVICE


/* Blacklist for Danube message dump */
DSL_DEV_MsgIdRange_t g_DANUBE_MsgDumpBlacklist[]=
{
   /* GroupId              First Adr   Last Adr */
   { DSL_CMV_GROUP_CNTL,        2,          2     },
   { DSL_CMV_GROUP_STAT,        0,          1     },
   { DSL_CMV_GROUP_STAT,       14,         18     },
   { DSL_CMV_GROUP_INFO,       30,         33     },
   { DSL_CMV_GROUP_INFO,       81,         83     },
   { DSL_CMV_GROUP_RATE,        0,          1     },
   { DSL_CMV_GROUP_PLAM,        0,         19     },
   { DSL_CMV_GROUP_PLAM,       24,         28     },
   { DSL_CMV_GROUP_PLAM,       33,         37     },
   { DSL_CMV_GROUP_PLAM,       42,         45     },
   { DSL_CMV_GROUP_CNFG,       12,         28     },
   { -1,                   0xFFFF,     0xFFFF     }
};

#ifdef INCLUDE_CALLBACK_SUPPORT
static DSL_int_t DSL_DRV_HandleEventCallback
(
   DSL_DEV_Device_t *pDev,
   DSL_BSP_CB_Type_t nCallbackType,
   DSL_BSP_CB_DATA_Union_t *pData
);
#endif /* #ifdef INCLUDE_CALLBACK_SUPPORT */

static DSL_Error_t DSL_DRV_DANUBE_ChannelRateUpdate(
   DSL_Context_t *pContext);

static DSL_Error_t DSL_DRV_DANUBE_ModemReset(
   DSL_Context_t *pContext);

static DSL_Error_t DSL_DRV_DANUBE_LineFeaturesUpdate(
   DSL_Context_t *pContext);

/* This routine sends ATSE configuration to the FW */
static DSL_uint32_t DSL_DRV_DANUBE_XTUSystemEnablingConfigSend
(
   DSL_IN DSL_Context_t *pContext
);

/*
   This function checks the System Interface Configuration parameters
   against supported by firmware
*/
static DSL_Error_t DSL_DRV_DANUBE_SystemInterfaceConfigCheck
(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_SystemInterfaceConfigData_t *pData
);

/* This routine retreives current mode from the the FW and
    stores it in the DSL Context structure */
static DSL_uint32_t DSL_DRV_DANUBE_XTUSystemEnablingStatusUpdate
(
   DSL_IN_OUT DSL_Context_t *pContext
);

/* This routine retreives current SNRM and
    stores it in the DSL Context structure */
static DSL_Error_t DSL_DRV_DANUBE_SnrmDsUpdate
(
   DSL_IN_OUT DSL_Context_t *pContext
);

/* This routine retreives current failure state and
    stores it in the DSL Context structure */
static DSL_Error_t DSL_DRV_DANUBE_LineFailuresUpdate
(
   DSL_IN_OUT DSL_Context_t *pContext
);

/* This function should be called when line is reached showtime state */
DSL_Error_t DSL_DRV_DANUBE_ShowtimeReached
(
   DSL_Context_t *pContext
);

/*
   This routine starts a DSL modem

   \param pContext
      pointer to the DSL context
*/
static DSL_Error_t DSL_DRV_DANUBE_ModemStart
(
   DSL_Context_t *pContext
);

/*
   This routine updates the version information for currently used
   - chip
   - low level MEI BSP driver
   and stores it within device specific context.

   \param pContext
      pointer to the DSL context
*/
DSL_Error_t DSL_DRV_DANUBE_LowLevelVersionsUpdate
(
   DSL_Context_t *pContext
);

/*
   This routine updates the version information for currently used firmware
   within device specific context.

   \param pContext
      pointer to the DSL context
*/
static DSL_Error_t DSL_DRV_DANUBE_FirmwareVersionUpdate
(
   DSL_Context_t *pContext
);

/*
   This routine writes common firmware configurations that has to be done
   between firmware download and link start.
   \attention All of this configurations will be written to the software
              independently of the autoboot thread state, means also if
              autoboot thread is disabled by using 'acs 0'.

   \param pContext
      pointer to the DSL context
*/
static DSL_Error_t DSL_DRV_DANUBE_FirmwareInit
(
   DSL_Context_t *pContext
);

/**
   Checks the currently used firmware version against the given one and returns
   the status.

   \param pContext
      pointer to the DSL context
   \param nFS
      Feature Set (major version) of  firmware to check
   \param nExtVer
      External version of firmware version to check. In case of giving -1 this
      value will be not processed
   \param nIntVer
      Internal version of firmware version to check. In case of giving -1 this
      value will be not processed
   \param nApp
      Application version of firmware version to check. In case of giving -1
      this value will be not processed
   \param pVerCheck
      Returns the status of the version comparison

   \return
   Return values are defined within the DSL_Error_t definition
   - DSL_SUCCESS in case of success
   - DSL_ERROR if operation failed
*/

static DSL_Error_t DSL_DRV_DANUBE_FirmwareVersionCheck
(
   DSL_Context_t *pContext,
   DSL_int_t nMajVer,
   DSL_int_t nMinVer,
   DSL_int_t nExtVer,
   DSL_int_t nIntVer,
   DSL_FirmwareAnnex_t nApp,
   DSL_DEV_VersionCheck_t *pVerCheck
);

/*
 * Reset the ARC.
 * This function resets the ARC.
 *
 * \return  DSL_SUCCESS or DSL_ERROR
 * \ingroup Internal
 */
static DSL_Error_t DSL_DRV_DANUBE_ResetDFE
(
   DSL_Context_t *pContext
);

/*
 * Check the L3 status
 * This function get the CPE Power Management Mode status
 * \return see DSL_G997_PowerManagement_t for possible values
 *    0: L0 Mode
 *    2: L2 Mode
 *    3: L3 Mode
 * \ingroup Internal
 */
static DSL_Error_t DSL_DRV_DANUBE_L3StatusGet(
   DSL_Context_t *pContext,
   DSL_G997_PowerManagement_t *pnPowerMode
);

/*
 * Get the hdlc status
 *
 * \return  HDLC status
 * \ingroup Internal
 */
static DSL_Error_t DSL_DRV_DANUBE_HdlcStatusGet (
   DSL_Context_t *pContext,
   DSL_uint16_t *pnStatus
);

/*
 * Check if the me is resolved.
 *
 * \param   status      the me status
 * \return  ME_HDLC_UNRESOLVED or ME_HDLC_RESOLVED
 * \ingroup Internal
 */
static DSL_int_t DSL_DRV_DANUBE_HdlcResolvedGet(
   DSL_Context_t *pContext,
   DSL_uint16_t nStatus
);

/*
 * Halt the ARC.
 * This function halts the ARC.
 *
 * \return  DSL_SUCCESS or DSL_ERROR
 * \ingroup Internal
 */
static DSL_Error_t DSL_DRV_DANUBE_HaltDFE
(
   DSL_Context_t *pContext
);

static DSL_Error_t DSL_DRV_DANUBE_OnLineInventoryFe
(
   DSL_Context_t *pContext
);

static DSL_Error_t DSL_DRV_DANUBE_FailReasonGet
(
   DSL_Context_t *pContext
);

static const char *const gGroupName[] =
{
   "??", "CNTL", "STAT", "INFO", "TEST", "OPTN", "RATE", "PLAM", "CNFG"
};

DSL_Error_t DSL_DRV_DANUBE_FirmwareInit(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_CALLBACK_SUPPORT
   DSL_DEV_VersionCheck_t nFwVer = DSL_VERSION_ERROR;
#endif

#ifdef INCLUDE_CALLBACK_SUPPORT
   nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext, DSL_MIN_FW_VERSION_CB,
      DSL_FW_ANNEX_NA, &nFwVer);

   do
   {
      DSL_uint16_t nVal = 0;

      if (nErrCode < DSL_SUCCESS) break;

      if (DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Couldn't lock bsp mutex!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }

      else
      {
         if (nFwVer >= DSL_VERSION_EQUAL)
         {
            /* configure reboot interrupt handler */
            nVal = 15;
            if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
               DSL_CMV_ADDRESS_INFO_REBOOT_INT_CONFIG, 0, 1, &nVal) != DSL_SUCCESS)
            {
               DSL_DEBUG(DSL_DBG_ERR, (pContext,
                  "DSL[%02d]: ERROR - Failed to get FW version!" DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));

               nErrCode = DSL_ERR_MSG_EXCHANGE;
            }
         }
      }
   } while (0);

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
#endif /* INCLUDE_CALLBACK_SUPPORT */

#if defined (INCLUDE_DSL_CPE_API_AMAZON_SE) || defined (INCLUDE_DSL_CPE_API_AMAZON_S)
   /* Workaround for Amazon-SE firmware problem with device version 1.2 */
   do
   {
      DSL_ChipVersion_t *pVer = DSL_NULL;
      DSL_uint16_t nVal = 0;

      if (nErrCode < DSL_SUCCESS) break;

      if (DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Couldn't lock bsp mutex!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }

      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Couldn't lock data mutex!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }
      
      pVer = &(pContext->pDevCtx->data.version.chipVersion);
      if (pVer == DSL_NULL)
      {
         nErrCode = DSL_ERR_POINTER;
         break;
      }

      if (pVer->bValid == DSL_FALSE)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: No chip version available to handle A12 workaround!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_WRN_DEVICE_NO_DATA;
         
         break;
      }

      if (pVer->nMajorVersion != 1)
      {
         DSL_DEBUG(DSL_DBG_WRN, (pContext,
            "DSL[%02d]: A12 workaround not necessary." DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      if (pVer->nMinorVersion == 1)
      {
         nVal = 0x0;
      }
      else if (pVer->nMinorVersion == 2)
      {
         nVal = 0x1;
      }
      else
      {
         DSL_DEBUG(DSL_DBG_WRN, (pContext,
            "DSL[%02d]: A12 workaround not necessary." DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }
                  
      if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_SYSTEM_INTERFACE, 5, 1, &nVal) != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Failed to send CMV to activate A12 workaround!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

         nErrCode = DSL_ERR_MSG_EXCHANGE;
      }
   } while (0);

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
#endif

   return nErrCode;
}

DSL_Error_t DSL_DRV_DANUBE_FirmwareVersionCheck(
   DSL_Context_t *pContext,
   DSL_int_t nMajVer,
   DSL_int_t nMinVer,
   DSL_int_t nExtVer,
   DSL_int_t nIntVer,
   DSL_FirmwareAnnex_t nApp,
   DSL_DEV_VersionCheck_t *pVerCheck)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t nActUsedVer = 0, nVer = 0;
   DSL_FwVersion_t *pVer = DSL_NULL;

   DSL_CHECK_POINTER(pContext, pVerCheck);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DANUBE_FirmwareVersionCheck" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   *pVerCheck = DSL_VERSION_ERROR;

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_ERROR;
   }

   do
   {
      if (nErrCode < DSL_SUCCESS) break;

      pVer = &(pContext->pDevCtx->data.version.fwVersion);
      if (pVer == DSL_NULL)
      {
         nErrCode = DSL_ERR_POINTER;
         break;
      }

      if (pVer->bValid == DSL_FALSE)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Firmware version check not possible" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         nErrCode = DSL_ERR_DEVICE_NO_DATA;
         break;
      }
            
      /* First check if the Major Version (device) and application types (Annex)
         matches or not */
      if ((nApp != -1 && pVer->nApplicationType != nApp) || (pVer->nMajorVersion != nMajVer))
      {
         *pVerCheck = DSL_VERSION_MISMATCH;
         break;
      }

      /* Build 32-bit value for given firmware version (nVer) and
         for currently used firmware version (nActUsedVer) */
      nVer = ((DSL_uint32_t)nMinVer) << 16;
      nActUsedVer = ((DSL_uint32_t)(pVer->nMinorVersion)) << 16;
      if (nExtVer != -1)
      {
         nVer |= ((DSL_uint32_t)nExtVer) << 8;
         nActUsedVer |= ((DSL_uint32_t)pVer->nExternalVersion) << 8;
      }
      if (nIntVer != -1)
      {
         nVer |= (DSL_uint32_t)nIntVer;
         nActUsedVer |= (DSL_uint32_t)pVer->nInternalVersion;
      }

      /* Check both generated 32-bit values */
      if (nActUsedVer < nVer)
      {
         *pVerCheck = DSL_VERSION_SMALLER;
      }
      else if (nActUsedVer > nVer)
      {
         *pVerCheck = DSL_VERSION_BIGGER;
      }
      else
      {
         *pVerCheck = DSL_VERSION_EQUAL;
      }
   } while (0);

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DANUBE_FirmwareVersionCheck" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_DEV_Handle_t DSL_DRV_DEV_DriverHandleGet(DSL_int_t nMaj, DSL_int_t nNum)
{
#ifdef INCLUDE_CALLBACK_SUPPORT
   DSL_BSP_EventCallBack_t evtCB;
   DSL_int_t i = 0;
#endif

   DSL_DEV_Handle_t tmp = DSL_BSP_DriverHandleGet(nMaj, nNum);

#ifdef INCLUDE_CALLBACK_SUPPORT
   /* Temporarily because of neccessary clarification within low level driver */
   memset (&evtCB, 0, sizeof(DSL_BSP_EventCallBack_t));
   evtCB.function = DSL_DRV_HandleEventCallback;
   evtCB.pData = DSL_NULL;

   for (i = (DSL_BSP_CB_FIRST + 1); i < DSL_BSP_CB_LAST; i++)
   {
      evtCB.event = i;
      if (DSL_BSP_EventCBRegister(&evtCB) != 0)
      {
         DSL_DEBUG(DSL_DBG_ERR, (DSL_NULL,
            "DSL[%02d]: DSL_BSP_DriverHandleGet: Regiser Event Callback (Type=%d) failed!"
            DSL_DRV_CRLF, nNum,i));
      }
      else
      {
         DSL_DEBUG(DSL_DBG_MSG, (DSL_NULL, "DSL[%02d]: DSL_BSP_DriverHandleGet: Regiser "
            "Event Callback (Type=%d) successful"DSL_DRV_CRLF, nNum, i));
      }
   }
#endif /* INCLUDE_CALLBACK_SUPPORT */

   return tmp;
}

DSL_Error_t DSL_DRV_DEV_DriverHandleDelete(
   DSL_DEV_Handle_t handle)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_CALLBACK_SUPPORT
   DSL_BSP_EventCallBack_t evtCB;
   DSL_int_t i = 0;

   /* Temporarily because of neccessary clarification within low level driver */
   memset (&evtCB, 0, sizeof(DSL_BSP_EventCallBack_t));
   evtCB.function = DSL_NULL;
   evtCB.pData = DSL_NULL;

   for (i = (DSL_BSP_CB_FIRST + 1); i < DSL_BSP_CB_LAST; i++)
   {
      evtCB.event = i;
      if (DSL_BSP_EventCBUnregister(&evtCB) != 0)
      {
         DSL_DEBUG(DSL_DBG_ERR, (DSL_NULL,
            "DSL_DEV_DriverHandleDelete: Unregiser Event Callback (Type=%d) Fail!"
            DSL_DRV_CRLF, i));

         nErrCode = DSL_ERROR;
      }
      else
      {
         DSL_DEBUG(DSL_DBG_MSG, (DSL_NULL, "DSL_DEV_DriverHandleDelete: Event"
            "Callback (Type=%d) unregistered successfully." DSL_DRV_CRLF,
            i, ifxDevices[i].lowHandle));
      }
   }
#endif /* INCLUDE_CALLBACK_SUPPORT */

   DSL_BSP_DriverHandleDelete(handle);
   return nErrCode;
}

#ifdef INCLUDE_CALLBACK_SUPPORT
/* handles events from the low level driver, it is called "new style" in order
   to split it from existing callback handling, later it should be renamed */
static DSL_int_t DSL_DRV_HandleEventCallback(
   DSL_DEV_Device_t *pDev,
   DSL_BSP_CB_Type_t nCallbackType,
   DSL_BSP_CB_DATA_Union_t *pData)
{
   DSL_int_t i = 0;
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal;

   if (pDev == DSL_NULL)
   {
      return -EIO;
   }

   if (ifxDevices[i].pContext != DSL_NULL)
   {
      DSL_DEBUG(DSL_DBG_MSG, (ifxDevices[i].pContext,
         "DSL[%02d]: DSL_DRV_HandleEventCallback: Callback event (type=%d) occured" DSL_DRV_CRLF,
         DSL_DEV_NUM(ifxDevices[i].pContext), nCallbackType));
   }

   switch (nCallbackType)
   {
   case DSL_BSP_CB_DYING_GASP:
      /* go through all devices we registered and determine for which of
         them an event has been occured */
      for (i = 0; i < DSL_MAX_DEVICE_INSTANCES; i++)
      {
         if (ifxDevices[i].lowHandle == pDev)
         {
            DSL_DEBUG(DSL_DBG_ERR, (ifxDevices[i].pContext,
               "DSL[%02d]: DSL_DRV_HandleEventCallback: Got Dying Gasp interrupt"
               DSL_DRV_CRLF, DSL_DEV_NUM(ifxDevices[i].pContext)));
         }
      }
      break;
#if defined(INCLUDE_DSL_CEOC)
   case DSL_BSP_CB_CEOC_IRQ:
      /* go through all devices we registered and determine for which of
         them an event has been occured */
      for (i = 0; i < DSL_MAX_DEVICE_INSTANCES; i++)
      {
         if (ifxDevices[i].lowHandle == pDev)
         {
            DSL_DEBUG(DSL_DBG_MSG, (ifxDevices[i].pContext,
               "DSL[%02d]: DSL_DRV_HandleEventCallback: Got CEOC interrupt" DSL_DRV_CRLF,
               DSL_DEV_NUM(ifxDevices[i].pContext)));

            nErrCode = DSL_CEOC_EventCB(ifxDevices[i].pContext);
         }
      }
      break;
#endif
   case DSL_BSP_CB_FIRMWARE_REBOOT:
      /* go through all devices we registered and determine for which of
         them an event has been occured */
      for (i = 0; i < DSL_MAX_DEVICE_INSTANCES; i++)
      {
         if (ifxDevices[i].lowHandle == pDev)
         {
               /* assume that if this interrupt has occured then it is supported */
               if (DSL_DRV_MUTEX_LOCK(ifxDevices[i].pContext->bspMutex))
               {
                  DSL_DEBUG(DSL_DBG_ERR, (ifxDevices[i].pContext,
                     DSL_DRV_CRLF"DSL[%02d]: ERROR - getting mei driver semaphore failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(ifxDevices[i].pContext)));
                     
                  nErrCode = DSL_ERROR;
                  
                  break;
               }
               else
               {
                  /* read a cause of the interrupt */
                  if (DSL_DRV_DANUBE_CmvRead(ifxDevices[i].pContext,
                     DSL_CMV_GROUP_INFO, DSL_CMV_ADDRESS_INFO_REBOOT_INT_INFO,
                     0, 1, &nVal) != DSL_SUCCESS)
                  {
                     DSL_DEBUG(DSL_DBG_ERR, (ifxDevices[i].pContext,
                     "DSL[%02d]: DSL_DRV_HandleEventCallback: Failed to get firmware interrupt "
                        "cause!"DSL_DRV_CRLF, DSL_DEV_NUM(ifxDevices[i].pContext)));
                        
                     nErrCode = DSL_ERR_MSG_EXCHANGE;
                     
                     break;
                  }
                  else
                  {
                     /* do something */
                     if (nVal & 0x1)
                     {
                        DSL_DEBUG(DSL_DBG_ERR, (ifxDevices[i].pContext,
                        "DSL[%02d]: DSL_DRV_HandleEventCallback: firmware reboot cause: LOS"
                        DSL_DRV_CRLF, DSL_DEV_NUM(ifxDevices[i].pContext)));
                     }
                     if (nVal & 0x2)
                     {
                        DSL_DEBUG(DSL_DBG_ERR, (ifxDevices[i].pContext,
                        "DSL[%02d]: DSL_DRV_HandleEventCallback: firmware reboot cause: "
                        "Negative Margin " DSL_DRV_CRLF, DSL_DEV_NUM(ifxDevices[i].pContext)));
                     }
                     if (nVal & 0x4)
                     {
                        DSL_DEBUG(DSL_DBG_ERR, (ifxDevices[i].pContext,
                        "DSL[%02d]: DSL_DRV_HandleEventCallback: firmware reboot cause: "
                        "Margin below the minimum margin "DSL_DRV_CRLF, DSL_DEV_NUM(ifxDevices[i].pContext)));
                     }
                     if (nVal & 0x8)
                     {
                        DSL_DEBUG(DSL_DBG_ERR, (ifxDevices[i].pContext,
                        "DSL: DSL_DRV_HandleEventCallback: firmware reboot cause: "
                        "Failed during initialization"DSL_DRV_CRLF, DSL_DEV_NUM(ifxDevices[i].pContext)));
                     }

                     /* restart autoboot */
                     DSL_DRV_AutobootStateSet(ifxDevices[i].pContext,
                        DSL_AUTOBOOTSTATE_RESTART, DSL_AUTOBOOT_RESTART_POLL_TIME);
                  }
               }
               break;
            }
         }
      break;
   default:
      /* skip an event */
      DSL_DEBUG(DSL_DBG_ERR, (ifxDevices[i].pContext,
         "DSL[%02d]: DSL_DRV_HandleEventCallback: Unknown/unhandled event type (%d)" DSL_DRV_CRLF,
         DSL_DEV_NUM(ifxDevices[i].pContext), nCallbackType));
      break;
   }

   return (nErrCode == DSL_SUCCESS ? 0 : -EFAULT);
}
#endif /* INCLUDE_CALLBACK_SUPPORT */

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_ModemStart(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_ModemStart"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))  /* disable CMV access until ARC ready */
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if ((nErrCode = DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_DSL_START,
      DSL_NULL)) != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext, "DSL[%02d]: Running ADSL modem "
         "firmware fail!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }
   else
   {
      DSL_DEBUG( DSL_DBG_MSG, (pContext,
         "DSL[%02d]: ADSL firmware is running..." DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Modem couldn't be started!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_ModemStart"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DANUBE_LowLevelVersionsUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_Version_t *pVer = DSL_NULL;
   DSL_DEV_Handle_t dev = DSL_NULL;
   DSL_DEV_Version_t bspVersion = { 0 };
   DSL_DEV_HwVersion_t chipVersion = { 0 };

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DANUBE_LowLevelVersionsUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   do
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Couldn't lock device mutex!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ERROR;
      }

      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
           (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
           DSL_DEV_NUM(pContext)));
           
         DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
         
         return DSL_ERROR;
      }

      pVer = &(pContext->pDevCtx->data.version);
      if (pVer == DSL_NULL)
      {
         nErrCode = DSL_ERR_POINTER;
         break;
      }

      /* Get device handle*/
      dev = DSL_DEVICE_LOWHANDLE(pContext);
      if (dev == DSL_NULL)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - failed to get LL driver handle" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         nErrCode = DSL_ERR_LOWLEVEL_DRIVER_HANDLE;
         
         break;
      }

      if (DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_GET_VERSION,
         (DSL_uint32_t *)&bspVersion) < DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Failed to get BSP version!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         nErrCode = DSL_ERR_LOW_LEVEL_DRIVER_ACCESS;
         
         break;
      }

      pVer->meiSwVersion.nMajorVersion = (DSL_uint8_t)bspVersion.major;
      pVer->meiSwVersion.nMinorVersion = (DSL_uint8_t)bspVersion.minor;
      pVer->meiSwVersion.nBuild = (DSL_uint8_t)bspVersion.revision;
      pVer->meiSwVersion.bValid = DSL_TRUE;

      nErrCode = DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_GET_CHIP_INFO,
         (DSL_uint32_t *)&chipVersion);

      if (nErrCode < DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Failed to get chip version!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         nErrCode = DSL_ERR_LOW_LEVEL_DRIVER_ACCESS;
         
         break;
      }
      else if ((chipVersion.major == 0) && (chipVersion.minor == 0))
      {
         DSL_DEBUG(DSL_DBG_WRN, (pContext,
            "DSL[%02d]: Got no valid chip version from LL driver" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         nErrCode = DSL_WRN_DEVICE_NO_DATA;
         
         break;
      }

      pVer->chipVersion.nMajorVersion = (DSL_uint8_t)chipVersion.major;
      pVer->chipVersion.nMinorVersion = (DSL_uint8_t)chipVersion.minor;
      pVer->chipVersion.bValid = DSL_TRUE;

   } while (0);

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DANUBE_LowLevelVersionsUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DANUBE_FirmwareVersionUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_Version_t *pVer = DSL_NULL;
   DSL_uint16_t nValVer[2];

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DANUBE_FirmwareVersionUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   do
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Couldn't lock device mutex!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         nErrCode = DSL_ERROR;
         
         break;
      }

      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Couldn't lock data mutex!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         nErrCode = DSL_ERROR;
         
         break;
      }

      pVer = &(pContext->pDevCtx->data.version);
      if (pVer == DSL_NULL)
      {
         nErrCode = DSL_ERR_POINTER;
         break;
      }

      /* Read the version */
      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
               DSL_CMV_ADDRESS_INFO_VERSION, 0, 2, nValVer) != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Failed to get FW version!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         nErrCode = DSL_ERR_MSG_EXCHANGE;
         
         break;
      }

      /* Extract the version information from CMVs and store it within context */
      pVer->fwVersion.nMajorVersion = (DSL_uint8_t)(nValVer[0] & 0xFF);
      pVer->fwVersion.nMinorVersion = (DSL_uint8_t)((nValVer[0] >> 8) & 0xFF);
      pVer->fwVersion.nExternalVersion = (DSL_uint8_t)((nValVer[1] >> 4) & 0xF);
      pVer->fwVersion.nInternalVersion = (DSL_uint8_t)(nValVer[1] & 0xF);
      pVer->fwVersion.nReleaseState = (DSL_uint8_t)((nValVer[1] >> 14) & 0x3);
      pVer->fwVersion.nApplicationType = (DSL_uint8_t)((nValVer[1] >> 8) & 0x3F);

      pVer->fwVersion.bValid = DSL_TRUE;

   } while (0);

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DANUBE_FirmwareVersionUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DANUBE_UsedTonesNumberGet(
   DSL_Context_t *pContext,
   const DSL_AccessDir_t nDirection,
   DSL_uint16_t *pTonesNum)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t XTSE[DSL_G997_NUM_XTSE_OCTETS] = {0};

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_UsedTonesNumberGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   /*Get current xTSE octets*/
   DSL_CTX_READ(pContext, nErrCode, xtseCurr, XTSE);

   /*
   ADSL   Annex   US/DS
   1      A       0...32/0...256
   1      B       0...64/0...256
   2      A,I,L   0...32/0...256
   2      B,J     0...64/0...256
   2+     A,I,L   0...32/0...512
   2+     B,J,M   0...64/0...512
   */
   if (nDirection == DSL_DOWNSTREAM)
   {
      if ( (XTSE[6-1] & (XTSE_6_01_A_5_NO |  XTSE_6_03_B_5_NO | XTSE_6_07_I_5_NO)) ||
           (XTSE[7-1] & (XTSE_7_01_J_5_NO |  XTSE_7_03_M_5_NO)))
      {
         *pTonesNum = 512;
      }
      else if( (XTSE[1-1] & (XTSE_1_03_A_1_NO | XTSE_1_05_B_1_NO)) ||
               (XTSE[3-1] & (XTSE_3_03_A_3_NO |  XTSE_3_05_B_3_NO)) ||
               (XTSE[4-1] & (XTSE_4_05_I_3_NO |  XTSE_4_07_J_3_NO)) ||
               (XTSE[5-1] & (XTSE_5_03_L_3_NO |  XTSE_5_04_L_3_NO | XTSE_5_07_M_3_NO)) )
      {
         *pTonesNum = 256;
      }
      else
      {
         *pTonesNum = 0;         
          
         nErrCode = DSL_ERROR;
            
         DSL_DEBUG(DSL_DBG_ERR,
           (pContext, "DSL[%02d]: ERROR - XTSE current settings not detected!"DSL_DRV_CRLF,
           DSL_DEV_NUM(pContext)));
      }
   }
   else
   {
      if ( (XTSE[1-1] & (XTSE_1_05_B_1_NO)) ||
           (XTSE[3-1] & (XTSE_3_05_B_3_NO)) ||
           (XTSE[4-1] & (XTSE_4_07_J_3_NO)) ||
           (XTSE[6-1] & (XTSE_6_03_B_5_NO)) ||
           (XTSE[7-1] & (XTSE_7_01_J_5_NO | XTSE_7_03_M_5_NO)) )
      {
         *pTonesNum = 64;
      }
      else if( (XTSE[1-1] & (XTSE_1_03_A_1_NO)) ||
               (XTSE[3-1] & (XTSE_3_03_A_3_NO)) ||
               (XTSE[4-1] & (XTSE_4_05_I_3_NO )) ||
               (XTSE[5-1] & (XTSE_5_03_L_3_NO |  XTSE_5_04_L_3_NO)) ||
               (XTSE[6-1] & (XTSE_6_01_A_5_NO |  XTSE_6_07_I_5_NO)) )
      {
         *pTonesNum = 32;
      }
      else
      {
         *pTonesNum = 0;         
           
         nErrCode = DSL_ERROR;
            
         DSL_DEBUG(DSL_DBG_ERR,
           (pContext, "DSL[%02d]: ERROR - XTSE current settings not detected!"DSL_DRV_CRLF,
           DSL_DEV_NUM(pContext)));
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_UsedTonesNumberGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

DSL_Error_t DSL_DRV_DEV_Reboot(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_boolean_t bModemReady = DSL_FALSE;
   DSL_int32_t nWaitCount = 0;
#if defined(INCLUDE_DSL_PM)
   DSL_uint32_t fwDwnldStartTime = 0, fwDwnldStopTime = 0, fwDwnldTime = 0;
#endif

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DEV_Reboot"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, DSL_FALSE);

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

#if defined(INCLUDE_DSL_PM)
   fwDwnldStartTime = DSL_DRV_ElapsedTimeMSecGet(0);
#endif
   /* Reboot device*/
   if (DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_REBOOT, DSL_NULL) != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - device DSL_FIO_BSP_HALT failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }
   else
   {
      /* Start device*/
      nErrCode = DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_DSL_START, DSL_NULL);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - device DSL_FIO_BSP_START failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      }
   }
#if defined(INCLUDE_DSL_PM)
         fwDwnldStopTime = DSL_DRV_ElapsedTimeMSecGet(0);
         fwDwnldTime     = fwDwnldStartTime > fwDwnldStopTime ?
                           fwDwnldStopTime :
                           ((fwDwnldStopTime - fwDwnldStartTime)/1000);

         /* Set FW unavailable time*/
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nFwUnavailableTime, fwDwnldTime);
#endif

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - device reboot failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Poll for modem ready*/
   /* wait for Modem Ready state max 4 sec */
   for (nWaitCount = 0; nWaitCount < 40; nWaitCount++)
   {
      bModemReady = DSL_DRV_DEV_ModemIsReady(pContext);
      if (bModemReady)
      {
         break;
      }
      
      DSL_DRV_MSecSleep(100);
   }


   if (!bModemReady)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Modem not Ready!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }
   else
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, DSL_TRUE);
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DEV_Reboot"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_FwDownload(
   DSL_Context_t *pContext,
   const DSL_char_t *pFw1,
   DSL_uint32_t nSize1,
   const DSL_char_t *pFw2,
   DSL_uint32_t nSize2,
   DSL_int32_t *pLoff,
   DSL_int32_t *pCurrentOff,
   DSL_boolean_t bLastChunk)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#if defined(INCLUDE_DSL_PM)
   DSL_uint32_t fwDwnldStartTime = 0, fwDwnldStopTime = 0, fwDwnldTime = 0;
#endif
   DSL_FwDownloadStatusData_t fwDwnlStatus;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DEV_FwDownload"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_ERR_CODE();

   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, DSL_FALSE);
#if defined(INCLUDE_DSL_PM)
   fwDwnldStartTime = DSL_DRV_ElapsedTimeMSecGet(0);
#endif

   if (*pLoff == 0)
   {
      if ((nErrCode = DSL_DRV_DANUBE_ModemReset(pContext)) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Could not reset a modem!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      }
   }

   if (nErrCode == DSL_SUCCESS)
   {
      if (DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, DSL_DRV_CRLF"DSL[%02d]: ERROR - getting mei driver semaphore failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
      }
      else
      {
         if (pFw1 == DSL_NULL || pFw2 != DSL_NULL
              || nSize1 == 0 || nSize2 != 0
              || pLoff == DSL_NULL || pCurrentOff == DSL_NULL )
         {
            DSL_DEBUG(DSL_DBG_ERR, (DSL_NULL, "DSL[%02d]: wrong parameters is passed "
               "to DSL_DRV_DEV_FwDownload(%p %p %p %p %d %d)" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext) ,pFw1, pFw2,
               pLoff, pCurrentOff, nSize1, nSize2));
               
            nErrCode = DSL_ERR_INVALID_PARAMETR;
         }

         if (nErrCode == DSL_SUCCESS
            && DSL_BSP_FWDownload(pContext->pDevCtx->lowHandle, pFw1, nSize1,
            pLoff, pCurrentOff) != DSL_DEV_MEI_ERR_SUCCESS)
         {
            nErrCode = DSL_ERROR;
         }

         DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
      }
   }

   if (nErrCode == DSL_SUCCESS && bLastChunk != DSL_FALSE)
   {
      nErrCode = DSL_DRV_DANUBE_ModemStart(pContext);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - couldn't start modem!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      }
      else
      {
         DSL_DRV_DANUBE_FirmwareVersionUpdate(pContext);

#if defined(INCLUDE_DSL_PM)
         fwDwnldStopTime = DSL_DRV_ElapsedTimeMSecGet(0);;
         fwDwnldTime     = fwDwnldStartTime > fwDwnldStopTime ?
                           fwDwnldStopTime :
                           ((fwDwnldStopTime - fwDwnldStartTime)/1000);

         /* Set FW unavailable time*/
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nFwUnavailableTime, fwDwnldTime);
#endif

         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, pDevCtx->bFirmwareReady, DSL_TRUE);

         /* Fill DSL_EVENT_S_FIRMWARE_DOWNLOAD_STATUS event structure*/
         fwDwnlStatus.nError  = nErrCode == DSL_SUCCESS ? DSL_FW_LOAD_SUCCESS: DSL_FW_LOAD_ERROR;
         fwDwnlStatus.nFwType = DSL_FW_REQUEST_ADSL;

         if( DSL_DRV_EventGenerate( pContext, 0, DSL_ACCESSDIR_NA,
                DSL_XTUDIR_NA, DSL_EVENT_S_FIRMWARE_DOWNLOAD_STATUS,
               (DSL_EventData_Union_t*)&fwDwnlStatus, sizeof(DSL_FwDownloadStatusData_t)) 
             != DSL_SUCCESS )
         {
             DSL_DEBUG( DSL_DBG_ERR,
                (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), DSL_EVENT_S_FIRMWARE_DOWNLOAD_STATUS));
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DEV_FwDownload, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_boolean_t DSL_DRV_DEV_ModemIsReady(DSL_Context_t *pContext)
{
   DSL_uint32_t nModemReady = 0;
   DSL_boolean_t nRet = DSL_FALSE;
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   if (nErrCode != DSL_SUCCESS)
   {
      return DSL_FALSE;
   }

   if (DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_IS_MODEM_READY, &nModemReady) !=
      DSL_SUCCESS)
   {
      return DSL_FALSE;
   }

   if (nModemReady == 1)
   {
      nRet = DSL_TRUE;
   }
   else
   {
      nRet = DSL_FALSE;
   }

   return nRet;
}

DSL_Error_t DSL_DRV_DANUBE_ActLatencyGet(
   DSL_Context_t *pContext,
   DSL_uint16_t nChannel,
   DSL_LatencyPath_t *nLPath)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   if (nChannel >= DSL_CHANNELS_PER_LINE)
   {
      DSL_DEBUG (DSL_DBG_ERR, (pContext, "DSL[%02d]: DSL_DRV_DANUBE_ActLatencyGet - "
         "invalid channel (%d)!" DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nChannel));

      return DSL_ERROR;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLPath[nChannel], *nLPath);

   /* Do not process disabled or invalid states */
   switch (*nLPath)
   {
   case DSL_LATENCY_DISABLED:
      nErrCode = DSL_ERROR;
      break;
   case DSL_LATENCY_IP_LP0:
   case DSL_LATENCY_FP_LP1:
      break;
   default:
      nErrCode = DSL_ERROR;
      break;
   }

   return nErrCode;
}

static DSL_boolean_t DSL_DRV_DANUBE_CmvAccessTypeGet(DSL_uint16_t cmv_word0)
{
   DSL_boolean_t bWrite = DSL_FALSE;
  
   switch ( (cmv_word0 >> 4) & 0xFF )
   {
   case DSL_CMV_OPCODE_H2D_CMV_WRITE:
   case DSL_CMV_OPCODE_D2H_CMV_WRITE_REPLY:
   case DSL_CMV_OPCODE_H2D_DEBUG_WRITE_DM:
   case DSL_CMV_OPCODE_D2H_DEBUG_WRITE_DM_REPLY:
   case DSL_CMV_OPCODE_H2D_DEBUG_WRITE_PM:
   case DSL_CMV_OPCODE_D2H_DEBUG_WRITE_FM_REPLY:
      bWrite = DSL_TRUE;
      break;
   case DSL_CMV_OPCODE_H2D_CMV_READ:
   case DSL_CMV_OPCODE_D2H_CMV_READ_REPLY:
   case DSL_CMV_OPCODE_H2D_DEBUG_READ_DM:
   case DSL_CMV_OPCODE_D2H_DEBUG_READ_DM_REPLY:
   case DSL_CMV_OPCODE_H2D_DEBUG_READ_PM:
   case DSL_CMV_OPCODE_D2H_DEBUG_READ_FM_REPLY:
   default:
      break;
   }

   return bWrite;
}

static DSL_uint16_t DSL_DRV_DANUBE_CmvByteCountGet(DSL_uint16_t cmv_word0)
{
   DSL_uint16_t nSize = 0, nBitSize = 0, nCount = 0;

   nBitSize = cmv_word0 & 0xC000;
   nCount   = cmv_word0 & 0xF;

   switch (nBitSize)
   {
   case DSL_CPE_CMV_BYTE_SIZE:
      nSize = nCount + DSL_CMV_HEADER_LENGTH * 2;
      break;
   case DSL_CPE_CMV_DWORD_SIZE:
      nSize = nCount * 4 + DSL_CMV_HEADER_LENGTH * 2;
      break;
   case DSL_CPE_CMV_WORD_SIZE:
   default:
      nSize = nCount * 2 + DSL_CMV_HEADER_LENGTH * 2;   
      break;
   }

   return nSize;
}

/*
   Reads a CMV message.

   \param pContext Pointer to DSL CPE API context structure, [I]
   \param nGroup   Message group number, [I]
   \param nAddress Message address, [I]
   \param nIndex   Message index, [I]
   \param nSize    Number of words to read, [I]
   \param pData    Pointer to the data in case of writing a CMV. In case of
                   reading a CMV is shall be DSL_NULL, [I]
   \param pMsg     Pointer to the CMV data that has been returned by the
                   firmware (excluding header info), [O]

   \ingroup Internal
*/
DSL_Error_t DSL_DRV_DANUBE_CmvRead (
   DSL_Context_t *pContext,
   DSL_uint8_t nGroup,
   DSL_uint16_t nAddress,
   DSL_uint16_t nIndex,
   DSL_int_t nSize,
   DSL_uint16_t *pData)
{
   DSL_uint16_t RxMessage[DSL_MAX_CMV_MSG_LENGTH]__attribute__ ((aligned(4)));
   DSL_uint16_t TxMessage[DSL_MAX_CMV_MSG_LENGTH]__attribute__ ((aligned(4)));
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t i = 0;

   memset(&RxMessage, 0, sizeof(RxMessage));
   memset(&TxMessage, 0, sizeof(TxMessage));

   DSL_DRV_DANUBE_CmvPrepare(pContext, DSL_CMV_OPCODE_H2D_CMV_READ, nGroup,
      nAddress, nIndex, nSize, DSL_NULL, TxMessage);

   nErrCode = DSL_DRV_DANUBE_CmvSend(pContext, TxMessage, DSL_TRUE, RxMessage);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - CMV send failed, read %s %d %d!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), gGroupName[nGroup], nAddress, nIndex));
   }
   else
   {
      for (i = 0; i < nSize; i++)
      {
         *(pData+i) = RxMessage[i+4];
      }
   }

   return nErrCode;
}

/*
   Writes a CMV message.

   \param pContext Pointer to DSL CPE API context structure, [I]
   \param nGroup   Message group number, [I]
   \param nAddress Message address, [I]
   \param nIndex   Message index, [I]
   \param nSize    Number of words to write, [I]
   \param pData    Pointer to the data in case of writing a CMV. In case of
                   reading a CMV is shall be DSL_NULL, [I]
   \param pMsg     Pointer to the CMV data that has been returned by the
                   firmware (excluding header info), [O]

   \ingroup Internal
*/
DSL_Error_t DSL_DRV_DANUBE_CmvWrite (
   DSL_Context_t *pContext,
   DSL_uint8_t nGroup,
   DSL_uint16_t nAddress,
   DSL_uint16_t nIndex,
   DSL_int_t nSize,
   DSL_uint16_t *pData)
{
   DSL_uint16_t RxMessage[DSL_MAX_CMV_MSG_LENGTH]__attribute__ ((aligned(4)));
   DSL_uint16_t TxMessage[DSL_MAX_CMV_MSG_LENGTH]__attribute__ ((aligned(4)));
   DSL_Error_t nErrCode = DSL_SUCCESS;

   memset(&RxMessage, 0, sizeof(RxMessage));
   memset(&TxMessage, 0, sizeof(TxMessage));

   DSL_DRV_DANUBE_CmvPrepare(pContext, DSL_CMV_OPCODE_H2D_CMV_WRITE, nGroup,
      nAddress, nIndex, nSize, pData, TxMessage);

   nErrCode = DSL_DRV_DANUBE_CmvSend(pContext, TxMessage, DSL_TRUE, RxMessage);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - CMV send failed, write %s %d %d!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), gGroupName[nGroup], nAddress, nIndex));
   }

   return nErrCode;
}

/*
   Checks if a firmware message has to be printed or not.
   This function takes into account
   + debug level of message dump module
   + defined message blacklist in case of 0x00 < DbgLvl < 0x80

   \param pContext Pointer to DSL CPE API context structure, [I]
   \param nGroup   Message group number, [I]
   \param nAddress Message address, [I]

   \ingroup Internal
 */
static DSL_boolean_t DSL_DRV_DANUBE_CmvCheckPrint(
   DSL_Context_t *pContext,
   const DSL_uint16_t *pMsg)
{
   DSL_DEV_MsgIdRange_t *pTable = &g_DANUBE_MsgDumpBlacklist[0];
   DSL_DBG_ModuleLevel_t dbgModuleLevel;
   DSL_boolean_t bPrint = DSL_TRUE, bWrite = DSL_FALSE;
   DSL_uint8_t nGroup = 0, nDir = 0;
   DSL_uint16_t nAddress = 0;

   /* Get module debug level*/
   memset(&dbgModuleLevel, 0, sizeof(dbgModuleLevel));
   dbgModuleLevel.data.nDbgModule = DSL_DBG_MESSAGE_DUMP;
   DSL_DRV_DBG_ModuleLevelGet(pContext, &dbgModuleLevel);

   /* Get message info*/
   nGroup   = (DSL_uint8_t)(*(pMsg + 1) & 0x7F);
   nAddress = *(pMsg + 2);
   /* nDir!=0 - Arc->ME, else ME->Arc*/
   nDir     = (*pMsg & 0x10);

   /* Check Rd/Wr access type*/
   bWrite = DSL_DRV_DANUBE_CmvAccessTypeGet(*pMsg);

   if (dbgModuleLevel.data.nDbgLevel > DSL_DBG_NONE)
   {
      if (dbgModuleLevel.data.nDbgLevel < DSL_DBG_MSG)
      {
         /* Check if the prepared message is listed within blacklist */
         for (bPrint = DSL_TRUE; pTable->nGroupId != -1; pTable++)
         {
            if ((DSL_int_t)nGroup != pTable->nGroupId)
            {
               continue;
            }

            if ((nAddress >= pTable->nAdrFirst) && (nAddress <= pTable->nAdrLast))
            {
               bPrint = DSL_FALSE;
               break;
            }
         }
      }
      else
      {
         bPrint = DSL_TRUE;
      }

      if (dbgModuleLevel.data.nDbgLevel != DSL_DBG_LOCAL && bPrint)
      {
         if ((bWrite && nDir) || (!bWrite && !nDir))
         {
            bPrint = DSL_FALSE;
         }
      }
   }
   else
   {
      bPrint = DSL_FALSE;
   }

   return bPrint;
}

static DSL_void_t DSL_DRV_DANUBE_CmvMessageDump(
   DSL_Context_t *pContext,
   const DSL_char_t *sName,
   const DSL_uint16_t *pMsg)
{
   DSL_boolean_t bPrint = DSL_TRUE;
   DSL_uint8_t nSize = 0, i = 0;
   
   bPrint = DSL_DRV_DANUBE_CmvCheckPrint(pContext, pMsg);

   if (bPrint)
   {
      /* Get CMV byte count*/
      nSize = (DSL_uint8_t)DSL_DRV_DANUBE_CmvByteCountGet(*pMsg);

      DSL_DRV_debug_printf(pContext, "DSL[%s]:",sName);

      /* Print message in the 16-bit format*/
      for (i = 0; i < (nSize/2 + nSize%2); i++)
      {
         DSL_DRV_debug_printf(pContext, " %04X", *(pMsg + i));
      }
      DSL_DRV_debug_printf(pContext, DSL_DRV_CRLF);
   }
}

/*
   Compose a message.
   This function compose a message from opcode, group, address, index, size,
   and data

   \param pContext Pointer to DSL CPE API context structure, [I]
   \param nOpcode  Message opcode, [I]
   \param nGroup   Message group number, [I]
   \param nAddress Message address, [I]
   \param nIndex   Message index, [I]
   \param nSize    Number of words to read/write, [I]
   \param pData    Pointer to the data in case of writing a CMV. In case of
                   reading a CMV is shall be DSL_NULL, [I]
   \param pMsg     Pointer to composed message, [O]

   \ingroup Internal
 */
DSL_void_t DSL_DRV_DANUBE_CmvPrepare(
   DSL_Context_t *pContext,
   DSL_uint8_t nOpcode,
   DSL_uint8_t nGroup,
   DSL_uint16_t nAddress,
   DSL_uint16_t nIndex,
   DSL_int_t nSize,
   DSL_uint16_t *pData,
   DSL_uint16_t *pMsg)
{
   pMsg[0]= (nOpcode << 4) | (nSize & 0xf);
   pMsg[1]= (((nIndex==0) ? 0 : 1) << 7) | (nGroup & 0x7f);
   pMsg[2]= nAddress;
   pMsg[3]= nIndex;

   if ((nOpcode == DSL_CMV_OPCODE_H2D_CMV_WRITE) && (pData != NULL))
   {
      memcpy(pMsg+4, pData, nSize*2);
   }

   return;
}


/*
   Send a message to ARC and read the response
   This function sends a message to arc, waits the response, and reads the
   responses.

   \param pContext Pointer to DSL CPE API context structure, [I]
   \param pTxMsg   Pointer to CMV message that will be written, [I]
   \param reply    Specifies whether to wait on a firmware (DSL_TRUE) reply or
                   not (DSL_FALSE), [I]
   \param pRxMsg   Pointer to CMV message that has been returned by the firmware
                   as a reply to TxMsg, [O]

   \return  values from the DSL_Error_t enum

   \ingroup Internal
 */
DSL_Error_t DSL_DRV_DANUBE_CmvSend(
   DSL_Context_t *pContext,
   DSL_uint16_t *pTxMsg,
   DSL_boolean_t bReply,
   DSL_uint16_t *pRxMsg)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   if (pContext == DSL_NULL || pContext->pDevCtx == DSL_NULL)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: DSL_DRV_DANUBE_CmvSend: DSL context or device "
            "context pointers is NULL"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
   }

   /* Dump message*/
   DSL_DRV_DANUBE_CmvMessageDump(pContext,"tx",pTxMsg);

   if (DSL_BSP_SendCMV(pContext->pDevCtx->lowHandle, pTxMsg,
      (DSL_int_t)bReply, pRxMsg) != DSL_DEV_MEI_ERR_SUCCESS)
   {
      nErrCode = DSL_ERROR;
   }

   /* Dump message*/
   DSL_DRV_DANUBE_CmvMessageDump(pContext,"rx",pRxMsg);

   return nErrCode;
}

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_LineFeaturesUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_LineFeaturesUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_OPTN,
      DSL_CMV_ADDRESS_OPTN_ALG_CONTROL, 0, 1, &nVal)) == DSL_SUCCESS)
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
         lineFeatureDataSts[DSL_DOWNSTREAM].bBitswapEnable,
         (nVal & (1<<5)) ? DSL_FALSE : DSL_TRUE);
   }

   if (nErrCode == DSL_SUCCESS &&
       (nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
        DSL_CMV_ADDRESS_STAT_MISC, 0, 1, &nVal)) == DSL_SUCCESS)
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
         lineFeatureDataSts[DSL_DOWNSTREAM].bTrellisEnable,
         (nVal & (1<<4)) ? DSL_TRUE : DSL_FALSE);

      DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
         lineFeatureDataSts[DSL_UPSTREAM].bTrellisEnable,
         (nVal & (1<<7)) ? DSL_TRUE : DSL_FALSE);
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_LineFeaturesUpdate, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   This function checks the System Interface Configuration parameters
   against supported by firmware
*/
static DSL_Error_t DSL_DRV_DANUBE_SystemInterfaceConfigCheck(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_SystemInterfaceConfigData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_SystemInterfaceConfigCheck" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   /* System Interface selection check*/
   if (pData->nSystemIf != DSL_SYSTEMIF_MII)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unsupported SystemIf=%d selected!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), pData->nSystemIf));

      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
   }

   /* TC Layer selection check*/
   switch(pData->nTcLayer)
   {
   case DSL_TC_EFM:
   case DSL_TC_EFM_FORCED:
   case DSL_TC_AUTO:
      /* Upstream TC configuration selection check*/
      switch(pData->nEfmTcConfigUs)
      {
      case DSL_EMF_TC_CLEANED:
      case DSL_EMF_TC_NORMAL:
      case DSL_EMF_TC_PRE_EMPTION:
         break;

      case DSL_EMF_TC_SHORT_PACKETS:
      default:
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Unsupported nEfmTcConfigUs=%d selected!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), pData->nEfmTcConfigUs));
         
         nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
         break;
      }

      /* Downstream TC configuration selection check*/      
      switch(pData->nEfmTcConfigDs)
      {
      case DSL_EMF_TC_CLEANED:
      case DSL_EMF_TC_NORMAL:
         break;

      case DSL_EMF_TC_PRE_EMPTION:
      case DSL_EMF_TC_SHORT_PACKETS:
      default:
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Unsupported nEfmTcConfigDs=%d selected!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), pData->nEfmTcConfigDs));
         
         nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
         break;
      }
      
      /* Check if at least one of the possible TC Us/Ds configuration is selected*/
      if ((pData->nEfmTcConfigUs == DSL_EMF_TC_CLEANED) &&
          (pData->nEfmTcConfigDs == DSL_EMF_TC_CLEANED))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - At least one of the TC Us/Ds "
            "configuration has to be selected!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         
         nErrCode = DSL_ERROR;
      }
   
      break;

   case DSL_TC_ATM:
      DSL_DEBUG( DSL_DBG_WRN, (pContext,
         "DSL[%02d]: WARNING - All EFM configurations are ignored "
         "while the ATM is selected" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      break;
      
   default:
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unsupported nTcLayer=%d selected!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), pData->nTcLayer));

      nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
      break;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_SystemInterfaceConfigCheck, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_SystemInterfaceConfigSet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_SystemInterfaceConfig_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_VersionCheck_t nFwVer = DSL_VERSION_ERROR;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_SystemInterfaceConfigSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext, DSL_MIN_FW_VERSION_PTM,
      DSL_FW_ANNEX_A, &nFwVer);

   if (nErrCode >= DSL_SUCCESS)
   {
      if (nFwVer < DSL_VERSION_EQUAL)
      {
         nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
      }
      else
      {
         nErrCode = DSL_DRV_DANUBE_SystemInterfaceConfigCheck(pContext, &pData->data);
         if (nErrCode == DSL_SUCCESS)
         {
            if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
            {
               DSL_DEBUG(DSL_DBG_ERR, (pContext,
                  "DSL[%02d]: ERROR - data mutex lock failed!" DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
                  
               return DSL_ERROR;
            }

            /* Copy System Configuration data to internal data of device context*/
            memcpy(
               &(pContext->pDevCtx->data.sysCIF),
               &(pData->data),
               sizeof(DSL_SystemInterfaceConfigData_t));

            DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
         }
      }
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_SystemInterfaceConfigSet, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_SystemInterfaceConfigGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_SystemInterfaceConfig_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_VersionCheck_t nFwVer = DSL_VERSION_ERROR;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_SystemInterfaceConfigGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext, DSL_MIN_FW_VERSION_PTM,
      DSL_FW_ANNEX_A, &nFwVer);

   if (nErrCode >= DSL_SUCCESS)
   {
      if (nFwVer < DSL_VERSION_EQUAL)
      {
         nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
      }
      else
      {
         if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
         {
            DSL_DEBUG(DSL_DBG_ERR, (pContext,
               "DSL[%02d]: ERROR - data mutex lock failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return DSL_ERROR;
         }

         /* Get System Configuration from the internal DANUBE device context*/
         memcpy(
            &(pData->data),
            &(pContext->pDevCtx->data.sysCIF),
            sizeof(DSL_SystemInterfaceConfigData_t));

         DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
      }
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_SystemInterfaceConfigGet, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_SystemInterfaceStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_SystemInterfaceStatus_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_VersionCheck_t nFwVer = DSL_VERSION_ERROR;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_SystemInterfaceStatusGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext, DSL_MIN_FW_VERSION_PTM,
      DSL_FW_ANNEX_A, &nFwVer);

   if (nErrCode >= DSL_SUCCESS)
   {
      if (nFwVer < DSL_VERSION_EQUAL)
      {
         nErrCode = DSL_ERR_NOT_SUPPORTED_BY_FIRMWARE;
      }
      else
      {
         if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
         {
            DSL_DEBUG(DSL_DBG_ERR, (pContext,
               "DSL[%02d]: ERROR - data mutex lock failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return DSL_ERROR;
         }

         /* Get System Configuration from the internal DANUBE device context*/
         memcpy(
            &(pData->data),
            &(pContext->pDevCtx->data.sysCIFSts),
            sizeof(DSL_SystemInterfaceConfigData_t));

         DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
      }
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_SystemInterfaceStatusGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_AutobootHandleStart(
   DSL_Context_t *pContext,
   DSL_boolean_t bLoopTest,
   DSL_boolean_t bShortInit)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_TestModeControlSet_t nTestMode = DSL_TESTMODE_DISABLE;
   DSL_DEV_VersionCheck_t nFwVer = DSL_VERSION_ERROR,
                          nFwVerFTTXPOTS = DSL_VERSION_ERROR;
   DSL_uint16_t nPtmConfig = 0, nVal = 0;
   DSL_uint16_t nOptn9, nOptn2, nOptn16;
   DSL_uint8_t nUtopiaAddr[DSL_CHANNELS_PER_LINE], i;
   DSL_boolean_t bTrellisUS = DSL_FALSE, bTrellisDS = DSL_FALSE;
   DSL_boolean_t bBitSwapUS = DSL_FALSE, bBitSwapDS = DSL_FALSE;
   DSL_InteropFeatureConfigData_t interopFeatureConfigData;

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleStart" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext, DSL_MIN_FW_VERSION_PTM,
      DSL_FW_ANNEX_NA, &nFwVer);

   if (nErrCode == DSL_SUCCESS)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nTestModeControl, nTestMode);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_FirmwareInit(pContext);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      /* Initialization of the LED has to be done with ARC idle */
      nErrCode = DSL_DRV_DEV_LED_FirmwareInit(pContext);
   }

#if defined(INCLUDE_DSL_CEOC)
   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CEOC_FirmwareInit(pContext);
   }
#endif /* defined(INCLUDE_DSL_CEOC) */

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_LineInventoryNeWrite(pContext);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_LineInventoryNeRead(pContext);

      memcpy(&pContext->lineInventoryNe.XTSECapabilities,
         &pContext->xtseCfg, DSL_G997_NUM_XTSE_OCTETS);
   }

   for (i = 0; i < DSL_CHANNELS_PER_LINE; i++)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode,
         pDevCtx->data.nUtopiaAddr[i], nUtopiaAddr[i]);
   }

   if ((nErrCode == DSL_SUCCESS) &&
      ((nErrCode = DSL_DRV_DANUBE_XTUSystemEnablingConfigSend(pContext))
      == DSL_SUCCESS))
   {
      if (DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext, DSL_DRV_CRLF"DSL[%02d]: getting mei driver semaphore "
            "failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         nErrCode = DSL_ERROR;
      }
      else
      {
         do
         {
            /*$$ ND: is this condition correct?? */
            if (nUtopiaAddr[0] != 0 && DSL_CHANNELS_PER_LINE == 2 &&
               nUtopiaAddr[1] != 0)
            {
               nVal = ((nUtopiaAddr[0] & 0x1F) << 5)
                  | ((DSL_CHANNELS_PER_LINE == 2) ? (nUtopiaAddr[1] & 0x1F) : 0 );

               if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_OPTN,
                  DSL_CMV_ADDRESS_OPTN_UTOPIA_ADDRESS, 0, 1, &nVal)
                  != DSL_SUCCESS)
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - OPTN(write) "
                     "Utopia address configuration failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                     
                  nErrCode = DSL_ERR_MSG_EXCHANGE;
                  
                  break;
               }
            }

            if (nTestMode == DSL_TESTMODE_QUIET)
            {
               nVal = 0;

               if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
                  DSL_CMV_ADDRESS_INFO_TX_POWER, 0, 1, &nVal)
                  != DSL_SUCCESS)
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - INFO(write) "
                     "TX power CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                     
                  nErrCode = DSL_ERROR;
                  
                  break;
               }
            }

            /* Check for DSL_MIN_FW_VERSION_FTTXPOTS*/ 
            nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext,
                          DSL_MIN_FW_VERSION_FTTXPOTS, DSL_FW_ANNEX_NA, &nFwVerFTTXPOTS);

            if (nErrCode != DSL_SUCCESS)
            {
               DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - FW version check failed!"
                  DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               break;
            }

            if (nFwVerFTTXPOTS >= DSL_VERSION_EQUAL)
            {
               /* Get interop configuration data*/
               DSL_CTX_READ(
                  pContext, nErrCode, interopFeatureConfigData, interopFeatureConfigData);

               /* Alternative Tx POTS highpass filter configuration. This is not mentioned
                  in the CMV spec. For more details please refer to the SMS00715335 issue*/
               if (interopFeatureConfigData.bFtTxPotsHp)
               {
                  nVal = 0x0080;
                  
                  if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
                     DSL_CMV_ADDRESS_INFO_SYSTEM_INTERFACE, 1, 1, &nVal)
                     != DSL_SUCCESS)
                  {
                     DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - INFO(write) "
                        "TX POTS filter CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                     
                     nErrCode = DSL_ERROR;
                    
                     break;
                  }
               }
            }

            if (nFwVer >= DSL_VERSION_EQUAL)
            {
               /* Apply System Interface configuration options */            
               if (DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: getting data semaphore "
                     "failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                     
                  nErrCode = DSL_ERROR;
                  
                  break;
               }
               else
               {
                  nErrCode = DSL_DRV_DANUBE_SystemInterfaceConfigCheck(pContext,
                     &pContext->pDevCtx->data.sysCIF);
                  if (nErrCode != DSL_SUCCESS)
                  {
                     DSL_DEBUG(DSL_DBG_ERR, (pContext,
                        "DSL[%02d]: ERROR - System Interface configuration check failed!"
                        DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                  }
                  else
                  {
                     nPtmConfig = DSL_CMV_SEL_TPS_TC;

                     if (pContext->pDevCtx->data.sysCIF.nTcLayer == DSL_TC_ATM ||
                        pContext->pDevCtx->data.sysCIF.nTcLayer == DSL_TC_EFM_FORCED ||
                        pContext->pDevCtx->data.sysCIF.nTcLayer == DSL_TC_AUTO)
                     {
                        nPtmConfig |= DSL_CMV_ATM_SELECTED;
                     }

                     if (pContext->pDevCtx->data.sysCIF.nTcLayer == DSL_TC_EFM ||
                        pContext->pDevCtx->data.sysCIF.nTcLayer == DSL_TC_EFM_FORCED ||
                        pContext->pDevCtx->data.sysCIF.nTcLayer == DSL_TC_AUTO)
                     {
                        nPtmConfig |= DSL_CMV_PTM_SELECTED;

                        if (pContext->pDevCtx->data.sysCIF.nEfmTcConfigDs &
                           DSL_EMF_TC_NORMAL)
                        {
                           nPtmConfig |= DSL_CMV_PTM_64_65_OCTET_ENCAP_SUPPORT_DS;
                        }

                        if (pContext->pDevCtx->data.sysCIF.nEfmTcConfigUs &
                           DSL_EMF_TC_NORMAL)
                        {
                           nPtmConfig |= DSL_CMV_PTM_64_65_OCTET_ENCAP_SUPPORT_US;
                        }

                        if (pContext->pDevCtx->data.sysCIF.nEfmTcConfigUs &
                           DSL_EMF_TC_PRE_EMPTION)
                        {
                           nPtmConfig |= DSL_CMV_PTM_64_65_OCTET_ENCAP_PRE_US;
                        }
                     }
                  }
                  DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

                  if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
                     DSL_CMV_ADDRESS_INFO_SYSTEM_INTERFACE,
                     DSL_CMV_SYSTEM_INTERFACE_INDEX_CONFIG, 1, &nPtmConfig)
                     != DSL_SUCCESS)
                  {
                     DSL_DEBUG(DSL_DBG_ERR, (pContext,
                        "DSL[%02d]: ERROR - Error: INFO(write) system interface "
                        "configuration CMV send failed"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                        
                     nErrCode = DSL_ERR_MSG_EXCHANGE;
                     
                     break;
                  }
                  if (pContext->pDevCtx->data.sysCIF.nTcLayer == DSL_TC_EFM_FORCED)
                  {
                     if ( DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
                        DSL_CMV_ADDRESS_INFO_SYSTEM_INTERFACE,
                        DSL_CMV_SYSTEM_INTERFACE_PTM_FORCE, 1, &nPtmConfig)
                        != DSL_SUCCESS)
                     {
                        DSL_DEBUG(DSL_DBG_ERR, (pContext,
                           "DSL[%02d]: ERROR - Error: INFO(write) system interface "
                           "configuration CMV send failed"DSL_DRV_CRLF,
                           DSL_DEV_NUM(pContext)));
                           
                        nErrCode = DSL_ERR_MSG_EXCHANGE;
                        
                        break;
                     }
                     nPtmConfig |= DSL_CMV_PTM_FORCED;
                     if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
                        DSL_CMV_ADDRESS_INFO_SYSTEM_INTERFACE,
                        DSL_CMV_SYSTEM_INTERFACE_PTM_FORCE, 1, &nPtmConfig)
                        != DSL_SUCCESS)
                     {
                        DSL_DEBUG(DSL_DBG_ERR, (pContext,
                           "DSL[%02d]: ERROR - Error: INFO(write) system interface "
                           "configuration CMV send failed"DSL_DRV_CRLF,
                           DSL_DEV_NUM(pContext)));
                           
                        nErrCode = DSL_ERR_MSG_EXCHANGE;
                        
                        break;
                     }
                     if ( DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
                        DSL_CMV_ADDRESS_INFO_SYSTEM_INTERFACE,
                        DSL_CMV_SYSTEM_INTERFACE_PTM_FORCE, 1, &nPtmConfig)
                        != DSL_SUCCESS)
                     {
                        DSL_DEBUG(DSL_DBG_ERR, (pContext,
                           "DSL[%02d]: ERROR - Error: INFO(write) system interface "
                           "configuration CMV send failed"DSL_DRV_CRLF,
                           DSL_DEV_NUM(pContext)));
                           
                        nErrCode = DSL_ERR_MSG_EXCHANGE;
                        
                        break;
                     }
                  }
               }
            }

            /* Apply G997 Line activation options */
            if (bLoopTest == DSL_TRUE || bShortInit == DSL_TRUE)
            {
               DSL_DEBUG(DSL_DBG_MSG,
                  (pContext, "DSL[%02d]: Enable diag or shortinit mode in the Firmware"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));

               if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_OPTN,
                  DSL_CMV_ADDRESS_OPTN_STATE_MACHINE_CTRL, 0, 1, &nOptn9)
                  == DSL_SUCCESS)
               {
                  if (bLoopTest == DSL_TRUE)
                  {
                     nOptn9 |= 1<<2;
                  }

                  if (bShortInit == DSL_TRUE)
                  {
                     nOptn9 |= 1<<5;
                  }

                  if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_OPTN,
                     DSL_CMV_ADDRESS_OPTN_STATE_MACHINE_CTRL, 0, 1, &nOptn9)
                     != DSL_SUCCESS)
                  {
                     DSL_DEBUG(DSL_DBG_ERR,
                        (pContext, "DSL[%02d]: ERROR - Error: OPTN(write) "
                        "state machine control CMV send failed"DSL_DRV_CRLF,
                        DSL_DEV_NUM(pContext)));
                        
                     nErrCode = DSL_ERROR;
                     
                     break;
                  }
               }
               else
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - OPTN(read) state "
                     "machine control CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                  nErrCode = DSL_ERROR;
                  break;
               }
            }

            /* apply line features */
            DSL_CTX_READ_SCALAR(pContext, nErrCode,
               lineFeatureDataCfg[DSL_UPSTREAM].bBitswapEnable, bBitSwapUS);
            DSL_CTX_READ_SCALAR(pContext, nErrCode,
               lineFeatureDataCfg[DSL_DOWNSTREAM].bBitswapEnable, bBitSwapDS);
            DSL_CTX_READ_SCALAR(pContext, nErrCode,
               lineFeatureDataCfg[DSL_DOWNSTREAM].bTrellisEnable, bTrellisDS);
            DSL_CTX_READ_SCALAR(pContext, nErrCode,
               lineFeatureDataCfg[DSL_UPSTREAM].bTrellisEnable, bTrellisUS);

            if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_OPTN,
               DSL_CMV_ADDRESS_OPTN_ALG_CONTROL, 0, 1, &nOptn2)
               == DSL_SUCCESS)
            {
               if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_OPTN,
                  DSL_CMV_ADDRESS_OPTN_ALG_CONTROL2, 0, 1, &nOptn16)
                  == DSL_SUCCESS)
               {
                  /* DS */
                  if (bBitSwapDS == DSL_FALSE)
                  {
                     nOptn16 &= ~(1<<10);
                     nOptn2 |= 1<<5;
                  }
                  else
                  {
                     nOptn16 &= ~(1<<10);
                     nOptn2 &= ~(1<<5);
                  }

                  /* US */
                  if (bBitSwapUS == DSL_FALSE)
                  {
                     nOptn2 |= 1<<15;
                  }
                  else
                  {
                     nOptn2 &= ~(1<<15);
                  }

                  /*$$ Is this correct? */
                  if (bTrellisDS == DSL_TRUE || bTrellisUS == DSL_TRUE)
                  {
                     nOptn2 &= ~(1<<13);
                  }
                  else
                  {
                     nOptn2 |= 1<<13;
                  }

                  if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_OPTN,
                     DSL_CMV_ADDRESS_OPTN_ALG_CONTROL, 0, 1, &nOptn2)
                     != DSL_SUCCESS)
                  {
                     DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - OPTN(write) "
                        "alg control CMV send failed"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                        
                     nErrCode = DSL_ERROR;
                     
                     break;
                  }

                  if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_OPTN,
                     DSL_CMV_ADDRESS_OPTN_ALG_CONTROL2, 0, 1, &nOptn16)
                     != DSL_SUCCESS)
                  {
                     DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - OPTN(write) "
                        "alg control2 CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                        
                     nErrCode = DSL_ERROR;
                     
                     break;
                  }
               }
               else
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - OPTN(read) state "
                     "alg2 CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                     
                  nErrCode = DSL_ERROR;
                  
                  break;
               }
            }
            else
            {
               DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - OPTN(read) state "
                  "alg CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                  
               nErrCode = DSL_ERROR;
               
               break;
            }

#ifdef INCLUDE_DSL_CPE_TRACE_BUFFER
            nVal = 0x800; /* enable showtime event logging */
            if ( (nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
               DSL_CMV_ADDRESS_INFO_DEBUGTRAIL_TRIGGER, 0, 1, &nVal))
               != DSL_SUCCESS)
            {
               DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - INFO(write) showtime "
                  "event logging control CMV send failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
                  
               break;
            }
            else
            {
               /* set buffer length */
               nVal = DSL_DEV_SHOWTIME_EVENT_LOGGING_BUFFER_LENGTH;
               if ( (nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
                  DSL_CMV_ADDRESS_INFO_DEBUGTRAIL_COLLECT, 1, 1, &nVal))
                  != DSL_SUCCESS)
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - INFO(write) showtime "
                     "event logging buffer length CMV send failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                     
                  break;
               }
            }
#endif
         } while (0);

         DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
      }
   }

   if (nErrCode != DSL_SUCCESS)
   {
      /* Set LINIT value to 'LINIT_UNKNOWN' in case of an error */
      DSL_DRV_HandleLinitValue(pContext, LINIT_UNKNOWN, LINIT_SUB_NONE);
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleStart, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_PowerStatusSet(
   DSL_Context_t *pContext,
   DSL_G997_PowerManagement_t nNewStatus)
{
   DSL_G997_PowerManagement_t nPreStatus;
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_PowerStatusSet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* get current status and send event if changed */
   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      powerMgmtStatus.nPowerManagementStatus, nPreStatus);
   if (nPreStatus != nNewStatus)
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
         powerMgmtStatus.nPowerManagementStatus, nNewStatus);

      nErrCode = DSL_DRV_EventGenerate(
                    pContext, 0, DSL_ACCESSDIR_NA, DSL_XTUDIR_NA,
                    DSL_EVENT_S_LINE_POWERMANAGEMENT_STATE,
                    (DSL_EventData_Union_t*)&pContext->powerMgmtStatus,
                    sizeof(DSL_G997_PowerManagementStatusData_t));
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_EVENT_S_LINE_POWERMANAGEMENT_STATE));
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_PowerStatusSet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_PowerStatusHandle(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_G997_PowerManagement_t nNewStatus;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_PowerStatusHandle"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nErrCode = DSL_DRV_DANUBE_L3StatusGet(pContext, &nNewStatus);
   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_PowerStatusSet(pContext, nNewStatus);
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_PowerStatusHandle, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_ShowtimeStatusUpdate(
   DSL_Context_t *pContext,
   DSL_boolean_t bInit)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal;
   DSL_int16_t nsVal = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_ShowtimeStatusUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   nErrCode = DSL_DRV_DANUBE_XTUSystemEnablingStatusUpdate(pContext);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - An error occured while XTU System Enabling"
         " Status update!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }
   else
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ERROR;
      }

      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
           (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
           DSL_DEV_NUM(pContext)));
           
         DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
         
         return DSL_ERROR;
      }

      /* 
         According to some interop issues and also SMS00718794 the handling for the
         min SNR margin applies 
      */
      /*Check for the Annex A FW*/
      if ((pContext->xtseCurr[0] & XTSE_1_03_A_1_NO) ||
          (pContext->xtseCurr[1] & XTSE_2_01_A_2_NO) ||
          (pContext->xtseCurr[2] & XTSE_3_03_A_3_NO) ||
          (pContext->xtseCurr[3] & XTSE_4_05_I_3_NO) ||
          (pContext->xtseCurr[4] & XTSE_5_03_L_3_NO) ||
          (pContext->xtseCurr[4] & XTSE_5_04_L_3_NO) ||
          (pContext->xtseCurr[4] & XTSE_5_07_M_3_NO) ||
          (pContext->xtseCurr[5] & XTSE_6_01_A_5_NO) ||
          (pContext->xtseCurr[5] & XTSE_6_07_I_5_NO) ||
          (pContext->xtseCurr[6] & XTSE_7_03_M_5_NO))

      {
         /* Ignore the retrain margin sent by the CO and always
            assume a retrain margin of 0dB*/
         pContext->nMinSnrmDs = 0;
      }
      /*Check for the ADSL1 Annex B FW*/
      else if ((pContext->xtseCurr[0] & XTSE_1_05_B_1_NO))
      {
         /* In case of DMT the values has to be read by using C-MSG-RA */
         if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
            DSL_CMV_ADDRESS_INFO_RCMSGRA, 1, 1, &nVal)) == DSL_SUCCESS)
         {
            /* C-MSG-RA returns returns a 6 bit signed value (2'complement)
               for min margin (G992.1 Table 10-12) */
            if (DSL_DRV_TwosComplement16_HexToInt(pContext, nVal, 6, &nsVal) == DSL_SUCCESS);
            {
               /* Global value for MINSNRM is stored in units of 0.1dB */
               nsVal *= 10;
            }

            pContext->nMinSnrmDs = nsVal;
         }
      }
      /*Check for the ADSL2/2+ Annex B FW*/      
      else if((pContext->xtseCurr[2] & XTSE_3_05_B_3_NO) ||
              (pContext->xtseCurr[3] & XTSE_4_07_J_3_NO) ||
              (pContext->xtseCurr[5] & XTSE_6_03_B_5_NO) ||
              (pContext->xtseCurr[6] & XTSE_7_01_J_5_NO))
      {
         if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
            DSL_CMV_ADDRESS_INFO_CMsgs1, 1, 1, &nVal)) == DSL_SUCCESS)
         {
            /* In case of ADSL2/2+ the value is already defined as multiple
               of 0.1 dB within C-MSG1*/
            nsVal = (DSL_int16_t)nVal;

            pContext->nMinSnrmDs = nsVal;
         }
      }
      else
      {
         pContext->nMinSnrmDs = 0;
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

      if (nErrCode == DSL_SUCCESS)
      {
         if (bInit != DSL_FALSE)
         {
            nErrCode = DSL_DRV_DANUBE_LineFeaturesUpdate(pContext);
            if (nErrCode == DSL_SUCCESS)
            {
               nErrCode = DSL_DRV_DANUBE_ChannelStatusUpdate(pContext);
            }
         }
         if (nErrCode == DSL_SUCCESS)
         {
            nErrCode = DSL_DRV_DANUBE_ChannelRateUpdate(pContext);
         }
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

      DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: MINSNRMds=%d/10dB"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nsVal));
   }

   if (nErrCode == DSL_SUCCESS)
   {
      if ((nErrCode = DSL_DRV_DANUBE_LineFailuresUpdate(pContext)) == DSL_SUCCESS)
      {
         nErrCode = DSL_DRV_DANUBE_SnrmDsUpdate(pContext);
      }
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_PowerStatusHandle(pContext);
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_ShowtimeStatusUpdate, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

#if defined(INCLUDE_DSL_PM)
#ifdef HAS_TO_BE_CLARIFIED
#define DSL_DRV_DANUBE_LINE_INIT_FAILURES_INC() \
   do { \
         DSL_CTX_READ_SCALAR(pContext, nErrCode, \
            bGotShortInitResponse, bResp); \
            if (bResp == DSL_TRUE) \
            { \
            DSL_CTX_WRITE_SCALAR(pContext, nErrCode, \
               pDevCtx->data.lineInitCounters.nFailedShortInits, \
               pContext->pDevCtx->data.lineInitCounters.nFailedShortInits + 1); \
         } \
      } \
   } while (0)
#endif
#endif

static DSL_Error_t DSL_DRV_DANUBE_SystemInterfaceStatusUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_SystemInterfaceConfigData_t stsData = { 0 };
   DSL_DEV_VersionCheck_t nFwVer = DSL_VERSION_ERROR;
   DSL_uint16_t nPtmSts = 0;

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DANUBE_SystemInterfaceStatusUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_ERR_CODE();

   nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext, DSL_MIN_FW_VERSION_PTM,
      DSL_FW_ANNEX_A, &nFwVer);

   do
   {
      if (nErrCode < DSL_SUCCESS) break;

      if (nFwVer < DSL_VERSION_EQUAL)
      {
         break;
      }

      if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Couldn't lock device mutex!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }

      if ( DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_SYSTEM_INTERFACE,
         DSL_CMV_SYSTEM_INTERFACE_INDEX_STATUS, 1, &nPtmSts) != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Error: INFO(read) system interface configuration CMV "
            "send failed" DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERR_MSG_EXCHANGE;
      }

      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

      if (nErrCode < DSL_SUCCESS) break;

      /* The only system interface that is supported for Amazon-SE and
         Danube is MII by now */
      stsData.nSystemIf = DSL_SYSTEMIF_MII;

	DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: MSG - nPtmSts = %#x %s\n" DSL_DRV_CRLF,
		DSL_DEV_NUM(pContext), nPtmSts, (nPtmSts & DSL_CMV_ATM_SELECTED)?"DSL_TC_ATM":
		(nPtmSts & DSL_CMV_PTM_SELECTED)?"DSL_TC_EFM":"Unknown System Interface Status"));
        
      if (nPtmSts & DSL_CMV_ATM_SELECTED)
      {
         stsData.nTcLayer = DSL_TC_ATM;
      }
      else if (nPtmSts & DSL_CMV_PTM_SELECTED)
      {
         /* Reset bitmasks to uninitialized before updating with decoded
            firmware information. */
         stsData.nEfmTcConfigUs = DSL_EMF_TC_CLEANED;
         stsData.nEfmTcConfigDs = DSL_EMF_TC_CLEANED;

         stsData.nTcLayer = DSL_TC_EFM;

         if (nPtmSts & DSL_CMV_PTM_64_65_OCTET_ENCAP_SUPPORT_DS)
         {
            stsData.nEfmTcConfigDs |= DSL_EMF_TC_NORMAL;
         }

         if (nPtmSts & DSL_CMV_PTM_64_65_OCTET_ENCAP_SUPPORT_US)
         {
            stsData.nEfmTcConfigUs |= DSL_EMF_TC_NORMAL;
         }

         if (nPtmSts & DSL_CMV_PTM_64_65_OCTET_ENCAP_PRE_US)
         {
            stsData.nEfmTcConfigUs |= DSL_EMF_TC_PRE_EMPTION;
         }
      }
      else
      {
         stsData.nTcLayer = DSL_TC_UNKNOWN;
      }

      if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Couldn't lock data mutex!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }

      /* Store current status within device context in any case. */
      memcpy(&(pContext->pDevCtx->data.sysCIFSts), &stsData,
         sizeof(DSL_SystemInterfaceConfigData_t));

      DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: MSG nSystemIf %#x, "
         "nTcLayer %#x, nEfmTcConfigUs %#x, nEfmTcConfigDs %#x\n" DSL_DRV_CRLF, 
	     DSL_DEV_NUM(pContext), stsData.nSystemIf, stsData.nTcLayer,
         stsData.nEfmTcConfigUs, stsData.nEfmTcConfigDs));
      
      DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

      if (stsData.nTcLayer == DSL_TC_UNKNOWN)
      {
         break;
      }

      nErrCode = DSL_DRV_EventGenerate(
                    pContext, 0, DSL_ACCESSDIR_NA, DSL_XTUDIR_NA,
                    DSL_EVENT_S_SYSTEM_INTERFACE_STATUS,
                    (DSL_EventData_Union_t*)&stsData,
                    sizeof(DSL_SystemInterfaceStatus_t));

      if( nErrCode < DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Event(%d) generate failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_EVENT_S_SYSTEM_INTERFACE_STATUS));
      }
   } while (0);

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DANUBE_SystemInterfaceStatusUpdate, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_AutobootHandleL3(
   DSL_Context_t *pContext,
   DSL_boolean_t bL3Forced)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_G997_PowerManagement_t nActualPms;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleL3"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if (bL3Forced == DSL_FALSE)
   {
      nErrCode = DSL_DRV_AutobootStateSet(pContext, DSL_AUTOBOOTSTATE_RESTART,
         DSL_AUTOBOOT_RESTART_POLL_TIME);
   }
   else
   {
      /* read actual power status */
      nErrCode = DSL_DRV_DANUBE_PowerStatusHandle(pContext);
      if (nErrCode == DSL_SUCCESS)
      {
         DSL_CTX_READ_SCALAR(pContext, nErrCode,
            powerMgmtStatus.nPowerManagementStatus, nActualPms);
         if (nActualPms != DSL_G997_PMS_L3)
         {
            DSL_DEBUG(DSL_DBG_ERR, (pContext,
               "DSL[%02d]: ERROR - Unexpected power management state "
               "returned"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleL3, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_AutobootHandleFwRequest(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleFwRequest"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleFwRequest"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_AutobootHandleTraining(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_VersionCheck_t nFwVer = DSL_VERSION_ERROR;
   DSL_TestModeControlSet_t nTestMode;
   DSL_LineStateValue_t nPrevLineState = DSL_LINESTATE_UNKNOWN;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;
   DSL_uint16_t nVal = 0;

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleTraining" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nTestModeControl, nTestMode);
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nPrevLineState);

   if ((nErrCode = DSL_DRV_LineStateUpdate(pContext)) == DSL_SUCCESS)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);
      if (nLineState != nPrevLineState)
      {
         switch (nLineState)
         {
            case DSL_LINESTATE_SILENT:
               DSL_DEBUG(DSL_DBG_MSG,
                  (pContext, "DSL[%02d]: SILENT state reached"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
               break;
            case DSL_LINESTATE_SHOWTIME_NO_SYNC:
               /* Clear bGotResponse flag*/
               pContext->bGotResponse = DSL_FALSE;
               /* Clear bGotFullInit flag*/
               pContext->bGotFullInit = DSL_FALSE;

#ifdef INCLUDE_DSL_PM
               /* Set bPmDataValid flag*/
               DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid = DSL_TRUE;
#endif /* INCLUDE_DSL_PM*/

               if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
               {
                  DSL_DEBUG( DSL_DBG_ERR, (pContext,
                     "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                     
                  return DSL_ERROR;
               }

               nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
                  DSL_CMV_ADDRESS_STAT_MISC, 0, 1, &nVal);

               DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

               if (nErrCode == DSL_SUCCESS)
               {
                  if (nVal & 0x02)
                  {
#ifdef INCLUDE_DSL_FIRMWARE_MEMORY_FREE
                     /* Free Resources of low level driver */
                     if (DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_FREE_RESOURCE,
                        DSL_NULL) != DSL_SUCCESS)
                     {
                        DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: DSL_DRV_DEV_BspIoctl("
                           "DSL_FIO_BSP_FREE_RESOURCE) failed" DSL_DRV_CRLF,
                           DSL_DEV_NUM(pContext)));
                     }
                     else
                     {
                        DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: DSL_DRV_DEV_BspIoctl("
                           "DSL_FIO_BSP_FREE_RESOURCE) - success" DSL_DRV_CRLF,
                           DSL_DEV_NUM(pContext)));
                     }
#endif /* INCLUDE_DSL_FIRMWARE_MEMORY_FREE*/

                     nErrCode = DSL_DRV_AutobootStateSet(
                                pContext,
                                DSL_AUTOBOOTSTATE_SHOWTIME,
                                DSL_AUTOBOOT_SHOWTIME_POLL_TIME);

                     if (nErrCode == DSL_SUCCESS)
                     {
                        DSL_DEBUG(DSL_DBG_MSG, (pContext,
                           "DSL[%02d]: SHOWTIME state reached"DSL_DRV_CRLF,
                           DSL_DEV_NUM(pContext)));

                        /* Set LINIT value to 'LINIT_SUCCESSFUL' here
                           because activation procedure
                           was finished without errors */
                        DSL_DRV_HandleLinitValue(pContext, LINIT_SUCCESSFUL, LINIT_SUB_NONE);

                        DSL_DRV_ShowtimeStatusUpdate(pContext, DSL_TRUE);
                        DSL_DRV_DANUBE_ShowtimeReached(pContext);

                        DSL_DRV_Timeout_AddEvent( pContext,
                           DSL_TIMEOUTEVENT_FE_STATUS,
                           DSL_DEV_TIMEOUT_FE_STATUS );
                     }
                  }
               }
               else
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - STAT(read) "
                     "code swaps complete CMV send failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                     
                  nErrCode = DSL_ERR_MSG_EXCHANGE;
                  
                  break;
               }

               break;
            case DSL_LINESTATE_EXCEPTION:
			/* Sercomm add for SYNC LED */
			{
				DSL_Error_t nErrCode = DSL_SUCCESS;
		   		DSL_uint16_t nData = 0;

			         /*  Let FW to handle ADSL Link LED */
				#if defined (INCLUDE_DSL_CPE_API_DANUBE)
			         nData=0x0a03;
				#elif defined (INCLUDE_DSL_CPE_API_AMAZON_SE)
			         nData=0x0900;
				#endif
			         if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,DSL_CMV_ADDRESS_INFO_GPIO_CONTROL, 4, 1, &nData) != DSL_SUCCESS)
				{
					DSL_DEBUG(DSL_DBG_ERR,(pContext, "DSL: ERROR - CMV send failed!"DSL_CRLF));
					nErrCode = DSL_ERR_MSG_EXCHANGE;
				}
			}								
#if defined(INCLUDE_DSL_PM)
               if (pContext->bGotFullInit)
               {
                  DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
                     pDevCtx->data.lineInitCounters.nFailedFullInits,
                     pContext->pDevCtx->data.lineInitCounters.nFailedFullInits + 1);
               }
#ifdef HAS_TO_BE_CLARIFIED
               if (nErrCode  == DSL_SUCCESS)
               {
                  DSL_DRV_DANUBE_LINE_INIT_FAILURES_INC();
               }
#endif /* HAS_TO_BE_CLARIFIED*/
#endif /* defined(INCLUDE_DSL_PM)*/
               DSL_DEBUG(DSL_DBG_MSG,
                  (pContext, "DSL[%02d]: EXCEPTION occurred"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));

               if (nTestMode == DSL_TESTMODE_TRAINING_LOCK)
               {
                  DSL_DEBUG(DSL_DBG_MSG,
                     (pContext, "DSL[%02d]: EXCEPTION - training lock is set, "
                     "exception state will not be set"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
               }
               else
               {
                  DSL_DEBUG(DSL_DBG_MSG,
                     (pContext, "DSL[%02d]: EXCEPTION - set exception state"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
                     
                  nErrCode = DSL_DRV_AutobootStateSet(
                                pContext,
                                DSL_AUTOBOOTSTATE_EXCEPTION,
                                DSL_AUTOBOOT_EXCEPTION_POLL_TIME);
               }
               break;
            case DSL_LINESTATE_HANDSHAKE:
#if 0				
               if (pContext->bGotResponse == DSL_FALSE)
               {
                  pContext->bGotResponse = DSL_TRUE;
                  pContext->autobootStartTime = DSL_DRV_TimeMSecGet();
                  pContext->nAutobootTimeoutLimit = 60;
               }

               nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext,
                  DSL_MIN_FW_VERSION_PTM, DSL_FW_ANNEX_A, &nFwVer);

               if ((nErrCode == DSL_SUCCESS) && (nFwVer >= DSL_VERSION_EQUAL))
               {
                  nErrCode = DSL_DRV_DANUBE_SystemInterfaceStatusUpdate(pContext);
               }
               break;
#endif
#if 1
			/* Sercomm add for SYNC LED */
			{
				DSL_Error_t nErrCode = DSL_SUCCESS;
		   		DSL_uint16_t nData = 0;

			         /*  Let FW to handle ADSL Link LED */
				#if defined (INCLUDE_DSL_CPE_API_DANUBE)
			         nData=0x0a03;
				#elif defined (INCLUDE_DSL_CPE_API_AMAZON_SE)
			         nData=0x0901;
				#endif
			         if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,DSL_CMV_ADDRESS_INFO_GPIO_CONTROL, 4, 1, &nData) != DSL_SUCCESS)
				{
					DSL_DEBUG(DSL_DBG_ERR,(pContext, "DSL: ERROR - CMV send failed!"DSL_CRLF));
					nErrCode = DSL_ERR_MSG_EXCHANGE;
				}
			}
#endif
               break;

            case DSL_LINESTATE_FULL_INIT:
               if (pContext->bGotFullInit == DSL_FALSE)
               {
                  pContext->bGotFullInit = DSL_TRUE;
#if defined(INCLUDE_DSL_PM)
                  DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
                     pDevCtx->data.lineInitCounters.nFullInits,
                     pContext->pDevCtx->data.lineInitCounters.nFullInits + 1);
#endif /* defined(INCLUDE_DSL_PM)*/
               }
               break;

            case DSL_LINESTATE_FASTRETRAIN:
#ifdef HAS_TO_BE_CLARIFIED
                     if (pContext->bGotShortInitResponse == DSL_FALSE)
                     {
                        pContext->bGotShortInitResponse = DSL_TRUE;
                        DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
                           pDevCtx->data.lineInitCounters.nShortInits,
                           pContext->pDevCtx->data.lineInitCounters.nShortInits + 1);
                     }
               break;
#endif /* HAS_TO_BE_CLARIFIED*/

            case DSL_LINESTATE_LOOPDIAGNOSTIC_ACTIVE:
               nErrCode = DSL_DRV_AutobootStateSet(
                             pContext,
                             DSL_AUTOBOOTSTATE_DIAGNOSTIC,
                             DSL_AUTOBOOT_DIAGNOSTIC_POLL_TIME);

               DSL_CTX_WRITE_SCALAR(pContext, nErrCode, bLoopDiagnosticsCompleted, DSL_FALSE);
               pContext->nAutobootTimeoutLimit = 120;
               if (pContext->bGotResponse == DSL_FALSE)
               {
                  pContext->bGotResponse = DSL_TRUE;
                  pContext->autobootStartTime = DSL_DRV_TimeMSecGet();

                  /* Set LINIT value to 'LINIT_SUCCESSFUL' here
                     because activation procedure
                     was finished without errors */
                  DSL_DRV_HandleLinitValue(pContext, LINIT_SUCCESSFUL, LINIT_SUB_NONE);
               }

               /* In case of DIAGNOSTIC mode use this state to identify FULL_INIT*/
               if (pContext->bGotFullInit == DSL_FALSE)
               {
                  pContext->bGotFullInit = DSL_TRUE;
#if defined(INCLUDE_DSL_PM)
                  DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
                     pDevCtx->data.lineInitCounters.nFullInits,
                     pContext->pDevCtx->data.lineInitCounters.nFullInits + 1);
#endif /* defined(INCLUDE_DSL_PM)*/
               }
               break;
            case DSL_LINESTATE_LOOPDIAGNOSTIC_COMPLETE:
               DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
                  bLoopDiagnosticsCompleted, DSL_TRUE);
               nErrCode = DSL_DRV_DANUBE_XTUSystemEnablingStatusUpdate(pContext);
               if (nErrCode != DSL_SUCCESS)
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - Could not "
                     "read current mode!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               }

               nErrCode = DSL_DRV_DANUBE_G997_LoopDiagnosticCompleted(pContext);
               if (nErrCode != DSL_SUCCESS)
               {
                  DSL_DEBUG(DSL_DBG_WRN, (pContext, "DSL[%02d]: WARNING - Could not "
                     "complete loop diagnostic mode!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
               }
            /* Pass through */
            case DSL_LINESTATE_IDLE:
#if defined(INCLUDE_DSL_PM)
               if ( (pContext->bGotFullInit == DSL_TRUE) &&
                    (nPrevLineState != DSL_LINESTATE_LOOPDIAGNOSTIC_ACTIVE))
               {
                  DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
                     pDevCtx->data.lineInitCounters.nFailedFullInits,
                     pContext->pDevCtx->data.lineInitCounters.nFailedFullInits + 1);
               }
#endif /* defined(INCLUDE_DSL_PM)*/

               /* Get Modem Fail Reason*/
               nErrCode = DSL_DRV_DANUBE_FailReasonGet( pContext );
               if( nErrCode != DSL_SUCCESS )
               {
                  DSL_DEBUG(DSL_DBG_WRN, (pContext, "DSL[%02d]: WARNING - Can't get "
                     "modem Fail reason!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               }

               nErrCode = DSL_DRV_AutobootStateSet(
                             pContext,
                             DSL_AUTOBOOTSTATE_RESTART,
                             DSL_AUTOBOOT_TRAINING_POLL_TIME);
               break;
            default:
               DSL_DEBUG(DSL_DBG_ERR, (pContext,
                        "DSL[%02d]: ERROR - Unexpected line state (%X) while training!"DSL_DRV_CRLF,
                        DSL_DEV_NUM(pContext), nLineState));
                        
               nErrCode = DSL_ERROR;
               
               break;
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleTraining, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
/*
DSL_Error_t DSL_DRV_DEV_AutobootHandleRestart(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "IN - DSL_DRV_DEV_AutobootHandleRestart"DSL_DRV_CRLF));


   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "OUT - DSL_DRV_DEV_AutobootHandleRestart, retCode=%d" DSL_DRV_CRLF, nErrCode));

   return nErrCode;
}
*/

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_AutobootHandleException(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_DSL_CPE_TRACE_BUFFER
   DSL_uint32_t i;
   DSL_uint16_t nVal;
#endif

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_AutobootHandleException"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   /* Get Modem Fail Reason*/
   nErrCode = DSL_DRV_DANUBE_FailReasonGet( pContext );
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_WRN, (pContext, "DSL[%02d]: WARNING - Can't get "
         "modem Fail reason!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_LineFailuresUpdate(pContext);
   }

#ifdef INCLUDE_DSL_CPE_TRACE_BUFFER
   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
      
      return DSL_ERROR;
   }

   for (i = 0; i < DSL_DEV_SHOWTIME_EVENT_LOGGING_BUFFER_LENGTH; i++)
   {
      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_DATA_LOGGING_BUFFER, i, 1, &nVal) != DSL_SUCCESS)
      {
         nErrCode = DSL_ERR_MSG_EXCHANGE;
         break;
      }

      pContext->loggingBuffer[i] = nVal;
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: DEBUG:1:: showtime event logging data (after DSL_DRV_DANUBE_CmvRead() calls):",
      DSL_DEV_NUM(pContext)));
   for (i = 0; i < DSL_DEV_SHOWTIME_EVENT_LOGGING_BUFFER_LENGTH; i++)
   {
      if (i % 10 == 0) DSL_DEBUG(DSL_DBG_MSG, (pContext, DSL_DRV_CRLF));
      DSL_DEBUG(DSL_DBG_MSG, (pContext, "0x%04X ", pContext->loggingBuffer[i]));
   }
   DSL_DEBUG(DSL_DBG_MSG, (pContext, DSL_DRV_CRLF DSL_DRV_CRLF));

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   /* Inform upper software about a showtime event data is ready */
   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_EventGenerate(pContext, 0, DSL_ACCESSDIR_NA, DSL_XTUDIR_NA,
         DSL_EVENT_S_SHOWTIME_LOGGING, DSL_NULL, 0);
   }
#endif

   /* Update Power Management Status*/
   nErrCode = DSL_DRV_DANUBE_PowerStatusSet(pContext, DSL_G997_PMS_L3);
   if(nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Power Management State set failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_AutobootHandleException, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_DeviceInit(
   DSL_Context_t *pContext,
   DSL_Init_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_DeviceInit"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);

   /* Initialize possible XTSE configurations*/
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, xtsePoss[1-1], 0x17);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, xtsePoss[2-1], 0x01);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, xtsePoss[3-1], 0x14);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, xtsePoss[4-1], 0x51);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, xtsePoss[5-1], 0x4C);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, xtsePoss[6-1], 0x45);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, xtsePoss[7-1], 0x05);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, xtsePoss[8-1], 0x00);

   memcpy( &(pContext->pDevCtx->data.sysCIF),
           &(pData->data.nDeviceCfg.sysCIF),
           sizeof(DSL_SystemInterfaceConfigData_t) );

   DSL_DRV_DANUBE_LowLevelVersionsUpdate(pContext);

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_DeviceInit, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_ModemReset(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_ModemReset"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_DRV_DANUBE_ResetDFE(pContext);
   DSL_DRV_DANUBE_HaltDFE(pContext);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_ModemReset, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));
      
   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_LinkReset(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   
   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_LinkReset"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Update Power Management Status*/
   nErrCode = DSL_DRV_DANUBE_PowerStatusSet(pContext, DSL_G997_PMS_L3);
   if(nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Power Management State set failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_LinkReset, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_LinkActivate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal = 0;
#if 0
   DSL_LineStateValue_t nLineState;

   /* Get device line state*/
   nErrCode = DSL_DRV_DEV_LineStateGet(pContext, &nLineState);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Line state get failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   if (nLineState != DSL_LINESTATE_IDLE)
   {
      DSL_DEBUG( DSL_DBG_ERR,
        (pContext, "DSL[%02d]: ERROR - Expecting IDLE line state while Link "
        "Activation, actual state=%d!"DSL_DRV_CRLF,
        DSL_DEV_NUM(pContext), nLineState));
        
      return DSL_ERROR;
   }
#endif

   if (DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext, DSL_DRV_CRLF"DSL[%02d]: getting mei driver semaphore "
         "failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
      nErrCode = DSL_ERROR;
   }
   else
   {
      nVal = DSL_CMV_ADDRESS_ARG_CNTL_MODEM_START;
      if ( DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_CNTL,
           DSL_CMV_ADDRESS_CNTL_MODEM_CONTROL, 0, 1, &nVal)
           != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - CNTL(write) modem "
            "control CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Link Activation Failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      /* SILENT line state reached*/
      DSL_DRV_LineStateSet(pContext, DSL_LINESTATE_SILENT);
   }
  
   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_BspIoctl(
   DSL_Context_t *pContext,
   DSL_uint_t nCommand,
   DSL_uint32_t *nArg)
{
   DSL_DEV_Handle_t dev;
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_ERR_CODE();

   nErrCode = DSL_ERROR;
   dev = DSL_DRV_LowDeviceGet(pContext->pDevCtx);
   if (dev != NULL)
   {
      if ( DSL_BSP_KernelIoctls(dev, nCommand, (unsigned long)nArg) == 0 )
      {
         nErrCode = DSL_SUCCESS;
      }
   }

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_FirmwareVersionGet(
   DSL_Context_t *pContext,
   DSL_char_t    *pString)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_FwVersion_t *pVer = DSL_NULL;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_FirmwareVersionGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_POINTER(pContext, pString);
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERROR;
   }

   if (nErrCode >= DSL_SUCCESS)
   {
      pVer = &(pContext->pDevCtx->data.version.fwVersion);
      if (pVer == DSL_NULL)
      {
         nErrCode = DSL_ERR_POINTER;
      }
      else
      {
         if (pVer->bValid == DSL_TRUE)
         {
            DSL_DRV_snprintf(pString, MAX_INFO_STRING_LEN, "%hu.%hu.%hu.%hu.%hu.%hu",
               pVer->nMajorVersion, pVer->nMinorVersion, pVer->nExternalVersion,
               pVer->nInternalVersion, pVer->nReleaseState,
               pVer->nApplicationType);

            DSL_DEBUG(DSL_DBG_MSG, (pContext,
               "DSL[%02d]: DSL_DEV_FirmwareVersionGet: %s" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), pString));
         }
         else
         {
            DSL_DEBUG(DSL_DBG_ERR, (pContext,
               "DSL[%02d]: ERROR - No firmware version available" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), pString));
            
            nErrCode = DSL_WRN_DEVICE_NO_DATA;
         }
      }
   }

   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DRV_snprintf(pString, MAX_INFO_STRING_LEN, "n/a");
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_FirmwareVersionGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_ChipHardwareVersionGet(
   DSL_Context_t *pContext,
   DSL_char_t    *pString)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_ChipVersion_t *pVer = DSL_NULL;

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: DSL_DEV_ChipVersionStringGet: "
      "This function is not applicable."DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pString);
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERROR;
   }

   if (nErrCode >= DSL_SUCCESS)
   {
      pVer = &(pContext->pDevCtx->data.version.chipVersion);
      if (pVer == DSL_NULL)
      {
         nErrCode = DSL_ERR_POINTER;
      }
      else
      {
         if (pVer->bValid == DSL_TRUE)
         {
            DSL_DRV_snprintf(pString, MAX_INFO_STRING_LEN, "%hu.%hu",
               pVer->nMajorVersion, pVer->nMinorVersion);

            DSL_DEBUG(DSL_DBG_MSG, (pContext,
               "DSL: DSL_DEV_ChipHardwareVersionGet: %s" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), pString));
         }
         else
         {
            DSL_DEBUG(DSL_DBG_ERR, (pContext,
               "DSL[%02d]: ERROR - No chip version available" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), pString));
            
            nErrCode = DSL_WRN_DEVICE_NO_DATA;
         }
      }
   }

   if (nErrCode != DSL_SUCCESS)
   {
      strcpy(pString, "n/a");
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_DriverVersionGet(
   DSL_Context_t *pContext,
   DSL_char_t    *pString)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_MeiSwVersion_t *pVer = DSL_NULL;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_DriverVersionGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_POINTER(pContext, pString);
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      nErrCode = DSL_ERROR;
   }

   if (nErrCode >= DSL_SUCCESS)
   {
      pVer = &(pContext->pDevCtx->data.version.meiSwVersion);
      if (pVer == DSL_NULL)
      {
         nErrCode = DSL_ERR_POINTER;
      }
      else
      {
         if (pVer->bValid == DSL_TRUE)
         {
            DSL_DRV_snprintf(pString, MAX_INFO_STRING_LEN, "%hu.%hu.%hu",
               pVer->nMajorVersion, pVer->nMinorVersion, pVer->nBuild);

            DSL_DEBUG(DSL_DBG_MSG, (pContext,
               "DSL[%02d]: DSL_DEV_DriverVersionGet: %s" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), pString));
         }
         else
         {
            DSL_DEBUG(DSL_DBG_ERR, (pContext,
               "DSL[%02d]: ERROR - No low level driver version available!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), pString));
            
            nErrCode = DSL_WRN_DEVICE_NO_DATA;
         }
      }
   }

   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DRV_snprintf(pString, MAX_INFO_STRING_LEN, "n/a");
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t DSL_DRV_DEV_LineStateGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_LineStateValue_t *pnLineState)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nLineState;
   DSL_LineStateValue_t nCurrLineState = DSL_LINESTATE_UNKNOWN;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_LineStateGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_POINTER(pContext, pnLineState);
   DSL_CHECK_ERR_CODE();

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrLineState);
   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Could not lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
                        DSL_CMV_ADDRESS_STAT_MACRO_STATE, 0, 1,
                        &nLineState);

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: Got %d state"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nLineState));

   switch (nLineState)
   {
      case DSL_DEV_STAT_InitState:
         *pnLineState = DSL_LINESTATE_NOT_INITIALIZED;
         break;
      case DSL_DEV_STAT_FailState:
         *pnLineState = DSL_LINESTATE_EXCEPTION;
         break;
      case DSL_DEV_STAT_IdleState:
         if (nCurrLineState == DSL_LINESTATE_LOOPDIAGNOSTIC_ACTIVE)
         {
            *pnLineState = DSL_LINESTATE_LOOPDIAGNOSTIC_COMPLETE;
         }
         else
         {
            *pnLineState = DSL_LINESTATE_IDLE;
         }
         break;
      case DSL_DEV_STAT_GhsState:
         *pnLineState = DSL_LINESTATE_HANDSHAKE;
         break;
      case DSL_DEV_STAT_FullInitState:
         *pnLineState = DSL_LINESTATE_FULL_INIT;
         break;
      case DSL_DEV_STAT_ShowTimeState:
         if (nCurrLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC)
         {
            *pnLineState = DSL_LINESTATE_SHOWTIME_NO_SYNC;
         }
         else
         {
            *pnLineState = nCurrLineState;
         }
         break;
      case DSL_DEV_STAT_FastRetrainState:
         *pnLineState = DSL_LINESTATE_FASTRETRAIN;
         break;
      case DSL_DEV_STAT_LoopDiagMode:
         *pnLineState = DSL_LINESTATE_LOOPDIAGNOSTIC_ACTIVE;
         break;
      case DSL_DEV_STAT_ReadyState:
         *pnLineState = DSL_LINESTATE_SILENT;
         break;
      case DSL_DEV_STAT_ShortInit:
         *pnLineState = DSL_LINESTATE_FASTRETRAIN;
         break;
      /* [TD-20070118] Usage of fixed value (0xB) is a workaround for missing
         state of T.1413 (STAT 0 0 11). This state is an intermediate one which
         will be handled as handshake state to avoid restart of autoboot
         handling. */
      case 0x000B:
         *pnLineState = DSL_LINESTATE_HANDSHAKE;
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: T.1413 handled as handshake!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         break;
      case DSL_DEV_STAT_QuietState:
      default:
         DSL_DEBUG( DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - DSL_LINESTATE_UNKNOWN will be returned!"
            DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
         *pnLineState = DSL_LINESTATE_UNKNOWN;
         
         break;
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DEV_LineStateGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_LowLevelConfigurationSet(
   DSL_Context_t *pContext,
   DSL_DeviceLowLevelConfig_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_LowLevelConfigurationSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   nErrCode = DSL_ERR_NOT_IMPLEMENTED;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_LowLevelConfigurationSet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_UtopiaAddressConfigSet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_PhyAddressConfigData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_UtopiaAddressConfigSet(nChannel=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nChannel));

   DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
      pDevCtx->data.nUtopiaAddr[nChannel], pData->nPhyAdr);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_UtopiaAddressConfigSet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_UtopiaAddressConfigGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_PhyAddressConfigData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_UtopiaAddressConfigGet, (nChannel=%d)"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nChannel));

   DSL_CTX_READ_SCALAR(pContext, nErrCode,
      pDevCtx->data.nUtopiaAddr[nChannel], pData->nPhyAdr);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_UtopiaAddressConfigGet, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return (nErrCode);
}

DSL_Error_t DSL_DRV_DEV_XtseSettingsCheck(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN DSL_uint8_t *pXTSE)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_XtseSettingsCheck"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   /* Check if G992.3 Annex A mode specified in case of Annex L*/
   if (((pXTSE[5-1] & XTSE_5_04_L_3_NO) || (pXTSE[5-1] & XTSE_5_03_L_3_NO)) &&
       (!(pXTSE[3-1] & XTSE_3_03_A_3_NO)))
   {
      DSL_DEBUG(DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Annex L mode is only supported in conjunction with G992.3 Annex A"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      /* Set G992.3 Annex A mode*/
      pXTSE[3-1] |= XTSE_3_03_A_3_NO;
      
      nErrCode = DSL_WRN_INCONSISTENT_XTSE_CONFIGURATION;
   }

   /* Check if both narrow and wide Annex L modes are set*/   
   if ((pXTSE[5-1] & (XTSE_5_04_L_3_NO | XTSE_5_03_L_3_NO)) &&
       ((pXTSE[5-1] & (XTSE_5_04_L_3_NO | XTSE_5_03_L_3_NO)) != (XTSE_5_04_L_3_NO | XTSE_5_03_L_3_NO)))
   {
      DSL_DEBUG(DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Setting only both (narrow, wide) Annex L allowed"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      /* Set both Annex L modes*/
      pXTSE[5-1] |= (XTSE_5_04_L_3_NO | XTSE_5_03_L_3_NO);

      nErrCode = DSL_WRN_INCONSISTENT_XTSE_CONFIGURATION;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_XtseSettingsCheck"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_ResetDFE(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_ResetDFE"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nErrCode = DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_RESET, DSL_NULL);

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_ResetDFE"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_HaltDFE(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_HaltDFE"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR -  Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nErrCode = DSL_DRV_DEV_BspIoctl(pContext, DSL_FIO_BSP_HALT, DSL_NULL);

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HaltDFE"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_Error_t
DSL_DRV_DEV_InitDataPrepare(
   DSL_Context_t *pContext,
   DSL_boolean_t bIsInKernel,
   DSL_Init_t *pInit)
{
   /* Nothing to be done for Danube */
   return DSL_SUCCESS;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_void_t
DSL_DRV_DEV_InitDataFree(DSL_Init_t *pInit)
{
   /* Nothing to be done for Danube */
   return;
}

static DSL_Error_t DSL_DRV_DANUBE_ModeControlSet(
   DSL_Context_t *pContext,
   DSL_FirmwareAnnex_t nAnnex,
   DSL_uint16_t *pnOptn,
   DSL_uint16_t *pnOptn1)
{
   DSL_uint16_t nOptn = 0, nOptn1 = 0;
   DSL_uint32_t i;
#ifdef HAS_TO_BE_CLARIFIED
   DSL_uint32_t nBitCnt;
#endif /* HAS_TO_BE_CLARIFIED*/
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_ModeControlSet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pnOptn);
   DSL_CHECK_POINTER(pContext, pnOptn1);
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: XTSE Octets: ",
      DSL_DEV_NUM(pContext)));
   
   for (i = 0; i < DSL_G997_NUM_XTSE_OCTETS; i++)
   {
      DSL_DEBUG(DSL_DBG_MSG, (pContext, "%02X ", pContext->xtseCfg[i]));
   }
   DSL_DEBUG(DSL_DBG_MSG, (pContext, ""DSL_DRV_CRLF));

   if ((nAnnex == DSL_FW_ANNEX_A) || (nAnnex == DSL_FW_ANNEX_UNKNOWN))
   {
      nOptn1 |= (pContext->xtseCfg[6] & XTSE_7_03_M_5_NO) ? 4 : 0;
      nOptn  |= (pContext->xtseCfg[4] & XTSE_5_07_M_3_NO) ? 1 << 13 : 0;
      nOptn  |= (pContext->xtseCfg[5] & XTSE_6_01_A_5_NO) ? 1 << 15 : 0;
      nOptn  |= (pContext->xtseCfg[1] & XTSE_2_01_A_2_NO) ? 1 << 1 : 0;
      nOptn  |= (pContext->xtseCfg[0] & XTSE_1_03_A_1_NO) ? 1 << 2 : 0;
      nOptn1 |= (pContext->xtseCfg[5] & XTSE_6_07_I_5_NO) ? 1 : 0;
      nOptn  |= (pContext->xtseCfg[3] & XTSE_4_05_I_3_NO) ? 1 << 10 : 0;
      nOptn  |= ((pContext->xtseCfg[2] & XTSE_3_03_A_3_NO) ||
                 (pContext->xtseCfg[4] & XTSE_5_03_L_3_NO) ||
                 (pContext->xtseCfg[4] & XTSE_5_04_L_3_NO)) << 8;
      nOptn  |= ((pContext->xtseCfg[4] & XTSE_5_03_L_3_NO) ||
                 (pContext->xtseCfg[4] & XTSE_5_04_L_3_NO)) << 12;
   }

   if ((nAnnex == DSL_FW_ANNEX_B) || (nAnnex == DSL_FW_ANNEX_UNKNOWN))
   {
      nOptn  |= (pContext->xtseCfg[5] & XTSE_6_03_B_5_NO) ? 1 << 14 : 0;
      nOptn  |= (pContext->xtseCfg[2] & XTSE_3_05_B_3_NO) ? 1 << 9 : 0;
      nOptn  |= (pContext->xtseCfg[0] & XTSE_1_05_B_1_NO) ? 1 << 3 : 0;
      nOptn1 |= (pContext->xtseCfg[6] & XTSE_7_01_J_5_NO) ? 2 : 0;
      nOptn  |= (pContext->xtseCfg[3] & XTSE_4_07_J_3_NO) ? 1 << 11 : 0;
   }

   nOptn  |= pContext->xtseCfg[0] & XTSE_1_01_A_T1_413;

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   if (nOptn == 0 && nOptn1 == 0)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - There are no possible modes "
         "enabled in the configuration. Please check it!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_ERR_INVALID_PARAMETR;
   }

/*
   [Joerg Dauerer]: The statement in the CMV spec regarding the limitations in multimode is wrong.
   The current FW supports multimode for all relevant annexes (i.e. A/L/I/M or B/M/J).
   [KAv]: This code has to be removed after CMV specification update
*/
#ifdef HAS_TO_BE_CLARIFIED
   /* Check modes */
   if ((nOptn & (1 << 2)) &&
       (nOptn & (1 << 8)) &&
       (nOptn & (1 << 10)) &&
       (nOptn & (1 << 13)) &&
       (nOptn & (1 << 15)) &&
       (nOptn1 & (1)) &&
       (nOptn1 & (1 << 2)))
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Turning on all modes "
         "for annexes A/I/M at the same moment is not possible!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nBitCnt  = (nOptn1 & (1 << 1)) ? 1 : 0;
   nBitCnt += (nOptn1 & (1 << 2)) ? 1 : 0;
   for (i = 0; i < 16; i++)
   {
      if (i == 3 || i == 9 || i == 11 || i == 13 || i == 14)
      {
         if (nOptn & (1 << i))
         {
            nBitCnt++;
         }
      }
   }

   if (nBitCnt > 2)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Turning on more then 2 groups "
         "for annexes B/J/M at the same moment is not possible!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* HAS_TO_BE_CLARIFIED*/

   *pnOptn = nOptn;
   *pnOptn1 = nOptn1;

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_G997_ModeControlSet"
      ", retCode=0, nOptn=%x, nOptn1=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nOptn, nOptn1));

   return DSL_SUCCESS;
}

static DSL_uint32_t DSL_DRV_DANUBE_XTUSystemEnablingConfigSend(
   DSL_IN DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal;
   DSL_FirmwareAnnex_t nAnnex = DSL_FW_ANNEX_UNKNOWN;
   DSL_uint16_t nOptn0, nOptn7;

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: IN - DSL_DRV_DANUBE_XTUSystemEnablingConfigSend"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
      DSL_CMV_ADDRESS_INFO_VERSION, 1, 1, &nVal);

   if (nErrCode == DSL_SUCCESS)
   {
      nAnnex = (nVal>>8) & 0x3f;
      switch (nAnnex)
      {
         case DSL_FW_ANNEX_A:
               DSL_DEBUG(DSL_DBG_MSG, (pContext,
                  "DSL[%02d]: AnnexA FW is detected"DSL_DRV_CRLF, 
                  DSL_DEV_NUM(pContext)));
                  
         case DSL_FW_ANNEX_B:
            if (nAnnex == DSL_FW_ANNEX_B)
            {
               DSL_DEBUG(DSL_DBG_MSG, (pContext,
                  "DSL[%02d]: AnnexB FW is detected"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
            }
            nErrCode = DSL_DRV_DANUBE_ModeControlSet(pContext, nAnnex,
               &nOptn0, &nOptn7);
            if (nErrCode != DSL_SUCCESS)
            {
               DSL_DEBUG(DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - retreiving operation mode"
                  " information failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            }
            break;
         default:
            DSL_DEBUG(DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Unknown Annex of "
               "the Firmware (0x%X)!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nAnnex));
               
            nErrCode = DSL_DRV_AutobootStateSet(
                          pContext,
                          DSL_AUTOBOOTSTATE_EXCEPTION,
                          DSL_AUTOBOOT_IDLE_POLL_TIME);

            nErrCode = DSL_ERROR;
            break;
      }

      if (nErrCode == DSL_SUCCESS)
      {
         if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_OPTN,
            DSL_CMV_ADDRESS_OPTN_MODECONTROL, 0, 1, &nOptn0 ) != DSL_SUCCESS)
         {
            DSL_DEBUG(DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - OPTN mode control CMV "
               "send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
            nErrCode = DSL_ERROR;
         }
         else
         {
            if (DSL_DRV_DANUBE_CmvWrite(pContext,DSL_CMV_GROUP_OPTN,
               DSL_CMV_ADDRESS_OPTN_MODECONTROL1, 0, 1, &nOptn7)
               != DSL_SUCCESS)
            {
               DSL_DEBUG(DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - OPTN mode control "
                  "CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                  
               nErrCode = DSL_ERROR;
            }
         }
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DANUBE_XTUSystemEnablingConfigSend, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

static DSL_uint32_t DSL_DRV_DANUBE_XTUSystemEnablingStatusUpdate(
   DSL_IN_OUT DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_XTUSystemEnablingStatusUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
      
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   DSL_DRV_MemSet(pContext->xtseCurr, 0, DSL_G997_NUM_XTSE_OCTETS);

   if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
      DSL_CMV_ADDRESS_STAT_MODE, 0, 1, &nVal)) == DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_MSG, (pContext,
         "DSL[%02d]: STAT1 = %04X"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nVal));
         
      pContext->pDevCtx->data.nAdslMode = nVal;
      
      if (nVal == 0)
      {
         if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
            DSL_CMV_ADDRESS_STAT_MODE1, 0, 1, &nVal)) == DSL_SUCCESS)
         {
            DSL_DEBUG(DSL_DBG_MSG, (pContext,
               "DSL[%02d]: STAT17 = %04X"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nVal));
               
            pContext->pDevCtx->data.nAdslMode1 = nVal;
            
            switch (nVal)
            {
               case DSL_CMV_STAT_MODE_G992_5_I:
                  pContext->xtseCurr[5] |= XTSE_6_07_I_5_NO;
                  pContext->nXDslMode = DSL_XDSLMODE_G_992_5;
                  pContext->nAnnexType = DSL_ANNEX_I;
                  break;
               case DSL_CMV_STAT_MODE_G992_5_J:
                  pContext->xtseCurr[6] |= XTSE_7_01_J_5_NO;
                  pContext->nXDslMode = DSL_XDSLMODE_G_992_5;
                  pContext->nAnnexType = DSL_ANNEX_J;
                  break;
               case DSL_CMV_STAT_MODE_G992_5_M:
                  pContext->xtseCurr[6] |= XTSE_7_03_M_5_NO;
                  pContext->nXDslMode = DSL_XDSLMODE_G_992_5;
                  pContext->nAnnexType = DSL_ANNEX_M;
                  break;
               case 0:
               default:
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - Unknown mode "
                     "is returned in the CMV!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                     
                  break;
            }
         }
         else
         {
            DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - Read current mode "
               "CMV send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         }
      }
      else
      {
         pContext->pDevCtx->data.nAdslMode1 = 0;
         switch (nVal)
         {
            case DSL_CMV_STAT_MODE_T1_413:
               pContext->xtseCurr[0] |= XTSE_1_01_A_T1_413;
               pContext->nXDslMode = DSL_XDSLMODE_T1_413;
               pContext->nAnnexType = DSL_ANNEX_A;
               break;
            case DSL_CMV_STAT_MODE_G992_1_A:
               pContext->xtseCurr[0] |= XTSE_1_03_A_1_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_1;
               pContext->nAnnexType = DSL_ANNEX_A;
               break;
            case DSL_CMV_STAT_MODE_G992_1_B:
               pContext->xtseCurr[0] |= XTSE_1_05_B_1_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_1;
               pContext->nAnnexType = DSL_ANNEX_B;
               break;
            case DSL_CMV_STAT_MODE_G992_2_A:
               pContext->xtseCurr[1] |= XTSE_2_01_A_2_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_1;
               pContext->nAnnexType = DSL_ANNEX_B;
               break;
            case DSL_CMV_STAT_MODE_G992_3_A:
               pContext->xtseCurr[2] |= XTSE_3_03_A_3_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_3;
               pContext->nAnnexType = DSL_ANNEX_A;
               break;
            case DSL_CMV_STAT_MODE_G992_3_B:
               pContext->xtseCurr[2] |= XTSE_3_05_B_3_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_3;
               pContext->nAnnexType = DSL_ANNEX_B;
               break;
            case DSL_CMV_STAT_MODE_G992_3_I:
               pContext->xtseCurr[3] |= XTSE_4_05_I_3_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_3;
               pContext->nAnnexType = DSL_ANNEX_I;
               break;
            case DSL_CMV_STAT_MODE_G992_3_J:
               pContext->xtseCurr[3] |= XTSE_4_07_J_3_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_3;
               pContext->nAnnexType = DSL_ANNEX_J;
               break;
            case DSL_CMV_STAT_MODE_G992_3_L:
               if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
                  DSL_CMV_ADDRESS_STAT_MISC, 0, 1, &nVal)) == DSL_SUCCESS)
               {
                  DSL_DEBUG(DSL_DBG_MSG, (pContext,
                     "DSL[%02d]: STAT1 = %04X"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nVal));

                  if (nVal & DSL_CMV_STAT_MISC_ANNEX_L_MASK_1)
                  {
                     pContext->xtseCurr[4] |= XTSE_5_03_L_3_NO;
                  }
                  else if (nVal & DSL_CMV_STAT_MISC_ANNEX_L_MASK_2)
                  {
                     pContext->xtseCurr[4] |= XTSE_5_04_L_3_NO;
                  }
                  else
                  {
                     DSL_DEBUG(DSL_DBG_WRN, (pContext, "DSL[%02d]: WARNING - Unkhown Annex L "
                     "Mask received!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
                  }
               }
               else
               {
                  DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - STAT1 CMV "
                  "send failed!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               }
            
               pContext->nXDslMode = DSL_XDSLMODE_G_992_3;
               pContext->nAnnexType = DSL_ANNEX_L;
               break;
            case DSL_CMV_STAT_MODE_G992_3_M:
               pContext->xtseCurr[4] |= XTSE_5_07_M_3_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_3;
               pContext->nAnnexType = DSL_ANNEX_M;
               break;
            case DSL_CMV_STAT_MODE_G992_5_B:
               pContext->xtseCurr[5] |= XTSE_6_03_B_5_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_5;
               pContext->nAnnexType = DSL_ANNEX_B;
               break;
            case DSL_CMV_STAT_MODE_G992_5_A:
               pContext->xtseCurr[5] |= XTSE_6_01_A_5_NO;
               pContext->nXDslMode = DSL_XDSLMODE_G_992_5;
               pContext->nAnnexType = DSL_ANNEX_A;
               break;
            case 0:
            default:
               DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - Unknown mode "
                  "is returned in the CMV!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               break;
         }
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);
   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_XTUSystemEnablingStatusUpdate, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DANUBE_ShowtimeReached(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LatencyPath_t nLPath = DSL_LATENCY_DISABLED;
   DSL_uint16_t nVal;
   DSL_uint32_t nRate = 0, nFast = 0, nIntl = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_ShowtimeReached"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }


/*$$*/
#if 0
   meierr = DSL_BSP_ReadAdslMode ();

#ifdef DSL_BSP_DYING_GASP_SUPPORT
   meierr = DSL_BSP_GetDyingGaspAddr (adsl_dev);
#endif /* DSL_BSP_DYING_GASP_SUPPORT */

#ifdef INCLUDE_DSL_ADSL_MIB
   mei_mib_adsl_link_up ();
#endif
#endif

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nUAS, nVal);
   if ((nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_PLAM,
      DSL_CMV_ADDRESS_PLAM_NEAR_END_UASL_CNT, 0, 1, &nVal))
      != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - CMV fail, Group 7 Address 10 Index 0!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      DSL_DRV_LineStateSet(pContext, DSL_LINESTATE_SHOWTIME_TC_SYNC);
      if ( (nErrCode != DSL_DRV_DANUBE_ChannelRateUpdate(pContext)) != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Could not update data rates!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      }
      else
      {
         /* \note Review implementation in case of using dual latency!
                  By now the interface for low level driver is used
                  to differentiate between fast and interleaved for ADSL1
                  only. In case of ADSL2/2+ datarate will be always given
                  by "nIntl" value. */
         if (DSL_DRV_DANUBE_ActLatencyGet(pContext, 0, &nLPath) >= DSL_SUCCESS)
         {
            DSL_CTX_READ_SCALAR(pContext, nErrCode, nChannelActualDataRateUs[0],
               nRate);

            if (nLPath == DSL_LATENCY_IP_LP0)
            {
               nIntl = nRate;
            }
            else
            {
               nFast = nRate;
            }

            DSL_DEBUG( DSL_DBG_WRN,
               (pContext, "DSL[%02d]: DSL_BSP_Showtime(nFast=%u, nIntl=%u)"
               DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nFast, nIntl));

            if ( DSL_BSP_Showtime (pContext->pDevCtx->lowHandle, nFast,
               nIntl) != DSL_DEV_MEI_ERR_SUCCESS )
            {
               nErrCode = DSL_ERROR;
            }
         }
      }
   }
   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_ShowtimeReached, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_uint32_t DSL_DRV_DANUBE_ChannelStatusUpdate(
   DSL_IN DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LatencyPath_t nLPath = 0;
   DSL_uint16_t nAddrPath = 0, nDataPath = 0;
   DSL_uint8_t nChannel = 0;
   DSL_AccessDir_t nDirection = DSL_ACCESSDIR_NA;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_ChannelStatusUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   for (nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++)
   {
      if (nChannel == 1)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR -  nChannel = 1 is currently unsupported!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERR_NOT_SUPPORTED;
      }
      else
      {
         for (nDirection = DSL_UPSTREAM; nDirection <= DSL_DOWNSTREAM; nDirection++)
         {
            if (nDirection == DSL_UPSTREAM)
            {
               nAddrPath = DSL_CMV_ADDRESS_STAT_PATH_US;
            }
            else if (nDirection == DSL_DOWNSTREAM)
            {
               nAddrPath = DSL_CMV_ADDRESS_STAT_PATH_DS;
            }

            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT, nAddrPath,
               0, 1, &nDataPath);
            if (nErrCode == DSL_SUCCESS)
            {
               if (nDataPath & 0x1)
               {
                  nLPath = DSL_LATENCY_IP_LP0;
               }
               else if (nDataPath & 0x2)
               {
                  nLPath = DSL_LATENCY_FP_LP1;
               }
               else
               {
                  nLPath = DSL_LATENCY_DISABLED;
               }

               DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nLPath[nChannel], nLPath);
            }
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_ChannelStatusUpdate, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_intern_g997.h
*/
DSL_uint32_t DSL_DRV_DANUBE_ChannelStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN DSL_uint16_t nChannel,
   DSL_IN DSL_AccessDir_t nDirection,
   DSL_IN_OUT DSL_G997_ChannelStatusData_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nAddrProps = 0, nOffset = 0, nData = 0;
   /* $$TD: The handling for the latency path/bearer channel status update
      should be included in a central place after showtime is reached and
      stored within context (channel specific) to be used here for example.
      NOTE:
      Update for all bearer channels should be done together in this separate
      function (implementation below handles only bearer channel 0 which is the
      only possible at the moment).
   */
   DSL_LatencyPath_t nLPath = 0;
   DSL_uint32_t nDelay = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_ChannelStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if (nDirection == DSL_UPSTREAM)
   {
      nAddrProps = DSL_CMV_ADDRESS_INFO_LATENCY_PATH_PROPS_US;
   }
   else if (nDirection == DSL_DOWNSTREAM)
   {
      nAddrProps = DSL_CMV_ADDRESS_INFO_LATENCY_PATH_PROPS_DS;
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
        (pContext, "DSL[%02d]: ERROR - Direction was not selected!"DSL_DRV_CRLF,
        DSL_DEV_NUM(pContext)));
        
      return DSL_ERROR;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLPath[nChannel], nLPath);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR -  Could not read latency path configuration!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* Calculate offset according to current cofiguration */
   if (nLPath == DSL_LATENCY_IP_LP0)
   {
      nOffset = 0;
   }
   else if (nLPath == DSL_LATENCY_FP_LP1)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddrProps,
         0, 1, &nOffset);
   }
   else
   {
      DSL_DEBUG(DSL_DBG_ERR,
        (pContext, "DSL[%02d]: ERROR - Latency path (%d) is disabled."DSL_DRV_CRLF,
        DSL_DEV_NUM(pContext), nChannel));
        
      return DSL_ERROR;
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddrProps,
         (nOffset + 2), 1, &nData);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      /* Unit within firmware: (INP * 10), unit within G.997.1: (INP * 2)
         => INP(fw) / 5 = INP (G.997.1) */
      pData->ActualImpulseNoiseProtection = nData / 5;
   }

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO, nAddrProps,
      (nOffset + 1), 1, &nData);

   if (nErrCode == DSL_SUCCESS)
   {
      /* Unit within firmware: (LATENCY * 4), unit within G.997.1: (LATENCY * 100)
         => (LATENCY(fw) * 100) / 4 = LATENCY (G.997.1) */
      nDelay = (nData * 100) / 4;
      pData->ActualInterleaveDelay = nDelay;
   }


   pData->ActualDataRate   = nDirection == DSL_UPSTREAM ?
                             pContext->nChannelActualDataRateUs[nChannel]:
                             pContext->nChannelActualDataRateDs[nChannel];
   pData->PreviousDataRate = nDirection == DSL_UPSTREAM ?
                             pContext->nChannelPreviousDataRateUs[nChannel]:
                             pContext->nChannelPreviousDataRateDs[nChannel];

   return nErrCode;
}

/*
   This function should be called with locked bsp.
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_ChannelRateUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;
   DSL_G997_ChannelStatusData_t chStsData;
   DSL_G997_DataRateThresholdCrossingType_t nDataRateCrossingType;
   DSL_LatencyPath_t nLPath = DSL_LATENCY_DISABLED;
   DSL_uint32_t nActualDataRate = 0, nDataRateShiftThresh = 0;
   DSL_uint32_t nDataRate, nDataRate_remain, nOldDataRate, nKp;
   DSL_uint16_t nRate16[2] = {0};
   DSL_uint16_t nMode = 0, nMode1 = 0;
   DSL_uint16_t nCh;
   DSL_uint16_t nMp, nLp, nTp, nRp, nBpn;
   DSL_boolean_t bPrevDataRateValid = DSL_FALSE;
   DSL_boolean_t bNotifyDataRateShift = {DSL_FALSE};
#ifdef INCLUDE_DSL_ADSL_MIB
   DSL_MIB_ADSL_Thresholds_t nMibThresholds = 0;
#endif

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_ChannelRateUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode, nMode);
   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode1, nMode1);
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

   /* Get the previous data rate valid flag */
   DSL_CTX_READ_SCALAR(pContext, nErrCode, bPrevDataRateValid, bPrevDataRateValid);

   if (nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC &&
      nLineState != DSL_LINESTATE_SHOWTIME_NO_SYNC)
   {
      nErrCode = DSL_ERROR;
   }
   else
   {
      for (nCh = 0; nCh < DSL_CHANNELS_PER_LINE; nCh++)
      {
         if (DSL_DRV_DANUBE_ActLatencyGet(pContext, nCh, &nLPath) < DSL_SUCCESS)
         {
            continue;
         }

         DSL_CTX_READ_SCALAR(pContext, nErrCode, nChannelActualDataRatePrevUs[nCh],
            nOldDataRate);

         /* In case of ADSL1 the values for the data rates can be directly
            read via CMVs */
         if (nMode <= 8 && nMode1 == 0)
         {
            if (nLPath == DSL_LATENCY_IP_LP0)
            {
               nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_RATE,
                  DSL_CMV_ADDRESS_RATE_UsRate, 0, 2, nRate16);
            }
            else if (nLPath == DSL_LATENCY_FP_LP1)
            {
               nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_RATE,
                  DSL_CMV_ADDRESS_RATE_UsRate, 2, 2, nRate16);
            }

            if (nErrCode == DSL_SUCCESS)
            {
               nActualDataRate = (nRate16[1] << 16) + nRate16[0];
            }
         }
         /* In case of ADSL2/2+ the values for data rates has to be calculated
            using framing parameters */
         else
         {
            nMp = nLp = nTp = nRp = nKp = nBpn = nDataRate =
               nDataRate_remain = 0;

            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
               DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_LP, nCh, 1,
               &nLp);

            if (nErrCode == DSL_SUCCESS)
            {
               nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                  DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_RP, nCh, 1,
                  &nRp);
            }

            if (nErrCode == DSL_SUCCESS)
            {
               nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                  DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_MP, nCh, 1,
                  &nMp);
            }

            if (nErrCode == DSL_SUCCESS)
            {
               nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                  DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_TP, nCh, 1,
                  &nTp);
            }

            if (nErrCode == DSL_SUCCESS)
            {
               nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                  DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_KP, 2* nCh, 2,
                  nRate16);
            }

            if (nErrCode == DSL_SUCCESS)
            {
               nBpn = nRate16[0] + nRate16[1];
               nKp = nBpn + 1;

               if (nTp == 0 || (nKp * nMp + nRp) == 0)
               {
                  nDataRate = 0;
               }
               else
               {
                  nDataRate = ((nTp * (nBpn + 1) -
                     1) * nMp * nLp * 4) / (nTp * (nKp * nMp + nRp));

                  nDataRate_remain = ((((nTp * (nBpn + 1) -
                     1) * nMp * nLp * 4) % (nTp * (nKp * nMp + nRp)))
                     * 1000) / (nTp * (nKp * nMp + nRp));
               }

               nActualDataRate = nDataRate * 1000 + nDataRate_remain;
            }
         }

         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRatePrevUs[nCh],
            nActualDataRate);

         /* Update Actual Data Rate for the US direction*/
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRateUs[nCh],
            nActualDataRate);

         if (nOldDataRate != nActualDataRate && nErrCode == DSL_SUCCESS)
         {
            /* Set previous data rate value */
            DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelPreviousDataRateUs[nCh],
               nOldDataRate);

            DSL_DEBUG (DSL_DBG_MSG, (pContext, "DSL[%02d]: Datarate channel %d US datarate"
               "= %d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nActualDataRate));

            nErrCode = DSL_DRV_DANUBE_ChannelStatusGet(pContext, nCh,
               DSL_UPSTREAM, &chStsData);
            if( nErrCode != DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Channel status get failed!" DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
            }
            else
            {
               chStsData.ActualDataRate   = nActualDataRate;
               chStsData.PreviousDataRate = nOldDataRate;

               nErrCode = DSL_DRV_EventGenerate(
                  pContext, nCh, DSL_UPSTREAM, DSL_XTUDIR_NA,
                  DSL_EVENT_S_CHANNEL_DATARATE,
                  (DSL_EventData_Union_t*)&chStsData,
                  sizeof(DSL_G997_ChannelStatusData_t));
            }

            if( nErrCode != DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), DSL_EVENT_S_CHANNEL_DATARATE));
            }
         }

         if (nOldDataRate != 0 && nErrCode == DSL_SUCCESS)
         {
            /* Check for the Data Rate Upshift */
            if( nActualDataRate > nOldDataRate )
            {
               DSL_CTX_READ_SCALAR(pContext, nErrCode,
                  channelDataRateThresholdUs.nDataRateThresholdUpshift,
                  nDataRateShiftThresh);

               if( nActualDataRate - nOldDataRate > nDataRateShiftThresh)
               {
                  bNotifyDataRateShift = DSL_TRUE;
                  nDataRateCrossingType = DSL_G997_DATARATE_THRESHOLD_UPSHIFT;
               }
            }

            /* Check for the Data Rate Downshift*/
            if( nActualDataRate < nOldDataRate && nErrCode == DSL_SUCCESS )
            {
               DSL_CTX_READ_SCALAR(pContext, nErrCode,
                  channelDataRateThresholdUs.nDataRateThresholdDownshift,
                  nDataRateShiftThresh);

               if( nOldDataRate - nActualDataRate > nDataRateShiftThresh)
               {
                  bNotifyDataRateShift = DSL_TRUE;
                  nDataRateCrossingType = DSL_G997_DATARATE_THRESHOLD_DOWNSHIFT;
               }
            }

            if( bNotifyDataRateShift == DSL_TRUE
               && nErrCode == DSL_SUCCESS)
            {
               nErrCode = DSL_DRV_EventGenerate(
                  pContext, nCh, DSL_UPSTREAM, DSL_XTUDIR_NA,
                  DSL_EVENT_I_CHANNEL_DATARATE_SHIFT_THRESHOLD_CROSSING,
                  (DSL_EventData_Union_t*)&nDataRateCrossingType,
                  sizeof(DSL_G997_DataRateThresholdCrossingType_t));

               if( nErrCode != DSL_SUCCESS )
               {
                  DSL_DEBUG( DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext), DSL_EVENT_I_CHANNEL_DATARATE_SHIFT_THRESHOLD_CROSSING));
                  return nErrCode;
               }
#ifdef INCLUDE_DSL_ADSL_MIB
               else
               {
                  DSL_CTX_READ_SCALAR(pContext, nErrCode, MibAdslCtx.nThresholds,
                     nMibThresholds);
                  DSL_CTX_WRITE_SCALAR(pContext, nErrCode, MibAdslCtx.nThresholds,
                     nMibThresholds | DSL_MIB_THRESHOLD_ATUR_RATE_CHANGE_FLAG);
               }
#endif
            }
         }

         if (nErrCode == DSL_SUCCESS)
         {
            if( bPrevDataRateValid == DSL_TRUE )
            {
               bNotifyDataRateShift = DSL_FALSE;
            }

            DSL_CTX_READ_SCALAR(pContext, nErrCode, nChannelActualDataRatePrevDs[nCh],
               nOldDataRate);

            /* In case of ADSL1 the values for the data rates can be directly
               read via CMVs */
            if (nMode <= 8 && nMode1 == 0)
            {
               if (nLPath == DSL_LATENCY_IP_LP0)
               {
                  nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_RATE,
                     DSL_CMV_ADDRESS_RATE_DsRate, 0, 2, nRate16);
               }
               else if (nLPath == DSL_LATENCY_FP_LP1)
               {
                  nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_RATE,
                     DSL_CMV_ADDRESS_RATE_DsRate, 2, 2, nRate16);
               }

               if (nErrCode == DSL_SUCCESS)
               {
                  nActualDataRate = (nRate16[1] << 16) + nRate16[0];
               }
            }
            else
            {
               nMp = nLp = nTp = nRp = nKp = nBpn = nDataRate =
                  nDataRate_remain = 0;

               nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                  DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_LP, nCh, 1,
                  &nLp);

               if (nErrCode == DSL_SUCCESS)
               {
                  nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                     DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_RP, nCh, 1,
                     &nRp);
               }

               if (nErrCode == DSL_SUCCESS)
               {
                  nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                     DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_MP, nCh, 1,
                     &nMp);
               }

               if (nErrCode == DSL_SUCCESS)
               {
                  nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                     DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_TP, nCh, 1,
                     &nTp);
               }

               if (nErrCode == DSL_SUCCESS)
               {
                  nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
                     DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_KP, 2* nCh, 2,
                     nRate16);
               }

               if (nErrCode == DSL_SUCCESS)
               {
                  nBpn = nRate16[0] + nRate16[1];
                  nKp = nBpn + 1;

                  if (nTp == 0 || (nKp * nMp + nRp) == 0)
                  {
                     nDataRate = 0;
                  }
                  else
                  {
                     nDataRate = ((nTp * (nBpn + 1) -
                        1) * nMp * nLp * 4) / (nTp * (nKp * nMp + nRp));

                     nDataRate_remain = ((((nTp * (nBpn + 1) -
                        1) * nMp * nLp * 4) % (nTp * (nKp * nMp + nRp)))
                        * 1000) / (nTp * (nKp * nMp + nRp));
                  }

                  nActualDataRate = nDataRate * 1000 + nDataRate_remain;
               }
            }
         }

         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRatePrevDs[nCh],
            nActualDataRate);

         /* Update Actual Data Rate for the DS direction*/
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelActualDataRateDs[nCh],
            nActualDataRate);
         
         if (nErrCode == DSL_SUCCESS)
         {
            if (nOldDataRate != nActualDataRate && nErrCode == DSL_SUCCESS)
            {
               /* Set previous data rate value */
               DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nChannelPreviousDataRateDs[nCh],
                  nOldDataRate);

               DSL_DEBUG (DSL_DBG_MSG, (pContext, "DSL[%02d]: Datarate channel %d DS datarate"
                  "= %d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nActualDataRate));

               nErrCode = DSL_DRV_DANUBE_ChannelStatusGet(pContext, nCh,
                  DSL_DOWNSTREAM, &chStsData);
               if( nErrCode != DSL_SUCCESS )
               {
                  DSL_DEBUG( DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: ERROR - Channel status get failed!" DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext)));
               }
               else
               {
                  chStsData.ActualDataRate   = nActualDataRate;
                  chStsData.PreviousDataRate = nOldDataRate;

                  nErrCode = DSL_DRV_EventGenerate(
                     pContext, nCh, DSL_DOWNSTREAM, DSL_XTUDIR_NA,
                     DSL_EVENT_S_CHANNEL_DATARATE,
                     (DSL_EventData_Union_t*)&chStsData,
                     sizeof(DSL_G997_ChannelStatusData_t));
               }

               if( nErrCode != DSL_SUCCESS )
               {
                  DSL_DEBUG( DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), DSL_EVENT_S_CHANNEL_DATARATE));
               }
            }

            if (nOldDataRate != 0 && nErrCode == DSL_SUCCESS)
            {
               /* Check for the Data Rate Upshift */
               if( nActualDataRate > nOldDataRate )
               {
                  DSL_CTX_READ_SCALAR(pContext, nErrCode,
                     channelDataRateThresholdDs.nDataRateThresholdUpshift,
                     nDataRateShiftThresh);

                  if( nActualDataRate - nOldDataRate > nDataRateShiftThresh)
                  {
                     bNotifyDataRateShift = DSL_TRUE;
                     nDataRateCrossingType = DSL_G997_DATARATE_THRESHOLD_UPSHIFT;
                  }
               }

               /* Check for the Data Rate Downshift*/
               if( nActualDataRate < nOldDataRate && nErrCode == DSL_SUCCESS )
               {
                  DSL_CTX_READ_SCALAR(pContext, nErrCode,
                     channelDataRateThresholdDs.nDataRateThresholdDownshift,
                     nDataRateShiftThresh);

                  if( nOldDataRate - nActualDataRate > nDataRateShiftThresh)
                  {
                     bNotifyDataRateShift = DSL_TRUE;
                     nDataRateCrossingType = DSL_G997_DATARATE_THRESHOLD_DOWNSHIFT;
                  }
               }

               if( bNotifyDataRateShift == DSL_TRUE
                  && nErrCode == DSL_SUCCESS)
               {
                  nErrCode = DSL_DRV_EventGenerate(
                     pContext, nCh, DSL_DOWNSTREAM, DSL_XTUDIR_NA,
                     DSL_EVENT_I_CHANNEL_DATARATE_SHIFT_THRESHOLD_CROSSING,
                     (DSL_EventData_Union_t*)&nDataRateCrossingType,
                     sizeof(DSL_G997_DataRateThresholdCrossingType_t));

                  if( nErrCode != DSL_SUCCESS )
                  {
                     DSL_DEBUG( DSL_DBG_ERR,
                        (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
                        DSL_DEV_NUM(pContext),
                        DSL_EVENT_I_CHANNEL_DATARATE_SHIFT_THRESHOLD_CROSSING));
                        
                     return nErrCode;
                  }
#ifdef INCLUDE_DSL_ADSL_MIB
                  else
                  {
                     DSL_CTX_READ_SCALAR(pContext, nErrCode, MibAdslCtx.nThresholds,
                        nMibThresholds);
                     DSL_CTX_WRITE_SCALAR(pContext, nErrCode, MibAdslCtx.nThresholds,
                        nMibThresholds | DSL_MIB_THRESHOLD_ATUC_RATE_CHANGE_FLAG);
                  }
#endif
               }
            }
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_ChannelRateUpdate, retCode=%d"DSL_DRV_CRLF, 
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DANUBE_SnrmDsUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_SnrmDsUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
      DSL_CMV_ADDRESS_PLAM_SNRM_0_1DB, 0, 1, &nVal)) == DSL_SUCCESS)
   {
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nSnrmDs, (DSL_int16_t)nVal);
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_SnrmDsUpdate, retCode=%d"
      DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DANUBE_LineFailuresUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nVal, nVal2, i;
   DSL_uint32_t nFailures;
   DSL_uint32_t nFailures2[DSL_CHANNELS_PER_LINE];
   DSL_uint32_t nCfgDPFM_NE = 0;
   DSL_uint32_t nCfgDPFM_FE = 0;
   DSL_uint32_t nCfgLFM_NE = 0;
   DSL_uint32_t nCfgLFM_FE = 0;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_LineFailuresUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nFailures = nFailures2[0] = 0;
   if (DSL_CHANNELS_PER_LINE == 2)
   {
      nFailures2[1] = 0;
   }

   /* Get line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nLineState);

   if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
      DSL_CMV_ADDRESS_PLAM_NE_FAILURES, 0, 1, &nVal)) == DSL_SUCCESS)
   {
      if (nVal & 0x1)
      {
         nFailures |= DSL_G997_LINEFAILURE_LOS;
         /* Check for transition to L3 */
         if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
            DSL_CMV_ADDRESS_STAT_MISC, 0, 1, &nVal2)) == DSL_SUCCESS)
         {
            if (nVal2 & (1 << 14))
            {
               DSL_CTX_WRITE_SCALAR(pContext, nErrCode,
                  powerMgmtStatus.nPowerManagementStatus, DSL_G997_PMS_L3);
            }
         }
         else
         {
            DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: ERROR - Could not"
               " read CO L3 request status CMV"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
         }
      }
      if (nVal & 0x2)
      {
         nFailures |= DSL_G997_LINEFAILURE_LOF;
      }
      if (nVal & 0x4)
      {
         nFailures |= DSL_G997_LINEFAILURE_LPR;
      }
      if (pContext->nXDslMode >= DSL_XDSLMODE_G_992_3)
      {
         if (nVal & (1 << 8))
         {
            nFailures2[0] |= DSL_G997_DATAPATHFAILURE_NCD;
         }
         if (DSL_CHANNELS_PER_LINE == 2 && (1 << 9))
         {
            nFailures2[1] |= DSL_G997_DATAPATHFAILURE_NCD;
         }
         if (nVal & (1 << 10))
         {
            nFailures2[0] |= DSL_G997_DATAPATHFAILURE_LCD;
         }
         if (DSL_CHANNELS_PER_LINE == 2 && nVal & (1 << 11))
         {
            nFailures2[1] |= DSL_G997_DATAPATHFAILURE_LCD;
         }
      }
      else
      {
         for (i = 0; i < DSL_CHANNELS_PER_LINE; i++)
         {
            if (pContext->nLPath[i] == DSL_LATENCY_IP_LP0)
            {
               if (nVal & (1 << 4))
               {
                  nFailures2[i] |= DSL_G997_DATAPATHFAILURE_NCD;
               }
               if (nVal & (1 << 6))
               {
                  nFailures2[i] |= DSL_G997_DATAPATHFAILURE_LCD;
               }
            }
            else if (pContext->nLPath[i] == DSL_LATENCY_FP_LP1)
            {
               if (nVal & (1 << 5))
               {
                  nFailures2[i] |= DSL_G997_DATAPATHFAILURE_NCD;
               }
               if (nVal & (1 << 7))
               {
                  nFailures2[i] |= DSL_G997_DATAPATHFAILURE_LCD;
               }
            }
         }
      }

      DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: DSL_DRV_DANUBE_LineFailuresUpdate: NE "
         "nVal = %d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nVal));

      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nLineFailuresNe, nFailures);
      
      DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: nLineFailuresNe = %d"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nFailures));

      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nDataPathFailuresNe[0], nFailures2[0]);

      DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: nDataPathFailuresNe[0] = %d"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nFailures2[0]));

      if (DSL_CHANNELS_PER_LINE == 2 && nErrCode == DSL_SUCCESS)
      {
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nDataPathFailuresNe[1], nFailures2[1]);

         DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: nDataPathFailuresNe[1] = %d"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nFailures2[1]));
      }
   }

   if (nErrCode == DSL_SUCCESS)
   {
      if( nLineState == DSL_LINESTATE_SHOWTIME_TC_SYNC  ||
          nLineState == DSL_LINESTATE_SHOWTIME_NO_SYNC )
      {
         /* Get (user) mask configuration for line failure event handling */
         DSL_CTX_READ_SCALAR(pContext, nErrCode, nDataPathFailuresNeAlarmMask, nCfgDPFM_NE);
         DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineFailuresNeAlarmMask, nCfgLFM_NE);

         nFailures &= nCfgLFM_NE;
         if (nFailures != 0)
         {
            nErrCode = DSL_DRV_EventGenerate(
                           pContext, 0, DSL_ACCESSDIR_NA, DSL_NEAR_END,
                           DSL_EVENT_I_LINE_FAILURES,
                           (DSL_EventData_Union_t*)&nFailures,
                           sizeof(DSL_G997_LineFailuresData_t));
            if( nErrCode != DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Event(%d) generation failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), DSL_EVENT_I_LINE_FAILURES));
                  
               return nErrCode;
            }
         }

         nFailures2[0] &= nCfgDPFM_NE;
         if (nFailures2[0] != 0)
         {
            nErrCode = DSL_DRV_EventGenerate(
                           pContext, 0, DSL_ACCESSDIR_NA, DSL_NEAR_END,
                           DSL_EVENT_I_DATA_PATH_FAILURES,
                           (DSL_EventData_Union_t*)&nFailures2[0],
                           sizeof(DSL_G997_DataPathFailuresData_t));
            if( nErrCode != DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Event(%d) generation failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), DSL_EVENT_I_DATA_PATH_FAILURES));
                  
               return nErrCode;
            }
         }

         if (DSL_CHANNELS_PER_LINE == 2)
         {
            nFailures2[1] &= nCfgDPFM_NE;

            if (nFailures2[1] != 0)
            {
               nErrCode = DSL_DRV_EventGenerate(
                             pContext, 1, DSL_ACCESSDIR_NA, DSL_NEAR_END,
                             DSL_EVENT_I_DATA_PATH_FAILURES,
                             (DSL_EventData_Union_t*)&nFailures2[1],
                             sizeof(DSL_G997_DataPathFailuresData_t));
               if( nErrCode != DSL_SUCCESS )
               {
                  DSL_DEBUG( DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: ERROR - Event(%d) generation failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext), DSL_EVENT_I_DATA_PATH_FAILURES));
                     
                  return nErrCode;
               }
            }
         }
      }
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nFailures = 0;
      if (DSL_CHANNELS_PER_LINE == 2)
      {
         nFailures2[1] = 0;
      }

      if ((nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_PLAM,
         DSL_CMV_ADDRESS_PLAM_FE_FAILURES, 0, 1, &nVal)) == DSL_SUCCESS)
      {
         if (nVal & 0x1)
         {
            nFailures |= DSL_G997_LINEFAILURE_LOS;
         }
         if (nVal & 0x2)
         {
            nFailures |= DSL_G997_LINEFAILURE_LOF;
         }
         if (nVal & 0x4)
         {
            nFailures |= DSL_G997_LINEFAILURE_LPR;
         }
         if (nVal & 0x100)
         {
            nFailures2[0] |= DSL_G997_DATAPATHFAILURE_NCD;
         }
         if (DSL_CHANNELS_PER_LINE == 2 && nVal & 0x200)
         {
            nFailures2[1] |= DSL_G997_DATAPATHFAILURE_NCD;
         }
         if (nVal & 0x400)
         {
            nFailures2[0] |= DSL_G997_DATAPATHFAILURE_LCD;
         }
         if (DSL_CHANNELS_PER_LINE == 2 && nVal & 0x800)
         {
            nFailures2[1] |= DSL_G997_DATAPATHFAILURE_LCD;
         }

         DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: DSL_DRV_DANUBE_LineFailuresUpdate: FE "
            "nVal = %d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nVal));

         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nLineFailuresFe, nFailures);
         
         DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: nLineFailuresFe = %d"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nFailures));
            
         DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nDataPathFailuresFe[0], nFailures2[0]);

         DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: nDataPathFailuresFe[0] = %d"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nFailures2[0]));
            
         if (DSL_CHANNELS_PER_LINE == 2 && nErrCode == DSL_SUCCESS)
         {
            DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nDataPathFailuresFe[1], nFailures2[1]);

            DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: nDataPathFailuresFe[1] = %d"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext), nFailures2[1]));
         }
      }
   }

   if (nErrCode == DSL_SUCCESS)
   {
      if( nLineState == DSL_LINESTATE_SHOWTIME_TC_SYNC  ||
          nLineState == DSL_LINESTATE_SHOWTIME_NO_SYNC )
      {
         /* Get (user) mask configuration for line failure event handling */
         DSL_CTX_READ_SCALAR(pContext, nErrCode, nDataPathFailuresFeAlarmMask, nCfgDPFM_FE);
         DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineFailuresFeAlarmMask, nCfgLFM_FE);

         nFailures &= nCfgLFM_FE;
         if (nFailures != 0)
         {
            nErrCode = DSL_DRV_EventGenerate(
                           pContext, 0, DSL_ACCESSDIR_NA, DSL_FAR_END,
                           DSL_EVENT_I_LINE_FAILURES,
                           (DSL_EventData_Union_t*)&nFailures,
                           sizeof(DSL_G997_LineFailuresData_t));
            if( nErrCode != DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Event(%d) generation failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), DSL_EVENT_I_LINE_FAILURES));
                  
               return nErrCode;
            }
         }

         nFailures2[0] &= nCfgDPFM_FE;
         if (nFailures2[0] != 0)
         {
            nErrCode = DSL_DRV_EventGenerate(
                           pContext, 0, DSL_ACCESSDIR_NA, DSL_FAR_END,
                           DSL_EVENT_I_DATA_PATH_FAILURES,
                           (DSL_EventData_Union_t*)&nFailures2[0],
                           sizeof(DSL_G997_DataPathFailuresData_t));
            if( nErrCode != DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Event(%d) generation failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), DSL_EVENT_I_DATA_PATH_FAILURES));
                  
               return nErrCode;
            }
         }

         if (DSL_CHANNELS_PER_LINE == 2)
         {
            nFailures2[1] &= nCfgDPFM_FE;

            if (nFailures2[1] != 0)
            {
               nErrCode = DSL_DRV_EventGenerate(
                             pContext, 1, DSL_ACCESSDIR_NA, DSL_FAR_END,
                             DSL_EVENT_I_DATA_PATH_FAILURES,
                             (DSL_EventData_Union_t*)&nFailures2[1],
                             sizeof(DSL_G997_DataPathFailuresData_t));
               if( nErrCode != DSL_SUCCESS )
               {
                  DSL_DEBUG( DSL_DBG_ERR,
                     (pContext, "DSL[%02d]: ERROR - Event(%d) generation failed!"DSL_DRV_CRLF,
                     DSL_DEV_NUM(pContext), DSL_EVENT_I_DATA_PATH_FAILURES));
                     
                  return nErrCode;
               }
            }
         }
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_LineFailuresUpdate, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_FramingParameterStatusGet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_FramingParameterStatus_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nCh = pData->nChannel, nSEQ;
   DSL_uint16_t nAddrMp, nAddrTp, nAddrBp, nAddrMSGc, nAddrMSGLp, nIdxBp;
   DSL_LatencyPath_t nLp = 0;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
   
   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_FramingParameterStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pData);
   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_DIRECTION(pData->nDirection);
   DSL_CHECK_CHANNEL_RANGE(pData->nChannel);
   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   /* Get current line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   /* Only proceed if the line is in SHOWTIME state.*/
   if ( (nCurrentState != DSL_LINESTATE_SHOWTIME_TC_SYNC) &&
        (nCurrentState != DSL_LINESTATE_SHOWTIME_NO_SYNC) )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Function is only available in the SHOWTIME!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));

      return DSL_ERR_ONLY_AVAILABLE_IN_SHOWTIME;
   }

   if (pData->nDirection == DSL_UPSTREAM)
   {
      nAddrMp = DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_MP;
      nAddrTp = DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_TP;
      nAddrBp = DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_KP;
      nAddrMSGLp = DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_MSGLp;
      nAddrMSGc = DSL_CMV_ADDRESS_CNFG_DATA_RATEUS_FLAG_ADSL2_MSGc;
   }
   else
   {
      nAddrMp = DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_MP;
      nAddrTp = DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_TP;
      nAddrBp = DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_KP;
      nAddrMSGLp = DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_MSGLp;
      nAddrMSGc = DSL_CMV_ADDRESS_CNFG_DATA_RATEDS_FLAG_ADSL2_MSGc;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* Update channel status for the case if it was not apdated yet */
   nErrCode = DSL_DRV_DANUBE_ChannelStatusUpdate(pContext);

   if (nErrCode == DSL_SUCCESS)
   {
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nLPath[pData->nChannel], nLp);
      nIdxBp = nCh;
      if (nLp == DSL_LATENCY_FP_LP1)
      {
         nIdxBp += 2;
      }
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
         nAddrMp, nCh, 1, &pData->data.nMp);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
         nAddrTp, nCh, 1, &pData->data.nTp);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
         nAddrBp, nIdxBp, 1, &pData->data.nBP);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
         nAddrMSGc, nCh, 1, &pData->data.nMSGC);
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNFG,
         nAddrMSGLp, nCh, 1, &pData->data.nMSGLP);
   }

   /* Calculate the value for SEQ only in ADSL2/2+.
      For a description of the rules to set SEQ value please refer to
      ITU-T Rec. G.992.3, Table 7-7 */
   if (nErrCode == DSL_SUCCESS)
   {
      if (nCh == 0)
      {
         if (pData->data.nMSGLP == 0)
         {
            nSEQ = pData->data.nMSGC + 6;
         }
         else
         {
            nSEQ = 6;
         }
      }
      else
      {
         if (pData->data.nMSGLP == 0)
         {
            nSEQ = 2;
         }
         else
         {
            nSEQ = pData->data.nMSGC + 2;
         }
      }
      pData->data.nSEQ = nSEQ;
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_FramingParameterStatusGet, "
      "retCode(%d)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/* HDLC block */

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_HdlcStatusGet (
   DSL_Context_t *pContext,
   DSL_uint16_t *pnStatus)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_HdlcStatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Get HDLC status */
   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
      DSL_CMV_ADDRESS_STAT_ME_HDLC, 0, 1, pnStatus);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcStatusGet, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
 * Check if the me is resolved.
 *
 * \param   status      the me status
 * \return  ME_HDLC_UNRESOLVED or ME_HDLC_RESOLVED
 * \ingroup Internal
 */
/*
   This function does not check pContext pointer
*/
static DSL_int_t DSL_DRV_DANUBE_HdlcResolvedGet(
   DSL_Context_t *pContext,
   DSL_uint16_t nStatus)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nData;
   DSL_uint16_t nMode = 0, nMode1 = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_HdlcResolvedGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode, nMode);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not retreive internal Adsl mode "
         " from the device specific data of DSL Context!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ME_HDLC_UNRESOLVED;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, pDevCtx->data.nAdslMode1, nMode1);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not retreive internal Adsl mode 1 "
         " from the device specific data of DSL Context!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ME_HDLC_UNRESOLVED;
   }

   if (nMode <= 8 && nMode1 == 0) /* adsl mode */
   {
      /* Get ME-HDLC Control */
      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNTL,
         DSL_CMV_ADDRESS_CNTL_ME_HDLC, 0, 1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcResolvedGet"
            ", retCode=(DSL_ME_HDLC_URESOLVED)"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         return DSL_ME_HDLC_UNRESOLVED;
      }

      if (nData & (1 << 0))
      {
         DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcResolvedGet"
            ", retCode=(DSL_ME_HDLC_URESOLVED)"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ME_HDLC_UNRESOLVED;
      }
   }
   else
   {
      if (nStatus == DSL_ME_HDLC_MSG_QUEUED || nStatus == DSL_ME_HDLC_MSG_SENT)
      {
         DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcResolvedGet"
            ", retCode=(DSL_ME_HDLC_URESOLVED)"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ME_HDLC_UNRESOLVED;
      }

      if (nStatus == DSL_ME_HDLC_IDLE)
      {
         /* Get ME-HDLC Control */
         if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_CNTL,
            DSL_CMV_ADDRESS_CNTL_ME_HDLC, 0, 1, &nData) != DSL_SUCCESS)
         {
            DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcResolvedGet"
               ", retCode=(DSL_ME_HDLC_URESOLVED)"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return DSL_ME_HDLC_UNRESOLVED;
         }

         if (nData & (1 << 0))
         {
            DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcResolvedGet"
               ", retCode=(DSL_ME_HDLC_URESOLVED)"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return DSL_ME_HDLC_UNRESOLVED;
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcResolvedGet"
      ", retCode=(DSL_ME_HDLC_RESOLVED)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   return DSL_ME_HDLC_RESOLVED;
}

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_HdlcSend(
   DSL_Context_t *pContext,
   DSL_uint8_t *pHdlcPkt,
   DSL_int_t nPktLen,
   DSL_int_t nMaxLength)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t nData = 0;
   DSL_uint16_t nLen = 0;
   DSL_uint16_t nRxLength = 0;
   DSL_int_t nWriteSize = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_HdlcSend"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if (nPktLen > nMaxLength)
   {
      /* Get ME-HDLC Control */
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_RX_CLEAR_EOC, 2, 1, &nRxLength);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - RX Length for HDLC get failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      if (nRxLength + nMaxLength < nPktLen)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - Exceed maximum eoc "
            "rx(%d)+tx(%d) message length!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nRxLength, nMaxLength));
            
         return DSL_ERROR;
      }

      nData = 1;
      /* disable RX EOC */
      nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_RX_CLEAR_EOC, 6, 1, &nData);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Could not disable RX EOC"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   }

   while (nLen < nPktLen)
   {
      nWriteSize = nPktLen - nLen;
      if (nWriteSize > 24)
      {
         nWriteSize = 24;
      }

      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_ME_HDLC_TX_BUFFER, nLen / 2,
         (nWriteSize + 1) / 2, (DSL_uint16_t*)(pHdlcPkt + nLen));
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR, (pContext,
            "DSL[%02d]: ERROR - Could not write TX buffer!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      nLen += nWriteSize;
   }

   /* Update tx message length */
   nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
      DSL_CMV_ADDRESS_INFO_ME_HDLC_PARAMS, 2, 1, &nLen);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not write TX buffer length!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Start to send */
   nData = (1 << 0);
   nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_CNTL,
      DSL_CMV_ADDRESS_CNTL_ME_HDLC, 0, 1, &nData);
   if (nErrCode != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - Could not write TX buffer length!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcSend"
      ", retCode=(DSL_SUCCESS)"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   return DSL_SUCCESS;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_danube.h'
*/
DSL_Error_t DSL_DRV_DANUBE_HdlcWrite (
   DSL_Context_t *pContext,
   DSL_boolean_t bDeviceLock,
   DSL_uint8_t *pHdlcPkt,
   DSL_int_t nHdlcPktLen)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_uint16_t nHdlcStatus = 0;
   DSL_uint16_t nMaxHdlcTxLength = 0;
   DSL_uint16_t nRetry = 0;
   DSL_G997_PowerManagement_t nPowerMode = DSL_G997_PMS_NA;
   DSL_int_t nSendBusyCounter = 0;
   DSL_boolean_t bSendRetry = DSL_FALSE;
   DSL_uint16_t nData = 0;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_HdlcWrite"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pHdlcPkt);
   DSL_CHECK_ERR_CODE();

   if (bDeviceLock == DSL_TRUE)
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ERROR;
      }
   }

   for (;;)
   {
      /* retry 1000 times (10 seconds) */
      while (nRetry < 1000)
      {
         /* In L2 power mode, do not read the OHC related parameters, instead give
            the indication to the calling IOCTL, that the readout fails (just
            return DSL_ERR_DEVICE_BUSY).  */
         nErrCode = DSL_DRV_DANUBE_L3StatusGet(pContext, &nPowerMode);
         if (nErrCode != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Could not get power mode!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            break;
         }

         if (nPowerMode == DSL_G997_PMS_L2)
         {
            nErrCode = DSL_ERR_DEVICE_BUSY;
            break;
         }

         nErrCode = DSL_DRV_DANUBE_HdlcStatusGet(pContext, &nHdlcStatus);
         if (nErrCode != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Could not get HDLC status!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            break;
         }

         /* arc ready to send HDLC message */
         if (DSL_DRV_DANUBE_HdlcResolvedGet (pContext, nHdlcStatus)
            == DSL_ME_HDLC_RESOLVED)
         {
            /* Get Maximum Allowed HDLC Tx Message Length */
            nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
               DSL_CMV_ADDRESS_INFO_ME_HDLC_PARAMS, 0, 1, &nMaxHdlcTxLength);

            if (nErrCode == DSL_SUCCESS)
            {
               nErrCode = DSL_DRV_DANUBE_HdlcSend (pContext, pHdlcPkt, nHdlcPktLen, nMaxHdlcTxLength);

               if (bDeviceLock == DSL_TRUE)
               {
                  DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
               }

               DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcWrite, "
                  "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

               return nErrCode;
            }
         }
         else
         {
            if (nHdlcStatus == DSL_ME_HDLC_MSG_SENT)
            {
               nSendBusyCounter++;
            }
         }
         nRetry++;
         DSL_WAIT (1);
      }

      if (nErrCode != DSL_SUCCESS)
      {
         break;
      }

      /* wait 10 seconds and FW still report busy -> reset FW HDLC status */
      if (nSendBusyCounter > 950 && bSendRetry == DSL_FALSE)
      {
         bSendRetry = DSL_TRUE;
         nRetry = 0;
         
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext, "DSL[%02d]: WARNING - Reset FW HDLC status!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nSendBusyCounter = 0;
         nData = 2;

         /* force reset to */
         nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_CNTL,
            DSL_CMV_ADDRESS_CNTL_ME_HDLC, 0, 1, &nData);

         if (nErrCode != DSL_SUCCESS)
         {
            break;
         }
         continue;
      }
      else
      {
         nErrCode = DSL_ERR_DEVICE_BUSY;

         DSL_DEBUG( DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - Hdlc Write failed! "
            "Return ERR_DEVICE_BUSY"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

         break;
      }
   }

   if (bDeviceLock == DSL_TRUE)
   {
      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcWrite, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_danube.h'
*/
DSL_Error_t DSL_DRV_DANUBE_HdlcRead (
   DSL_Context_t *pContext,
   DSL_boolean_t bDeviceLock,
   DSL_uint8_t *pHdlcPkt,
   DSL_int_t nMaxHdlcPktLen,
   DSL_int_t *pnRead)
{
   DSL_int_t nMsgReadLen, nRetry = 0, nCurrentSize = 0;
   DSL_uint16_t nHdlcStatus = 0, nPktLen = 0;
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint16_t buf[DSL_MAX_CMV_MSG_LENGTH];

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_HdlcRead"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pHdlcPkt);
   DSL_CHECK_POINTER(pContext, pnRead);
   DSL_CHECK_ERR_CODE();

   *pnRead = 0;

   if (bDeviceLock == DSL_TRUE)
   {
      if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ERROR;
      }
   }

   while (nRetry < 10)
   {
      nErrCode = DSL_DRV_DANUBE_HdlcStatusGet(pContext, &nHdlcStatus);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Could not get HDLC status!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, "DSL[%02d]: DSL_DRV_DANUBE_HdlcRead: HDLC status = %hu"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nHdlcStatus));

      if (nHdlcStatus == DSL_ME_HDLC_RESP_RCVD)
      {
         /* Get EoC packet length */
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
            DSL_CMV_ADDRESS_INFO_ME_HDLC_PARAMS, 3, 1, &nPktLen);

         if (nErrCode == DSL_SUCCESS)
         {
            if (nPktLen > nMaxHdlcPktLen)
            {
               DSL_DEBUG(DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: DSL_DRV_DANUBE_HdlcRead: Buffer length too small "
                  "(received %hu against %d allocated)"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), nPktLen, nMaxHdlcPktLen));

               nErrCode = DSL_ERR_MEMORY;
               
               break;
            }
            else
            {
               while (nCurrentSize < nPktLen)
               {
                  if (nPktLen - nCurrentSize > (DSL_MAX_CMV_MSG_LENGTH * 2 - 8))
                  {
                     nMsgReadLen = DSL_MAX_CMV_MSG_LENGTH * 2 - 8;
                  }
                  else
                  {
                     nMsgReadLen = nPktLen - nCurrentSize;
                  }

                  /* Get EoC packet length */
                  nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
                     DSL_CMV_ADDRESS_INFO_ME_HDLC_RX_BUFFER,
                     0 + (nCurrentSize / 2), (nMsgReadLen + 1) / 2, buf);

                  if (nErrCode == DSL_SUCCESS)
                  {
                     memcpy (pHdlcPkt + nCurrentSize, buf, nMsgReadLen);
                     nCurrentSize += nMsgReadLen;
                  }
                  else
                  {
                     break;
                  }
               }
               if (nErrCode == DSL_SUCCESS)
               {
                  *pnRead = nCurrentSize;
               }
               else
               {
                  break;
               }
            }
         }
         else
         {
            DSL_DEBUG(DSL_DBG_ERR,
              (pContext, "DSL[%02d]: DSL_DRV_DANUBE_HdlcRead: HDLC status = %hu"DSL_DRV_CRLF,
              DSL_DEV_NUM(pContext), nHdlcStatus));
              
            break;
         }
         break;
      }
      else
      {
         nErrCode = DSL_ERR_DEVICE_NO_DATA;
      }

      nRetry++;

      DSL_WAIT (10);
   }

   if (bDeviceLock == DSL_TRUE)
   {
      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_HdlcRead, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/* Power management block */

/*
   This function does not check pContext pointer
*/
static DSL_Error_t DSL_DRV_DANUBE_L3StatusGet(
   DSL_Context_t *pContext,
   DSL_G997_PowerManagement_t *pnPowerMode)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
   DSL_uint16_t nData;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_L3StatusGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Default mode is L3 (power off/no link) */
   *pnPowerMode = DSL_G997_PMS_L3;

   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: ERROR - lineState for the L3 handling get failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   if (DSL_DRV_DEV_ModemIsReady(pContext) == DSL_TRUE)
   {
      /* Only proceed if the specified line is in SHOWTIME state*/
      if ((nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC) ||
          (nCurrentState == DSL_LINESTATE_SHOWTIME_NO_SYNC))
      {
         nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
            DSL_CMV_ADDRESS_STAT_POWER_STATE, 0, 1, &nData);
         if (nErrCode == DSL_SUCCESS)
         {
            *pnPowerMode = nData;
         }
      }
   }

   DSL_DEBUG(DSL_DBG_MSG, (pContext,
      "DSL[%02d]: OUT - DSL_DRV_DANUBE_L3StatusGet, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_DBG_DeviceMessageSend(
   DSL_Context_t *pContext,
   DSL_DeviceMessageData_t *pMsg )
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_uint16_t nSize = 0;
   DSL_uint16_t RxMessage[DSL_MAX_CMV_MSG_LENGTH]__attribute__ ((aligned(4)));
   DSL_boolean_t bWrite = DSL_FALSE;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_DBG_DeviceMessageSend"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pMsg);
   DSL_CHECK_ERR_CODE();

   if( ((*((DSL_uint16_t*)pMsg->pMsg)) & 0x1000) )
   {
      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - not supported CMV!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERR_NOT_SUPPORTED;   
   }
   
   /* Check Rd/Wr access*/
   bWrite = DSL_DRV_DANUBE_CmvAccessTypeGet(*((DSL_uint16_t*)pMsg->pMsg));

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if (DSL_DRV_DANUBE_CmvSend(pContext, (DSL_uint16_t*)pMsg->pMsg, DSL_TRUE, RxMessage)
      != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR, (pContext, "DSL[%02d]: ERROR - CMV send failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      
      nErrCode = DSL_ERR_MSG_EXCHANGE;
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   if (!bWrite)
   {
      /* Get CMV byte count*/
      nSize = DSL_DRV_DANUBE_CmvByteCountGet(RxMessage[0]);
   }

   if (nErrCode == DSL_SUCCESS && nSize != 0)
   {
      memcpy(pMsg->pMsg, RxMessage, nSize);
      pMsg->nSizeRx = nSize;
   }
   else
   {
      pMsg->nSizeRx = 0;   
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_DBG_DeviceMessageSend"
      ", retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

static DSL_Error_t DSL_DRV_DANUBE_FailReasonGet(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_G997_LineInit_t lineInitStatus = LINIT_UNKNOWN;
   DSL_uint16_t nVal = 0;

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DANUBE_FailReasonGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   nErrCode = DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_STAT,
      DSL_CMV_ADDRESS_STAT_FAIL, 0, 1, &nVal);
   if (nErrCode == DSL_SUCCESS)
   {
      switch(nVal)
      {
      case DSL_CMV_STAT_FAIL_SUCCESS:
         lineInitStatus = LINIT_SUCCESSFUL;
         break;
      case DSL_CMV_STAT_FAIL_CONFIGURATION:
         lineInitStatus = LINIT_CONFIG_ERROR;
         break;
      case DSL_CMV_STAT_FAIL_CONFIGURATION_NOT_FEASIBLE:
         lineInitStatus = LINIT_CONFIG_NOT_FEASIBLE;
         break;
      case DSL_CMV_STAT_FAIL_COMMUNICATION_PROBLEM:
         lineInitStatus = LINIT_COMMUNICATION_PROBLEM;
         break;
      case DSL_CMV_STAT_FAIL_NO_PEER_DETECTED:
          lineInitStatus = LINIT_NO_PEER_XTU;
         break;
      case DSL_CMV_STAT_FAIL_UNKNOWN:
      default:
         lineInitStatus = LINIT_UNKNOWN;
         break;
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DRV_HandleLinitValue(pContext, lineInitStatus, LINIT_SUB_NONE);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_FailReasonGet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}

/*
   DANUBE Device specific implementation of the DSL_DRV_OnTimeoutEvent() function
*/
DSL_Error_t DSL_DRV_DEV_OnTimeoutEvent(
   DSL_Context_t *pContext,
   DSL_int_t nEventType,
   DSL_uint32_t nTimeoutID)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG(DSL_DBG_MSG,(pContext, "DSL[%02d]: DSL_DRV_DEV_OnTimeoutEvent: "
      "nEventType=%d, nTimeoutID=0x%08X" DSL_DRV_CRLF ,
      DSL_DEV_NUM(pContext), nEventType, nTimeoutID));

   switch (nEventType)
   {
   case DSL_TIMEOUTEVENT_FE_STATUS:
      nErrCode = DSL_DRV_DANUBE_OnLineInventoryFe(pContext);
      break;

   default:
      DSL_DEBUG(DSL_DBG_ERR, (pContext,
         "DSL[%02d]: Unexpected nEventType (%d)" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nEventType));
      break;
   }

   return (nErrCode);
}

static DSL_Error_t DSL_DRV_DANUBE_OnLineInventoryFe(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* Retreive all FE inventory information*/
   nErrCode = DSL_DRV_DANUBE_LineInventoryFeRead(pContext);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - FE inventory get failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   if (nErrCode == DSL_SUCCESS)
   {
      nErrCode = DSL_DRV_EventGenerate(
                    pContext, 0, DSL_ACCESSDIR_NA, DSL_FAR_END,
                    DSL_EVENT_S_FE_INVENTORY_AVAILABLE,
                    DSL_NULL,
                    0);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Event(%d) generate failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), DSL_EVENT_S_CHANNEL_DATARATE));
            
         return nErrCode;
      }
   }

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_device_danube.h
*/
DSL_uint32_t DSL_DRV_DANUBE_LineInventoryNeWrite(
   DSL_IN DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t i;
   DSL_uint16_t nData;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Send inventory info..."DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
      
      return DSL_ERROR;
   }

   for (i = 0; i < 4; i++)
   {
      nData = (pContext->lineInventoryNe.SystemVendorID[2*i+1] << 8) |
         pContext->lineInventoryNe.SystemVendorID[2*i];

      if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
                           DSL_CMV_ADDRESS_INFO_VENDORID_NE,
                           i, 1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't send vendor ID!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }
   }

   for (i = 0; i < 8; i++)
   {
      nData = (pContext->lineInventoryNe.VersionNumber[2*i+1] << 8) |
         pContext->lineInventoryNe.VersionNumber[2*i];

      if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
                           DSL_CMV_ADDRESS_INFO_VERSIONNUM_NE,
                           i, 1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't send version num!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }
   }

   for (i = 0; i < 16; i++)
   {
      nData = (pContext->lineInventoryNe.SerialNumber[2*i+1] << 8) |
         pContext->lineInventoryNe.SerialNumber[2*i];

      if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
                           DSL_CMV_ADDRESS_INFO_SERIALNUM_NE,
                           i, 1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't send serial number!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         
         nErrCode = DSL_ERROR;
         
         break;
      }
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_LineInventoryWrite, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_device_danube.h
*/
DSL_uint32_t DSL_DRV_DANUBE_LineInventoryNeRead(
   DSL_IN DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t i;
   DSL_uint16_t nData;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Read inventory info..."DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   if(DSL_DRV_MUTEX_LOCK(pContext->dataMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock data mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
      
      return DSL_ERROR;
   }

   DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: G994 VID:"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));
   
   for (i = 0; i < DSL_G997_LI_MAXLEN_VENDOR_ID / 2; i++)
   {
      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_VENDORID_G994_NE, i, 1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't get G994 vendor ID!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }

      pContext->lineInventoryNe.G994VendorID[2*i + 1] = (nData >> 8) & 0xFF;
      pContext->lineInventoryNe.G994VendorID[2*i] = nData & 0xFF;
      DSL_DEBUG( DSL_DBG_MSG, (pContext, "%02x %02x ",
         pContext->lineInventoryNe.G994VendorID[2*i],
         pContext->lineInventoryNe.G994VendorID[2*i + 1]));
   }
   DSL_DEBUG( DSL_DBG_MSG, (pContext, ""DSL_DRV_CRLF));

   if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
      DSL_CMV_ADDRESS_INFO_SELFTEST, 0, 2,
      (DSL_uint16_t*)&pContext->lineInventoryNe.SelfTestResult) != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't get self test result!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }
   DSL_DEBUG( DSL_DBG_MSG, (pContext, "DSL[%02d]: Test result: %d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), pContext->lineInventoryNe.SelfTestResult));

   DSL_DRV_MUTEX_UNLOCK(pContext->dataMutex);

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_LineInventoryNeRead, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   For a detailed description please refer to the drv_dsl_cpe_device_danube.h
*/
DSL_uint32_t DSL_DRV_DANUBE_LineInventoryFeRead(
   DSL_IN DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t i;
   DSL_uint16_t nData;
   DSL_uint16_t hdlcCmd[2];
   DSL_uint16_t hdlcRxBuffer[32];
   DSL_int_t nHdlcRxLen = 0;
   DSL_uint8_t hdlcOctet = 0;
   /* 5 SPAR octets, 6th octet is dummy and
      is needed due to the CMV 16-bit format*/
   DSL_uint8_t spar[5 + 1];
   DSL_uint8_t xtse[DSL_G997_NUM_XTSE_OCTETS] = {0};

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: Read inventory info..."DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_MODEM_IS_READY();
   DSL_CHECK_ERR_CODE();

   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* Get G994 Vendor ID information*/
   for (i = 0; i < DSL_G997_LI_MAXLEN_VENDOR_ID / 2; i++)
   {
      if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_VENDORID_G994_FE, i, 1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't get G994 vendor ID!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERROR;
         
         break;
      }

      hdlcOctet = (nData >> 8) & 0xFF;
      DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.G994VendorID[2*i + 1], hdlcOctet);
      hdlcOctet = nData & 0xFF;
      DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.G994VendorID[2*i], hdlcOctet);
   }

   /* Fill HDLC command*/
   hdlcCmd[0] = 0x0143;
   hdlcCmd[1] = 0x0;

   if (DSL_DRV_DANUBE_HdlcWrite (pContext, DSL_FALSE, (char *) hdlcCmd, 4)
      == DSL_SUCCESS)
   {
      DSL_WAIT (1);

      if (DSL_DRV_DANUBE_HdlcRead (pContext, DSL_FALSE,
             (char *)&hdlcRxBuffer, 32 * 2, &nHdlcRxLen) == DSL_SUCCESS)
      {
         if (nHdlcRxLen <= 0)
         {
            DSL_DEBUG( DSL_DBG_WRN,
               (pContext, "DSL[%02d]: WARNING - HDLC read invalid RxLen!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));

            nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
         }
         else
         {
            /* Get System Vendor ID information*/
            for (i = 0; i < DSL_G997_LI_MAXLEN_VENDOR_ID / 2; i++)
            {
               hdlcOctet = hdlcRxBuffer[i + 1] & 0xFF;
               DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.SystemVendorID[2*i + 1], hdlcOctet);
               hdlcOctet = (hdlcRxBuffer[i + 1] >> 8) & 0xFF;
               DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.SystemVendorID[2*i], hdlcOctet);
            }
            
            /* Get Version Number information*/
            for (i = 0; i < DSL_G997_LI_MAXLEN_VERSION / 2; i++)
            {
               hdlcOctet = hdlcRxBuffer[i + 5] & 0xFF;
               DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.VersionNumber[2*i + 1], hdlcOctet);
               hdlcOctet = (hdlcRxBuffer[i + 5] >> 8) & 0xFF;
               DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.VersionNumber[2*i], hdlcOctet);
            }
             
            /* Get Serial Number information*/
            for (i = 0; i < DSL_G997_LI_MAXLEN_SERIAL / 2; i++)
            {
               hdlcOctet = hdlcRxBuffer[i + 13] & 0xFF;
               DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.SerialNumber[2*i + 1], hdlcOctet);
               hdlcOctet = (hdlcRxBuffer[i + 13] >> 8) & 0xFF;
               DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.SerialNumber[2*i], hdlcOctet);
            }
         }
      }
      else
      {
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext, "DSL[%02d]: WARNING - HDLC Read failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
                  
         nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: WARNING - HDLC Send failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
          
      nErrCode = DSL_WRN_INCOMPLETE_RETURN_VALUES;
   }


   /* Get 5 SPAR octets*/  
   if (DSL_DRV_DANUBE_CmvRead(
                 pContext, DSL_CMV_GROUP_INFO,
                 DSL_CMV_ADDRESS_INFO_XTSE_FE,
                 0, sizeof(spar)/sizeof(DSL_uint16_t), (DSL_uint16_t*)spar) != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't get FE SPAR information!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
            
      nErrCode = DSL_ERROR;
   }
   else
   {
      /* 
         Map SPAR1 octet to the XTSE capabilities\
      */
      /* G992.1 Annex A*/
      xtse[1-1] |= (spar[1] & 0x1) ? XTSE_1_03_A_1_NO : 0x0;
      /* G992.1 Annex B*/
      xtse[1-1] |= (spar[1] & 0x2) ? XTSE_1_05_B_1_NO : 0x0;
      /* G992.1 Annex C*/
      xtse[1-1] |= (spar[1] & 0x4) ? XTSE_1_02_C_TS_101388 : 0x0;
      /* G992.2 Annex A*/
      xtse[2-1] |= (spar[1] & 0x8) ? XTSE_2_01_A_2_NO : 0x0;
      
      /* 
         Map SPAR2 octet to the XTSE capabilities
      */
      /* No ADSL info*/

      /* 
         Map SPAR3 octet to the XTSE capabilities
      */
      /* G992.3 Annex A*/
      xtse[3-1] |= (spar[3] & 0x1) ? XTSE_3_03_A_3_NO : 0x0;
      /* G992.3 Annex L*/
      xtse[5-1] |= (spar[3] & 0x1) ? (XTSE_5_03_L_3_NO | XTSE_5_04_L_3_NO) : 0x0;
      /* G992.3 Annex B*/
      xtse[3-1] |= (spar[3] & 0x2) ? XTSE_3_05_B_3_NO : 0x0;
      /* G992.3 Annex I*/
      xtse[4-1] |= (spar[3] & 0x4) ? XTSE_4_05_I_3_NO : 0x0;
      /* G992.3 Annex J*/
      xtse[4-1] |= (spar[3] & 0x8) ? XTSE_4_07_J_3_NO : 0x0;
      /* G992.4 Annex A*/
      xtse[4-1] |= (spar[3] & 0x10) ? XTSE_4_01_A_4_NO : 0x0;
      
      /* 
         Map SPAR4 octet to the XTSE capabilities
      */
      /* G992.5 Annex A*/
      xtse[6-1] |= (spar[2] & 0x1) ? XTSE_6_01_A_5_NO : 0x0;
      /* G992.5 Annex B*/
      xtse[6-1] |= (spar[2] & 0x2) ? XTSE_6_03_B_5_NO : 0x0;
      /* G992.5 Annex I*/
      xtse[6-1] |= (spar[2] & 0x4) ? XTSE_6_07_I_5_NO : 0x0;
      /* G992.3 Annex M*/
      xtse[5-1] |= (spar[2] & 0x8) ? XTSE_5_07_M_3_NO : 0x0;
      /* G992.5 Annex J*/
      xtse[7-1] |= (spar[2] & 0x10) ? XTSE_7_01_J_5_NO : 0x0;
      
      /*
         Map SPAR5 octet to the XTSE capabilities
      */
      /* G992.5 Annex M*/
      xtse[7-1] |= (spar[5] & 0x1) ? XTSE_7_03_M_5_NO : 0x0;

      /* Set FE XTSE capabilities in the DSL CPE Context*/
      DSL_CTX_WRITE(pContext, nErrCode, lineInventoryFe.XTSECapabilities, xtse);
   }

   /* Get Self Test Result information*/
   if (DSL_DRV_DANUBE_CmvRead(pContext, DSL_CMV_GROUP_INFO,
      DSL_CMV_ADDRESS_INFO_SELFTEST, 2, 2,
      (DSL_uint16_t*)&pContext->lineInventoryFe.SelfTestResult) != DSL_SUCCESS)
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't get self test result!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);

   DSL_DEBUG(DSL_DBG_MSG, (pContext, "DSL[%02d]: OUT - DSL_DRV_DANUBE_LineInventoryFeRead, "
      "retCode=%d"DSL_DRV_CRLF, DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_ShowtimeRebootCheck(
   DSL_Context_t *pContext,
   DSL_DEV_RebootCheckResult_t *pResult)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEV_VersionCheck_t nFwVer = DSL_VERSION_ERROR;

   DSL_CHECK_POINTER(pContext, pResult);
   DSL_CHECK_ERR_CODE();

#ifdef INCLUDE_CALLBACK_SUPPORT
   nErrCode = DSL_DRV_DANUBE_FirmwareVersionCheck(pContext, DSL_MIN_FW_VERSION_CB,
      DSL_FW_ANNEX_NA, &nFwVer);

   if (nErrCode >= DSL_SUCCESS)
   {
      if (nFwVer >= DSL_VERSION_EQUAL)
      {
         /* Autoboot should not reboot because firmware supports reboot itself */
         *pResult = DSL_DEV_REBOOT_NO_ACTION;
      }
      else
      {
         /* Autoboot should reboot firmware */
         *pResult = DSL_DEV_REBOOT_UNHANDLED;
      }
   }
#else
   *pResult = DSL_DEV_REBOOT_UNHANDLED;
#endif

   return nErrCode;
}

DSL_Error_t DSL_DRV_DEV_TestModeControlSet(
   DSL_IN DSL_Context_t *pContext,
   DSL_IN_OUT DSL_TestModeControl_t *pData)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_TestModeControlSet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CTX_WRITE(pContext, nErrCode, nTestModeControl, pData->data.nTestMode);
   
   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_TestModeControlSet"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return (nErrCode);
}


/* LED block */

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device.h'
*/
DSL_uint32_t DSL_DRV_DEV_LED_FirmwareInit(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* Configure GPIO[10] for OUTPUT */
   DSL_uint16_t nData=(1<<10);

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_DEV_LED_FirmwareInit"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, pContext->pDevCtx);
   DSL_CHECK_ERR_CODE();
   if(DSL_DRV_MUTEX_LOCK(pContext->bspMutex))
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock device mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
   DSL_BSP_AdslLedInit(pContext->pDevCtx->lowHandle, DSL_LED_LINK_ID,
                            DSL_LED_LINK_TYPE, DSL_LED_HD_FW);

#if defined(INCLUDE_ADSL_LED) && \
   (defined(INCLUDE_DSL_CPE_API_AMAZON_SE) || defined(INCLUDE_DSL_CPE_API_AMAZON_S))
   /* Configure GPIO[9] for OUTPUT */
   nData |= (1<<9);
#endif

   /*  Setup ADSL Link/Data LED */
   /*  setup ADSL GPIO as output mode */
   if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
      DSL_CMV_ADDRESS_INFO_GPIO_CONTROL, 0, 1, &nData) != DSL_SUCCESS)
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - CMV send failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERR_MSG_EXCHANGE;
   }
   else
   {
      /*  setup ADSL GPIO mask */
      if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_GPIO_CONTROL, 2, 1, &nData) != DSL_SUCCESS)
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - CMV send failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         nErrCode = DSL_ERR_MSG_EXCHANGE;
      }
      else
      {
/* Sercomm Mark for LED Control */
#if 0      
         /*  Let FW to handle ADSL Link LED */
#if defined (INCLUDE_DSL_CPE_API_DANUBE)
         nData=0x0a03;
#elif (defined (INCLUDE_DSL_CPE_API_AMAZON_SE) || defined (INCLUDE_DSL_CPE_API_AMAZON_S))
         nData=0x0901;
#endif
         if (DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
            DSL_CMV_ADDRESS_INFO_GPIO_CONTROL, 4, 1, &nData) != DSL_SUCCESS)
         {
            DSL_DEBUG(DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - CMV send failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            nErrCode = DSL_ERR_MSG_EXCHANGE;
         }
 #endif
      }
   }

#if defined(INCLUDE_ADSL_LED) && \
   (defined(INCLUDE_DSL_CPE_API_AMAZON_SE) || defined (INCLUDE_DSL_CPE_API_AMAZON_S))
   if (nErrCode == DSL_SUCCESS)
   {
      /*  Turn ADSL Data LED on */
      nData=0x0A00;
      nErrCode = DSL_DRV_DANUBE_CmvWrite(pContext, DSL_CMV_GROUP_INFO,
         DSL_CMV_ADDRESS_INFO_GPIO_CONTROL, 5, 1, &nData);

      if (nErrCode == DSL_SUCCESS)
      {
         DSL_BSP_AdslLedInit(pContext->pDevCtx->lowHandle, DSL_LED_DATA_ID,
            DSL_LED_DATA_TYPE, DSL_LED_HD_FW);
      }
   }
#endif

   DSL_DRV_MUTEX_UNLOCK(pContext->bspMutex);
   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_DEV_LED_FirmwareInit"
      ", retCode=%d"DSL_DRV_CRLF, nErrCode, DSL_DEV_NUM(pContext)));

   return nErrCode;
}
#endif /* #if defined (INCLUDE_DSL_CPE_API_DANUBE) || defined (INCLUDE_DSL_CPE_API_AMAZON_SE) ||
              defined (INCLUDE_DSL_CPE_API_AMAZON_S)*/
